import "dst.q"

function H = hyperbol_shearlet_radialfilters(sz : vec,J)
    [M, N] = sz[0..1]
    assert(type(M,"scalar"))
    assert(type(N,"scalar"))
    [x, y] = meshgrid((1..N)-N/2-1,((1..M)-M/2-1)*(N/M))

    a = 0.1
    #pragma force_parallel
    for m=0..M-1
        for n=0..N-1
            if abs(x[m,n])<1e-1
                x[m,n]=1e-1*sign(x[m,n])
            endif
            if abs(y[m,n])<1e-1
                y[m,n]=1e-1*sign(y[m,n])
            endif
        end
    end
    r1 = ((-a*(x+y)+sqrt(a^2*(x-y).^2+4*x.*y))./(2*x.*y)) .* (x .* y >= 0)
    r2 = ((-a*(x+y)-sqrt(a^2*(x-y).^2-4*x.*y))./(2*x.*y)) .* (x .* y < 0)
    r = zeros(size(r1))

    max_r = max(abs(r1))/(N*8)
    assert(type(max_r,"scalar"))

    #pragma force_parallel
    for m=0..M-1
        for n=0..N-1
            r[m,n] = x[0,n] .* y[m,0] == 0 ? 0.0 : _
                     x[0,n] .* y[m,0] >= 0 ? r1[m,n] / max_r : r2[m,n] / max_r
        end
    end

    function H=meyer_radial_wavelet_filters(sz,J)

        N=256    
        omega=(0..N-1)*(pi/N)
        v = x -> 3*x.^2-2*x.^3
        
        H0 = (omega < pi/2) + _
             cos((pi/2)*v(2*(omega/pi)-1)).*((pi/2)<=omega).*(omega<=pi)
        G0 = sin((pi/2)*v(2*(omega/pi)-1)).*((pi/2)<=omega).*(omega<=pi) + _
             (omega > pi)

        % cascade algorithm for subsequent scales
        H=cell(J+1)
        H[0] = copy(G0)
        lo = copy(H0)
        for j=1..J-1
            H[j] = copy(lo)
            v=0..N/(2^j)-1
            H[j][v] = H[j][v] .* G0[0..(2^j)..N-1]        
            lo[v]   = lo[v]   .* H0[0..(2^j)..N-1]
            lo[N/(2^j)..N-1] = 0 
        end
        H[J]=lo

        for j=0..J
            H[j] = repmat(fftshift2(interp1(0..N-1,H[j],r)),[1,1,sz[2]])
        end
    end

    r = (N*8)./r
    assert(type(r,"mat"))
    
    H = meyer_radial_wavelet_filters(sz,J)
end

#pragma loop_parallelizer(off)

function [] = main()    

    [M,N] = [512,512]
    J = 4
    K = 8
    H = hyperbol_shearlet_radialfilters([M,N,3], J)
    G = wedge_filter([M,N,3],K,0.5)

    imshow(fftshift2(H[2].*G[0]),[0,2])
    %return
    img = imread("lena_big.tif")

    w = shearlet_analysis(img,J,K,H,G,0)
    band=w[4][0]
    %band[size(band,0)/2,size(band,1)/2] = 1.0
    img_out = shearlet_synthesis(w,J,K,H,G,0)
    

%    print min(r),",",max(r)
%    imshow(log(r+1e-8),[])
    %imshow(fftshift2(H[3]),[])
    imshow(img_out,[])
    %imshow(abs(ifftshift2(fft2(img_out))),[])

end