%==============================================================================
% imboundary.q
% Helper module for dealing with the image boundaries in a generic way - 
% used by functions that require the image boundary access method to be 
% variable (for example set using a parameter)
%==============================================================================

type boundextfunction : [__device__ (cube, ivec3) -> scalar]
type boundextfunction_RGB : [__device__ (cube, ivec2) -> vec3]

% Zero boundary extension
%boundext_zeros : boundextfunction
boundext_zeros = __device__ (x : cube, pos : ivec3) -> x[pos]

%boundext_zeros_RGB : boundextfunction_RGB
boundext_zeros_RGB = __device__ (x : cube, pos : ivec2) -> x[pos[0],pos[1],0..2]

% Circular boundary extension
%boundext_circular : boundextfunction
boundext_circular = __device__ (x : cube'circular, pos : ivec3) -> x[pos]

%boundext_circular_RGB : boundextfunction_RGB
boundext_circular_RGB = __device__ (x : cube, pos : ivec2) -> _
    x[periodize(pos[0],size(x,0)),periodize(pos[1],size(x,1)),0..2]
    
% Mirrored boundary extension
%boundext_mirror : boundextfunction
boundext_mirror = __device__ (x : cube'mirror, pos : ivec3) -> x[pos]

%boundext_mirror_RGB : boundextfunction_RGB
boundext_mirror_RGB = __device__ (x : cube, pos : ivec2) -> _
    x[mirror_ext(pos[0],size(x,0)),mirror_ext(pos[1],size(x,1)),0..2]

% Clamped boundary extension
%boundext_clamped : boundextfunction
boundext_clamped = __device__ (x : cube'clamped, pos : ivec3) -> x[pos]

%boundext_clamped_RGB : boundextfunction_RGB
boundext_clamped_RGB = __device__ (x : cube, pos : ivec2) -> _
    x[clamp(pos[0],size(x,0)),clamp(pos[1],size(x,1)),0..2]
