%==============================================================================
% generic_linkedlist.q
% Implementations of linked lists: so far only doubly linked is available.
% The implementation is not particularily safe or shielded, so beware!
% Because it is a bad habit to linearily index linked lists, I won't enable you!
% Simon Donné, November 2015
% Bart Goossens, Feb 2017
%==============================================================================

{!author name="Simon Donné, Bart Goossens"}
{!doc category="Data structures"}

type LinkedListElement[T] : mutable class
    value : ^T
    previous : ^LinkedListElement[T]
    next : ^LinkedListElement[T]
end

type LinkedList[T] : mutable class
    first : ^LinkedListElement[T]
    last : ^LinkedListElement[T]
    count : int
end

function [node:LinkedListElement[T]] = LinkedListElement[T](t:T)
    node = LinkedListElement[T](value:=t,previous:=nullptr,next:=nullptr)
end
    
function [list:LinkedList[T]] = LinkedList[T](t:T)
    element = LinkedListElement[T](t) % very quirky requirement, does not do anything but is required to allow execution
    list = LinkedList[T](first:=nullptr,last:=nullptr,count:=0)
end

function [element:^LinkedListElement[T]] = insert_before[T](self:^LinkedList[T],node:^LinkedListElement[T],value:T)
    element = LinkedListElement[T](value)
    if self.first == node
        self.first = element
    else
        node.previous.next = element
    endif
    
    element.next = node
    element.previous = node.previous
    node.previous = element
    self.count += 1
end

function [element:^LinkedListElement[T]] = insert_first[T](self:^LinkedList[T], value:T)
    % first element in the linked list
    if self.first == nullptr
        element = LinkedListElement[T](value)
        self.first = element
        self.last  = element
        self.count = 1
    else
        element = self.insert_before(self.first, value)
    endif
end


function [element:^LinkedListElement[T]] = insert_after[T](self:^LinkedList[T],node:^LinkedListElement[T],value:T)
    element = LinkedListElement[T](value)
    if self.last == node
        self.last = element
    else
        node.next.previous = element
    endif
    
    element.previous = node
    element.next = node.next
    node.next = element
    self.count += 1
end

function [element:^LinkedListElement[T]] = insert_last[T](self:^LinkedList[T], value:T)
    last = self.last
    if last == nullptr
        element = LinkedListElement[T](value)
        self.first = element
        self.last  = element
        self.count = 1
    else
        element = self.insert_after(last, value)
    endif
end

function [] = remove[T](self:^LinkedList[T],node:^LinkedListElement[T])
    if self.first == node
        self.first = node.next
    else
        node.previous.next = node.next
    endif
    if self.last == node
        self.last = node.previous
    else
        node.next.previous = node.previous
    endif
end

function [] = display[T](self:^LinkedList[T])
    print("LinkedList:")
    element = self.first
    if element == nullptr
        print("<empty>")
    else
        while element != nullptr
            print(element.value)
            element = element.next
        end
    endif
end

% TODO: clear

function [] = main()
    value : mat = zeros(0,0)
    %initialisation test
    list = LinkedList[cube](value)
    
    %insert_last test
    cub = ones(2,2) * 2
    entry2 = list.insert_last(cub)
    cub = ones(2,2) * 5
    entry5 = list.insert_last(cub)
    list.display()
    
    assert(list.first.value[1,1] == 2)
    assert(list.last.value[1,1]  == 5)
    
    %insert tests
    element = list.first
    cub = ones(2,2) * 1
    entry1 = list.insert_before(element,cub)
    list.display()
    cub = ones(2,2) * 3
    entry3 = list.insert_after(element,cub)
    list.display()
    
    element = list.last
    cub = ones(2,2) * 6
    entry6 = list.insert_after(element,cub)
    list.display()
    cub = ones(2,2) * 4
    entry4 = list.insert_before(element,cub)
    list.display()
    
    element = list.first
    for i = 1..6
        assert(element!=nullptr)
        assert(element.value[1,1] == i)
        element = element.next
    end
    
    %remove tests
    list.remove(entry2)
    list.remove(entry5)
    list.display()
    assert(list.first.next.value[1,1] == 3)
    assert(list.first.next.previous.value[1,1] == 1)
    
    list.remove(entry1)
    list.remove(entry6)
    list.display()
    assert(list.first.previous == nullptr)
    assert(list.last.next == nullptr)
    assert(list.first.next == list.last)
    assert(list.last.previous == list.first)
    
    list.remove(entry3)
    list.remove(entry4)
    list.display()
    assert(list.first == nullptr)
    assert(list.last == nullptr)
    
    %congratulatory message
    print("All tests succeeded, code functional!")
end