
function I : mat = __device__ inv(X : mat)

    %[rows,cols] = size(X,0..1)
    rows = size(X,0)
    cols = size(X,1)
    if rows != cols
        error "(inv) Matrix must be square!"
    endif
   
    Y = copy(X)
    I = eye(rows) 
    
    for i=0..rows-1
        {!parallel for}
        for pos = 0..cols-1
            if pos != i
                Yii = Y[i,i]
                I[i,pos] = I[i,pos] / Yii
                Y[i,pos] = Y[i,pos] / Yii
            endif
        endfor
        I[i,i] = I[i,i] / Y[i,i]
        Y[i,i] = 1.0
        
        {!parallel for}
        for m = 0..rows-1
            for n = 0..cols-1
                if m != i
                    Yii = Y[m, i]
                    I[m,n] = I[m,n] - I[i, n] * Yii
                    if n != i
                        Y[m,n] = Y[m,n] - Y[i, n] * Yii
                    endif
                endif
            endfor
        endfor
        {!parallel for}
        for pos = 0..rows-1
            if pos != i
                Y[pos,i] = 0
            endif
        endfor
    endfor
endfunction

function y = __device__ det(A:mat'unchecked(2,2))
    y = A[0,0] * A[1,1] - A[0,1] * A[1,0];
endfunction

function U = __device__ inv(A:mat'unchecked(2,2))
    d = det(A)
    if d == 0
        error "Matrix not invertible"
    endif
    inv_det = 1/d
    U = [[A[1,1], -A[0,1]], [-A[1,0], A[0,0]]] * inv_det
endfunction

function y = __device__ det(A:mat'unchecked(3,3))
    y = A[0, 0] * (A[1, 1] * A[2, 2] - A[2, 1] * A[1, 2]) -
        A[0, 1] * (A[1, 0] * A[2, 2] - A[1, 2] * A[2, 0]) +
        A[0, 2] * (A[1, 0] * A[2, 1] - A[1, 1] * A[2, 0]);
endfunction

function U = __device__ inv(A:mat'unchecked(3,3))
    d = det(A)
    if d == 0
        error "Matrix not invertible"
    endif
    invdet = 1 / d

    U = uninit(3,3)
    U[0, 0] = (A[1, 1] * A[2, 2] - A[2, 1] * A[1, 2]) * invdet
    U[0, 1] = (A[0, 2] * A[2, 1] - A[0, 1] * A[2, 2]) * invdet
    U[0, 2] = (A[0, 1] * A[1, 2] - A[0, 2] * A[1, 1]) * invdet
    U[1, 0] = (A[1, 2] * A[2, 0] - A[1, 0] * A[2, 2]) * invdet
    U[1, 1] = (A[0, 0] * A[2, 2] - A[0, 2] * A[2, 0]) * invdet
    U[1, 2] = (A[1, 0] * A[0, 2] - A[0, 0] * A[1, 2]) * invdet
    U[2, 0] = (A[1, 0] * A[2, 1] - A[2, 0] * A[1, 1]) * invdet
    U[2, 1] = (A[2, 0] * A[0, 1] - A[0, 0] * A[2, 1]) * invdet
    U[2, 2] = (A[0, 0] * A[1, 1] - A[1, 0] * A[0, 1]) * invdet
endfunction

function U = __device__ inv(A:mat'unchecked(4,4))
    U = uninit(4,4)
    U[0,0] = A[1,1]  * A[2,2] * A[3,3] -
             A[1,1]  * A[2,3] * A[3,2] -
             A[2,1]  * A[1,2]  * A[3,3] +
             A[2,1]  * A[1,3]  * A[3,2] +
             A[3,1] * A[1,2]  * A[2,3] -
             A[3,1] * A[1,3]  * A[2,2];

    U[1,0] = -A[1,0]  * A[2,2] * A[3,3] +
              A[1,0]  * A[2,3] * A[3,2] +
              A[2,0]  * A[1,2]  * A[3,3] -
              A[2,0]  * A[1,3]  * A[3,2] -
              A[3,0] * A[1,2]  * A[2,3] +
              A[3,0] * A[1,3]  * A[2,2];

    U[2,0] = A[1,0]  * A[2,1] * A[3,3] -
             A[1,0]  * A[2,3] * A[3,1] -
             A[2,0]  * A[1,1] * A[3,3] +
             A[2,0]  * A[1,3] * A[3,1] +
             A[3,0] * A[1,1] * A[2,3] -
             A[3,0] * A[1,3] * A[2,1];

    U[3,0] = -A[1,0]  * A[2,1] * A[3,2] +
               A[1,0]  * A[2,2] * A[3,1] +
               A[2,0]  * A[1,1] * A[3,2] -
               A[2,0]  * A[1,2] * A[3,1] -
               A[3,0] * A[1,1] * A[2,2] +
               A[3,0] * A[1,2] * A[2,1];

    U[0,1] = -A[0,1]  * A[2,2] * A[3,3] +
              A[0,1]  * A[2,3] * A[3,2] +
              A[2,1]  * A[0,2] * A[3,3] -
              A[2,1]  * A[0,3] * A[3,2] -
              A[3,1] * A[0,2] * A[2,3] +
              A[3,1] * A[0,3] * A[2,2];

    U[1,1] = A[0,0]  * A[2,2] * A[3,3] -
             A[0,0]  * A[2,3] * A[3,2] -
             A[2,0]  * A[0,2] * A[3,3] +
             A[2,0]  * A[0,3] * A[3,2] +
             A[3,0] * A[0,2] * A[2,3] -
             A[3,0] * A[0,3] * A[2,2];

    U[2,1] = -A[0,0]  * A[2,1] * A[3,3] +
              A[0,0]  * A[2,3] * A[3,1] +
              A[2,0]  * A[0,1] * A[3,3] -
              A[2,0]  * A[0,3] * A[3,1] -
              A[3,0] * A[0,1] * A[2,3] +
              A[3,0] * A[0,3] * A[2,1];

    U[3,1] = A[0,0]  * A[2,1] * A[3,2] -
              A[0,0]  * A[2,2] * A[3,1] -
              A[2,0]  * A[0,1] * A[3,2] +
              A[2,0]  * A[0,2] * A[3,1] +
              A[3,0] * A[0,1] * A[2,2] -
              A[3,0] * A[0,2] * A[2,1];

    U[0,2] = A[0,1]  * A[1,2] * A[3,3] -
             A[0,1]  * A[1,3] * A[3,2] -
             A[1,1]  * A[0,2] * A[3,3] +
             A[1,1]  * A[0,3] * A[3,2] +
             A[3,1] * A[0,2] * A[1,3] -
             A[3,1] * A[0,3] * A[1,2];

    U[1,2] = -A[0,0]  * A[1,2] * A[3,3] +
              A[0,0]  * A[1,3] * A[3,2] +
              A[1,0]  * A[0,2] * A[3,3] -
              A[1,0]  * A[0,3] * A[3,2] -
              A[3,0] * A[0,2] * A[1,3] +
              A[3,0] * A[0,3] * A[1,2];

    U[2,2] = A[0,0]  * A[1,1] * A[3,3] -
              A[0,0]  * A[1,3] * A[3,1] -
              A[1,0]  * A[0,1] * A[3,3] +
              A[1,0]  * A[0,3] * A[3,1] +
              A[3,0] * A[0,1] * A[1,3] -
              A[3,0] * A[0,3] * A[1,1];

    U[3,2] = -A[0,0]  * A[1,1] * A[3,2] +
               A[0,0]  * A[1,2] * A[3,1] +
               A[1,0]  * A[0,1] * A[3,2] -
               A[1,0]  * A[0,2] * A[3,1] -
               A[3,0] * A[0,1] * A[1,2] +
               A[3,0] * A[0,2] * A[1,1];

    U[0,3] = -A[0,1] * A[1,2] * A[2,3] +
              A[0,1] * A[1,3] * A[2,2] +
              A[1,1] * A[0,2] * A[2,3] -
              A[1,1] * A[0,3] * A[2,2] -
              A[2,1] * A[0,2] * A[1,3] +
              A[2,1] * A[0,3] * A[1,2];

    U[1,3] = A[0,0] * A[1,2] * A[2,3] -
             A[0,0] * A[1,3] * A[2,2] -
             A[1,0] * A[0,2] * A[2,3] +
             A[1,0] * A[0,3] * A[2,2] +
             A[2,0] * A[0,2] * A[1,3] -
             A[2,0] * A[0,3] * A[1,2];

    U[2,3] = -A[0,0] * A[1,1] * A[2,3] +
               A[0,0] * A[1,3] * A[2,1] +
               A[1,0] * A[0,1] * A[2,3] -
               A[1,0] * A[0,3] * A[2,1] -
               A[2,0] * A[0,1] * A[1,3] +
               A[2,0] * A[0,3] * A[1,1];

    U[3,3] = A[0,0] * A[1,1] * A[2,2] -
              A[0,0] * A[1,2] * A[2,1] -
              A[1,0] * A[0,1] * A[2,2] +
              A[1,0] * A[0,2] * A[2,1] +
              A[2,0] * A[0,1] * A[1,2] -
              A[2,0] * A[0,2] * A[1,1];

    det = A[0,0] * U[0,0] + A[0,1] * U[1,0] + A[0,2] * U[2,0] + A[0,3] * U[3,0]
    if det == 0
        error "Matrix not invertible"
    endif
    U *= 1.0 / det;
endfunction

function U = __device__ cholesky(A:mat'unchecked(4,4))
    U = zeros(4,4)
    U[0,0] = sqrt(A[0,0])
    U[1,0] = A[1,0] / U[0,0]
    U[1,1] = sqrt(A[1,1] - U[1,0]^2)
    U[2,0] = A[2,0] / U[0,0]
    U[2,1] = (A[2,1] - U[2,0]*U[1,0]) / U[1,1]
    U[2,2] = sqrt(A[2,2] - (U[2,0]^2 + U[2,1]^2))
    U[3,0] = A[3,0] / U[0,0]
    U[3,1] = (A[3,1] - U[3,0]*U[1,0]) / U[1,1]
    U[3,2] = (A[3,2] - (U[3,0]*U[2,0] + U[3,1]*U[2,1])) / U[2,2]
    U[3,3] = sqrt(A[3,3] - (U[3,0]^2 + U[3,1]^2 + U[3,2]^2)) 
endfunction

function U = __device__ cholesky(A:mat'unchecked(3,3))
    U = zeros(3,3)
    U[0,0] = sqrt(A[0,0])
    U[1,0] = A[1,0] / U[0,0]
    U[1,1] = sqrt(A[1,1] - U[1,0]^2)
    U[2,0] = A[2,0] / U[0,0]
    U[2,1] = (A[2,1] - U[2,0]*U[1,0]) / U[1,1]
    U[2,2] = sqrt(A[2,2] - (U[2,0]^2 + U[2,1]^2))
endfunction

function U = __device__ cholesky(A:mat'unchecked(2,2))
    U = [[sqrt(A[0,0]), 0], _
         [A[1,0] / sqrt(A[0,0]), sqrt(A[1,1] - (A[1,0] / sqrt(A[0,0]))^2)]]
endfunction

function [] = main()
    x2 = randn(2,2); print(inv(x2)*x2)
    x3 = randn(3,3); print(inv(x3)*x3)
    x4 = randn(4,4); print(inv(x4)*x4)

    print cholesky(x2*transpose(x2))
    print cholesky(x3*transpose(x3))
    print cholesky(x4*transpose(x4))
endfunction
