import "SLIC.q"
import "imresize.q"
import "inttypes.q"
import "Quasar.UI.dll"
import "Quasar.Video.dll"

%TODO: standing issue - drift to the top left corner
%something spawns squares that move to the left and top. The squares DO change colour, but never shape or size

type VideoSLIC : mutable class
    cell_size : int
    kseeds : cube
    klabels : cube[int16]
    nlabels : mat
    maxlab : mat
    maxxy : mat
end

reduction (v:VideoSLIC, img:cube) -> v.apply_to(img) = slicvid_draw_labels(v,img)
reduction (sz:int) -> VideoSLIC(sz) = VideoSLIC(cell_size:=sz,kseeds:=zeros(0,0,0),klabels:=cube[int16](0,0,0),nlabels:=zeros(0,0),maxlab:=zeros(0,0),maxxy:=zeros(0,0))
reduction (v:VideoSLIC, img:cube, its:int) -> v.process(img, its) = SLICish(img, v, its)
reduction (v:VideoSLIC) -> v.get_glass_image() = slicvid_get_glass_image(v)

function [] = slicvid_draw_labels(v:VideoSLIC, img:cube)
    if numel(v.nlabels) == 0
        v.nlabels = enforce_connectivity(v.klabels, v.cell_size)
    endif
    
    apply_gSLIC_connected_borders(img,v.nlabels)
end
function [image:cube] = slicvid_get_glass_image(v:VideoSLIC)
    [M,N] = size(v.klabels,0..1)
    image = zeros(M,N,3)
    function [] = __kernel__ fill_image(klabels:cube[int16], kseeds:cube, pos:ivec2)
        yk = klabels[pos[0],pos[1],0]
        xk = klabels[pos[0],pos[1],1]
        for c = 0..2
            image[pos[0],pos[1],c] = kseeds[yk,xk,c]
        end
    end
    parallel_do([M,N], v.klabels, v.kseeds, fill_image)
end

function [] = main()
%    webcamslic()
    
%    stop
    
    img = imread("lena_big.tif")
    iterations = 1
    
    v = VideoSLIC(256)
    
    repeat
        tic()
        v.process(img, iterations)
        toc()
        
        dimg = img+0
        v.apply_to(dimg)
        imshow(dimg)
    until !hold("on")
end

function [] = webcamslic()
    cams = vidcamlist()
    if numel(cams)==0
        error "Unfortunately, we could not find a webcam in your system."
    endif
    cam = cams[0]
    stream = vidopen(sprintf("cam:video=""%s"",video_size=640x480,frame_rate=30",cam))
    
    
    [M,N,C] = size(stream.rgb_data)

    v = VideoSLIC(256)
    
    framenr = 1
    
    repeat
        vidreadframe(stream)
        new_frame = float(stream.rgb_data)
        
        %flip the frame to correct orientation
        function [] = __kernel__ verticalflip(image:cube,pos:ivec3)
            t = image[M-1-pos[0],N-1-pos[1],pos[2]]
            image[M-1-pos[0],N-1-pos[1],pos[2]] = image[pos]
            image[pos] = t
        end
%        parallel_do([int(M/2),N,C],new_frame,verticalflip)
        
        %perform SLIC.q
        tic()
        v.process(new_frame, 1)
        toc()
        
        img = v.get_glass_image()
        img2 = img + 0
        v.apply_to(img2)
%        imwrite(sprintf("glass%04d.png",framenr),img)
%        imwrite(sprintf("applied%04d.png",framenr),img2)
%        imwrite(sprintf("input%04d.png",framenr),new_frame)
%        imwrite(sprintf("frame%04d.png",framenr),cat(cat(new_frame,img2),img))
        imshow(img2)
        
        framenr = framenr + 1
        
    until !hold("on")
end

function [] = SLICish(image:cube, v:VideoSLIC, iterations:int)
    [M,N] = size(image, 0..1)
    superpixel_size = v.cell_size
    step_size = int(sqrt(superpixel_size)+0.5)
    
    if numel(v.kseeds) == 0
        seed_perturbation = true
        hexgrid = true
        v.kseeds  = get_LabXY_seeds_for_given_stepsize(step_size, seed_perturbation, image, hexgrid)
        
        perform_superpixel_SLICish(v, image, iterations)
    else
        perform_superpixel_SLICish(v, image, iterations)
    endif
    
    %invalidate the nlabels
    v.nlabels = zeros(0,0)
end

%//===========================================================================
%/// GetLABXYSeeds_ForGivenStepSize
%///
%/// The k seed values are taken as uniform spatial pixel samples.
%//===========================================================================
function [kseeds:cube] = get_LabXY_seeds_for_given_stepsize(step_size:int, seed_perturbation:int, image:cube, hexgrid:int, additional_fields:int=0)
    [M,N] = size(image, 0..1)
    
    ystrips = int(0.5+(M/step_size))
    yerr = int(M - step_size*ystrips)
    if yerr < 0
        ystrips-=1
        yerr = M - step_size*ystrips
    endif
    yerrperstrip = yerr/ystrips
    
    xstrips = int(0.5+(N/step_size))
    xerr = int(N - step_size*xstrips)
    if xerr < 0
        xstrips-=1
        xerr = N - step_size*xstrips
    endif
    xerrperstrip = xerr/xstrips
    
    yoff = int(step_size/2)
    xoff = int(step_size/2)
    
    kseeds = zeros(ystrips, xstrips, 5+additional_fields)
    
    function [] = __kernel__ get_kseeds(hexgrid:int, pos:ivec2)
        seedx = pos[1]*step_size+xoff+pos[1] * xerrperstrip
        if(hexgrid)
            seedx = pos[1]*step_size+pos[1] * xerrperstrip + (mod(pos[0],2)?2*xoff:xoff)
            seedx = min(N-1.,seedx)
        endif
        seedy = pos[0]*step_size+yoff+pos[0] * yerrperstrip
        kseeds[pos[0], pos[1], 0] = image[seedy,seedx,0]
        kseeds[pos[0], pos[1], 1] = image[seedy,seedx,1]
        kseeds[pos[0], pos[1], 2] = image[seedy,seedx,2]
        kseeds[pos[0], pos[1], 3] = seedy
        kseeds[pos[0], pos[1], 4] = seedx
    end
    parallel_do([ystrips,xstrips],hexgrid,get_kseeds)
    
    if(seed_perturbation)
        edgemag = detect_edges(image)    
        perturb_seeds(kseeds, edgemag, image)
    endif
end

function [] = perturb_seeds(kseeds:cube, edgemag:mat, image:cube)
    dx8 = [-1,-1,0,1,1,1,0,-1]
    dy8 = [0,-1,-1,-1,0,1,1,1]
    function [] = __kernel__ perturbation_kernel(pos:ivec2)
        oy = kseeds[pos[0], pos[1], 3]
        ox = kseeds[pos[0], pos[1], 4]
        besty = oy
        bestx = ox
        for i = 0..7
            ny = oy + dy8[i]
            nx = ox + dx8[i]    
            if edgemag[ny,nx] < edgemag[besty,bestx]
                besty = ny
                bestx = nx
            endif
        end
        if oy!=besty || ox!=bestx
            kseeds[pos[0], pos[1], 0] = image[besty,bestx,0]
            kseeds[pos[0], pos[1], 1] = image[besty,bestx,1]
            kseeds[pos[0], pos[1], 2] = image[besty,bestx,2]
            kseeds[pos[0],pos[1],3] = besty
            kseeds[pos[0],pos[1],4] = bestx
        endif
    end
    parallel_do(size(edgemag), perturbation_kernel)
end

function [edgemag:mat] = detect_edges(image:cube)
    edgemag = uninit(size(image, 0..1))
    function [] = __kernel__ get_mags(image_in:cube'const, edgemag_out:mat, pos:ivec2)
        y = 1+clamp(pos[0]-1, size(image_in,0)-3)
        x = 1+clamp(pos[1]-1, size(image_in,1)-3)
        dx = (image_in[y,x+1,0]-image_in[y,x-1,0])^2+(image_in[y,x+1,1]-image_in[y,x-1,1])^2+(image_in[y,x+1,2]-image_in[y,x-1,2])^2
        dy = (image_in[y+1,x,0]-image_in[y-1,x,0])^2+(image_in[y+1,x,1]-image_in[y-1,x,1])^2+(image_in[y+1,x,2]-image_in[y-1,x,2])^2
        edgemag_out[pos] = dx^2 + dy^2
    end
    parallel_do(size(edgemag), image, edgemag, get_mags)
end

%assign the max color distance for each cluster
function [] = __device__ update_maxes(klabels:cube[int16], maxlab:mat, distlab:mat, maxxy:mat, distxy:mat)
    for i = 0..size(klabels, 0)
    for j = 0..size(klabels, 1)
        yk = klabels[i,j,0]
        xk = klabels[i,j,1]
        maxlab[yk,xk] = max(distlab[i,j],maxlab[yk,xk])
        maxxy[yk,xk] = max(distxy[i,j],maxxy[yk,xk])
    end
    end
end

%todo: current issue: if a superpixel becomes empty, it is never used again
function [] = perform_superpixel_SLICish(v:VideoSLIC, image:cube, iterations:int)
    [M,N]   = size(image, 0..1)
    [Mk,Nk] = size(v.kseeds, 0..1)
    superpixel_size = v.cell_size
    step_size = int(sqrt(superpixel_size)+0.5)
    
    if numel(v.klabels) == 0
        v.klabels = cube[int16](M,N,2)
        function [] = __kernel__ initlabels(target:cube[int16], pos:ivec2)
            target[pos[0],pos[1],0] = (pos[0]*Mk)/M
            target[pos[0],pos[1],1] = (pos[1]*Nk)/N
        end
        parallel_do([M,N], v.klabels, initlabels)
        v.maxlab = ones(Mk,Nk)*10^2   %decides the variable value of the compactness. Start with 10²
        v.maxxy  = ones(Mk,Nk)*step_size^1.25 %decides the variable value of the subpixel size. Start with step²
    endif
    
    offset = step_size<10?int(step_size*1.5):step_size
    
    dists  = -ones(M,N)
    distlab = -ones(M,N)
    distxy  = -ones(M,N)
    
    invxywt = 1/step_size^2
    
    for iteration = 1..iterations
        dists = -ones(M,N)
        
        function [] = __kernel__ firstloop(klabels:cube[int16], kseeds:cube, maxlab:mat, maxxy:mat, pos:ivec2)
            y1 = max(0,int(kseeds[pos[0],pos[1],3]-offset))
            y2 = min(M, int(kseeds[pos[0],pos[1],3]+offset))
            x1 = max(0,int(kseeds[pos[0],pos[1],4]-offset))
            x2 = min(N, int(kseeds[pos[0],pos[1],4]+offset))
            for y = y1..y2-1
            for x = x1..x2-1
                distlab[y,x] = (image[y,x,0]-kseeds[pos[0],pos[1],0])^2+(image[y,x,1]-kseeds[pos[0],pos[1],1])^2+(image[y,x,2]-kseeds[pos[0],pos[1],2])^2
                distxy[y,x]  = (y-kseeds[pos[0],pos[1],3])^2+(x-kseeds[pos[0],pos[1],4])^2
                
%                dist = distlab[y,x]/maxlab[pos] + distxy[y,x]*invxywt %only varying compactness, prettier superpixels
                dist = distlab[y,x]/maxlab[pos] + distxy[y,x]/maxxy[pos] %varying both compactness and step_size
                
                if(dist<dists[y,x] || dists[y,x] < 0)
                    dists[y,x] = dist
                    klabels[y,x,0] = pos[0]
                    klabels[y,x,1] = pos[1]
                endif
            end
            end
        end
        parallel_do([Mk,Nk], v.klabels, v.kseeds, v.maxlab, v.maxxy, firstloop)
        
%        v.maxlab = zeros(size(v.maxlab))
%        v.maxxy = zeros(size(v.maxxy))
%        update_maxes(v.klabels, v.maxlab, distlab, v.maxxy, distxy)
        
        %recalculate the centroid and store in the seed values
        v.kseeds = zeros(Mk,Nk,5)
        cluster_sizes = zeros(Mk,Nk)
        function [] = __kernel__ fillsigmas(klabels:cube[int16], kseeds:cube, pos:ivec2)
            kseeds[klabels[pos[0],pos[1],0],klabels[pos[0],pos[1],1],0] += image[pos[0],pos[1],0]
            kseeds[klabels[pos[0],pos[1],0],klabels[pos[0],pos[1],1],1] += image[pos[0],pos[1],1]
            kseeds[klabels[pos[0],pos[1],0],klabels[pos[0],pos[1],1],2] += image[pos[0],pos[1],2]
            kseeds[klabels[pos[0],pos[1],0],klabels[pos[0],pos[1],1],3] += pos[0]
            kseeds[klabels[pos[0],pos[1],0],klabels[pos[0],pos[1],1],4] += pos[1]
            cluster_sizes[klabels[pos[0],pos[1],0],klabels[pos[0],pos[1],1]] += 1
        end
        parallel_do([M,N],v.klabels, v.kseeds, fillsigmas)
                
        v.kseeds = v.kseeds./repmat(max(1, cluster_sizes),[1,1,5])
    end
end