%RUN_DISABLED
{!author name="Bart Goossens"}
{!doc category="Samples/Advanced CUDA/SIMD processing demo"}
{!title name="SIMD processing demo"}

% Section: Overview
%
% Demonstrates how to use the integer SIMD vectorization
% facilities of Quasar. On GPU: four 8-bit integer operations
% are mapped onto a 32-bit integer access. On x64 CPU, sixteen
% 8-bit integers are combined to form an 128-bit vector that is 
% processed using SSE2 instructions.
K : int'const = 8

type u8vec : vec[uint8]
type u8vec4 : vec[uint8](4)

function [im_out] = im_to_uint8[N](im : mat)
    type rmat : mat[uint8'sat]
    im_out = rmat(size(im))
    for m=0..size(im_out,0)-1
        for n=0..size(im_out,1)-1
            {!kernel_transform enable="simdprocessing"; target="gpu"; simd_width=N}
            {!kernel_transform enable="simdprocessing"; target="cpu"}
            im_out[m,n] = im[m,n]
        endfor
    endfor
endfunction

function [] = __kernel__ filter_kernel(im8 : mat[uint8], im_out : mat[uint8]'unchecked, pos : ivec2)
    r = 0 
    for x=0..K-1         
        r += im8[pos[0],pos[1]+x]
    endfor     
    im_out[pos[0],pos[1]] = int(r/(2*K))
endfunction

im = imread("lena_big.tif", "rgba")
im8 : mat[uint8](1024,1024) = im_to_uint8[4](repmat(im[:,:,1],[2,2]))
im_out = mat[uint8](size(im8))

runs = 100

tic()
for k=1..runs
    {!parallel for}
    for m=0..size(im8,0)-1
        for n=0..size(im8,1)-1
            {!kernel_transform enable="simdprocessing"}
            r = 0
            for x=0..K-1         
                r += im8[m,n+x]
            endfor     
            im_out[m,n] = int(r/(2*K))
        endfor
    endfor
endfor
syncthreads(host)
toc("For loop")

tic()
for k=1..runs
    parallel_do(size(im_out),im8,im_out,filter_kernel)
endfor
syncthreads(host)
toc("Kernel (unvectorized)")

tic()
for k=1..runs
    {!parallel for}
    for m=0..size(im8,0)-1
        for n=0..4..size(im8,1)-1
            r2 = vec[int](4)
            for x=0..K-1         
                r2 += im8[m,n+x+(0..3)]
            endfor     
            im_out[m,n+(0..3)] = int(r2/(2*K))
        endfor
    endfor
endfor
syncthreads(host)
toc("For loop2 (explicitly vectorized)")

