{!author name="Bart Goossens"}
{!doc category="Samples/Advanced CUDA/Kernel fusion"}
{!title name="Demonstrates how kernel fusion can be used to speed up code"}

% Section: Overview
%
% In this example, a 3x3 covariance matrix of an image is calculated, in a relative hardware-
% agnostic way. Without kernel fusion, this leads to at least 4 kernel function launches.
% 
function [C : mat(3,3)] = calculate_covariance[kernel_fusion](im : cube(:,:,3))
    if kernel_fusion
        {!kernel_fusion strategy="manual"}
    endif
    
    mu = [mean(im[:,:,0]),mean(im[:,:,1]),mean(im[:,:,2])]
    C = zeros(3,3)
    
    {!parallel for; dim=4}
    for [m,n] = 0..size(im,0..1)-1
        for [k,l]=0..2
            C[k,l] += (im[m,n,k] - mu[k]) * (im[m,n,l] - mu[l])
        endfor
    endfor
    
    C = C / (size(im,0) * size(im,1))
endfunction


im = imread("lena_big.tif")

approx = (x,y)->mean(abs(x-y))/max(mean(abs(x)),mean(abs(y))) < 0.01

runs = 1000
tic()
for run=1..runs
cov = calculate_covariance[false](im)
endfor
toc("without kernel fusion")


tic()
for run=1..runs
cov = calculate_covariance[true](im)
endfor
toc("with kernel fusion")
ref = [[2406, 2279, 1130], _
        [2279, 2796, 1640], _
        [1130, 1640, 1160]]
print "ref=",ref
print "cov=",cov
print success = approx(cov,ref)