{!author name="Bart Goossens"}
{!doc category="Samples/Advanced CUDA/No copy arguments"}
{!title name="No copy arguments to avoid unnecessary memory transfers to the GPU device"}

% Section: Overview
%
% 'nocopy is a special modifier that indicates that an array/matrix does not need to be transferred
% to the target device because the kernel is going to overwrite it completely. By doing so, a 
% useless memory transfer is avoided. By default, the compiler tries to identify automatically
% the parameters to which 'nocopy can safely be applied. In some cases, it may be useful to 
% manually annotate the parameter types (for example, when using a large scratchpad with values
% that are "don't care").
% 
function [] = default_copy()
    function [] = __kernel__ kernel(x : cube, y : cube, pos : ivec3)
        y[pos] = x[pos]
    end
    
    x = zeros(768,1024,3)
    y = uninit(size(x))
    parallel_do(size(x),x,y,kernel)
end

function [] = nocopy_copy()
    function [] = __kernel__ kernel(x : cube, y : cube'nocopy, pos : ivec3)
        y[pos] = x[pos]
    end

    x = zeros(768,1024,3)
    y = uninit(size(x))
    parallel_do(size(x),x,y,kernel)
end

% Note: the two functions above only differ in the 'nocopy modifier. Their performance is
% the same, due to the compiler also adding 'nocopy automatically to the first kernel.

num_runs=1000
tic()
for i=0..num_runs-1
    default_copy()
end
toc("default_copy")

tic()
for i=0..num_runs-1
    nocopy_copy()
end
toc("nocopy_copy")