import "Quasar.CompMath.dll" % New Computational Math library
% Sample uses Algorithmic Differentiation library. See documentation for the explanation.

{!title name="Total variation with Gradient Descent"}
{!author name="Bart Goossens"}
{!doc category="Samples/Algorithmic Differentiation"}

reduction x -> $diff(abs(x),x) = sign(x)

% C = sum((g - b).^2) + lambda*|dg/dx| + lambda*|dg/dy|

function C = costfunc(g : mat, b : mat, lambda : scalar)
    C = 0.0
    for m=0..size(g,0)-1
        for n=0..size(g,1)-1
            C += (g[m,n] - b[m,n]).^2 % data fidelity
            C += lambda*abs(g[m,n-1] - g[m,n]) % horizontal partial derivative
            C += lambda*abs(g[m-1,n] - g[m,n]) % vertical partial derivative
        endfor
    endfor
endfunction

% gradient_descent is a generic function in Quasar (like C++ templates) because it 
% requires access to cost_fn. This is: as long as cost_fn cannot be determined at
% compile-time, the function cannot be used from user-code.
function y = gradient_descent(cost_fn, x, step_size=0.01, max_iter=100, epsilon=1e-4)    
    % Maple-like syntax for obtaining the gradient
    gradient = $unapply($diff(cost_fn(x), x), x)

    % Initial solution
    y = x

    for k=0..max_iter
        y_old = y
        y = y_old - step_size * gradient(y)

        print "iteration ",k+1, ": ", cost_fn(y)

        % Stop condition
        if sum(abs(y-y_old)) < epsilon
            break
        endif
    endfor

endfunction

function [] = main()
    
    im = imread("lena_big.tif")[:,:,1]
    im_noisy = im + 25 * randn(size(im))
    lambda = 40

    % Call the gradient descent algorithm to derive the cost function
    % The explicit lambda syntax is required for closure variable binding. This
    % way, the variables im_noisy and lambda, which are defined locally, can be
    % passed to the cost function. 
    im_den = gradient_descent(((x) -> costfunc(x, im_noisy, lambda)), im_noisy)

    %imshow(im_den,[0,255])

    psnr = (x, y) -> 10*log10(255^2/mean((x - y).^2))
    print "psnr_in=", psnr(im_noisy, im), " dB"
    print "psnr_out=", psnr(im_den, im), " dB"

endfunction