{!author name="Bart Goossens"}
{!doc category="Samples/Basic/Color temperature"}
{!title name="A simple color temperature image filter"}

% Section: Overview
%
% Demonstrates applying a single kernel to an input image.
% The Color Temperature filter allows chaning color temperature of the light source in an image. Depending on the temperature set, the image can look more reddish or more blueish. This example is mainly for illustrational purposes of the parallel programming techniques.
%

% lerp needs to be annotated with __device__ in order to be called from a kernel function.
lerp = __device__ (a,b,c) -> a*c+b*(1-c)

function [] = __kernel__ color_temperature(x : cube, y : cube, temp : scalar, cold : vec3, hot : vec3, pos : vec2)

    input = x[pos[0],pos[1],0..2]
    if temp<0
        output = lerp(input,cold,(-0.25)*temp)
    else
        output = lerp(input,hot,0.25*temp)
    endif
    y[pos[0],pos[1],0..2] = output

endfunction

function [img_out] = apply(img_in, temp)
    hot = [1,0.2,0]*255
    cold = [0.3,0.4,1]*255
    img_out = zeros(size(img_in))
    parallel_do(size(img_out,0..1),img_in,img_out,temp,cold,hot,color_temperature);
endfunction

function [] = main()
   
    img_in = imread("lena_big.tif")
	img_out = apply(img_in, temp:=1)
    imshow(img_out)

endfunction
