%
% gamma_correction.q
%
% Bart Goossens, Aug 2012
%
{!author name="Bart Goossens"}
{!doc category="Samples/Basic/Gamma correction"}
{!title name="Gamma correction implemented as a kernel function"}

% Section: Overview
%
% Illustration of a simple point-wise operation in Quasar: a gamma correction, using the
% kernel function:
%
% :gamma_correction = __kernel__ (pos) -> _
% :             y[pos] = 255*(y[pos]*(1.0/255))^gamma
%
% In this example, no data types need to be specified to allow compilation. The type of 'pos' is 
% determined using type inference
%
x = imread("lena_big.tif")
y = copy(x)
gamma = 0.4

% Define the kernel function
gamma_correction = __kernel__ (pos) -> _
	y[pos] = 255*(y[pos]*(1.0/255))^gamma

% Apply the gamma correction in parallel
parallel_do(size(y),gamma_correction)

imshow(y)
