{!author name="Bart Goossens"}
{!doc category="Samples/Basic/Lookup table and sharpening"}
{!title name="Illustrates basic operations, such as filtering and lookup tables"}

% Section: Overview
%
% This example illustrates basic operations, such as filtering and lookup tables

lut = clamp(255*(linspace(0,255,65536)/255).^1.9, 255)

% Function: kernel
% 
% The main kernel in this program. Performs sharpening as well as a gamma correction (via a lookup 
% table). The hardware texturing unit is used for this purpose, with clamped access.
% 
% Usage:
%   : function [] = __kernel__ kernel(img : cube'const'clamped'hwtex_nearest(4), img_out : cube'unchecked, lut : vec'const'clamped'hwtex_nearest, pos : ivec'const'unchecked(2))
% 
% Parameters:
% img - the input image
% img_out - the output image
% lut - the lookup table
% pos - the position in the image
function [] = __kernel__ kernel(img : cube'hwtex_nearest(4)'clamped, img_out : cube'unchecked, lut : vec'hwtex_nearest'clamped, pos : ivec2)

    sharpen = img[...pos,0..2] * 2.0 - 0.25 * (
        img[pos[0],pos[1]-1,0..2] +
        img[pos[0],pos[1]+1,0..2] +
        img[pos[0]-1,pos[1],0..2] +
        img[pos[0]+1,pos[1],0..2])

    img_out[...pos,0..2] = [lut[sharpen[0]], lut[sharpen[1]], lut[sharpen[2]]]

endfunction

img = imread("lighthouse.png")*256
img_out = uninit(size(img))

parallel_do(size(img,0..1), img, img_out, lut, kernel)

imshow(img_out/256)

