%
% subdivide_tile.q
%
% Bart Goossens, Sept 2020
%
{!author name="Bart Goossens"}
{!doc category="Samples/Basic/Subdivide tile"}
{!title name="Shows how to use matrix slices to subdivide an image recursively"}

function [] = subdivide(im : cube, remaining_depth : int, m1 : int, n1 : int, m2 : int, n2 : int, quarter : int)

    m3 = int((m1 + m2)/2)
    n3 = int((n1 + n2)/2)
    
    block = im[m1..2..m2-1,n1..2..n2-1,:]
    im[m1..m3-1,n1..n3-1,:] = block
    im[m3..m2-1,n1..n3-1,:] = block
    im[m1..m3-1,n3..n2-1,:] = block
    im[m3..m2-1,n3..n2-1,:] = block

    if remaining_depth > 0
        match quarter with 
        | 0 ->
            subdivide(im, remaining_depth - 1, m1, n1, m3, n3, quarter)
        | 1 ->
            subdivide(im, remaining_depth - 1, m3, n1, m2, n3, quarter)
        | 2 ->
            subdivide(im, remaining_depth - 1, m1, n3, m3, n2, quarter)
        | 3 ->
            subdivide(im, remaining_depth - 1, m3, n3, m2, n2, quarter)
        endmatch
    else
        
    
    endif

endfunction

im = repmat(imread("lighthouse.png"), [2,2,1])

N = 4
subdivide(im, N, 0, 0, size(im,0)/2, size(im,1)/2, 3)
subdivide(im, N, size(im,0)/2, 0, size(im,0), size(im,1)/2, 2)
subdivide(im, N, 0, size(im,1)/2, size(im,0)/2, size(im,1), 1)
subdivide(im, N, size(im,0)/2, size(im,1)/2, size(im,0), size(im,1), 0)

imshow(im)
