%REQUIRES_DISPLAY
{!author name="Bart Goossens"}
{!doc category="Samples/Basic/Subplot"}
{!title name="Displaying multiple images using the subplot function"}

% Section: Overview
%
% Example of using the subplot function for 
% displaying multiple images within the same window, using the function <qdisplay/subplot>
%
import "imresize.q"
import "Quasar.UI.dll"

function [] = main()

    % Load an image and subsample it
    im = imread("lena_big.tif")
    im_subsampled = im[0..2..511, 0..2..511, :]

    new_size = [512,512,3]
    
    % Try different interpolation methods
    im_bilin   = imresize(im_subsampled, new_size, "linear")
    im_cubic = imresize(im_subsampled, new_size, "cubic")
    im_sinc = imresize(im_subsampled, new_size, "sinc")
    
    frm = form("Subplot demo")
    disp = frm.add_display()
    disp.subplot(2,2,0)
    h1=disp.imshow(im)
    disp.title("<b>Original image</b>")
    disp.subplot(2,2,1)
    h2=disp.imshow(im_bilin)
    disp.title("<b>Bilinear interpolation</b>")
    disp.subplot(2,2,2)
    h3=disp.imshow(im_cubic)
    disp.title("<b><i>Cubic interpolation</i></b>")
    disp.subplot(2,2,3)
    h4=disp.imshow(im_sinc)
    disp.title("<b><span color='red'>Sinc interpolation</span></b>")

    % Connects the different windows
    h1.connect(h2)
    h1.connect(h3)
    h1.connect(h4)
    frm.wait()
end
