%
% imshow.q
%
% Bart Goossens, Aug-Sept 2012
%
{!author name="Bart Goossens"}
{!doc category="Samples/Graphics/Showing images and videos"}
{!title name="Displaying images"}

import "Quasar.UI.dll"

% Section: Overview
%
% This demo show how the imshow functions can be used to 
% display video. This is possible due to the OpenGL interoperability.
% Here, images containing white noise are generated, using the function 
% randn. When the program is run using the CUDA computation engine, the noise 
% samples are stored in GPU memory. The visualization then displays the 
% noise samples directly from GPU memory.
%
% For the CPU computation engine, the data is transferred to the video
% memory using OpenGL streaming buffers.
%
frm = form("Quasar OpenGL rendering demo")
frm.width = 800
disp1 = frm.add_display()
disp2 = frm.add_display()
frm.show()

% Uncomment if you only want to display 20 frames/second
disp1.sync_framerate(20)
disp2.sync_framerate(20)

tic()
frames = 0
while !frm.closed()
    disp1.imshow(randn(256,512,3),[-1,1]) % color noise
    disp2.imshow(randn(256,512),[-1,1])   % monochrome noise
    pause(0)
    frames += 1
end
time = toc()

print frames
print frames/(time)*1000, " FPS"
