%REQUIRES_DISPLAY
% opengl_cube.q
% Nov 2012
{!author name="Bart Goossens"}
{!doc category="Samples/Graphics/OpenGL cube"}
{!title name="OpenGL cube visualization demo"}

% Section: Overview
%
% This sample demonstrates how OpenGL vertex buffers can be used from Quasar.
% It suffices to define the right data structures (with position, normal and texcoord).
% The data structures will be automatically mapped onto OpenGL vertex formats. 
% Then, using a generic vector construction, an OpenGL vertex buffer can be constructed
% and displayed. This is useful for interactive 3D visualization.

import "Quasar.UI.dll"

type texcoord : class
    u : scalar
    v : scalar
end

type vector3 : class
    x : scalar
    y : scalar
    z : scalar
end

type vertex : class
    position : vector3
    normal : vector3
    texcoord : texcoord
    %texture : int
end

type vertex_buffer : vec[vertex]

function [v : vec[vertex]] = build_cube()
    v = vertex_buffer(24)

    % Front face
    v[0] = vertex(position:=vector3(x:=-0.5,y:= 0.5,z:=-0.5),normal:=vector3(x:=0,y:=0,z:=-1),texcoord:=texcoord(u:=0,v:=1))
    v[1] = vertex(position:=vector3(x:= 0.5,y:= 0.5,z:=-0.5),normal:=vector3(x:=0,y:=0,z:=-1),texcoord:=texcoord(u:=1,v:=1))
    v[2] = vertex(position:=vector3(x:= 0.5,y:=-0.5,z:=-0.5),normal:=vector3(x:=0,y:=0,z:=-1),texcoord:=texcoord(u:=1,v:=0))
    v[3] = vertex(position:=vector3(x:=-0.5,y:=-0.5,z:=-0.5),normal:=vector3(x:=0,y:=0,z:=-1),texcoord:=texcoord(u:=0,v:=0))

    % Back face
    v[4] = vertex(position:=vector3(x:=-0.5,y:=-0.5,z:=0.5),normal:=vector3(x:=0,y:=0,z:=1),texcoord:=texcoord(u:=0,v:=0))
    v[5] = vertex(position:=vector3(x:= 0.5,y:=-0.5,z:=0.5),normal:=vector3(x:=0,y:=0,z:=1),texcoord:=texcoord(u:=1,v:=0))
    v[6] = vertex(position:=vector3(x:= 0.5,y:= 0.5,z:=0.5),normal:=vector3(x:=0,y:=0,z:=1),texcoord:=texcoord(u:=1,v:=1))
    v[7] = vertex(position:=vector3(x:=-0.5,y:= 0.5,z:=0.5),normal:=vector3(x:=0,y:=0,z:=1),texcoord:=texcoord(u:=0,v:=1))

    % Other faces
    {!parallel for}
    for k = 0..7
        v[8+k] =  vertex(position:=vector3(x:=v[k].position.z,y:=v[k].position.x,z:=v[k].position.y), _
                         normal  :=vector3(x:=v[k].normal.z,  y:=v[k].normal.x,  z:=v[k].normal.y),texcoord:=v[k].texcoord)
        v[16+k] = vertex(position:=vector3(x:=v[k].position.y,y:=v[k].position.z,z:=v[k].position.x), _
                         normal  :=vector3(x:=v[k].normal.y,  y:=v[k].normal.z,  z:=v[k].normal.x),texcoord:=v[k].texcoord)
    end
end   

function [x : mat, y : mat] = meshgrid(range_x : vec, range_y : vec)
    x = ones(numel(range_y),1) * range_x
    y = transpose(range_y) * ones(1,numel(range_x))
end

function [] = main()

    im = imread("Quasar.jpg")    
    frm = form("Simple OpenGL visualization demo")
    frm.width = 800
    disp = frm.add_display()
    disp.sync_framerate(40) % Render at 40 FPS
    renderer = disp.create_opengl_renderer()
    renderer.add(build_cube(), "quads")
    frm.show()

    t = 0.0
    wave = zeros(size(im))
    [x,y] = meshgrid(linspace(-0.5,0.5,size(im,1)),linspace(-0.5,0.5,size(im,0)))

    while !frm.closed()
        t += 0.1
        wave[:,:,0] = wave[:,:,1] = wave[:,:,2] = cos(24*sqrt(x.^2+y.^2)-t).^8*32
        renderer.set_texture((im+wave)/255)
        pause(10)
    end

end