%REQUIRES_DISPLAY
% opengl_cube.q
% Nov 2012
{!author name="Bart Goossens"}
{!doc category="Samples/Graphics/OpenGL pointcloud"}
{!title name="OpenGL pointcloud visualization demo"}

% Section: Overview
%
% This sample demonstrates how OpenGL vertex buffers can be used from Quasar.
% It suffices to define the right data structures (with position, normal and texcoord).
% The data structures will be automatically mapped onto OpenGL vertex formats. 
% Then, using a generic vector construction, an OpenGL vertex buffer can be constructed
% and displayed. This is useful for interactive 3D visualization.

import "Quasar.UI.dll"

type vector3 : class
    x : scalar
    y : scalar
    z : scalar
end

type rgb_color : class
    x : scalar
    y : scalar
    z : scalar
end

type vertex : class
    position : vector3
    color : rgb_color
end

function [] = update_pointcloud(v : vec[vertex], A : mat, t : scalar)    

    % Calculate the coordinates based on the current time-index
    coord = 12*abs(A).^(3+2*sin(t)).*sign(A)

    % Fill the vertex buffer with the vertices
    {!parallel for}
    for k = 0..numel(v)-1
        r = sum(abs(coord[k,0..2]))
        v[k] = vertex(position:=vector3(x:=coord[k,0], y:=coord[k,1], z:=coord[k,2]), color:=rgb_color(x:=r, y:=0.0, z:=1-r))
    end
end   

function [] = main()
    frm = form("OpenGL point-cloud visualization demo")
    frm.width = 800                 % Sets the width of the form
    disp = frm.add_display()        % Attaches a display object
    disp.sync_framerate(40)         % Render at 40 FPS
    renderer = disp.create_opengl_renderer() % Creates the OpenGL 3D renderer
    layer = new(qvectorlayer3d)     % Creates a 3D vector layer
    renderer.add(layer, "layer")    % Adds the vector layer to the renderer
    frm.show()                      % Displays the form (otherwise the form remains hidden)

    % Generate a set of random coordinates
    %A = rand(1000000,3)-0.5
    A = 2*rand(1000000,3)-1.0
    print min(A), "-", max(A)
    v = vec[vertex](size(A,0))      % Initializes the vertex buffer
    layer.drawpoints(v)             % Draws the vertices onto the 3D vector layer
    % Note that the vertices have not been initialized at this stage.
    t = 0.0
    while !frm.closed()
        t += 0.01                   % Time step
        update_pointcloud(v, A, t)  % Updates the point cloud with new vertex data
        layer.update()              % Marks the surface for updating
        pause(10)
    end
end