%REQUIRES_DISPLAY
{!author name="Bart Goossens"}
{!doc category="Samples/Graphics/Quiver plot"}
{!title name="Quiver plot example program"}

% Section: Overview
% 
% Demonstrates how to add a vector layer to an image, in order to display a quiver plot.
% The quiver plot is generated by first filling a vertex buffer (containing the coordinates of the
% arrows). Next, the vertex buffer is rendered using OpenGL, via layer.drawlines.
import "Quasar.Video.dll"
import "optical_flow_farneback.q"

function [] = quiver(layer : qvectorlayer, vis : cmat, arrow_size : scalar, B : scalar)
    layer.setpencolor("limegreen")
    
    vertex_buffer = uninit(numel(vis) * 12)
    
    {!parallel for}
    for m=0..size(vis,0)-1
        for n=0..size(vis,1)-1
            % Draw arrow
            c = vis[m,n]
            if abs(c) > 0 || true
                d = cos(pi/6)*1i + sin(pi/6)
                p1 = B.*([n,m] + 0.5)
                p2 = p1 + arrow_size * [imag(c), real(c)]
                p3 = p2 - 0.3 * arrow_size * [imag(c*d), real(c*d)]
                p4 = p2 - 0.3 * arrow_size * [imag(c*conj(d)), real(c*conj(d))]
                index = pos2ind(size(vis), [m,n]) * 12
                vertex_buffer[index + (0..1)] = p1
                vertex_buffer[index + (2..3)] = p2
                vertex_buffer[index + (4..5)] = p2
                vertex_buffer[index + (6..7)] = p3
                vertex_buffer[index + (8..9)] = p2
                vertex_buffer[index + (10..11)] = p4
            endif
        endfor
    endfor  
    layer.drawlines(vertex_buffer)
endfunction

function [] = main() 
    pyramid_factor = 1/4
    window_size = 15
    its = 3
    poly_n = 5
    poly_sigma = 0.7
    
    %read in an entire video sequence
    stream = vidopen("flowers.avi")
    
    frames = vec[mat](floor(stream.duration_sec * stream.avg_frame_rate))

    tic()
    if (vidreadframe(stream))
        new_frame = rgb2gray(float(stream.rgb_data))
        frm = form("Quiverplot sample")
        [frm.width, frm.height] = [stream.frame_width + 40, stream.frame_height + 80]
        dsp = frm.add_display()
        img = dsp.imshow(new_frame)
        layer = new(qvectorlayer)
        img.addlayer(layer)
        
        dsp.sync_framerate(25)
    
        while vidreadframe(stream) && !frm.closed()
            old_frame = new_frame
            new_frame = rgb2gray(float(stream.rgb_data))
            flow_p = optical_flow_farneback_parallel(new_frame, old_frame, pyramid_factor, window_size, its, poly_n, poly_sigma)
            dsp.imshow(new_frame)
            
            layer.clear()
            [M,N] = size(flow_p,0..1)
            B = 8
            quiver(layer, complex(flow_p[0..B..M-1,0..B..N-1,1], flow_p[0..B..M-1,0..B..N-1,0]), 4, B)
        endwhile
    else
        error "There was a problem reading this video (apparently there are no frames)"
    endif
    time = toc()        
    
endfunction