%REQUIRES_DISPLAY
{!author name="Bart Goossens"}
{!doc category="Samples/Graphics/Stereoview"}
{!title name="OpenGL stereo rendering sample"}

% Section: Overview
%
% Demonstration of stereo rendering using OpenGL.
% 
import "Quasar.UI.dll"
import "inttypes.q"

type vertexnormal : class
    position : vec3
    normal : vec3
end

% Draws a cube with dimensions [0,0,0]-[1,1,1] onto the given 3D vector layer.
function [] = draw_cube(layer : qvectorlayer3d)
    indices = [0,1,2,0,2,3, % Front
               5,4,7,5,7,6, % Back
               4,0,3,4,3,7, % Left
               1,5,6,1,6,2, % Right
               3,2,6,3,6,7, % Top
               4,5,1,4,1,0] % Bottom
    layer.translatetransform([-0.5,-0.5,0])
    p = vec[vertexnormal](8)
    p[0] = vertexnormal(position:=[0,0,1],normal:=[0,0,1])
    p[1] = vertexnormal(position:=[1,0,1],normal:=[1,0,1])
    p[2] = vertexnormal(position:=[1,1,1],normal:=[1,1,1])
    p[3] = vertexnormal(position:=[0,1,1],normal:=[0,1,1])
    p[4] = vertexnormal(position:=[0,0,0],normal:=[0,0,0])
    p[5] = vertexnormal(position:=[1,0,0],normal:=[1,0,0])
    p[6] = vertexnormal(position:=[1,1,0],normal:=[1,1,0])
    p[7] = vertexnormal(position:=[0,1,0],normal:=[0,1,0])
    layer.fillindexedtriangles(p, float_to_int32(indices))
    layer.translatetransform([0.5,0.5,0])
end

function [] = setup_lights(gl : qgldisplay)
    % Setup the scene lights
    light = gl.lights[0]
    light.position = [2,2,2]
    light.ambientcolor = [0.2,0.2,0.2,1]
    light.diffusecolor = [1,1,1,1]
    light.specularcolor = [1,1,1,1]
    light.enabled = true
end

function [] = draw_scene(layer : qvectorlayer3d)     
    
    layer.enable_lighting = true  
    
    % bottom plane
    layer.setcolor("purple")
    layer.drawbox([-0.5,-0.5,0],[0.5,0.5,0.5])
    layer.fillrect([0,0,0],[1,0,0],[0,1,0])
    
    layer.setcolor("yellow")
    layer.pushtransform()
    layer.translatetransform([-0.2,-0.2,0])
    layer.scaletransform(0.3*[1,1,1])
    draw_cube(layer)
    layer.poptransform()

end


function [] = main()
    visualise_3d_scene_stereoscopic()
end

function [] = visualise_3d_scene_stereoscopic()
    frm = form("Stereoscopic visualization")
    [frm.width, frm.height] = [800, 600]
    hor_layout = frm.add_horizontallayout()
    
    dispL = hor_layout.add_display()
    dispR = hor_layout.add_display()

    rendererL = dispL.create_opengl_renderer()
    rendererR = dispR.create_opengl_renderer()
    frm.show()
    
    rendererL.position = [ 0.1,0,0]
    rendererR.position = [-0.1,0,0]  
    
    setup_lights(rendererL)
    setup_lights(rendererR)
       
    % Create a vector layer and add it to the left and right renderers
    layer = new(qvectorlayer3d)
    draw_scene(layer)
    rendererL.add(layer, "layer")
    rendererR.add(layer, "layer")

    while !frm.closed() 
        rendererR.zoom  = rendererL.zoom    
        rendererR.pitch = rendererL.pitch  
        rendererR.yaw   = rendererL.yaw
        rendererR.roll  = rendererL.roll
        pause(10)
    end
end