{!author name="Bart Goossens"}
{!doc category="Samples/Image Processing/Anisotropic diffusion"}
{!title name="Shows how to implement an anisotropic diffusion filter"}

% Section: Overview
%
% This demo illustrates the Quasar implementation of an (an)istropic diffusion
% algorithm in Quasar, with suitable visualization.


import "imfilter.q"

function [] = __kernel__ diffusion(imarray : cube, result : cube, K : scalar, alpha : scalar, deltat : scalar, pos : ivec2)

    deltaIxu = imarray[pos[0], pos[1]+1, 0..2] - imarray[pos[0], pos[1], 0..2]
    deltaIxl = imarray[pos[0], pos[1], 0..2]   - imarray[pos[0], pos[1]-1, 0..2]
    
    deltaIyu = imarray[pos[0]+1, pos[1], 0..2] - imarray[pos[0], pos[1], 0..2]
    deltaIyl = imarray[pos[0], pos[1], 0..2]   - imarray[pos[0]-1, pos[1], 0..2]

    cxu = 1.0 ./ (1.0 + pow(abs(deltaIxu) / K, 1 + alpha))
    cxl = 1.0 ./ (1.0 + pow(abs(deltaIxl) / K, 1 + alpha))
    
    cyu = 1.0 ./ (1.0 + pow(abs(deltaIyu) / K, 1 + alpha))
    cyl = 1.0 ./ (1.0 + pow(abs(deltaIyl) / K, 1 + alpha))
    
    deltaI = (cxu .* deltaIxu - cxl .* deltaIxl) + (cyu .* deltaIyu - cyl .* deltaIyl)
    
    result[pos[0], pos[1], 0..2] = imarray[pos[0], pos[1], 0..2] + deltat * deltaI
end

function [] = main()
    im = imread("F35.jpg")
    sigma = 50
    noisy_im = im + sigma * randn(size(im))
    denoised_im = copy(noisy_im)
    result = uninit(size(denoised_im))
    K = 2
    alpha = 0.5
    deltat = 1
    num_iter = 450
    anisotropic = true
    
    for iter = 1..num_iter
        if anisotropic
            parallel_do(size(denoised_im), denoised_im, result, K, alpha, deltat, diffusion)
            denoised_im = copy(result)
        else
            denoised_im = gaussian_filter(denoised_im, 1, 0, "mirror")
        endif
        
        imshow(denoised_im,[0, 255])
        title(sprintf("Iteration %d", iter))
        hold("on")
    end
    
end