%REQUIRES_DISPLAY
{!author name="Bart Goossens"}
{!doc category="Samples/Image Processing/Optical flow"}
{!title name="Farneback optical flow demonstration"}

% Section: Overview
%
% Illustration of the Farneback optical flow algorithm on video.
% The program also draws a motion vector field using an imshow
% vector layer. When the visualization is done with OpenGL (default),
% the motion vectors are then drawn with OpenGL line primitives in
% a separate stage (hence unaffecting the displayed image). This can
% be noted when zooming in.
% 
import "optical_flow_farneback.q"

function [] = main()
    filename = "Media\streetColor.avi"

    pyramid_factor = 1/4
    window_size = 25
    its = 5
    poly_n = 5
    poly_sigma = 1
    
    stream = vidopen(filename)   
    frames = vec[mat](floor(stream.duration_sec * stream.avg_frame_rate))

    tic()
    if !vidreadframe(stream)
        error "There was a problem reading this video (apparently there are no frames)"
    endif
    i=0
    new_frame = rgb2gray(float(stream.rgb_data))
    
    frm = form()
    dsp = frm.add_display()
    dsp.sync_framerate(stream.avg_frame_rate)  
    
    % Please make sure that the layer is added when the image is displayed
    % for the first time. Later, the image will only be updated.
    h = dsp.imshow(stream.rgb_data)
    layer = new(qvectorlayer)
    h.addlayer(layer)
        
    while !frm.closed()
        if !vidreadframe(stream)
            vidseek(stream, 0)
        endif
        old_frame = new_frame
        new_frame = rgb2gray(float(stream.rgb_data))
        flow_p = optical_flow_farneback_parallel(new_frame, old_frame, pyramid_factor, window_size, its, poly_n, poly_sigma)
        
        % Update the layers
        layer.clear()
        layer.setpencolor([1,0,0])            
        theta = pi/6
        R = -0.4
        A = uninit(stream.frame_height/16,stream.frame_width/16,12)
        
        % Generate a motion vector field
        #pragma force_parallel
        for m0=0..int(stream.frame_height/16)-1
            for n0=0..int(stream.frame_width/16)-1
                [m, n] = [16 * m0, 16 * n0]
                p = [n, m]
                q = 1 + [n, m] - [flow_p[m,n,0], flow_p[m,n,1]]
                v = [flow_p[m,n,0], flow_p[m,n,1]]
                A[m0,n0,0..1] = p
                A[m0,n0,2..3] = q
                A[m0,n0,4..5] = q
                A[m0,n0,6..7] = q - R * sin(theta) * v - R * cos(theta) * [v[1],v[0]]
                A[m0,n0,8..9] = q
                A[m0,n0,10..11] = q - R * sin(theta) * v + R * cos(theta) * [v[1],v[0]]
            end
        end
        layer.drawlines(A)
        
        % Then: update the image
        %dsp.imshow(show_flow(flow_p[:,:,1], flow_p[:,:,0], 4))   
        dsp.imshow(stream.rgb_data) 
    end      
end
