# Interop samples - C++ API

This folder contains various Quasar vs. C++ interop samples: we demonstrate how Quasar functions can be integrated in C++.

The following sample programs are available:

1. `sample1_functioncall.cpp`: calling Quasar functions from C++
2. `sample2_paralleldo.cpp`: launching a Quasar kernel from C++
3. `sample3_inlineprogram.cpp`: embedding a Quasar program as a string in C++ code
4. `sample4_opencv.cpp`: mixing OpenCV and Quasar functions
5. `sample5_opencv_facedetection.cpp`: face detection via OpenCV and visualization via Quasar (OpenGL)
6. `sample6_cellmatrices.cpp`: using Quasar cell matrices in C++
7. `sample7_usertypes.cpp`: defining types in C++ that can be used from Quasar programs
8. `sample8_vectorization.cpp`:  code vectorization sample
9. `sample9_calculations.cpp`: performing calculations on Quasar matrices
10. `sample10_kernel_dynamicmemory.cpp`: demonstrates some of Quasar's dynamic memory allocation algorithms.
11. `sample11_cuda_interop.cpp`: combining CUDA code with Quasar code
12. `sample12_opencl_interop.cpp`: combining OpenCL code with Quasar code
13. `sample13_higherdimmatrices.cpp`: using higher dimensional arrays
15. `sample15_errorhandling.cpp`: using the custom exception handler mechanism for catching Quasar exceptions
16. `sample16_usertypematrix.cpp`: creating arrays of user types (structs).
17. `sample17_lambda.cpp`: wrapping C++ functions and lambdas so that they can be called from Quasar
18. `sample18_reductions.cpp`: defining Quasar reductions in C++
19. `sample20_multigpu.cpp`: multi-GPU processing sample
20. `sample21_deviceselect.cpp`: enumerating available computation devices and selecting a device dynamically.
21. `sample22_pointerwrapping.cpp`: hiding native C++ pointers in Quasar objects, for example for defining C++ classes that can be used from Quasar
22. `sample23_nativeclasses.cpp`: defining C++ classes that can be used from Quasar

A CMake file `CMakeLists.txt` is included to compile the samples. The program can be compiled as follows:

    # Linux
    cmake .  
    make

    # Windows (Visual Studio 2017)
    cmake -G "Visual Studio 15 2017" -A x64

Note that the environment variable `QUASAR_INCL_PATH` needs to point to the Quasar include directory (containing `quasar.h`, `quasar_dsl.h`), this may require some readjustment.