// This example demonstrates how to use the dynamic memory allocators included
// in the quasar runtime. The CPU allocator is significantly faster than the
// standard malloc/free functions, especially in multi-threaded mode, where a
// larger number of threads are allocating/freeing memory at the same time.
// There are however a few restrictions on the memory size that can be allocated,
// for the specific details we refer to the manual, on the topic of "Kernel
// dynamic memory allocation".
#define ENABLE_DYNAMIC_KERNEL_MEM
#include <ctime>
#include "quasar.h"

void sample10(void)
{
    using namespace quasar;


    DynMemConfiguration::GPUAllocator* allocator = new DynMemConfiguration::GPUAllocator();

	printf("Allocator size: %d\n", sizeof(*allocator));
    memset(allocator, 0, sizeof(*allocator));

	// Memory allocator stress test.
	printf("TEST1 : allocate and free... ");


	int data_races = 0;
	int t = clock();

	#pragma omp parallel for
    for (int k = 0; k < 100000000; k++)
    {
		int sz = rand()*32700/RAND_MAX;
        char *data = (char *)allocator->allocate(sz);
		int esz = allocator->get_size(data);
		if(esz != sz) {
			_atomicAdd(&data_races, 1);
			printf("%d ", sz);
		}
		if (!data)
			printf("Failed allocation!\n");
        allocator->free(data);
    }

	printf("%d msec\n", (int) ((clock() - t)*1000/CLOCKS_PER_SEC));

	if (data_races)
		printf("Data races detected: %d!\n", data_races);

	t = clock();
	printf("TEST2 : allocate and free...");
	const int N = 10000;
	char **items = new char* [N];
	#pragma omp parallel for
    for (int k = 0; k < N; k++)
    {
        items[k] = (char *)allocator->allocate(12);

		if (!items[k])
			printf("Failed allocation!\n");
    }

	#pragma omp parallel for
    for (int k = 0; k < N; k++)
    {
        allocator->free(items[k]);
    }
	printf("%d msec\n", (int) ((clock() - t)*1000/CLOCKS_PER_SEC));


	// Should be 0
	printf("count_reserved=%d\n", allocator->count_reserved());
	//printf("count_reserved_small=%d\n", allocator->count_reserved_small());
    delete allocator;
}
