// This sample demonstrates how to use higher-dimensional matrices
// using the C++ interface
#include "quasar_dsl.h"

void sample13(void)
{
    using namespace quasar;
	using quasar::ref;

	try
	{
		// Creates the quasar host and loads the specified module
		ref<IQuasarHost> host = IQuasarHost::Create(_T("cuda"), false);

		// Problem loading Quasar
		if (host == NULL)
			return;

		Function print(_T("print()"));
		Function sum(_T("sum(?\?)"));

		intN<4> dims = make_intN(2,3,4,2);
		QValue Q = QValue::CreateNCube<scalar, 4>(dims);
		NCubeBase<scalar, 4> Qdata = host->LockNCube<scalar, 4>(Q);
		for (int m = 0; m < prod(dims); m ++)
			Qdata.data[m] = m;
		host->UnlockNCube(Q);

		print(_T("The matrix is:"), Q);
		print(_T("The sum is:"), sum(Q));
		Q(0,0,3,0) = 58;
		print(Q(0,0,3,0));
	}
	catch (exception_t ex)
	{
		tprintf(_T("An error occurred:\nSource: %s\nMessage: %s\nStack trace: %s"),
			ex.source.get_buf(),
			ex.message.get_buf(),
			ex.stack_trace.get_buf());
	}
}
