#include "quasar_dsl.h"

bool custom_exception_handler(const quasar::exception_t & ex)
{
	tprintf(_T("Custom exception handler:\nsource: %s\nmessage: %s\nstack trace: %s\n"),
		(LPCTSTR) ex.source,
		(LPCTSTR) ex.message,
		(LPCTSTR) ex.stack_trace);
	return true;
}

void sample15(void)
{
    using namespace quasar;
	using quasar::ref;

	LPCTSTR errorMsg = NULL;

	try
	{
		// Creates the quasar host and loads the specified module
		ref<IQuasarHost> host = IQuasarHost::Create(_T("cuda"), true);

		// Problem loading Quasar
		if (host == NULL)
			return;


		QValue e(2.718281828f);
		Function ln(_T("log()"));
		Function print(_T("print(...)"));

		assert(host->FunctionExists(_T("log")));
		QValue result;

		// Default exception handler -> throw a C++ exception
		try
		{
			result = ln(e); // 'log' bound to function without arguments
		}
		catch (exception_t ex)
		{
			print(_T("An error occurred: "), ex.message);
		}

		host->SetCustomExceptionHandler(custom_exception_handler);

		// Try again, but with custom exception handler
		result = ln(e);

		print(_T("The logarithm of e is "), result);

		// Load a simple program and run it
		LPCTSTR sourceProgram =
			_T("function [] = fail()\n")
			_T("print \"Program to generate an error\";  \n")
			_T("error \"I fail, right here!\"\n")
			_T("end\n");

		if (!host->LoadModuleFromSource(_T("sample15"), sourceProgram, &errorMsg))
		{
			tprintf(_T("Error while compiling program:\n%s\n"), errorMsg);
			return;
		}

		Function fail(_T("fail()"));
		fail();
	}
	catch (exception_t ex)
	{
		tprintf(_T("An error occurred:\nSource: %s\nMessage: %s\nStack trace: %s"),
			ex.source.get_buf(),
			ex.message.get_buf(),
			ex.stack_trace.get_buf());
	}
}
