#include "quasar_dsl.h"

// Illustrates the construction of user-type matrices, via the C++ external interface
using namespace quasar;

struct vertex
{
    float x;
    float y;
	float z;
};

namespace quasar
{
    DECLARE_STRUCT_TYPE(vertex); // Generates type information for our struct
    IMPLEMENT_TYPE(TYPE_TYPEDOBJECT, vertex, _T("vertex"))
}

void sample16(void)
{
    using namespace quasar;
	using quasar::ref;


	try
	{
		// Creates the quasar host and loads the specified module
		ref<IQuasarHost> host = IQuasarHost::Create(_T("cuda"), true);

		Function print(_T("print()"));

		TypeBuilder builder(_T("sample16"), _T("vertex"));
		builder.AddField(_T("x"), Type(_T("scalar'single")));
		builder.AddField(_T("y"), Type(_T("scalar'single")));
		builder.AddField(_T("z"), Type(_T("scalar'single"))); // Single precision scalar number
		Type type = builder.CreateType();

		vertex vertices[3] = { {0,1,2 }, {2,0,1}, {1,2,0} };
		QValue array = vertices;

		QValue elem = QValue::FromObj(vertices[2]);
		print(_T("The vertex is: "), elem);

		QValue str = QValue(_T("Test string"));
		string_t strVal = str.operator string_t();
		tprintf(_T("String value: %s\n"), (LPCTSTR)strVal);

		vertex v = array(0).as<vertex>();
		tprintf(_T("Vertex read from array: (%g %g %g)\n"), v.x, v.y, v.z);
		v.x = 5;
		array(1) = v;

		print(_T("The list is: "), array);
		print(_T("The element at position 0 is: "), array(0));
	}
	catch (exception_t ex)
	{
		tprintf(_T("An error occurred:\nSource: %s\nMessage: %s\nStack trace: %s"),
			ex.source.get_buf(),
			ex.message.get_buf(),
			ex.stack_trace.get_buf());
	}
}
