#include "quasar_dsl.h"

using namespace quasar;

QValue calcsum_m3f_0(QValue f, QValue g, QValue x)
{
	QValue y = QValue();
	y = (QValue) f(x) + (QValue) g(x);
	return y;
}

void sample17(void)
{
	using namespace quasar;
	using quasar::ref;

	try
	{
		// Creates the quasar host and loads the specified module
		ref<IQuasarHost> host = IQuasarHost::Create(_T("cuda"), true);

#ifdef CPLUSPLUS14

		Function print(_T("print(...)"));

		/*auto*/std::function<QValue(QValue, QValue)> sum_two = [](QValue a, QValue b)
		{
			return a + b;
		};
		auto sum_two_int = [](int a, int b)
		{
			return a + b;
		};
		auto sum_all = [&](initializer_list<QValue> args)
		{
			QValue ret = 0.0;
			for (auto arg = args.begin(); arg < args.end(); arg++)
				ret = ret + *arg;
			return ret;
		};
		auto sum_variadic = [](const QValue *args, int N)
		{
			QValue sum = 0.0;
			for (int i = 0; i < N; i++)
				sum = sum + args[i];
			return sum;
		};
		print(sum_two(1.0, 2.0));
		print(sum_all({ 1.0, 2.0, 3.0 }));

		// Wrap the C++ lambda in a Lambda expression that can be used from Quasar. The
		// lambda function can then be passed to Quasar functions etc. For this purpose,
		// we explicitly specify the type of the function.
		Function sum_two_wrap = Function::CreateLambda(Type(_T("[(?\?,?\?)->?\?]")), (std::function<QValue(int, int)>) sum_two_int);

		// Wrap a variadic function
		Function sum_var_wrap = Function::CreateVariadicLambda(Type(_T("[(...?\?)->?\?]")), sum_variadic);

		// Call the wrapped lambda expression using Quasar
		print(sum_two_wrap(2.0, 3.0));
		print(sum_var_wrap(1.0, 2.0, 3.0));
		print(type(sum_var_wrap).ToString());

		// Lambda expression returning two values
		std::function<std::tuple<QValue, QValue>(QValue, QValue)> swap = [](QValue a, QValue b)
		{
			return std::tuple<QValue, QValue>(b, a);
		};

		std::function<std::tuple<QValue, QValue>()> give_two = []()
		{
			return std::tuple<QValue, QValue>(20, 39);
		};


		Function swap_wrap = Function::CreateLambda(Type(_T("[(?\?,?\?)->(?\?,?\?)]")), swap);
		Function give_two_wrap = Function::CreateLambda(Type(_T("[()->(?\?,?\?)]")), give_two);

		QValue qv_swap = swap_wrap;
		QValue qv_give_two = give_two_wrap;
		print(qv_swap(qv_give_two()));

		float a, b;
		//std::tie(a, b) = swap(2.0, 1.0);
		//std::tie(a, b) = swap_wrap.apply_multi<2>(2.0, 1.0);
		std::tie(a, b) = swap_wrap.operator() < 2 > (2, 1);
		//a = swap_wrap(2.0, 1.0);
		print(a, _T(" "), b);
		std::tuple<QValue, QValue> two = give_two_wrap.operator() < 2 > ();
#endif

		host->RunApp();
	}
	catch (exception_t ex)
	{
		tprintf(_T("An error occurred:\nSource: %s\nMessage: %s\nStack trace: %s"),
			ex.source.get_buf(),
			ex.message.get_buf(),
			ex.stack_trace.get_buf());
	}
}
