#include "quasar_dsl.h"

using namespace quasar;

QValue global_is_symmetric(QValue A)
{
	return true;
}

void sample18(void)
{
	using namespace quasar;
	using quasar::ref;

	try
	{
		// Creates the quasar host and loads the specified module
		ref<IQuasarHost> host = IQuasarHost::Create(_T("cuda"), true);
		CoreLib * coreLib = new CoreLib();

#ifdef CPLUSPLUS14

		std::function<QValue(QValue)> is_symmetric = [coreLib](QValue A)
		{
			return coreLib->min(A.ElemwiseEquals(coreLib->transpose(A)));
		};

		std::function<QValue(QValue)> is_notsymmetric = [is_symmetric](QValue A)
		{
			return !is_symmetric(A);
		};

		std::function<QValue(QValue)> symmetric_func = [coreLib](QValue A)
		{
			coreLib->print_((QValue)_T("The matrix is symmetric!"));
			return QValue(0);
		};

		std::function<QValue(QValue)> asymmetric_func = [coreLib](QValue A)
		{
			coreLib->print_((QValue)_T("The matrix is asymmetric!"));
			return QValue(1);
		};

		// Define a reduction dispatch
		//Reduction red1(_T("(A : mat) -> test_matrix(A)"),
		//	Function::CreateLambda(_T("[?\? -> ?\?]"), symmetric_func),
		//	Function::CreateLambda(_T("[?\? -> ?\?]"), is_symmetric));

		Reduction red1(_T("(A : mat) -> test_matrix(A)"),
			Function::CreateLambda(_T("[?\? -> ?\?]"), symmetric_func),
			global_is_symmetric);

		Reduction red2(_T("(A : mat) -> test_matrix(A)"),
			Function::CreateLambda(_T("[?\? -> ?\?]"), asymmetric_func),
			Function::CreateLambda(_T("[?\? -> ?\?]"), is_notsymmetric));


		Function test_matrix(_T("test_matrix(mat)"));
		QValue B = coreLib->eye(4);
		QValue C = coreLib->rand(4, 4);

		coreLib->print_(test_matrix(B));
		coreLib->print_(test_matrix(C));

#endif

		host->RunApp();
		delete coreLib;
	}
	catch (exception_t ex)
	{
		tprintf(_T("An error occurred:\nSource: %s\nMessage: %s\nStack trace: %s"),
			ex.source.get_buf(),
			ex.message.get_buf(),
			ex.stack_trace.get_buf());
	}
}
