#include "quasar_dsl.h"

void sample1(void)
{
    using namespace quasar;
	using quasar::ref;

    try
    {
        // Creates the quasar host and loads the specified module
        ref<IQuasarHost> host = IQuasarHost::Create(_T(""), true);

        RuntimeSettings settings = { 0 };
        settings.structSize = sizeof(RuntimeSettings);
        settings.scalarType = SCALARTYPE_SINGLE;
        settings.loadCompiler = true;
        settings.enableConcurrentKernelExecution = false;
        host->OpenDevice(_T("cuda"), &settings);

    
        LPCTSTR errorMsg;
    	//if (!host->LoadBinaryModule(_T("dwt.qlib"), &errorMsg))
        if (!host->LoadSourceModule(_T("dwt.q"), &errorMsg))
        {
            tprintf(_T("%s\n"), errorMsg);
            return;
        }

        // Checks whether the function 'dwt2d' is available
        wprintf(L"The function 'dwt2d' exists: %d\n",
                host->FunctionExists(_T("dwt2d")));

    	QValue dwt_dim0;
    	wprintf(L"The function 'dwt_dim0' exists: %d\n",
    		QValue::ReadHostVariable(_T("dwt_dim0"), dwt_dim0));

        // Create a matrix (qvalue_t is a dynamic type)
        QValue A = QValue::CreateMatrix<scalar>(4, 4);
        Matrix Adata = host->LockMatrix<scalar>(A);
        for (int m = 0; m < Adata.dim1; m++)
            for (int n = 0; n < Adata.dim2; n++)
            {
                Adata.data[m * Adata.dim2 + n] = m + n;
            }
        host->UnlockMatrix(A);

    	Function print(_T("print()"));
        print(_T("The matrix is:"), A);
    }
    catch (exception_t ex)
    {
        tprintf(_T("An error occurred: %s"), (LPCTSTR) ex.message);
    }
}
