#include "quasar_dsl.h"

using namespace quasar;

void sample20(void)
{
	using namespace quasar;

	LPCTSTR sourceProgram =
		_T("function[] = __kernel__ filter_kernel(x:cube, y : cube, offset : ivec3, pos : ivec3)\n")
		_T("	dpos = pos + offset;\n")
		_T("	M = 9\n")
		_T("	sum = 0.0\n")
		_T("	for m = -M..M\n")
		_T("		for n = -M..M\n")
		_T("			sum += x[dpos + [m, n, 0]]\n")
		_T("		end\n")
		_T("	end\n")
		_T("	y[pos] = sum ./ (2 * M + 1) ^ 2\n")
		_T("end\n");

	LPCTSTR errorMsg = NULL;

	using quasar::ref;

	try
	{
		ref<IQuasarHost> host = IQuasarHost::Create(_T("hycuda2.device.xml"));

		// Export Quasar functions to C++
		Function tic(_T("tic()"));
		Function toc(_T("toc(...??)"));
		Function imread(_T("imread(string)"));
		Function imshow(_T("imshow(cube)"));
		Function parallel_do(_T("parallel_do(...??)"));
		Function uninit(_T("uninit(??)"));

		tprintf(_T("Compiling code...\n"));
		tic();
		if (!host->LoadModuleFromSource(_T("sample20"), sourceProgram, &errorMsg))
		{
			tprintf(_T("Error while compiling program:\n%s\n"), errorMsg);
			return;
		}
		toc();

		QValue kernelFunc;
		if (!QValue::ReadHostVariable(_T("filter_kernel"), kernelFunc))
			tprintf(_T("Error reading variable 'filter_kernel'!\n"));

		// Load an image
		QValue img = imread(_T("lena_big.tif"));
		QValue img_out = uninit(size(img));
		RuntimeControl runtimeCtl;
		//Method sched(type(runtimeCtl), _T("sched(...??)");

		tic();
		for (int k = 0; k < 50; k++)
		{
			// Select the GPU
			runtimeCtl.ScheduleGPU(k % 2);

			// Execute the kernel function
			parallel_do(size(img), img, img_out, make_scalar3(0, 0, 0), kernelFunc);
		}
		toc();

		// Switch back to the automatic scheduling mode
		//sched(runtimeCtl, NamedParam(_T("gpu_index", 0));
		runtimeCtl.ScheduleAuto();

		host->RunApp();
	}
	catch (exception_t ex)
	{
		tprintf(_T("An error occurred:\nSource: %s\nMessage: %s\nStack trace: %s"),
			ex.source.get_buf(),
			ex.message.get_buf(),
			ex.stack_trace.get_buf());
	}
}
