#include "quasar_dsl.h"

using namespace quasar;

void sample21(void)
{
	using namespace quasar;

	LPCTSTR errorMsg = NULL;

	using quasar::ref;

	try
	{
		ref<IQuasarHost> host = IQuasarHost::Create();

		// Print info
		const MachineInfo machineInfo = host->GetMachineInfo();
		tprintf(_T("Host name: %s\n"), machineInfo.hostName);
		tprintf(_T("Domain name: %s\n"), machineInfo.domainName);
		tprintf(_T("IP address: %s\n"), machineInfo.ipAddress);
		tprintf(_T("CPU: %s, cores: %d, total memory: %ll GB\n"), machineInfo.cpu.cpuName, machineInfo.cpu.numCPUcores,
			(int)(machineInfo.cpu.totalMemory / (long long)(1024 * 1024 * 1024)));

		string_t cudaRuntimeVer = machineInfo.cuda.runtimeVersion.ToString();
		string_t cudaDriverVer = machineInfo.cuda.driverVersion.ToString();

		tprintf(_T("CUDA installed: %d (runtime ver %s, driver ver %s), OpenCL installed: %d\n"),
			machineInfo.cuda.installed,
			(LPCTSTR)cudaRuntimeVer,
			(LPCTSTR)cudaDriverVer,
			machineInfo.opencl.installed);

		tprintf(_T("\nAvailable computation devices:\n"));
		for (int i = 0; i < machineInfo.numComputationDevices; i++)
		{
			const ComputationDeviceInfo *dev = machineInfo.computationDevices[i];
			tprintf(_T("%d. name: '%s', type: '%s'\n"), i + 1, dev->name,
				dev->target == DEVTYPE_CPU ? _T("cpu") :
				dev->target == DEVTYPE_CUDA ? _T("cuda") :
				dev->target == DEVTYPE_OPENCL ? _T("opencl") :
				dev->target == DEVTYPE_HYPERION ? _T("hyperion") : _T("unknown"));
		}

		// Select a Hyperion device
		string_t deviceName;
		for (int i = 0; i < machineInfo.numComputationDevices; i++)
			if (machineInfo.computationDevices[i]->target == DEVTYPE_HYPERION)
			{
				deviceName = machineInfo.computationDevices[i]->deviceId;
				break;
			}

		if (deviceName.get_length() == 0)
		{
			tprintf(_T("No hyperion device found in the current configuration!\n"));
			host->FreeMachineInfo(const_cast<MachineInfo&>(machineInfo));
			return;
		}

		RuntimeSettings settings;
		settings.structSize = sizeof(settings);
		settings.loadCompiler = false;
		settings.scalarType = SCALARTYPE_SINGLE;
		host->OpenDevice(deviceName, &settings);
		tprintf(_T("Device successfully opened!\n"));

		host->CloseDevice();

		host->FreeMachineInfo(const_cast<MachineInfo&>(machineInfo));
	}
	catch (exception_t ex)
	{
		tprintf(_T("An error occurred: %s\n"), (LPCTSTR)ex.message);
	}
}
