#include "quasar_dsl.h"

void sample2(void)
{
    using namespace quasar;
	using quasar::ref;

    try
    {
        // Creates the quasar host and loads the specified module
        ref<IQuasarHost> host = IQuasarHost::Create(_T("cuda"), false);

        LPCTSTR errorMsg;
        //if (!host->LoadSourceModule(_T("color_temperature.q"), &errorMsg))
        if (!host->LoadBinaryModule(_T("color_temperature.qlib"), &errorMsg))
        {
            tprintf(_T("%s\n"), errorMsg);
            return;
        }

        Function imread(_T("imread(string)"));
        Function parallel_do(_T("parallel_do()"));
        Function imshow(_T("imshow(cube)"));

        // Checks whether the function 'apply' is available
        tprintf(_T("The function 'apply' exists: %d\n"),
                host->FunctionExists(_T("apply")));

        // Load an image and show it
        QValue img = imread(_T("lena_big.tif"));
        tprintf(_T("The image has dimensions %dx%dx%d\n"), size(img, 0), size(img, 1),
                size(img, 2));

        // Call a kernel function
        QValue kernelFunc;
        if (!QValue::ReadHostVariable(_T("color_temperature_"), kernelFunc))
             tprintf(_T("Error reading variable 'color_temperature'!\n"));

        // x : cube, y : cube, temp : scalar, cold : vec3, hot : vec3, pos : vec2
        // hot = [1,0.2,0]*255;
        // cold = [0.3,0.4,1]*255
        scalar sz[2] = {size(img, 0), size(img, 1)};
        scalar hot[3] = {1 * 255, 0.2 * 255, 0};
        scalar cold[3] = {0.3 * 255, 0.4 * 255, 255};
        parallel_do((QValue) sz, img, img, 0.5, (QValue) cold, (QValue) hot, kernelFunc);

        imshow(img);

        // Wait until all windows are closed
        host->RunApp();
    }
    catch (exception_t ex)
    {
        tprintf(_T("An error occurred:\nSource: %s\nMessage: %s\nStack trace: %s"),
            ex.source.get_buf(),
            ex.message.get_buf(),
            ex.stack_trace.get_buf());
    }
}
