#include "quasar_dsl.h"

void sample3(void)
{
    using namespace quasar;

    // Inline definition of a Quasar program, similar to OpenGL/DirectX shader
    // language
    LPCTSTR sourceProgram =
        _T("function y = gaussian_filter(x, fc, n)\n")
        _T("	function [] = __kernel__ gaussian_filter_hor(x : cube, y : ")
        _T("cube'unchecked, fc : vec'unchecked, n : int, pos : vec3)\n")
        _T("		sum = 0.\n")
        _T("		for i=0..numel(fc)-1\n")
        _T("			sum = sum + x[pos + [0,i-n,0]] * fc[i]\n")
        _T("		end\n")
        _T("		y[pos] = sum\n")
        _T("	end\n")
        _T("\n")
        _T("	function [] = __kernel__ gaussian_filter_ver(x : cube, y : ")
        _T("cube'unchecked, fc : vec'unchecked, n : int, pos : vec3)\n")
        _T("		sum = 0.\n")
        _T("		for i=0..numel(fc)-1\n")
        _T("			sum = sum + x[pos + [i-n,0,0]] * fc[i]\n")
        _T("		end\n")
        _T("		y[pos] = sum\n")
        _T("	end\n")
        _T("\n")
        _T("	z = uninit(size(x))\n")
        _T("	y = uninit(size(x))\n")
        _T("	parallel_do (size(y), x, z, fc, n, gaussian_filter_hor)\n")
        _T("	parallel_do (size(y), z, y, fc, n, gaussian_filter_ver)\n")
        _T("end");

    LPCTSTR errorMsg = NULL;

	using quasar::ref;

	try
	{
		ref<IQuasarHost> host = IQuasarHost::Create(_T("cuda"));
		// Export Quasar functions to C++
		Function tic(_T("tic()"));
		Function toc(_T("toc()"));
		Function imread(_T("imread(string)"));
		Function imshow(_T("imshow(cube)"));

        tprintf(_T("Compiling code...\n"));
        tic();
        if (!host->LoadModuleFromSource(_T("sample3"), sourceProgram, &errorMsg))
        {
            tprintf(_T("Error while compiling program:\n%s\n"), errorMsg);
            return;
        }
        toc();

        // The function gaussian_filter is now available
        Function gaussian_filter(_T("gaussian_filter(cube,vec,int)"));


        // Load an image
        QValue img = imread(_T("lena_big.tif"));
        //tprintf(_T("The image has dimensions %dx%dx%d\n"), (int)size(img, 0), (int)size(img, 1),
        //        (int)size(img, 2));

    	// Apply the Gaussian filter
        scalar fcoeff[] = {1.0 / 9, 2.0 / 9, 3.0 / 9, 2.0 / 9, 1.0 / 9};
        const int N = 2;
        QValue img_out = gaussian_filter(img, (QValue) fcoeff, N);

        // Show the result
        QValue p1 = imshow(img);
        QValue p2 = imshow(img_out);

        Type qimshow = type(p1);
        Method connect(qimshow, _T("connect(\?\?)"));
        connect(p1, p2); // Call the function p1.connect(p2)!

        host->RunApp();
	}
	catch (exception_t ex)
	{
		tprintf(_T("An error occurred:\nSource: %s\nMessage: %s\nStack trace: %s"),
			ex.source.get_buf(),
			ex.message.get_buf(),
			ex.stack_trace.get_buf());
	}
}
