#include "quasar_dsl.h"

using namespace quasar;

void sample6(void)
{
	try
	{
		// Creates the quasar host and loads the specified module
		quasar::ref<IQuasarHost> host = IQuasarHost::Create(_T("cuda"));
		Function print(_T("print()"));
		Function tic(_T("tic()"));
		Function toc(_T("toc()"));

		// Construct a cell matrix
		QValue A = QValue::CreateMatrix<qvalue_t>(256, 256);

		// The following is equivalent to the Quasar code:
		// tic()
		// for m=0..size(A,0)-1
		// 	 for n=0..size(A,1)-1
		//		A[m,n] = m + n
		// 	 end
		// end
		// toc()
		// Note that this programming pattern is quite slow, as it involves
		// a lot of calls to Quasar (the loop is not parallelized).

		tic();
		for (int m = 0; m < size(A, 0); m++)
			for (int n = 0; n < size(A, 1); n++)
			{
				A(m, n) = m + n;
			}
		toc();

		// print(A);
		print((QValue)A(1, 1));

		// Reading out one element!
		scalar b = A(1, 1).as<scalar>();

		scalar vals[] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16};
		QValue B = QValue::CreateMatrix<scalar>(4, 4, vals);
		A(0, 1) = B;
		print(A(0, 1));
		print(A(0, 1)(1, 3));

		scalar row1[] = { 1, 2, 3, 4 };
		scalar row2[] = { 4, 3, 2, 1 };
		QValue rows[] = { QValue::CreateVector<scalar>(4, row1),
							QValue::CreateVector<scalar>(4, row2) };
		QValue C = QValue::CreateMatrix(rows);
		#ifdef CPLUSPLUS11
		C = QValue::CreateMatrix({ QValue::CreateVector<scalar>(4, row1),
									QValue::CreateVector<scalar>(4, row2) });
		#endif
		print(C);

		//*****************************
		int NDim = 3;
		int dims[] = { 1, 4, 1 };

		qhandle elem_type = host->GetTypeHandle<NCube<4>>();

		qvalue_t value = host->CreateNCube(elem_type, NDim, dims, false);

		Function print_f(_T("print()"));
		Function type_f(_T("type()"));
		print_f(type_f(value));



	}
	catch (exception_t ex)
	{
		tprintf(_T("An error occurred:\nSource: %s\nMessage: %s\nStack trace: %s"),
			ex.source.get_buf(),
			ex.message.get_buf(),
			ex.stack_trace.get_buf());
	}
}
