#include "quasar_dsl.h"

void sample9(void)
{
    using namespace quasar;
	using quasar::ref;

	try
	{
		// Creates the quasar host
		ref<IQuasarHost> host = IQuasarHost::Create(_T("cuda"));

		// Problem loading Quasar
		if (host == NULL)
			return;

		Function print(_T("print()"));

		// Create a matrix of scalars
		QValue A = QValue::CreateMatrix<scalar>(4, 4);
		Matrix Adata = host->LockMatrix<scalar>(A);
		for (int m = 0; m < Adata.dim1; m++)
			for (int n = 0; n < Adata.dim2; n++)
			{
				Adata.data[m * Adata.dim2 + n] = m + n;
			}
		host->UnlockMatrix(A);

		// C++ hack to get the point-wise multiplication. Use at your own risk!
		//#define _ ElemWise()
		// QValue B = A ._* QValue(4.0);

		// Preferred style pointwise multiplication
		QValue B = A.ElemWise() * QValue(4.0) + QValue(0.5);

		print(_T("The matrix is:"), B);
	}
	catch (exception_t ex)
	{
		tprintf(_T("An error occurred:\nSource: %s\nMessage: %s\nStack trace: %s"),
			ex.source.get_buf(),
			ex.message.get_buf(),
			ex.stack_trace.get_buf());
	}
}
