//
// cpp_sample.cu
// 
// Example CUDA module that can be used from Quasar. The module defines
// a CPU kernel function and a CPU define function. The use of the macro
// ENTRY() is required, in order to perform the correct name mangling for
// the export symbol.
//
// (c) 2013 Bart Goossens
//
#include "quasar.h"
 
using namespace quasar;
 
// A CPU kernel function
extern "C" EXPORT void ENTRY(fill_matrix)(void* closure, Matrix mtx, 
    int3 _gridDim, int3 _blockDim, int _nThreads)
{
    for (int m=0;m<mtx.dim1;m++)
        for (int n=0;n<mtx.dim2;n++)
        {
            mtx.data[m*mtx.dim2+n] = m+n;
        }
}

// A CPU device function
extern "C" EXPORT __device__ scalar ENTRY(count_zeros)(void *closure, Matrix mtx)
{
	int cnt = 0;
	for (int m=0;m<mtx.dim1;m++)
        for (int n=0;n<mtx.dim2;n++)
	{
		if (mtx.data[m*mtx.dim2+n] == 0)
			cnt ++;
		mtx.data[m*mtx.dim2+n] ++;
	}
	return 10+cnt;
}