import "decklinkcapture.cpp"
import "Quasar.UI.dll"

function index = indexof(list, text)
    index = -1
    for i=0..numel(list)-1
        if list[i] == text
            index = i
        endif
    endfor
endfunction

devices = decklink_get_devices()
print devices

device_name = "DeckLink Mini Recorder"   % note: currently only one capture card at the time can be opened
display_mode = "1080i59.94"              % note: needs to match the camera settings

device_index = indexof(devices, device_name)
if device_index<0
    error "Could not select device: ", device_name, ". Device unknown"
endif

decklink_open(device_index)
display_modes = decklink_get_displaymodes()



display_mode_index = indexof(display_modes, display_mode)
if display_mode_index<0
    error "Could not select display mode: ", display_mode
endif

print display_modes

decklink_startcapture(display_mode_index)

frm = form(sprintf("Camera: %s, display mode: %s", devices[device_index], display_mode))
frm.width = 1900
disp = frm.add_display()
frm.show()

pause(0.5)

function [] = __kernel__ convert_YUV422_to_RGB(im_in : mat[uint16]'const, im_out : cube, pos : ivec2)
    val = [im_in[pos[0],pos[1]*2],im_in[pos[0],pos[1]*2+1]]

% From Decklink SDK HD YUV to RGB conversion
% For HD:
%R = max(0, min(255, 1.164(Y - 16) + 1.793(Cr - 128)));
%G = max(0, min(255, 1.164(Y - 16) - 0.534(Cr - 128) - 0.213(Cb - 128)));
%B = max(0, min(255, 1.164(Y - 16) + 2.115(Cr - 128)));
% For SD:
%R = max(0, min(255, 1.164(Y - 16) + 1.596(Cr - 128)));
%G = max(0, min(255, 1.164(Y - 16) - 0.813(Cr - 128) - 0.391(Cb - 128)));
%B = max(0, min(255, 1.164(Y - 16) + 2.018(Cr - 128)));

    Rc = [1.164,  1.596,  0.0]
    Gc = [1.164, -0.813, -0.391]
    Bc = [1.164,  0.0,    2.018]

%    Rc = [1.164,  1.793,  0.0]
%    Gc = [1.164, -0.534, -0.213]
%    Bc = [1.164,  0.0,    2.115]

    Y1 = shr(and(val[0],0ff00h),8)-16
    Y2 = shr(and(val[1],0ff00h),8)-16
    V  =     (and(val[0],000ffh)-128)
    U  =     (and(val[1],000ffh)-128)

    R1 = clamp(dotprod(Rc,[Y1,U,V]),255)
    G1 = clamp(dotprod(Gc,[Y1,U,V]),255)
    B1 = clamp(dotprod(Bc,[Y1,U,V]),255)

    R2 = clamp(dotprod(Rc,[Y2,U,V]),255)
    G2 = clamp(dotprod(Gc,[Y2,U,V]),255)
    B2 = clamp(dotprod(Bc,[Y2,U,V]),255)

    im_out[pos[0],pos[1]*2+0,0..2] = [R1,G1,B1]
    im_out[pos[0],pos[1]*2+1,0..2] = [R2,G2,B2]
endfunction

first = true
{!interpreted for}
while !frm.closed()
    im = decklink_readframe()
    if first
        imc = zeros(size(im,0),size(im,1),3)
        first = false
    endif
    parallel_do([size(im,0),size(im,1)/2],im,imc,convert_YUV422_to_RGB)

    disp.imshow(imc,[0,255])

    pause(0)
endwhile

decklink_close()
