// Demo application for using the DeckLinkVideoCapture in standalone mode

#include "DeckLinkAPI_h.h"
#include "DeckLinkDevice.h"

class ScreenPreviewCallback : public IDeckLinkScreenPreviewCallback
{
private:
	ULONG m_cRef;

public:
	ScreenPreviewCallback();
	ULONG AddRef();
	ULONG Release();
	HRESULT QueryInterface(REFIID  riid, LPVOID * ppvObj);
	HRESULT STDMETHODCALLTYPE ScreenPreviewCallback::DrawFrame(
		IDeckLinkVideoFrame *theFrame);
};

ScreenPreviewCallback::ScreenPreviewCallback() : m_cRef(1)
{

}

HRESULT STDMETHODCALLTYPE ScreenPreviewCallback::DrawFrame(
	IDeckLinkVideoFrame *theFrame)
{
	wprintf(L"Received a frame of size %dx%d\n", theFrame->GetHeight(),
		theFrame->GetWidth());
	unsigned char *buf = NULL;
	if (theFrame->GetBytes((void **)&buf) == S_OK)
	{
		wprintf(L"RGB: %d %d %d\n", buf[0], buf[1], buf[2]);
	}
	return S_OK;
}

HRESULT ScreenPreviewCallback::QueryInterface(REFIID  riid, LPVOID * ppvObj)
{
	// Always set out parameter to NULL, validating it first.
	if (!ppvObj)
		return E_INVALIDARG;
	*ppvObj = NULL;
	if (riid == IID_IUnknown)
	{
		// Increment the reference count and return the pointer.
		*ppvObj = (LPVOID)this;
		AddRef();
		return NOERROR;
	}
	return E_NOINTERFACE;
}

ULONG ScreenPreviewCallback::AddRef()
{
	InterlockedIncrement(&m_cRef);
	return m_cRef;
}
ULONG ScreenPreviewCallback::Release()
{
	// Decrement the object's internal counter.
	ULONG ulRefCount = InterlockedDecrement(&m_cRef);
	if (0 == m_cRef)
	{
		delete this;
	}
	return ulRefCount;
}


int main()
{
	CoInitialize(NULL);

	const int device_id = 0;
	const int display_mode = 8;

	DeckLinkDeviceDiscovery * discovery = new DeckLinkDeviceDiscovery();
	discovery->Enable();

	// Wait for the devices to be enumerated
	if (!discovery->WaitForDevices())
	{
		wprintf(L"No DeckLink devices discovered!\n");
		return 0;
	}

	// Print the devices
	for (int index = 0; index < discovery->Count(); index++)
	{
		DeckLinkDevice * dd = new DeckLinkDevice(discovery->GetDeckLink(index));
		dd->Init();
		wprintf(L"Device %d: %s\n", index, dd->GetDeviceName().c_str());
		dd->Release();
	}
	
	// Open device 0
	DeckLinkDevice * device = new DeckLinkDevice(discovery->GetDeckLink(device_id));
	device->Init();

	std::vector<std::wstring> displayModeNames;
	device->GetDisplayModeNames(displayModeNames);
	int index = 0;
	for (auto it = displayModeNames.begin(); it != displayModeNames.end(); it++, index ++)
	{
		wprintf(L"Display mode %d: %s\n", index, it->c_str());
	}

	ScreenPreviewCallback *callback = new ScreenPreviewCallback();
	device->StartCapture(display_mode, callback, true);
	callback->Release();

	Sleep(2000);

	device->StopCapture();

	device->Release();

	discovery->Release();

	CoUninitialize();
    return 0;
}

