﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Quasar.Computations;
using DirectShowLib;

namespace DirectShowCam
{
    [QObject.ClassVisible("qdirectshowcamview")]
    public class DirectShowCamView : QObject
    {
        private IBaseFilter _sourceFilter;
        private IAMVideoProcAmp videoProcAmp;
        private IAMCameraControl cameraProcAmp;

        public DirectShowCamView(IBaseFilter sourceFilter)
        {
            this._sourceFilter = sourceFilter;

            this.videoProcAmp = sourceFilter as IAMVideoProcAmp;
            this.cameraProcAmp = sourceFilter as IAMCameraControl;
        }

        #region CameraControlProperties

        private void SetProperty(CameraControlProperty prop, int value)
        {
            int min, max, step, @default;
            CameraControlFlags flags;
            cameraProcAmp.GetRange(prop, out min, out max, out step, out @default, out flags);

            if (value < min) value = min;
            if (value > max) value = max;

            cameraProcAmp.Set(prop, value, CameraControlFlags.Manual);
        }

        private int GetProperty(CameraControlProperty prop)
        {
            int value;
            CameraControlFlags flags;
            cameraProcAmp.Get(prop, out value, out flags);
            return value;
        }

        private bool GetPropertyAuto(CameraControlProperty prop)
        {
            int value;
            CameraControlFlags flags;
            cameraProcAmp.Get(prop, out value, out flags);
            return flags == CameraControlFlags.Auto;
        }

        private void SetPropertyAuto(CameraControlProperty prop, bool value)
        {
            int oldValue;
            CameraControlFlags oldFlags;
            cameraProcAmp.Get(prop, out oldValue, out oldFlags);
            cameraProcAmp.Set(prop, oldValue, value ? CameraControlFlags.Auto : CameraControlFlags.None);
        }

        #endregion


        [MemberVisible]
        public bool auto_pan { get { return GetPropertyAuto(CameraControlProperty.Pan); } set { SetPropertyAuto(CameraControlProperty.Pan, value); } }
        [MemberVisible]
        public bool auto_exposure { get { return GetPropertyAuto(CameraControlProperty.Exposure); } set { SetPropertyAuto(CameraControlProperty.Exposure, value); } }
        [MemberVisible]
        public bool auto_focus { get { return GetPropertyAuto(CameraControlProperty.Focus); } set { SetPropertyAuto(CameraControlProperty.Focus, value); } }
        [MemberVisible]
        public bool auto_iris { get { return GetPropertyAuto(CameraControlProperty.Iris); } set { SetPropertyAuto(CameraControlProperty.Iris, value); } }
        [MemberVisible]
        public bool auto_roll { get { return GetPropertyAuto(CameraControlProperty.Roll); } set { SetPropertyAuto(CameraControlProperty.Roll, value); } }
        [MemberVisible]
        public bool auto_tilt { get { return GetPropertyAuto(CameraControlProperty.Tilt); } set { SetPropertyAuto(CameraControlProperty.Tilt, value); } }
        [MemberVisible]
        public bool auto_zoom { get { return GetPropertyAuto(CameraControlProperty.Zoom); } set { SetPropertyAuto(CameraControlProperty.Zoom, value); } }

        [MemberVisible]
        public int pan { get { return GetProperty(CameraControlProperty.Pan); } set { SetProperty(CameraControlProperty.Pan, value); } }
        [MemberVisible]
        public int exposure { get { return GetProperty(CameraControlProperty.Exposure); } set { SetProperty(CameraControlProperty.Exposure, value); } }
        [MemberVisible]
        public int focus { get { return GetProperty(CameraControlProperty.Focus); } set { SetProperty(CameraControlProperty.Focus, value); } }
        [MemberVisible]
        public int iris { get { return GetProperty(CameraControlProperty.Iris); } set { SetProperty(CameraControlProperty.Iris, value); } }
        [MemberVisible]
        public int roll { get { return GetProperty(CameraControlProperty.Roll); } set { SetProperty(CameraControlProperty.Roll, value); } }
        [MemberVisible]
        public int tilt { get { return GetProperty(CameraControlProperty.Tilt); } set { SetProperty(CameraControlProperty.Tilt, value); } }
        [MemberVisible]
        public int zoom { get { return GetProperty(CameraControlProperty.Zoom); } set { SetProperty(CameraControlProperty.Zoom, value); } }

        #region VideoProcAmpProperties

        private void SetProperty(VideoProcAmpProperty prop, int value)
        {
            int min, max, step, @default;
            VideoProcAmpFlags flags;
            videoProcAmp.GetRange(prop, out min, out max, out step, out @default, out flags);

            if (value < min) value = min;
            if (value > max) value = max;

            videoProcAmp.Set(prop, value, VideoProcAmpFlags.Manual);
        }

        private int GetProperty(VideoProcAmpProperty prop)
        {
            int value;
            VideoProcAmpFlags flags;
            videoProcAmp.Get(prop, out value, out flags);
            return value;
        }

        private bool GetPropertyAuto(VideoProcAmpProperty prop)
        {
            int value;
            VideoProcAmpFlags flags;
            videoProcAmp.Get(prop, out value, out flags);
            return flags == VideoProcAmpFlags.Auto;
        }

        private void SetPropertyAuto(VideoProcAmpProperty prop, bool value)
        {
            int oldValue;
            VideoProcAmpFlags oldFlags;
            videoProcAmp.Get(prop, out oldValue, out oldFlags);
            videoProcAmp.Set(prop, oldValue, value ? VideoProcAmpFlags.Auto : VideoProcAmpFlags.None);
        }

        #endregion

        [MemberVisible]
        public bool auto_brightness { get { return GetPropertyAuto(VideoProcAmpProperty.Brightness); } set { SetPropertyAuto(VideoProcAmpProperty.Brightness, value); } }
        [MemberVisible]
        public bool auto_contrast { get { return GetPropertyAuto(VideoProcAmpProperty.Contrast); } set { SetPropertyAuto(VideoProcAmpProperty.Contrast, value); } }
        [MemberVisible]
        public bool auto_hue { get { return GetPropertyAuto(VideoProcAmpProperty.Hue); } set { SetPropertyAuto(VideoProcAmpProperty.Hue, value); } }
        [MemberVisible]
        public bool auto_saturation { get { return GetPropertyAuto(VideoProcAmpProperty.Saturation); } set { SetPropertyAuto(VideoProcAmpProperty.Saturation, value); } }
        [MemberVisible]
        public bool auto_sharpness { get { return GetPropertyAuto(VideoProcAmpProperty.Sharpness); } set { SetPropertyAuto(VideoProcAmpProperty.Sharpness, value); } }
        [MemberVisible]
        public bool auto_gamma { get { return GetPropertyAuto(VideoProcAmpProperty.Gamma); } set { SetPropertyAuto(VideoProcAmpProperty.Gamma, value); } }
        [MemberVisible]
        public bool auto_colorenable { get { return GetPropertyAuto(VideoProcAmpProperty.ColorEnable); } set { SetPropertyAuto(VideoProcAmpProperty.ColorEnable, value); } }
        [MemberVisible]
        public bool auto_whitebalance { get { return GetPropertyAuto(VideoProcAmpProperty.WhiteBalance); } set { SetPropertyAuto(VideoProcAmpProperty.WhiteBalance, value); } }
        [MemberVisible]
        public bool auto_backlightcompensation { get { return GetPropertyAuto(VideoProcAmpProperty.BacklightCompensation); } set { SetPropertyAuto(VideoProcAmpProperty.BacklightCompensation, value); } }
        [MemberVisible]
        public bool auto_gain { get { return GetPropertyAuto(VideoProcAmpProperty.Gain); } set { SetPropertyAuto(VideoProcAmpProperty.Gain, value); } }

        [MemberVisible]
        public int brightness { get { return GetProperty(VideoProcAmpProperty.Brightness); } set { SetProperty(VideoProcAmpProperty.Brightness, value); } }
        [MemberVisible]
        public int contrast { get { return GetProperty(VideoProcAmpProperty.Contrast); } set { SetProperty(VideoProcAmpProperty.Contrast, value); } }
        [MemberVisible]
        public int hue { get { return GetProperty(VideoProcAmpProperty.Hue); } set { SetProperty(VideoProcAmpProperty.Hue, value); } }
        [MemberVisible]
        public int saturation { get { return GetProperty(VideoProcAmpProperty.Saturation); } set { SetProperty(VideoProcAmpProperty.Saturation, value); } }
        [MemberVisible]
        public int sharpness { get { return GetProperty(VideoProcAmpProperty.Sharpness); } set { SetProperty(VideoProcAmpProperty.Sharpness, value); } }
        [MemberVisible]
        public int gamma { get { return GetProperty(VideoProcAmpProperty.Gamma); } set { SetProperty(VideoProcAmpProperty.Gamma, value); } }
        [MemberVisible]
        public int colorenable { get { return GetProperty(VideoProcAmpProperty.ColorEnable); } set { SetProperty(VideoProcAmpProperty.ColorEnable, value); } }
        [MemberVisible]
        public int whitebalance { get { return GetProperty(VideoProcAmpProperty.WhiteBalance); } set { SetProperty(VideoProcAmpProperty.WhiteBalance, value); } }
        [MemberVisible]
        public int backlightcompensation { get { return GetProperty(VideoProcAmpProperty.BacklightCompensation); } set { SetProperty(VideoProcAmpProperty.BacklightCompensation, value); } }
        [MemberVisible]
        public int gain { get { return GetProperty(VideoProcAmpProperty.Gain); } set { SetProperty(VideoProcAmpProperty.Gain, value); } }



    }
}
