% Example of digital camera playback, using the DirectShow library (Windows only)
% The advantage of this library is that all windows cameras/webcams can be directly
% used.
import "DirectShowCam.dll"
import "Quasar.UI.dll"
import "inttypes.q"

% List cameras
cams = dx_listcam()
print cams
camera_index = 1

% Create a display window
frm = form("Video capture demo")
frm.move(0,0)
frm.width = 1900
disp = frm.add_display()
frm.show()

function [] = __kernel__ swapRB(im : cube, pos : ivec2)
	p = im[pos[0],pos[1],0..2]
	im[pos[0],pos[1],0..2] = [p[2], p[1], p[0]]
end

% The camera callback function
function [] = camera_frame_callback(camera, index)
	if !frm.closed()
		% Remark: the frame buffer is in 8-bit BGR format. For processing, 
		% it may be useful to convert the data to a floating point format,
		% using the float(.) function.
		im = float(camera.frame_buffer)

		% Toggle R and B
		parallel_do(size(im,0..1),im,swapRB)		

		disp.imshow(im,[0,255])
	endif
end

% Start camera capturing...
cam = dx_capture(camera_index, camera_frame_callback)
frm.add_heading(sprintf("Camera %d - %s (%d x %d, %d FPS)", cam.Source, cam.Name, cam.Width, cam.Height, cam.FramesPerSecond))


frm.wait() % Wait until the form is closed

% Close the camera
cam = null
print "Finished!"
