using System;
using System.Linq;
using Quasar;
using Quasar.Computations;
using Quasar.Computations.Runtime;

namespace TIFFIO
{
	public class MainClass
	{
		/// <summary>
		/// A main function - for testing
		/// </summary>
		/// <param name="args">The command-line arguments.</param>
		public static void Main(string [] args)
		{
			// Initialize the runtime system
			RuntimeSystem.Initialize();

            // Setup the runtime settings for 32-bit float
            var settings = new RuntimeSettings();
            settings.TypeEnvironment = new Quasar.TypeSystem.TypeEnvironment();
            settings.TypeEnvironment.scalarType = Quasar.TypeSystem.ScalarTypes.Single;            

            var engine = RuntimeSystem.CreateComputationEngine(RuntimeSystem.ComputationPreference.GPU, settings);

            // Initialize the core library
            CoreLibrary.run(engine, engine.EvaluationStack);

            // Initialize the TIFF Reader library
            TIFFIO.run(engine, engine.EvaluationStack);
            CoreLibrary.tic();
			//TIFFIO.tiffread(@"C:\Bart\Programs\2011\Quasar\bin\x86\1k image.tif");
            TIFFIO.tiffread(@"F:\Data\dmbr_jonas\MAM data-set\Stack.tif", new QValue(10));
            CoreLibrary.toc("Loading");

            // Display the result!
            QObject result = (QObject) engine.EvaluationStack.Pop().value;
            //CoreLibrary.imshow(result["data"], engine.MatrixFactory.New(new float[] {}, 1, 0));
            UILibrary.imshow(result["data"], engine.MatrixFactory.New(AllocationFlags.None, new float[] { 0,255 }, 1, 2));

            // Wait until all windows have been closed
            RuntimeSystem.WaitTillFinished();
		}
	}
}

