%==============================================================================
% TIFFIO_example.q - TIF input/output library for Quasar
%
% Bart Goossens, May 2013
%==============================================================================
% Supports more advanced TIF usage scenarios:
% - 8-bit, 16-bit and 32-bit TIF files
% - selection of difference slices from multi-frame TIF files
%
% How to use:
% ```
% import "TIFFIO.dll"
% img = tiffread("mytif.tif", 1)  % only read slice 1 (base 0 = the second slice!)
% img = tiffread("mytif.tif", 0..5) % the first six slices
% img = tiffread("mytif.tif", [0, 2, 4]) % slice 0, 2 and 4
% ```
% The individual slices can be accessed through;
%
%    img.slices : vec[cube]
%
% Or, for single-slice tiff files:
%
%    img.data : cube
%
% Note: for multi-slice tif files, img.data just points to the first slice.
%==============================================================================
%
% This example shows how to read multislice TIF files in Quasar.
%==============================================================================
import "TIFFIO.dll"

% Note: sliceRange can be either an integer or a vector
% img is a structure (object()) with relevant TIFF fields.
% - img.data will always point to the first slice
% - img.slices is a cell array with all individual slices
sliceRange = 0..3
img = tiffread("lena_big.tif")
print type(img)
print img
imshow(img.slices[0],[])


