%COMPILE_MUSTFAIL: Vector index out of bounds
{!author name="Bart Goossens"}
{!doc category="Samples/Misc/Boundary checking"}
{!title name="Array boundary checking demonstration"}


% Section: overview
%
% Example of writing to a vector from a kernel functions, where the vector index
% exceeds the boundaries of the vector. The situation is detected at compile-time
% and an error message is generated:
%
% Error while compiling the instruction 'a[-1]=0': Vector index out of bounds 
% (-1), allowed values are from 0 to 4.
%
% So in this case, the compiler is able to figure out the unsafe boundary access for
% variable 'a'. In cases that this is not possible, a runtime error is generated.

% Sample kernel function
function [] = __kernel__ my_kernel (index : int, c : vec'checked, pos : int)
    c[pos] = 1
    a = zeros(5)
    a[index] = 0
endfunction
c = zeros(100)
parallel_do(256,-1,c,my_kernel)
