{!author name="Bart Goossens"}
{!doc category="Samples/Advanced CUDA/Typeless GPU programming"}
{!title name="Typeless GPU programming"}

% Section: Overview
%
% Demonstrates some of the type inference capabilities of the Quasar compiler. When type information
% is omitted, the compiler attempts to derive the type information from the context. In particular,
% a kernel function is required to be specialized, because dynamically typed variables are not 
% allowed to be used on the GPU (for performance reasons). This may even cause calling functions
% to be specialized.
% 
% This technique in combination with lambda expressions, leads to readable and concise code, yet
% it enjoys the optimizing features of the Quasar compiler. For example, gradient_filter takes
% advantage of shared memory. In addition, the compiler is able to optimize the block dimensions
% of gradient_filter, knowing that the image, which is in RGBA format, has four color components.

im = imread("lena_big.tif", "rgba")

parallel_do(size(im), __kernel__ (pos) -> im[pos] = 255 * (im[pos] / 255) .^ 0.4)

function [] = __kernel__ gradient_filter (X, Y, pos)
    {!kernel_transform enable="localwindow"}
    Y[pos] = 4*X[pos] - 
             X[pos + [ 0,-1, 0]] - 
             X[pos + [ 0, 1, 0]] -
             X[pos + [-1, 0, 0]] -
             X[pos + [ 1, 0, 0]]             
end

im_out = zeros(size(im))
parallel_do(size(im_out), im, im_out, gradient_filter)


parallel_do(size(im_out), __kernel__ (pos) ->
    im_out[pos] = 4*im[pos] -
             im[pos + [ 0,-1, 0]] - 
             im[pos + [ 0, 1, 0]] -
             im[pos + [-1, 0, 0]] -
             im[pos + [ 1, 0, 0]])
             
imshow(im_out)