{!author name="Bart Goossens"}
{!doc category="Samples/Parallelization/Auto Parallelize"}
{!title name="Automatic parallelization example"}


%
% Section: Overview
%
% Demonstration of automatic parallelization of for-loops
% using the Quasar optimizer. The program performs a
% joint summation of the intensities in the image and a
% convolution.
%
% Using the pragma:
% :#pragma loop_parallelizer(off|on),
%
% the automatic loop parallelizer (ALP) can be activated/de-activated.
% The ALP detects "ideal" parallel loops and converts these loops
% to appropriate kernel functions, taking the data dependencies into account.
%
% With the pragma:
% :{!parallel for}
%
% it is possible to indicate that a for-loop needs to be parallelized.
% In case the parallelization fails, a compiler error is generated.

% Aug 18, 2012, Bart Goossens

% Use mirror boundary access outside the domain of im.
im : cube'mirror = imread("lena_big.tif")
y = zeros(size(im))
N = 5
mask = ones(2*N+1,2*N+1)/(2*N+1)^2
dim = size(im)
num_runs = 1
total_sum = zeros(3)

tic()
for k1=1..num_runs
    % Actually, this {!parallel for} is optional, but forces
    % the compiler to generate an error message in case the
    % parallelisation fails, guiding the user with messages
    % how to solve the cause of the failure.
    {!parallel for}
    for m=0..size(im,0)-1
        for n=0..size(im,1)-1
            total_sum += im[m,n,0..2]
            a = [0.,0.,0.]
            for k=-N..N
                for l=-N..N
                    a += mask[N+k,N+l] * im[m+k,n+l,0..2]
                end
            end
            y[m,n,0..2] = a
        end
    end
end

toc()
print total_sum,",",sum(sum(im,0),1)
imshow(y)
