{!author name="Bart Goossens"}
{!doc category="Samples/Parallelization/NaN mean"}
{!title name="mean calculation (excluding NaN values)"}

% Section: Overview
%
% This example demonstrates how to implement a mean function, that also takes into account that
% certain components of the matrix can be NaN. If this is the case, the values are excluded.

function y = nanmean(x : cube)
    function [total : scalar, count : int] = __kernel__ kernel(x : vec, pos : int)
        val = x[pos]
        if !isnan(val)
            total += val
            count += 1
        endif
    endfunction

    [total, count] = parallel_do(numel(x),x,kernel)
    y = total/count
endfunction

function [] = main()
    A = nan * ones(4,4)
    B = randn(200,200)
    print A
    print nanmean(B)
endfunction