{!author name="Bart Goossens"}
{!doc category="Samples/Parallelization/Partial parallel prefix sum"}
{!title name="Automatic code generation for the partial parallel prefix sum algorithm"}

% Section: Overview
%
% Calculation of a partial parallel prefix sum. The following for-loop calculates a cumulative sum 
% along the second dimension `cumsum(.,1)` using a for loop. The for-loop will be optimized to 
% automatically use shared memory of the GPU and a parallel reduction algorithm. The advantage is 
% that the for-loop body can be modified, while the code remains being executed as a "parallel 
% reduction".
% 

function [] = main()
    im : cube'safe = imread("lena_big.tif")
    
    runs = 1000
    
    tic()
    {!interpreted for}
    for run = 0..runs-1   
        im_integral = zeros(size(im)) 
        
        for m=0..size(im,0)-1
            for n=0..size(im,1)-1
                for k=0..2
                    im_integral[m,n,k] = im_integral[m,n-1,k] + im[m,n,k]
                endfor
            endfor
        endfor
    endfor
    toc("User sum")
    
    tic()
    for run = 0..runs-1
        im_integral2 =  cumsum(im,1)
    endfor
    toc("Builtin function")
    
    imshow(im_integral)
    imshow(im_integral2)

endfunction
