{!author name="Bart Goossens"}
{!doc category="Samples/Parallelization/Partial parallel reduction"}
{!title name="Automatic code generation for the partial parallel reduction algorithm"}

% Section: Overview
%
% Calculation of a partial parallel reduction. The following for-loop calculates a sum along the 
% second dimension `sum(.,1)` using a for loop. The for-loop will be optimized to automatically
% use shared memory of the GPU and a parallel reduction algorithm. The advantage is that the 
% for-loop body can be modified, while the code remains being executed as a "parallel reduction".
% 
% Another operation that is accelerated is a cumulative sum (cumsum(.,1))

function [] = main()
    im : cube = imread("lena_big.tif")
    
    runs = 1000
    
    tic()
    {!interpreted for}
    for run = 0..runs-1   
        proj = zeros(size(im,0), size(im,2)) 
        
        for m=0..size(im,0)-1
            for n=0..size(im,1)-1
                for k=0..2
                    proj[m,k] += im[m,n,k].^2
                endfor
            endfor
        endfor
    endfor
    toc("User sum")
    
    tic()
    for run = 0..runs-1
        proj2 = squeeze(sum(im.^2,1))
    endfor
    toc("Builtin function")
    
    plot(proj)
    plot(proj2)
    assert(proj == proj2, "The results were not the same!")

endfunction
