# Example of launching a Quasar kernel from Python
# Python equivalent of the color_temperature.q sample file
import pyquasar as q

kernel_src = """
lerp = __device__ (a,b,c) -> a*c+b*(1-c)

function [] = __kernel__ color_temperature(x : cube, y : cube, temp : scalar, cold : vec3, hot : vec3, pos : vec2)

    input = x[pos[0],pos[1],0..2]
    if temp<0
        output = lerp(input,cold,(-0.25)*temp)
    else
        output = lerp(input,hot,0.25*temp)
    endif
    y[pos[0],pos[1],0..2] = output
endfunction
""";


q.init("cuda", load_compiler=1)  # 1 to load the compiler (note: libhelios does not provide a compiler)
q.compile_string("color_temperature", kernel_src)

def apply(img_in, temp):
    hot = q.value([1,0.2,0]) * 255
    cold = q.value([0.3,0.4,1]) * 255
    img_out = q.zeros(q.size(img_in))
    q.parallel_do(q.size(img_out,[0,1]),img_in, img_out, temp, cold, hot, q.color_temperature)
    return img_out

img_in = q.imread("lena_big.tif")
temp = 1
img_out = apply(img_in, temp)
q.imshow(img_out)
q.run_gui()