{!author name="Bart Goossens"}
{!doc category="Samples/Python"}
{!title name="Use NumPy from Quasar through Python"}

% Note: see the PyQuasar reference manual for explanation of the programming 
% techniques being used.

import "pyquasar.${BITNESS}.${NATIVEEXT}"

py = pyimport("builtins")
np = pyimport("numpy")

% Reduction to calculate the transpose of a matrix in a numpy way.
reduction x -> x.T = np.transpose(x)

function [] = main()
    py.print(py.type(5.0))
    py.print(py.type(pyint(5.0)))

	v = np.array([1,2,3])  % v has shape (3,)
	w = np.array([4,5])    % w has shape (2,)

    print(pyint(1e20).to_string())
    print(v.to_string())
	py.print("v",v,sep:="=")

	% To compute an outer product, we first reshape v to be a column
	% vector of shape (3, 1); we can then broadcast it against w to yield
	% an output of shape (3, 2), which is the outer product of v and w:
	% [[ 4  5]
	%  [ 8 10]
	%  [12 15]]
    py.print(np.reshape(v, pyint([3, 1])))

	% Add a vector to each row of a matrix
	x = np.array([[1,2,3], [4,5,6]])
	py.print(x)

	% x has shape (2, 3) and v has shape (3,) so they broadcast to (2, 3),
	% giving the following matrix:
	% [[2 4 6]
	%  [5 7 9]]
	py.print(x + v)

	% Add a vector to each column of a matrix
	% x has shape (2, 3) and w has shape (2,).
	% If we transpose x then it has shape (3, 2) and can be broadcast
	% against w to yield a result of shape (3, 2); transposing this result
	% yields the final result of shape (2, 3) which is the matrix x with
	% the vector w added to each column. Gives the following matrix:
	% [[ 5  6  7]
	%  [ 9 10 11]]
	py.print((x.T + w).T)
	
	% Another solution is to reshape w to be a column vector of shape (2, 1);
	% we can then broadcast it directly against x to produce the same
	% output.
	py.print(x + np.reshape(w, pyint([2, 1])))

	% Multiply a matrix by a constant:
	% x has shape (2, 3). Numpy treats scalars as arrays of shape ();
	% these can be broadcast together to shape (2, 3), producing the
	% following array:
	% [[ 2  4  6]
	%  [ 8 10 12]]
	py.print(x * 2)
	print(x.from_py())

	% Tuple test
	py.print(py.tuple({1,2,3,4}))

endfunction
