{!author name="Bart Goossens"}
{!doc category="Samples/Python/TensorFlow"}
{!title name="TensorFlow example program"}

% Section: TensorFlow example program
%
% Demonstrates how to make TensorFlow calls from Quasar. Uses PyQuasar (Python bindings
% library for Quasar). PyQuasar is included in your Quasar installation.
% Notes: 
% * make sure Python 3.7 (Windows)/3.6 (Ubuntu) is installed with tensorflow
% (pip install tensorflow)
% * Windows users: TensorFlow may complain about some CUDA dlls not being in the path.
% If this is the case, adjust your system PATH.
%
import "pyquasar.${BITNESS}.${NATIVEEXT}"

function y : cube'unchecked = uint8_to_float(x : cube[uint8]'unchecked)
    y = uninit(size(x))
    {!parallel for}
    for [m,n,p]=0..size(x,0..2)-1
        y[m,n,p] = float(x[m,n,p])
    endfor
endfunction

function [] = main()
    py = pyimport("builtins")
    np = pyimport("numpy")
    tf = pyimport("tensorflow")

    % The following code limits the amount of GPU memory used by TensorFlow, so that
    % Quasar can also still allocate some memory.
    gpu_devices = tf.config.experimental.list_physical_devices("GPU")    
    gpu_device = gpu_devices[pyint(0)]
    tf.config.experimental.set_virtual_device_configuration(gpu_devices[pyint(0)], py.list({tf.config.experimental.VirtualDeviceConfiguration(memory_limit:=1024)}))
     
    mnist = tf.keras.datasets.mnist

    [train, test] = mnist.load_data().from_py()
    [x_train, y_train] = train 
    [x_test, y_test] = test 

    x_train = uint8_to_float(x_train) / 255.0
    y_train = reshape(uint8_to_float(y_train) / 255.0,[60000,1])  % Interpreter: dimensions changed?
    print "size(x_train)=", size(x_train), " size(y_train)=", size(y_train)

    y_train = np.reshape(y_train, pyint([60000,1,1]))
    y_test = np.reshape(y_test, pyint([10000,1,1]))

    model = tf.keras.models.Sequential(py.list({
      tf.keras.layers.Flatten(input_shape:=py.list(pyint([28, 28]))),
      tf.keras.layers.Dense(128, activation:="relu"),
      tf.keras.layers.Dropout(0.2),
      tf.keras.layers.Dense(10, activation:="softmax")
    }))

    model.compile(optimizer="adam",
                  loss="sparse_categorical_crossentropy",
                  metrics=py.list({"accuracy"}))

    model.fit(x_train, y_train, epochs:=pyint(5))
    model.evaluate(x_test, y_test, verbose:=pyint(2))
endfunction

