%REQUIRES_DISPLAY
{!author name="Bart Goossens"}
{!doc category="Samples/Video Processing/Recursive filter"}
{!title name="Implementation of a temporally recursive video filter"}

% Section: Overview
%
% Demonstration of a temporally recursive video filter, using the
% Quasar video library.
% 

% Video player demonstration - 
% videos are rendered as 3D surfaces
import "Quasar.Video.dll"
import "Quasar.UI.dll"
import "system.q"
import "inttypes.q"

function [] = temporalrecursivefilter(frame_in : cube'unchecked, frame_out : cube'unchecked, _
        alpha : scalar)
    function [] = __kernel__ filter_kernel(pos : ivec3)

        frame_out[pos] = frame_out[pos] * alpha + frame_in[pos] * (1 - alpha)
    end

    parallel_do(size(frame_in),filter_kernel)
end

function [] = main()

    state = struct()
    state.rendered_initialized = false
    state.filter_initialized = false
    [state.is_playing, state.allow_seeking, state.show_next_frame] = [true, true, true]
    state.last_frame = 0

    file_name ="C:\Quasar\Demos\Media\big_buck_bunny_1080p_surround.avi"

    stream = vidopen(file_name) % Opens the specified video file for playing
    print stream % Gives information about this stream
    sz = [stream.frame_height,stream.frame_width,3]

    frm = form("Recursive filtering demo")
    frm.move(0,0)
    frm.width = 1200
    button_stop=frm.add_button("Stop")
    button_stop.icon = imread("Media/control_stop_blue.png")
    button_play=frm.add_button("Play")
    button_play.icon = imread("Media/control_play_blue.png")
    button_fullrewind=frm.add_button("Full rewind")
    button_fullrewind.icon = imread("Media/control_start_blue.png")
    position = frm.add_slider("Position",0,0,stream.duration_sec)
    button_stop.onclick.add(() -> state.is_playing = false)
    button_play.onclick.add(() -> state.is_playing = true)
    button_fullrewind.onclick.add(() -> (vidseek(stream, 0); state.show_next_frame = true))    
    position.onchange.add(() -> state.allow_seeking ? vidseek(stream, position.value) : [])

    frm.add_heading("Controls")
    sliderAlpha = frm.add_slider("Alpha",0.5,0,1)
    sliderNoise = frm.add_slider("Noise",0,0,50)
    disp = frm.add_display()
    disp.sync_framerate(25)
    state.renderer = disp.create_opengl_renderer()        

    frm.show()

    vidstate = object()

    frames = 0

    img = zeros(sz)
    tic()
    while !frm.closed() 
        if state.is_playing || state.show_next_frame            
            if state.is_playing ? !vidreadframe(stream) : false
                vidseek(stream,0) % Play in a loop
            endif       
            vidstate.frame = float(stream.rgb_data) + sliderNoise.value * randn(size(stream.rgb_data))

            temporalrecursivefilter(vidstate.frame, img, sliderAlpha.value)
            
            % Display the depth map in 2D
            h=disp.imshow(img,[0,255])
           
            state.allow_seeking = false
            position.value = stream.pts
            state.allow_seeking = true
            state.show_next_frame = false
        endif
        pause(0.5*1000 / stream.avg_frame_rate)
    end
    time = toc()
    print frames/(time)*1000, " FPS"


    print "Success!!"

end



