%REQUIRES_DISPLAY
{!author name="Bart Goossens"}
{!doc category="Samples/Video Processing/Video player"}
{!title name="A simple video viewer with GUI"}

% Section: Overview
% 
% This example demonstrates how Quasar <User interfaces> can be used in
% combination with the video library (<Video Processing>) in order to create
% an interactive video player with several filtering options. Even with a 
% very limited number of lines of Quasar code, a complex video player can be
% built.
%

% Writing a custom video player in Quasar
% Bart Goossens, Oct 28, 2013
import "Quasar.Video.dll"
import "Quasar.UI.dll"
import "system.q"
import "inttypes.q"
import "colortransform.q"
import "imhist.q"

function [] = main()
    %stream = vidopen("C:/Bart/Quasar/t/Samples/Media/streetColor.avi") % Opens the specified video file for playing
    stream = vidopen("Media/streetColor.avi") % Opens the specified video file for playing
    vidstate = object()
    [vidstate.is_playing, vidstate.allow_seeking, vidstate.show_next_frame] = [true, true, true]
    print "Video stream information: ",stream

    frm = form("Video player")
    frm.center()
    frm.show()
    [frm.width, frm.height] = [2*stream.frame_width + 80, 2*stream.frame_height + 120]
    button_stop=frm.add_button("Stop")
    button_stop.icon = imread("Media/control_stop_blue.png")
    button_play=frm.add_button("Play")
    button_play.icon = imread("Media/control_play_blue.png")
    button_fullrewind=frm.add_button("Full rewind")
    button_fullrewind.icon = imread("Media/control_start_blue.png")
    button_resetcolors=frm.add_button("Reset colors")
    button_resetcolors.icon = imread("Media/control_equalizer_blue.png")
    check_showhist=frm.add_checkbox("Show histogram", true)
    position = frm.add_slider("Position",0,0,stream.duration_sec)
    video_display = frm.add_display()
    video_display.sync_framerate(stream.avg_frame_rate) % Sets the frame rate
    button_stop.onclick.add(() -> vidstate.is_playing = false)
    button_play.onclick.add(() -> vidstate.is_playing = true)
    button_fullrewind.onclick.add(() -> (vidseek(stream, 0); vidstate.show_next_frame = true))    
    position.onchange.add(() -> vidstate.allow_seeking ? vidseek(stream, position.value) : [])
    
    slider_hue = frm.add_slider("Hue", 0, -1, 1)
    slider_sat = frm.add_slider("Saturation", 0, -1, 1)
    slider_light = frm.add_slider("Lightness", 0, -1, 1)
    slider_hue.onchange.add(() -> vidstate.show_next_frame = true)
    slider_sat.onchange.add(() -> vidstate.show_next_frame = true)
    slider_light.onchange.add(() -> vidstate.show_next_frame = true)
    button_resetcolors.onclick.add(() -> slider_hue.value = slider_sat.value = slider_light.value = 0)
    
    layer = new(qvectorlayer)
    hold("invisible")

    while true
        if vidstate.is_playing || vidstate.show_next_frame            
            if vidstate.is_playing ? !vidreadframe(stream) : false
                vidseek(stream,0) % Play in a loop
            endif            
            frame_buffer = float(stream.rgb_data)
            hsl_correct(frame_buffer, frame_buffer, slider_hue.value, slider_sat.value, slider_light.value)
            h = video_display.imshow(frame_buffer)
            if frm.closed()
                break
            endif
            h.clearlayers()
            if check_showhist.value
                h.addlayer(layer)
                hist = imhist(frame_buffer)
                p = plot(hist/max(hist))
                p.left_margin = p.top_margin = p.right_margin = p.bottom_margin = 18
                p.xtick = 0..64..256
                p.ytick = [0,0.5,1]
                %legend("Red","Green","Blue")
                title("Histogram")
                layer.clear()
                layer.drawplot([0.7,-0.3],[1.3,0.3],p)
            endif
            vidstate.allow_seeking = false
            position.value = stream.pts
            vidstate.allow_seeking = true
            vidstate.show_next_frame = false
        endif 
        pause(0)
    end
end