%REQUIRES_DISPLAY
{!author name="Bart Goossens"}
{!doc category="Samples/Video Processing/Webcam demo"}
{!title name="A simple webcam viewer"}

% Section: Overview
% 
% This example demonstrates how Quasar <User interfaces> can be used in
% combination with the video library (<Video Processing>) in order to create
% an interactive webcam viewer with several filtering options. 
%

% Writing a custom webcam in Quasar
% Bart Goossens, July 31, 2014
import "Quasar.Video.dll"
import "Quasar.UI.dll"
import "system.q"
import "inttypes.q"
import "colortransform.q"

function [] = main()
    cams = vidcamlist()
    if numel(cams)==0
        error "Unfortunately, we could not find a webcam in your system."
    endif
    % select the first webcam
    cam = cams[0]
    cam = "/dev/video1"

    stream = vidopen(sprintf("cam:video=""%s"",video_size=768x1024,frame_rate=30",cam)) % Opens the specified video file for playing
    vidstate = object()
    [vidstate.is_playing, vidstate.allow_seeking, vidstate.show_next_frame] = [true, true, true]
    print "Video stream information: ",stream

    frm = form(sprintf("Webcam viewer - %s", cam))
    frm.center()
    frm.show()
    [frm.width, frm.height] = [stream.frame_width + 80, stream.frame_height + 120]
    button_resetcolors=frm.add_button("Reset colors")
    button_resetcolors.icon = imread("Media/control_equalizer_blue.png")
    cam_selector=frm.add_combobox("Camera:", cams)
    cam_selector.selected_index = 0    
    video_display = frm.add_display()
    video_display.sync_framerate(stream.avg_frame_rate) % Sets the frame rate   
    
    slider_hue = frm.add_slider("Hue", 0, -1, 1)
    slider_sat = frm.add_slider("Saturation", 0, -1, 1)
    slider_light = frm.add_slider("Lightness", 0, -1, 1)
    slider_hue.onchange.add(() -> vidstate.show_next_frame = true)
    slider_sat.onchange.add(() -> vidstate.show_next_frame = true)
    slider_light.onchange.add(() -> vidstate.show_next_frame = true)
    button_resetcolors.onclick.add(() -> slider_hue.value = slider_sat.value = slider_light.value = 0)
    
    while !frm.closed()
        if vidstate.is_playing || vidstate.show_next_frame            
            if vidstate.is_playing ? !vidreadframe(stream) : false
                vidseek(stream,0) % Play in a loop
            endif            
            frame_buffer = float(stream.rgb_data)
            hsl_correct(frame_buffer, frame_buffer, slider_hue.value, slider_sat.value, slider_light.value)
            video_display.imshow(frame_buffer)
            vidstate.allow_seeking = false
            vidstate.allow_seeking = true
            vidstate.show_next_frame = false
        endif 
        pause(0)
    end
end