%
% PyTorch sample program
% Demonstrates how to make PyTorch calls from Quasar
% Requires the following dependencies
% - Python 3.6 (or higher)
% - PyTorch
% - PyQuasar
%

import "pyquasar.${BITNESS}.${NATIVEEXT}"

function [] = pytorch_sample()

	py = pyimport("builtins")
	np = pyimport("numpy")
	torch = pyimport("torch")
	nn = pyimport("torch.nn")

	dtype = torch.float32
	device = torch.device("cuda")

	[N, D_in, H, D_out] = [64, 1000, 100, 10]

	% Create random Tensors to hold input and outputs.
	x = torch.randn(N, D_in, device:=device, dtype:=dtype)
	y = torch.randn(N, D_out, device:=device, dtype:=dtype)

	% Create random Tensors for weights.
	w1 = torch.randn(D_in, H, device:=device, dtype:=dtype, requires_grad:=pybool(true))
	w2 = torch.randn(H, D_out, device:=device, dtype:=dtype, requires_grad:=pybool(true))

	learning_rate = 1e-6
	relu = pyfunc(nn.ReLU(inplace:=false))

    for t=1..500
		% Forward pass: compute predicted y using operations
		y_pred = relu(x.mm(w1)).mm(w2)

		% Compute and print loss
		loss = (y_pred - y).pow(2).sum()
        if mod(t, 100) == 99
			print t, loss.item()
        endif

		% Use autograd to compute the backward pass
		loss.backward()

		% Update weights using gradient descent
		no_grad = torch.no_grad()
		no_grad.enter()
		w1 -= w1.grad * learning_rate 
		w2 -= w2.grad * learning_rate
		no_grad.exit()
    endfor

endfunction
