% Differentiable programming project - Photometric registration
% Estimates a function to map the intensities from one image onto
% another image, using a gradient descent algorithm with 
% automatic differentiation.

import "Quasar.CompMath.dll"

function C = cost_func(a : vec'clamped(256), x : cube, y : cube)   
    C = 0.0
	% Data fitting term
    for [m,n,p]=0..size(x,0..2)-1
        {!function_transform enable="sharedmempromotion"}
        C += (x[m,n,p] - a[y[m,n,p]]).^2  
    endfor
	% Smoothness regularization term
    lambda = 0.5*numel(x)
    for i=1..254
        C += lambda * (a[i-1] - 2*a[i] + a[i]).^2
    endfor
endfunction

function y = gradient_descent(cost_fn, x, step_size=1e-15, max_iter=250, epsilon=1e-8)
    gradient = $unapply($diff(cost_fn(x), x), x)
    
    y = x   % Initial solution

    for k=0..max_iter
        y_old = y
        y = y_old - step_size * gradient(y)
        if sum(isnan(y))>0
            print "NaN detected at iteration ",k
            break
        endif
		
		if mod(k,100)==0
            print "iter ",k, " cost_fn=", cost_fn(y)
        endif

        % Stop condition
        if sum(abs(y-y_old)) < epsilon
            break
        endif
    endfor
endfunction

reduction (a : cube) -> a[:] = reshape(a,numel(a))

function [] = main()
    im1 = imread("lighthouse.png")
    im1 += 10*randn(size(im1))
    im1 = min(max(im1,10),240)
    im2 = 255 * (im1 / 255).^0.4 + 20 + 10*randn(size(im1))   

    if sum(isnan(im2))>0
        error "im2 contains NaNs"
    endif
        
    scatter(im1[:],im2[:]).set_renderer("opengl")
    hold("on")
    a = 0..255
	tic()
	% Use Gradient Descent (GD) to minimize the cost function. Note that in this
	% case, GD is incredibly slow, but it can be replaced by a faster algorithm
    a_out = gradient_descent(((a : vec(256)) -> cost_func(a, im2, im1)), a, 1e-6, 5000)
    toc()
    hold("on")
    plot(a_out)
endfunction