%
% New Quasar graphical user interface project
%
import "Quasar.UI.dll"
import "imhist.q"

function [] = main()
    img = imread("lena_big.tif")
    img_out = copy(img)

    %=================================
	% frm
	%
    frm = form("Quasar GUI demonstration")
    frm.move(150, 150)
    frm.width = 300
    frm.show()
    frm.add_heading("Heading")

    slider = frm.add_slider("Gamma",1.0,0.5,2)
    slider_min = frm.add_slider("Minimum",0,0,255)
    slider_max = frm.add_slider("Maximum",255,0,255)
    cb = frm.add_checkbox("Enable minimum and maximum clipping",true)
    cb2 = frm.add_checkbox("Output visible",true)
    frm.add_heading("Display")
    disp = frm.add_display()

    %=================================
	% frm_output
	%
    frm_output = form("Output window")
    frm_output.move(frm.left + frm.width, frm.top)
    frm_output.width = 700
    img_disp = frm_output.add_display()
    img_disp.set_aspect(4/3)
    frm_output.show()

    slider.value = 0.75

    function [] = update_display()
        if cb.value
            gamma = slider.value
            rng = [slider_min.value, slider_max.value]
            new_img = 255*(img / 255).^gamma
            new_img = new_img .* (new_img > rng[0]) + rng[0] .* (new_img <= rng[0])
            new_img = new_img .* (new_img < rng[1]) + rng[1] .* (new_img >= rng[1])
            img_out[:,:,:] = new_img
        else
            img_out[:,:,:] = img % copy content
            rng = [0, 255]
        endif
        h = imhist(img_out,0..255)
        h[:,rng[0]] = 0 % clip borders of the histogram
        h[:,rng[1]] = 0
        disp.plot(h)
        title("histogram")

        img_disp.imshow(img_out,[0,255])
    endfunction

    function [] = toggle_display()
        frm_output.visible = cb2.value
    endfunction

    slider.onchange.add(update_display)
    slider_min.onchange.add(update_display)
    slider_max.onchange.add(update_display)
    cb.onchange.add(update_display)
    cb2.onchange.add(toggle_display)
    update_display()

    frm3 = form("Form 3")
    frm3.move(frm.left,frm.top+frm.width)
    frm3.width=frm.width
    frm3.show()
    frm3.add_heading("Heading")
    button1=frm3.add_button("Display data")
    button2=frm3.add_button("Sine and cosine")
    button3=frm3.add_button("Show image")
    button4=frm3.add_button("Clear")

    disp2 = frm3.add_display()
    function [] = button1_click()
        disp2.print("Display")
        disp2.print("-------")
        disp2.print("Average intensity:",mean(img_out))
    endfunction

    function [] = button2_click()
        x = linspace(0,2*pi,255)
        disp2.plot(x,sin(x),x,cos(x))
        title("Sine and cosine")
    endfunction
    
    function [] = button3_click()
        disp2.imshow(img,[0,255])
    endfunction
    
    function [] = button4_click()
        disp2.remove_all()
    endfunction

    button1.onclick.add(button1_click)
    button2.onclick.add(button2_click)
    button3.onclick.add(button3_click)
    button4.onclick.add(button4_click)

    frm.wait() % wait until this form is closed
endfunction
