% This program demonstrates an unsharp masking filter and the 
% use of graphical user interfaces.

import "Quasar.UI.dll"
import "system.q"
import "imfilter.q"

function [y, mask] = unsharp_masking(x, sigma)
    
    mask = [[0,0,0],
            [0,1,0],
            [0,0,0]] -
           0.5 * sigma ^ 2 * 
            [[0, 1, 0], 
             [1, -4, 1], 
             [0, 1, 0]]
    y = imfilter(x, mask)
                        
endfunction

function [] = main()

    im = imread("F35.jpg")
    state = object()
    state.sigma_noise = 0
    state.sigma_blur = 0
    state.show_spectrum = false
    
    % Apply some Gaussian blur to the input image
    im = gaussian_filter(im, 2)
    
    frm = form("Unsharp masking demo")
    frm.width = 1200
    slider_noise = frm.add_slider("noise standard deviation", state.sigma_noise, 0, 100)
    slider_blur = frm.add_slider("blur sigma", state.sigma_blur, 0, 8)
    checkbox = frm.add_checkbox("show spectrum", state.show_spectrum)
    slider_noise.onchange.add(() -> state.sigma_noise = slider_noise.value)
    slider_blur.onchange.add(() -> state.sigma_blur = slider_blur.value)
    checkbox.onchange.add(() -> state.show_spectrum = checkbox.value)
    disp = frm.add_display()
	disp.sync_framerate(50) % 50 FPS
    
    layer = new(qvectorlayer)
    
    frm.show()
    while !frm.closed()
        im_noisy = im + state.sigma_noise * randn(size(im))
        disp.subplot(1, 2, 0)
        p1 = disp.imshow(im_noisy,[0,255])
        disp.title("Degraded image")
        disp.subplot(1, 2, 1)

        % Unsharp masking is done here.        
        [im_sharpened, mask] = unsharp_masking(im_noisy, state.sigma_blur)
        
        if state.show_spectrum == false        
            p2 = disp.imshow(im_sharpened, [0, 255])
        else
            p2 = disp.imshow(log(eps + fftshift2(abs(fft2(im_sharpened)))), [-1, 8])
        endif
        
        layer.clear()
        layer.setfont("Courier new", 12)        
        layer.setbrushcolor("white")
        layer.drawstring(sprintf("[%+.02f %+.02f %+.02f]", mask[0,0], mask[0,1], mask[0,2]), [10, 10], [400, 20])
        layer.drawstring(sprintf("[%+.02f %+.02f %+.02f]", mask[1,0], mask[1,1], mask[1,2]), [10, 25], [400, 35])
        layer.drawstring(sprintf("[%+.02f %+.02f %+.02f]", mask[2,0], mask[2,1], mask[2,2]), [10, 40], [400, 50])        
        
        p1.connect(p2)
        p2.clearlayers()
        p2.addlayer(layer)
        disp.title("Filtered image")
    endwhile

endfunction