%
% This example demonstrates the opening, filtering and displaying of videos in Quasar,
% using the functions <vidopen>, <vidseek>, <vidreadframe>, <sync_framerate> and 
% <imshow>.
%

import "Quasar.Video.dll"
import "inttypes.q"

% Below is a simple video filter to illustrate the real-time
% video processing capabilities of Quasar. Of course, much
% more complex kernels are possible.
function [] = filter(im)
    function [] = __kernel__ kernel (y : cube, pos : ivec2)
        x = y[pos[0],pos[1],0..2]

        if pos[0] > 100 && pos[0] < size(y,0)-100 && _
           pos[1] > 100 && pos[1] < size(y,1)-100

            % switch R and B
            tmp = x[0]
            x[2] = tmp
            x[0] = x[2]
            y[pos[0],pos[1],0..2] = x
        endif
    endfunction
    parallel_do(size(im,0..1),im,kernel)
endfunction

function [] = main()
    file_name = "cam:video=Integrated Webcam"
    %file_name = "Sample.mp4"

    stream = vidopen(file_name) % Opens the specified video file for playing
    print stream % Gives information about this stream

    sz = [stream.frame_height,stream.frame_width,3]   
    looping = false

    % Sets the frame rate for the imshow function
    sync_framerate(stream.avg_frame_rate) 

    repeat
        % Reads until there is no frame left. 
        if !vidreadframe(stream)
            if looping
                % Jump back to the first frame
                vidseek(stream, 0)
            else
                break % Quit!
            endif
        endif
        % Important: Quasar allows raw processing of video data in cube[uint8]
        % format. In this format, there are four color channels (R,G,B and A)
        frame = float(stream.rgb_data)

        % Our custom frame filter
        filter(frame)

        % Displays the frame. Note: the default intensity range is [0,255]
        imshow(frame)
    until !hold("on")
endfunction
