//==============================================================================
// String helper routines - to deal with 16-bit UNICODE string in a platform
// independent way. (wchar_t is 32-bit in linux and 16-bit in windows, so it
// cannot be used for this purpose). The string functions are placed here to
// have independence of the underlying string representation (e.g. for future
// platforms which may have different string encodings).
//==============================================================================
#include <stdio.h>
#include <limits.h>
#include <stdarg.h>
#include <string.h>
#include <stddef.h>
#include <stdlib.h>
#if !defined(_MSC_VER) || _MSC_VER >= 1400
#include <stdint.h>
#endif
#include <float.h>
#include <math.h>
#include <ctype.h>
#include <errno.h>
#include "char16_string.h"
#ifdef __GNUC__
#include <wctype.h>
#endif


QUASARHOSTAPI LPTSTR tcsncpy(LPTSTR sTo, LPCTSTR sFrom, size_t N)
{
#ifdef _MSC_VER
	wcsncpy_s(sTo, N, sFrom, N);
	return sTo;
#else
	return wcsncpy(sTo, sFrom, N);
#endif
}
QUASARHOSTAPI LPTSTR tcschr(LPCTSTR str, TCHAR c)
{
	return wcschr((LPTSTR)str, c);
}
QUASARHOSTAPI size_t tcslen(LPCTSTR str)
{
	return wcslen(str);
}
QUASARHOSTAPI size_t tcsnlen(LPCTSTR str, size_t N)
{
	return wcsnlen(str, N);
}
QUASARHOSTAPI int vtfprintf(FILE * handle, LPCTSTR fmt, va_list ap)
{
#ifdef _MSC_VER
	return vfwprintf(handle, fmt, ap);
#else
	// use printf for the printing. The reason is that in GCC runtime library
	// it is not possible to mix printf with wprintf. When using wprintf, all
	// MONO debug messages are gone.
	TCHAR buffer[4096];
	char mbs[4096];
	int result = vswprintf(buffer, 4095, fmt, ap);
  wcstombs(mbs, buffer, result);
	mbs[result] = '\0';
	printf("%s", mbs);
	return result;
#endif
}
QUASARHOSTAPI int vstprintf(LPTSTR buffer, size_t N, LPCTSTR fmt, va_list args)
{
	return vswprintf(buffer, N, fmt, args);
}
QUASARHOSTAPI int vtprintf(LPCTSTR fmt, va_list ap)
{
	return vtfprintf(stdout, fmt, ap);
}
QUASARHOSTAPI int stprintf(LPTSTR s, size_t n, LPCTSTR fmt, ...)
{
    int ret;
    va_list ap;
    va_start(ap, fmt);
    ret = vstprintf(s, n, fmt, ap);
    va_end(ap);
    return ret;
}

QUASARHOSTAPI int tprintf(LPCTSTR fmt, ...)
{
  int ret;
	va_list ap;
	va_start(ap, fmt);
	ret = vtprintf(fmt, ap);
	va_end(ap);
	return ret;
}

QUASARHOSTAPI int stscanf(LPCTSTR buf, LPCTSTR fmt, ...)
{
	int ret;
	va_list ap;
	va_start(ap, fmt);
	ret = vswscanf(buf, fmt, ap);
	va_end(ap);
	return ret;
}

QUASARHOSTAPI int tstrcmp(LPCTSTR s1, LPCTSTR s2)
{
	return wcscmp(s1, s2);
}

QUASARHOSTAPI int tstrncmp(LPCTSTR s1, LPCTSTR s2, size_t size)
{
	return wcsncmp(s1, s2, size);
}


QUASARHOSTAPI int istdigit(TCHAR chr)
{
	return chr>='0' && chr<='9';
}

QUASARHOSTAPI int istalpha(TCHAR chr)
{
	return chr>='A' && chr<='Z' || chr>='a' && chr<='z';
}

QUASARHOSTAPI int ttoi(LPCTSTR a)
{
	LPTSTR str;
	return (int) wcstol(a, &str, 10);
}

QUASARHOSTAPI TCHAR totupper(TCHAR chr)
{
#ifdef _MSC_VER
	return towupper(chr);
#else
	// TODO: locale sensitive conversion
	if (chr >= 'A' && chr <= 'Z') chr += ('a' - 'A');
	return chr;
#endif
}

QUASARHOSTAPI TCHAR totlower(TCHAR chr)
{
#ifdef _MSC_VER
	return towlower(chr);
#else
	// TODO: locale sensitive conversion
	if (chr >= 'a' && chr <= 'z') chr -= ('a' - 'A');
	return chr;
#endif
}
