#pragma once
#include <cstdarg>

#ifndef QUASARHOSTAPI
#ifdef QUASARHOST_DLLIMPORT
#define QUASARHOSTAPI __declspec(dllimport)
#elif defined(QUASARHOST_DLLEXPORT)
#define QUASARHOSTAPI __declspec(dllexport)
#else
#define QUASARHOSTAPI
#endif
#endif

namespace quasar
{
	namespace runtime
	{
	//============================================================================
	// Helper macros
	//============================================================================
	QUASARHOSTAPI void dbgAssert(const char *expr, const char *src_file, int line, const char *fmt, ...);
	QUASARHOSTAPI void dbgWarning(const char *src_file, int line, const char *fmt, ...);

	// Assertion macro: prints an error message and stops the program when
	//    the specified condition fails.
	#define Assert(EX,...) \
	  (void)((EX) || (dbgAssert (#EX, __FILE__, __LINE__, __VA_ARGS__),0))

	// Todo macro
	//#define Todo Assert(false, "Not implemented yet");

	// Warning macro: print warning information in DEBUG mode only
	#if _DEBUG
	#define Warning(...) \
		dbgWarning(__FILE__, __LINE__, __VA_ARGS__)
	#else
	#define Warning(...)
	#endif

	// NotReached macro: indicates that the current code path is never taken.
	#ifdef _MSC_VER
	#define NotReached __assume(0)
	#elif defined(__GNUG__) || defined (__clang__)
	#define NotReached __builtin_unreachable()
	#else
	#define NotReached
	#endif
	}
}