#!/bin/bash

SOURCE=$1
TARGET=$2
PLATFORM=$3
DEFINE=$4
DYNMEM=$5
COMPILER=$6

CPPFLAGS="$CPPFLAGS -D$DEFINE -D$DYNMEM"
CXXFLAGS="$CXXFLAGS -fdata-sections -ffunction-sections -fno-operator-names -fopenmp -ftls-model=global-dynamic"
LDFLAGS="-Wl,--gc-sections -Wno-unused"

# When using the Quasar Host API, we need to add some additional parameters.
# Note that this may require libglib2.0-dev to be installed.
if [[ "$USE_HOST_API" == "true" ]]; then
	CPPFLAGS="$CPPFLAGS `pkg-config --cflags glib-2.0 mono-2`"
	LDFLAGS="$LDFLAGS `pkg-config --libs glib-2.0 mono-2`"
fi

case "$COMPILER" in
Clang)
	CXX=clang++
	CMD="clang++ -x c++ $CPPFLAGS $CXXFLAGS $LDFLAGS "$SOURCE" -fPIC -shared -o "$TARGET""
	;;
IntelC_1[1-5])
	if [[ "$PLATFORM" == "-m32" ]]; then
		intel_platform="ia32"
	else
		intel_platform="intel64"
	fi
	source /opt/intel/bin/compilervars.sh $intel_platform

	CXX="icc"
	;;
*)
	if [ -f "/usr/bin/g++-7" ]; then
		CXX="g++-7"
	elif [ -f "/usr/bin/g++-6" ]; then
		CXX="g++-6"
	else
		CXX="g++"
	fi
	CXXFLAGS="$CXXFLAGS -fpermissive"
	;;
esac

set -x
$CXX $CPPFLAGS $CXXFLAGS $LDFLAGS -x c++ "$SOURCE" -fPIC -shared -o "$TARGET"
