#!/bin/bash

SOURCE=$1
TARGET=$2
PLATFORM=$3
ARCH=$4
JIT=$5

HOSTCC=${CXX:=/usr/bin/gcc}
HOSTCFLAGS="-w"

CPPFLAGS="$DEFINES"
NVCCFLAGS="$PLATFORM -lineinfo -arch=$ARCH -rdc=${CUDA_RDC:-false}"
PTXASFLAGS="--verbose"
if [ "$PLATFORM" == "-m64" ]; then
	NVCCFLAGS+=" -D_LP64"
fi

TARGET_FILENAME=$(basename "$TARGET")
TARGET_CUBIN_FILENAME="${TARGET_FILENAME%.*}.cubin"
TARGET_EXT=${TARGET_FILENAME##*.}

set -x

# We are building a native executable
if [ "$TARGET_EXT" == "64" ]; then

	CPPFLAGS="$CPPFLAGS --std=c++11 -Xcompiler `pkg-config --cflags glib-2.0 mono-2`"
	LDFLAGS="$LDFLAGS -Xlinker `pkg-config --libs glib-2.0 mono-2`"

	nvcc --compiler-bindir="$HOSTCC" -Xcompiler "$HOSTCFLAGS" --keep-dir=.  \
	    $CPPFLAGS $NVCCFLAGS $LDFLAGS -cudart static "$SOURCE" $CUDA_LINK_MODULES -o "$TARGET" -Xptxas --verbose
else
	if [ "$JIT" == "True" ]; then
		# We are building a PTX file
		nvcc --compiler-bindir="$HOSTCC" -Xcompiler "$HOSTCFLAGS" \
	    	$CPPFLAGS $NVCCFLAGS -Xptxas "$PTXASFLAGS" "$SOURCE" --ptx -o "$TARGET"
	else
    	nvcc --compiler-bindir="$HOSTCC" -Xcompiler "$HOSTCFLAGS" \
	    	$CPPFLAGS $NVCCFLAGS "$SOURCE" -cubin -o "$TARGET_CUBIN_FILENAME"
	fi
fi


