﻿//=============================================================================
// Quasar SIMD vector processing back-end
// Header file is automatically generated - do NOT modify!
// (C) 2017-2020 Gepura
//=============================================================================
#ifndef QUASAR_SIMD_H_INCLUDED
#define QUASAR_SIMD_H_INCLUDED

#ifndef QUASAR_H_INCLUDED
#error quasar_simd.h should not be included directly. Include quasar.h instead.
#endif

#ifdef HAS_AVX
#define HAS_SSE3
#define HAS_SSE41
#define HAS_SSE42
#endif

// Include files
#ifndef TARGET_CUDA
#if !defined(__ppc64__) && !defined(__aarch64__) && !defined(__arm__)
#include <emmintrin.h>
#include <xmmintrin.h>
#include <smmintrin.h>
#include <immintrin.h>
#endif
#elif defined(HAS_CUDA_HALF_SUPPORT)
#include "cuda_fp16.h"
#endif

// mm_set1_epi64x is not supported by older MSC compilers
#if !defined(_MSC_VER) || _MSC_VER >= 1800
#define HAS_mm_set1_epi64x
#endif

// Disable warning: "switch statement contains 'default' but no 'case' labels"
#ifdef _MSC_VER
#pragma warning (disable: 4065)
#endif

// CUDA implementation has no half operators
#if !defined(TARGET_CUDA) || __CUDA_ARCH__ < 750
#define __CUDA_NO_HALF_OPERATORS__
#endif

#ifndef FOR_ALL
#define FOR_ALL(X)                                                            \
    for (int i = 0; i < N; i++)                                               \
        (X);
#endif

// Helper intrinsics
#define _mm256_load1_ps(x) _mm256_set_ps(x, x, x, x, x, x, x, x)
#define _mm256_load1_pd(x) _mm256_set_pd(x, x, x, x)

#define _mm_not_epi32(a)	   _mm_andnot_si128((a), _mm_set1_epi32(~0))
#define _mm_cmpneq_epi32(a, b) _mm_not_epi32(_mm_cmpeq_epi32(a, b))
#define _mm_cmpge_epi32(a, b)  _mm_not_epi32(_mm_cmplt_epi32(a, b))
#define _mm_cmple_epi32(a, b)  _mm_not_epi32(_mm_cmpgt_epi32(a, b))

INLINE scalar1 __device__ __HOST_V__ operator -(scalar1 a)
{
	return make_scalar1(-a.x);
}
INLINE scalar2 __device__ __HOST_V__ operator -(scalar2 a)
{
	return make_scalar2(-a.x, -a.y);
}
INLINE scalar3 __device__ __HOST_V__ operator -(scalar3 a)
{
	return make_scalar3(-a.x, -a.y, -a.z);
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ operator -(scalarN<N> a)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = -a.el[i]);
    return y;
}
INLINE int1 __device__ __HOST_V__ operator -(int1 a)
{
	return make_int1(-a.x);
}
INLINE int2 __device__ __HOST_V__ operator -(int2 a)
{
	return make_int2(-a.x, -a.y);
}
INLINE int3 __device__ __HOST_V__ operator -(int3 a)
{
	return make_int3(-a.x, -a.y, -a.z);
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ operator -(intN<N> a)
{
    intN<N> y;
    FOR_ALL(y.el[i] = -a.el[i]);
    return y;
}
INLINE cscalar1 __device__ __HOST_V__ operator -(cscalar1 a)
{
	return make_cscalar1(-a.x);
}
INLINE cscalar2 __device__ __HOST_V__ operator -(cscalar2 a)
{
	return make_cscalar2(-a.x, -a.y);
}
INLINE cscalar3 __device__ __HOST_V__ operator -(cscalar3 a)
{
	return make_cscalar3(-a.x, -a.y, -a.z);
}
INLINE cscalar4 __device__ __HOST_V__ operator -(cscalar4 a)
{
	return make_cscalar4(-a.x, -a.y, -a.z, -a.w);
}
template <int N>
INLINE cscalarN<N> __device__ __HOST_V__ operator -(cscalarN<N> a)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = -a.el[i]);
    return y;
}
INLINE scalar1 __device__ __HOST_V__ operator +(scalar1 a, scalar1 b)
{
    return make_scalar1(a.x + b.x);
}
INLINE scalar1 __device__ __HOST_V__ operator +(scalar a, scalar1 b)
{
    return make_scalar1(a + b.x);
}
INLINE scalar1 __device__ __HOST_V__ operator +(scalar1 a, scalar b)
{
    return make_scalar1(a.x + b);
}
INLINE scalar2 __device__ __HOST_V__ operator +(scalar2 a, scalar2 b)
{
    return make_scalar2(a.x + b.x, a.y + b.y);
}
INLINE scalar2 __device__ __HOST_V__ operator +(scalar a, scalar2 b)
{
    return make_scalar2(a + b.x, a + b.y);
}
INLINE scalar2 __device__ __HOST_V__ operator +(scalar2 a, scalar b)
{
    return make_scalar2(a.x + b, a.y + b);
}
INLINE scalar3 __device__ __HOST_V__ operator +(scalar3 a, scalar3 b)
{
    return make_scalar3(a.x + b.x, a.y + b.y, a.z + b.z);
}
INLINE scalar3 __device__ __HOST_V__ operator +(scalar a, scalar3 b)
{
    return make_scalar3(a + b.x, a + b.y, a + b.z);
}
INLINE scalar3 __device__ __HOST_V__ operator +(scalar3 a, scalar b)
{
    return make_scalar3(a.x + b, a.y + b, a.z + b);
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ operator +(scalarN<N> a, scalarN<N> b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] + b.el[i]);
    return y;
}
template <int N> INLINE scalarN<N> __device__ __HOST_V__ operator +(scalar a, scalarN<N> b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a + b.el[i]);
    return y;
}
template <int N> INLINE scalarN<N> __device__ __HOST_V__ operator +(scalarN<N> a, scalar b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] + b);
    return y;
}
INLINE int1 __device__ __HOST_V__ operator +(int1 a, int1 b)
{
    return make_int1(a.x + b.x);
}
INLINE int1 __device__ __HOST_V__ operator +(int a, int1 b)
{
    return make_int1(a + b.x);
}
INLINE int1 __device__ __HOST_V__ operator +(int1 a, int b)
{
    return make_int1(a.x + b);
}
INLINE int2 __device__ __HOST_V__ operator +(int2 a, int2 b)
{
    return make_int2(a.x + b.x, a.y + b.y);
}
INLINE int2 __device__ __HOST_V__ operator +(int a, int2 b)
{
    return make_int2(a + b.x, a + b.y);
}
INLINE int2 __device__ __HOST_V__ operator +(int2 a, int b)
{
    return make_int2(a.x + b, a.y + b);
}
INLINE int3 __device__ __HOST_V__ operator +(int3 a, int3 b)
{
    return make_int3(a.x + b.x, a.y + b.y, a.z + b.z);
}
INLINE int3 __device__ __HOST_V__ operator +(int a, int3 b)
{
    return make_int3(a + b.x, a + b.y, a + b.z);
}
INLINE int3 __device__ __HOST_V__ operator +(int3 a, int b)
{
    return make_int3(a.x + b, a.y + b, a.z + b);
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ operator +(intN<N> a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] + b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator +(int a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a + b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator +(intN<N> a, int b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] + b);
    return y;
}
INLINE cscalar1 __device__ __HOST_V__ operator +(cscalar1 a, cscalar1 b)
{
    return make_cscalar1(a.x + b.x);
}
INLINE cscalar1 __device__ __HOST_V__ operator +(cscalar a, cscalar1 b)
{
    return make_cscalar1(a + b.x);
}
INLINE cscalar1 __device__ __HOST_V__ operator +(cscalar1 a, cscalar b)
{
    return make_cscalar1(a.x + b);
}
INLINE cscalar2 __device__ __HOST_V__ operator +(cscalar2 a, cscalar2 b)
{
    return make_cscalar2(a.x + b.x, a.y + b.y);
}
INLINE cscalar2 __device__ __HOST_V__ operator +(cscalar a, cscalar2 b)
{
    return make_cscalar2(a + b.x, a + b.y);
}
INLINE cscalar2 __device__ __HOST_V__ operator +(cscalar2 a, cscalar b)
{
    return make_cscalar2(a.x + b, a.y + b);
}
INLINE cscalar3 __device__ __HOST_V__ operator +(cscalar3 a, cscalar3 b)
{
    return make_cscalar3(a.x + b.x, a.y + b.y, a.z + b.z);
}
INLINE cscalar3 __device__ __HOST_V__ operator +(cscalar a, cscalar3 b)
{
    return make_cscalar3(a + b.x, a + b.y, a + b.z);
}
INLINE cscalar3 __device__ __HOST_V__ operator +(cscalar3 a, cscalar b)
{
    return make_cscalar3(a.x + b, a.y + b, a.z + b);
}
INLINE cscalar4 __device__ __HOST_V__ operator +(cscalar4 a, cscalar4 b)
{
    return make_cscalar4(a.x + b.x, a.y + b.y, a.z + b.z, a.w + b.w);
}
INLINE cscalar4 __device__ __HOST_V__ operator +(cscalar a, cscalar4 b)
{
    return make_cscalar4(a + b.x, a + b.y, a + b.z, a + b.w);
}
INLINE cscalar4 __device__ __HOST_V__ operator +(cscalar4 a, cscalar b)
{
    return make_cscalar4(a.x + b, a.y + b, a.z + b, a.w + b);
}
template <int N>
INLINE cscalarN<N> __device__ __HOST_V__ operator +(cscalarN<N> a, cscalarN<N> b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] + b.el[i]);
    return y;
}
template <int N> INLINE cscalarN<N> __device__ __HOST_V__ operator +(cscalar a, cscalarN<N> b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a + b.el[i]);
    return y;
}
template <int N> INLINE cscalarN<N> __device__ __HOST_V__ operator +(cscalarN<N> a, cscalar b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] + b);
    return y;
}
INLINE scalar1 __device__ __HOST_V__ operator +(int a, scalar1 b)
{
    return make_scalar1(a + b.x);
}
INLINE scalar2 __device__ __HOST_V__ operator +(int a, scalar2 b)
{
    return make_scalar2(a + b.x, a + b.y);
}
INLINE scalar3 __device__ __HOST_V__ operator +(int a, scalar3 b)
{
    return make_scalar3(a + b.x, a + b.y, a + b.z);
}
INLINE scalar4 __device__ __HOST_V__ operator +(int a, scalar4 b)
{
    return make_scalar4(a + b.x, a + b.y, a + b.z, a + b.w);
}
INLINE scalar1 __device__ __HOST_V__ operator +(scalar1 a, int b)
{
    return make_scalar1(a.x + b);
}
INLINE scalar2 __device__ __HOST_V__ operator +(scalar2 a, int b)
{
    return make_scalar2(a.x + b, a.y + b);
}
INLINE scalar3 __device__ __HOST_V__ operator +(scalar3 a, int b)
{
    return make_scalar3(a.x + b, a.y + b, a.z + b);
}
INLINE scalar4 __device__ __HOST_V__ operator +(scalar4 a, int b)
{
    return make_scalar4(a.x + b, a.y + b, a.z + b, a.w + b);
}
template <int N> INLINE scalarN<N> __device__ __HOST_V__ operator +(scalarN<N> a, int b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] + b);
    return y;
}
template <int N> INLINE scalarN<N> __device__ __HOST_V__ operator +(int a, scalarN<N> b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a + b.el[i]);
    return y;
}
INLINE scalar1 __device__ __HOST_V__ operator +(int1 a, scalar b)
{
    return make_scalar1(a.x + b);
}
INLINE scalar2 __device__ __HOST_V__ operator +(int2 a, scalar b)
{
    return make_scalar2(a.x + b, a.y + b);
}
INLINE scalar3 __device__ __HOST_V__ operator +(int3 a, scalar b)
{
    return make_scalar3(a.x + b, a.y + b, a.z + b);
}
INLINE scalar4 __device__ __HOST_V__ operator +(int4 a, scalar b)
{
    return make_scalar4(a.x + b, a.y + b, a.z + b, a.w + b);
}
INLINE scalar1 __device__ __HOST_V__ operator +(scalar a, int1 b)
{
    return make_scalar1(a + b.x);
}
INLINE scalar2 __device__ __HOST_V__ operator +(scalar a, int2 b)
{
    return make_scalar2(a + b.x, a + b.y);
}
INLINE scalar3 __device__ __HOST_V__ operator +(scalar a, int3 b)
{
    return make_scalar3(a + b.x, a + b.y, a + b.z);
}
INLINE scalar4 __device__ __HOST_V__ operator +(scalar a, int4 b)
{
    return make_scalar4(a + b.x, a + b.y, a + b.z, a + b.w);
}
template <int N> INLINE scalarN<N> __device__ __HOST_V__ operator +(scalar a, intN<N> b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a + b.el[i]);
    return y;
}
template <int N> INLINE scalarN<N> __device__ __HOST_V__ operator +(intN<N> a, scalar b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] + b);
    return y;
}
INLINE cscalar1 __device__ __HOST_V__ operator +(scalar a, cscalar1 b)
{
    return make_cscalar1(a + b.x);
}
INLINE cscalar2 __device__ __HOST_V__ operator +(scalar a, cscalar2 b)
{
    return make_cscalar2(a + b.x, a + b.y);
}
INLINE cscalar3 __device__ __HOST_V__ operator +(scalar a, cscalar3 b)
{
    return make_cscalar3(a + b.x, a + b.y, a + b.z);
}
INLINE cscalar4 __device__ __HOST_V__ operator +(scalar a, cscalar4 b)
{
    return make_cscalar4(a + b.x, a + b.y, a + b.z, a + b.w);
}
INLINE cscalar1 __device__ __HOST_V__ operator +(cscalar1 a, scalar b)
{
    return make_cscalar1(a.x + b);
}
INLINE cscalar2 __device__ __HOST_V__ operator +(cscalar2 a, scalar b)
{
    return make_cscalar2(a.x + b, a.y + b);
}
INLINE cscalar3 __device__ __HOST_V__ operator +(cscalar3 a, scalar b)
{
    return make_cscalar3(a.x + b, a.y + b, a.z + b);
}
INLINE cscalar4 __device__ __HOST_V__ operator +(cscalar4 a, scalar b)
{
    return make_cscalar4(a.x + b, a.y + b, a.z + b, a.w + b);
}
template <int N> INLINE cscalarN<N> __device__ __HOST_V__ operator +(cscalarN<N> a, scalar b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] + b);
    return y;
}
template <int N> INLINE cscalarN<N> __device__ __HOST_V__ operator +(scalar a, cscalarN<N> b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a + b.el[i]);
    return y;
}
INLINE cscalar1 __device__ __HOST_V__ operator +(scalar1 a, cscalar b)
{
    return make_cscalar1(a.x + b);
}
INLINE cscalar2 __device__ __HOST_V__ operator +(scalar2 a, cscalar b)
{
    return make_cscalar2(a.x + b, a.y + b);
}
INLINE cscalar3 __device__ __HOST_V__ operator +(scalar3 a, cscalar b)
{
    return make_cscalar3(a.x + b, a.y + b, a.z + b);
}
INLINE cscalar4 __device__ __HOST_V__ operator +(scalar4 a, cscalar b)
{
    return make_cscalar4(a.x + b, a.y + b, a.z + b, a.w + b);
}
INLINE cscalar1 __device__ __HOST_V__ operator +(cscalar a, scalar1 b)
{
    return make_cscalar1(a + b.x);
}
INLINE cscalar2 __device__ __HOST_V__ operator +(cscalar a, scalar2 b)
{
    return make_cscalar2(a + b.x, a + b.y);
}
INLINE cscalar3 __device__ __HOST_V__ operator +(cscalar a, scalar3 b)
{
    return make_cscalar3(a + b.x, a + b.y, a + b.z);
}
INLINE cscalar4 __device__ __HOST_V__ operator +(cscalar a, scalar4 b)
{
    return make_cscalar4(a + b.x, a + b.y, a + b.z, a + b.w);
}
template <int N> INLINE cscalarN<N> __device__ __HOST_V__ operator +(cscalar a, scalarN<N> b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a + b.el[i]);
    return y;
}
template <int N> INLINE cscalarN<N> __device__ __HOST_V__ operator +(scalarN<N> a, cscalar b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] + b);
    return y;
}
INLINE scalar1 __device__ __HOST_V__ operator +(int1 a, scalar1 b)
{
    return make_scalar1(a.x + b.x);
}
INLINE scalar2 __device__ __HOST_V__ operator +(int2 a, scalar2 b)
{
    return make_scalar2(a.x + b.x, a.y + b.y);
}
INLINE scalar3 __device__ __HOST_V__ operator +(int3 a, scalar3 b)
{
    return make_scalar3(a.x + b.x, a.y + b.y, a.z + b.z);
}
INLINE scalar4 __device__ __HOST_V__ operator +(int4 a, scalar4 b)
{
    return make_scalar4(a.x + b.x, a.y + b.y, a.z + b.z, a.w + b.w);
}
INLINE scalar1 __device__ __HOST_V__ operator +(scalar1 a, int1 b)
{
    return make_scalar1(a.x + b.x);
}
INLINE scalar2 __device__ __HOST_V__ operator +(scalar2 a, int2 b)
{
    return make_scalar2(a.x + b.x, a.y + b.y);
}
INLINE scalar3 __device__ __HOST_V__ operator +(scalar3 a, int3 b)
{
    return make_scalar3(a.x + b.x, a.y + b.y, a.z + b.z);
}
INLINE scalar4 __device__ __HOST_V__ operator +(scalar4 a, int4 b)
{
    return make_scalar4(a.x + b.x, a.y + b.y, a.z + b.z, a.w + b.w);
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ operator +(scalarN<N> a, intN<N> b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] + b.el[i]);
    return y;
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ operator +(intN<N> a, scalarN<N> b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] + b.el[i]);
    return y;
}
INLINE cscalar1 __device__ __HOST_V__ operator +(scalar1 a, cscalar1 b)
{
    return make_cscalar1(a.x + b.x);
}
INLINE cscalar2 __device__ __HOST_V__ operator +(scalar2 a, cscalar2 b)
{
    return make_cscalar2(a.x + b.x, a.y + b.y);
}
INLINE cscalar3 __device__ __HOST_V__ operator +(scalar3 a, cscalar3 b)
{
    return make_cscalar3(a.x + b.x, a.y + b.y, a.z + b.z);
}
INLINE cscalar4 __device__ __HOST_V__ operator +(scalar4 a, cscalar4 b)
{
    return make_cscalar4(a.x + b.x, a.y + b.y, a.z + b.z, a.w + b.w);
}
INLINE cscalar1 __device__ __HOST_V__ operator +(cscalar1 a, scalar1 b)
{
    return make_cscalar1(a.x + b.x);
}
INLINE cscalar2 __device__ __HOST_V__ operator +(cscalar2 a, scalar2 b)
{
    return make_cscalar2(a.x + b.x, a.y + b.y);
}
INLINE cscalar3 __device__ __HOST_V__ operator +(cscalar3 a, scalar3 b)
{
    return make_cscalar3(a.x + b.x, a.y + b.y, a.z + b.z);
}
INLINE cscalar4 __device__ __HOST_V__ operator +(cscalar4 a, scalar4 b)
{
    return make_cscalar4(a.x + b.x, a.y + b.y, a.z + b.z, a.w + b.w);
}
template <int N>
INLINE cscalarN<N> __device__ __HOST_V__ operator +(cscalarN<N> a, scalarN<N> b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] + b.el[i]);
    return y;
}
template <int N>
INLINE cscalarN<N> __device__ __HOST_V__ operator +(scalarN<N> a, cscalarN<N> b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] + b.el[i]);
    return y;
}
INLINE scalar1 __device__ __HOST_V__ operator -(scalar1 a, scalar1 b)
{
    return make_scalar1(a.x - b.x);
}
INLINE scalar1 __device__ __HOST_V__ operator -(scalar a, scalar1 b)
{
    return make_scalar1(a - b.x);
}
INLINE scalar1 __device__ __HOST_V__ operator -(scalar1 a, scalar b)
{
    return make_scalar1(a.x - b);
}
INLINE scalar2 __device__ __HOST_V__ operator -(scalar2 a, scalar2 b)
{
    return make_scalar2(a.x - b.x, a.y - b.y);
}
INLINE scalar2 __device__ __HOST_V__ operator -(scalar a, scalar2 b)
{
    return make_scalar2(a - b.x, a - b.y);
}
INLINE scalar2 __device__ __HOST_V__ operator -(scalar2 a, scalar b)
{
    return make_scalar2(a.x - b, a.y - b);
}
INLINE scalar3 __device__ __HOST_V__ operator -(scalar3 a, scalar3 b)
{
    return make_scalar3(a.x - b.x, a.y - b.y, a.z - b.z);
}
INLINE scalar3 __device__ __HOST_V__ operator -(scalar a, scalar3 b)
{
    return make_scalar3(a - b.x, a - b.y, a - b.z);
}
INLINE scalar3 __device__ __HOST_V__ operator -(scalar3 a, scalar b)
{
    return make_scalar3(a.x - b, a.y - b, a.z - b);
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ operator -(scalarN<N> a, scalarN<N> b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] - b.el[i]);
    return y;
}
template <int N> INLINE scalarN<N> __device__ __HOST_V__ operator -(scalar a, scalarN<N> b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a - b.el[i]);
    return y;
}
template <int N> INLINE scalarN<N> __device__ __HOST_V__ operator -(scalarN<N> a, scalar b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] - b);
    return y;
}
INLINE int1 __device__ __HOST_V__ operator -(int1 a, int1 b)
{
    return make_int1(a.x - b.x);
}
INLINE int1 __device__ __HOST_V__ operator -(int a, int1 b)
{
    return make_int1(a - b.x);
}
INLINE int1 __device__ __HOST_V__ operator -(int1 a, int b)
{
    return make_int1(a.x - b);
}
INLINE int2 __device__ __HOST_V__ operator -(int2 a, int2 b)
{
    return make_int2(a.x - b.x, a.y - b.y);
}
INLINE int2 __device__ __HOST_V__ operator -(int a, int2 b)
{
    return make_int2(a - b.x, a - b.y);
}
INLINE int2 __device__ __HOST_V__ operator -(int2 a, int b)
{
    return make_int2(a.x - b, a.y - b);
}
INLINE int3 __device__ __HOST_V__ operator -(int3 a, int3 b)
{
    return make_int3(a.x - b.x, a.y - b.y, a.z - b.z);
}
INLINE int3 __device__ __HOST_V__ operator -(int a, int3 b)
{
    return make_int3(a - b.x, a - b.y, a - b.z);
}
INLINE int3 __device__ __HOST_V__ operator -(int3 a, int b)
{
    return make_int3(a.x - b, a.y - b, a.z - b);
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ operator -(intN<N> a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] - b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator -(int a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a - b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator -(intN<N> a, int b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] - b);
    return y;
}
INLINE cscalar1 __device__ __HOST_V__ operator -(cscalar1 a, cscalar1 b)
{
    return make_cscalar1(a.x - b.x);
}
INLINE cscalar1 __device__ __HOST_V__ operator -(cscalar a, cscalar1 b)
{
    return make_cscalar1(a - b.x);
}
INLINE cscalar1 __device__ __HOST_V__ operator -(cscalar1 a, cscalar b)
{
    return make_cscalar1(a.x - b);
}
INLINE cscalar2 __device__ __HOST_V__ operator -(cscalar2 a, cscalar2 b)
{
    return make_cscalar2(a.x - b.x, a.y - b.y);
}
INLINE cscalar2 __device__ __HOST_V__ operator -(cscalar a, cscalar2 b)
{
    return make_cscalar2(a - b.x, a - b.y);
}
INLINE cscalar2 __device__ __HOST_V__ operator -(cscalar2 a, cscalar b)
{
    return make_cscalar2(a.x - b, a.y - b);
}
INLINE cscalar3 __device__ __HOST_V__ operator -(cscalar3 a, cscalar3 b)
{
    return make_cscalar3(a.x - b.x, a.y - b.y, a.z - b.z);
}
INLINE cscalar3 __device__ __HOST_V__ operator -(cscalar a, cscalar3 b)
{
    return make_cscalar3(a - b.x, a - b.y, a - b.z);
}
INLINE cscalar3 __device__ __HOST_V__ operator -(cscalar3 a, cscalar b)
{
    return make_cscalar3(a.x - b, a.y - b, a.z - b);
}
INLINE cscalar4 __device__ __HOST_V__ operator -(cscalar4 a, cscalar4 b)
{
    return make_cscalar4(a.x - b.x, a.y - b.y, a.z - b.z, a.w - b.w);
}
INLINE cscalar4 __device__ __HOST_V__ operator -(cscalar a, cscalar4 b)
{
    return make_cscalar4(a - b.x, a - b.y, a - b.z, a - b.w);
}
INLINE cscalar4 __device__ __HOST_V__ operator -(cscalar4 a, cscalar b)
{
    return make_cscalar4(a.x - b, a.y - b, a.z - b, a.w - b);
}
template <int N>
INLINE cscalarN<N> __device__ __HOST_V__ operator -(cscalarN<N> a, cscalarN<N> b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] - b.el[i]);
    return y;
}
template <int N> INLINE cscalarN<N> __device__ __HOST_V__ operator -(cscalar a, cscalarN<N> b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a - b.el[i]);
    return y;
}
template <int N> INLINE cscalarN<N> __device__ __HOST_V__ operator -(cscalarN<N> a, cscalar b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] - b);
    return y;
}
INLINE scalar1 __device__ __HOST_V__ operator -(int a, scalar1 b)
{
    return make_scalar1(a - b.x);
}
INLINE scalar2 __device__ __HOST_V__ operator -(int a, scalar2 b)
{
    return make_scalar2(a - b.x, a - b.y);
}
INLINE scalar3 __device__ __HOST_V__ operator -(int a, scalar3 b)
{
    return make_scalar3(a - b.x, a - b.y, a - b.z);
}
INLINE scalar4 __device__ __HOST_V__ operator -(int a, scalar4 b)
{
    return make_scalar4(a - b.x, a - b.y, a - b.z, a - b.w);
}
INLINE scalar1 __device__ __HOST_V__ operator -(scalar1 a, int b)
{
    return make_scalar1(a.x - b);
}
INLINE scalar2 __device__ __HOST_V__ operator -(scalar2 a, int b)
{
    return make_scalar2(a.x - b, a.y - b);
}
INLINE scalar3 __device__ __HOST_V__ operator -(scalar3 a, int b)
{
    return make_scalar3(a.x - b, a.y - b, a.z - b);
}
INLINE scalar4 __device__ __HOST_V__ operator -(scalar4 a, int b)
{
    return make_scalar4(a.x - b, a.y - b, a.z - b, a.w - b);
}
template <int N> INLINE scalarN<N> __device__ __HOST_V__ operator -(scalarN<N> a, int b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] - b);
    return y;
}
template <int N> INLINE scalarN<N> __device__ __HOST_V__ operator -(int a, scalarN<N> b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a - b.el[i]);
    return y;
}
INLINE scalar1 __device__ __HOST_V__ operator -(int1 a, scalar b)
{
    return make_scalar1(a.x - b);
}
INLINE scalar2 __device__ __HOST_V__ operator -(int2 a, scalar b)
{
    return make_scalar2(a.x - b, a.y - b);
}
INLINE scalar3 __device__ __HOST_V__ operator -(int3 a, scalar b)
{
    return make_scalar3(a.x - b, a.y - b, a.z - b);
}
INLINE scalar4 __device__ __HOST_V__ operator -(int4 a, scalar b)
{
    return make_scalar4(a.x - b, a.y - b, a.z - b, a.w - b);
}
INLINE scalar1 __device__ __HOST_V__ operator -(scalar a, int1 b)
{
    return make_scalar1(a - b.x);
}
INLINE scalar2 __device__ __HOST_V__ operator -(scalar a, int2 b)
{
    return make_scalar2(a - b.x, a - b.y);
}
INLINE scalar3 __device__ __HOST_V__ operator -(scalar a, int3 b)
{
    return make_scalar3(a - b.x, a - b.y, a - b.z);
}
INLINE scalar4 __device__ __HOST_V__ operator -(scalar a, int4 b)
{
    return make_scalar4(a - b.x, a - b.y, a - b.z, a - b.w);
}
template <int N> INLINE scalarN<N> __device__ __HOST_V__ operator -(scalar a, intN<N> b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a - b.el[i]);
    return y;
}
template <int N> INLINE scalarN<N> __device__ __HOST_V__ operator -(intN<N> a, scalar b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] - b);
    return y;
}
INLINE cscalar1 __device__ __HOST_V__ operator -(scalar a, cscalar1 b)
{
    return make_cscalar1(a - b.x);
}
INLINE cscalar2 __device__ __HOST_V__ operator -(scalar a, cscalar2 b)
{
    return make_cscalar2(a - b.x, a - b.y);
}
INLINE cscalar3 __device__ __HOST_V__ operator -(scalar a, cscalar3 b)
{
    return make_cscalar3(a - b.x, a - b.y, a - b.z);
}
INLINE cscalar4 __device__ __HOST_V__ operator -(scalar a, cscalar4 b)
{
    return make_cscalar4(a - b.x, a - b.y, a - b.z, a - b.w);
}
INLINE cscalar1 __device__ __HOST_V__ operator -(cscalar1 a, scalar b)
{
    return make_cscalar1(a.x - b);
}
INLINE cscalar2 __device__ __HOST_V__ operator -(cscalar2 a, scalar b)
{
    return make_cscalar2(a.x - b, a.y - b);
}
INLINE cscalar3 __device__ __HOST_V__ operator -(cscalar3 a, scalar b)
{
    return make_cscalar3(a.x - b, a.y - b, a.z - b);
}
INLINE cscalar4 __device__ __HOST_V__ operator -(cscalar4 a, scalar b)
{
    return make_cscalar4(a.x - b, a.y - b, a.z - b, a.w - b);
}
template <int N> INLINE cscalarN<N> __device__ __HOST_V__ operator -(cscalarN<N> a, scalar b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] - b);
    return y;
}
template <int N> INLINE cscalarN<N> __device__ __HOST_V__ operator -(scalar a, cscalarN<N> b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a - b.el[i]);
    return y;
}
INLINE cscalar1 __device__ __HOST_V__ operator -(scalar1 a, cscalar b)
{
    return make_cscalar1(a.x - b);
}
INLINE cscalar2 __device__ __HOST_V__ operator -(scalar2 a, cscalar b)
{
    return make_cscalar2(a.x - b, a.y - b);
}
INLINE cscalar3 __device__ __HOST_V__ operator -(scalar3 a, cscalar b)
{
    return make_cscalar3(a.x - b, a.y - b, a.z - b);
}
INLINE cscalar4 __device__ __HOST_V__ operator -(scalar4 a, cscalar b)
{
    return make_cscalar4(a.x - b, a.y - b, a.z - b, a.w - b);
}
INLINE cscalar1 __device__ __HOST_V__ operator -(cscalar a, scalar1 b)
{
    return make_cscalar1(a - b.x);
}
INLINE cscalar2 __device__ __HOST_V__ operator -(cscalar a, scalar2 b)
{
    return make_cscalar2(a - b.x, a - b.y);
}
INLINE cscalar3 __device__ __HOST_V__ operator -(cscalar a, scalar3 b)
{
    return make_cscalar3(a - b.x, a - b.y, a - b.z);
}
INLINE cscalar4 __device__ __HOST_V__ operator -(cscalar a, scalar4 b)
{
    return make_cscalar4(a - b.x, a - b.y, a - b.z, a - b.w);
}
template <int N> INLINE cscalarN<N> __device__ __HOST_V__ operator -(cscalar a, scalarN<N> b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a - b.el[i]);
    return y;
}
template <int N> INLINE cscalarN<N> __device__ __HOST_V__ operator -(scalarN<N> a, cscalar b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] - b);
    return y;
}
INLINE scalar1 __device__ __HOST_V__ operator -(int1 a, scalar1 b)
{
    return make_scalar1(a.x - b.x);
}
INLINE scalar2 __device__ __HOST_V__ operator -(int2 a, scalar2 b)
{
    return make_scalar2(a.x - b.x, a.y - b.y);
}
INLINE scalar3 __device__ __HOST_V__ operator -(int3 a, scalar3 b)
{
    return make_scalar3(a.x - b.x, a.y - b.y, a.z - b.z);
}
INLINE scalar4 __device__ __HOST_V__ operator -(int4 a, scalar4 b)
{
    return make_scalar4(a.x - b.x, a.y - b.y, a.z - b.z, a.w - b.w);
}
INLINE scalar1 __device__ __HOST_V__ operator -(scalar1 a, int1 b)
{
    return make_scalar1(a.x - b.x);
}
INLINE scalar2 __device__ __HOST_V__ operator -(scalar2 a, int2 b)
{
    return make_scalar2(a.x - b.x, a.y - b.y);
}
INLINE scalar3 __device__ __HOST_V__ operator -(scalar3 a, int3 b)
{
    return make_scalar3(a.x - b.x, a.y - b.y, a.z - b.z);
}
INLINE scalar4 __device__ __HOST_V__ operator -(scalar4 a, int4 b)
{
    return make_scalar4(a.x - b.x, a.y - b.y, a.z - b.z, a.w - b.w);
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ operator -(scalarN<N> a, intN<N> b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] - b.el[i]);
    return y;
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ operator -(intN<N> a, scalarN<N> b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] - b.el[i]);
    return y;
}
INLINE cscalar1 __device__ __HOST_V__ operator -(scalar1 a, cscalar1 b)
{
    return make_cscalar1(a.x - b.x);
}
INLINE cscalar2 __device__ __HOST_V__ operator -(scalar2 a, cscalar2 b)
{
    return make_cscalar2(a.x - b.x, a.y - b.y);
}
INLINE cscalar3 __device__ __HOST_V__ operator -(scalar3 a, cscalar3 b)
{
    return make_cscalar3(a.x - b.x, a.y - b.y, a.z - b.z);
}
INLINE cscalar4 __device__ __HOST_V__ operator -(scalar4 a, cscalar4 b)
{
    return make_cscalar4(a.x - b.x, a.y - b.y, a.z - b.z, a.w - b.w);
}
INLINE cscalar1 __device__ __HOST_V__ operator -(cscalar1 a, scalar1 b)
{
    return make_cscalar1(a.x - b.x);
}
INLINE cscalar2 __device__ __HOST_V__ operator -(cscalar2 a, scalar2 b)
{
    return make_cscalar2(a.x - b.x, a.y - b.y);
}
INLINE cscalar3 __device__ __HOST_V__ operator -(cscalar3 a, scalar3 b)
{
    return make_cscalar3(a.x - b.x, a.y - b.y, a.z - b.z);
}
INLINE cscalar4 __device__ __HOST_V__ operator -(cscalar4 a, scalar4 b)
{
    return make_cscalar4(a.x - b.x, a.y - b.y, a.z - b.z, a.w - b.w);
}
template <int N>
INLINE cscalarN<N> __device__ __HOST_V__ operator -(cscalarN<N> a, scalarN<N> b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] - b.el[i]);
    return y;
}
template <int N>
INLINE cscalarN<N> __device__ __HOST_V__ operator -(scalarN<N> a, cscalarN<N> b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] - b.el[i]);
    return y;
}
INLINE scalar1 __device__ __HOST_V__ operator *(scalar1 a, scalar1 b)
{
    return make_scalar1(a.x * b.x);
}
INLINE scalar1 __device__ __HOST_V__ operator *(scalar a, scalar1 b)
{
    return make_scalar1(a * b.x);
}
INLINE scalar1 __device__ __HOST_V__ operator *(scalar1 a, scalar b)
{
    return make_scalar1(a.x * b);
}
INLINE scalar2 __device__ __HOST_V__ operator *(scalar2 a, scalar2 b)
{
    return make_scalar2(a.x * b.x, a.y * b.y);
}
INLINE scalar2 __device__ __HOST_V__ operator *(scalar a, scalar2 b)
{
    return make_scalar2(a * b.x, a * b.y);
}
INLINE scalar2 __device__ __HOST_V__ operator *(scalar2 a, scalar b)
{
    return make_scalar2(a.x * b, a.y * b);
}
INLINE scalar3 __device__ __HOST_V__ operator *(scalar3 a, scalar3 b)
{
    return make_scalar3(a.x * b.x, a.y * b.y, a.z * b.z);
}
INLINE scalar3 __device__ __HOST_V__ operator *(scalar a, scalar3 b)
{
    return make_scalar3(a * b.x, a * b.y, a * b.z);
}
INLINE scalar3 __device__ __HOST_V__ operator *(scalar3 a, scalar b)
{
    return make_scalar3(a.x * b, a.y * b, a.z * b);
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ operator *(scalarN<N> a, scalarN<N> b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] * b.el[i]);
    return y;
}
template <int N> INLINE scalarN<N> __device__ __HOST_V__ operator *(scalar a, scalarN<N> b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a * b.el[i]);
    return y;
}
template <int N> INLINE scalarN<N> __device__ __HOST_V__ operator *(scalarN<N> a, scalar b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] * b);
    return y;
}
INLINE int1 __device__ __HOST_V__ operator *(int1 a, int1 b)
{
    return make_int1(a.x * b.x);
}
INLINE int1 __device__ __HOST_V__ operator *(int a, int1 b)
{
    return make_int1(a * b.x);
}
INLINE int1 __device__ __HOST_V__ operator *(int1 a, int b)
{
    return make_int1(a.x * b);
}
INLINE int2 __device__ __HOST_V__ operator *(int2 a, int2 b)
{
    return make_int2(a.x * b.x, a.y * b.y);
}
INLINE int2 __device__ __HOST_V__ operator *(int a, int2 b)
{
    return make_int2(a * b.x, a * b.y);
}
INLINE int2 __device__ __HOST_V__ operator *(int2 a, int b)
{
    return make_int2(a.x * b, a.y * b);
}
INLINE int3 __device__ __HOST_V__ operator *(int3 a, int3 b)
{
    return make_int3(a.x * b.x, a.y * b.y, a.z * b.z);
}
INLINE int3 __device__ __HOST_V__ operator *(int a, int3 b)
{
    return make_int3(a * b.x, a * b.y, a * b.z);
}
INLINE int3 __device__ __HOST_V__ operator *(int3 a, int b)
{
    return make_int3(a.x * b, a.y * b, a.z * b);
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ operator *(intN<N> a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] * b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator *(int a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a * b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator *(intN<N> a, int b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] * b);
    return y;
}
INLINE cscalar1 __device__ __HOST_V__ operator *(cscalar1 a, cscalar1 b)
{
    return make_cscalar1(a.x * b.x);
}
INLINE cscalar1 __device__ __HOST_V__ operator *(cscalar a, cscalar1 b)
{
    return make_cscalar1(a * b.x);
}
INLINE cscalar1 __device__ __HOST_V__ operator *(cscalar1 a, cscalar b)
{
    return make_cscalar1(a.x * b);
}
INLINE cscalar2 __device__ __HOST_V__ operator *(cscalar2 a, cscalar2 b)
{
    return make_cscalar2(a.x * b.x, a.y * b.y);
}
INLINE cscalar2 __device__ __HOST_V__ operator *(cscalar a, cscalar2 b)
{
    return make_cscalar2(a * b.x, a * b.y);
}
INLINE cscalar2 __device__ __HOST_V__ operator *(cscalar2 a, cscalar b)
{
    return make_cscalar2(a.x * b, a.y * b);
}
INLINE cscalar3 __device__ __HOST_V__ operator *(cscalar3 a, cscalar3 b)
{
    return make_cscalar3(a.x * b.x, a.y * b.y, a.z * b.z);
}
INLINE cscalar3 __device__ __HOST_V__ operator *(cscalar a, cscalar3 b)
{
    return make_cscalar3(a * b.x, a * b.y, a * b.z);
}
INLINE cscalar3 __device__ __HOST_V__ operator *(cscalar3 a, cscalar b)
{
    return make_cscalar3(a.x * b, a.y * b, a.z * b);
}
INLINE cscalar4 __device__ __HOST_V__ operator *(cscalar4 a, cscalar4 b)
{
    return make_cscalar4(a.x * b.x, a.y * b.y, a.z * b.z, a.w * b.w);
}
INLINE cscalar4 __device__ __HOST_V__ operator *(cscalar a, cscalar4 b)
{
    return make_cscalar4(a * b.x, a * b.y, a * b.z, a * b.w);
}
INLINE cscalar4 __device__ __HOST_V__ operator *(cscalar4 a, cscalar b)
{
    return make_cscalar4(a.x * b, a.y * b, a.z * b, a.w * b);
}
template <int N>
INLINE cscalarN<N> __device__ __HOST_V__ operator *(cscalarN<N> a, cscalarN<N> b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] * b.el[i]);
    return y;
}
template <int N> INLINE cscalarN<N> __device__ __HOST_V__ operator *(cscalar a, cscalarN<N> b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a * b.el[i]);
    return y;
}
template <int N> INLINE cscalarN<N> __device__ __HOST_V__ operator *(cscalarN<N> a, cscalar b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] * b);
    return y;
}
INLINE scalar1 __device__ __HOST_V__ operator *(int a, scalar1 b)
{
    return make_scalar1(a * b.x);
}
INLINE scalar2 __device__ __HOST_V__ operator *(int a, scalar2 b)
{
    return make_scalar2(a * b.x, a * b.y);
}
INLINE scalar3 __device__ __HOST_V__ operator *(int a, scalar3 b)
{
    return make_scalar3(a * b.x, a * b.y, a * b.z);
}
INLINE scalar4 __device__ __HOST_V__ operator *(int a, scalar4 b)
{
    return make_scalar4(a * b.x, a * b.y, a * b.z, a * b.w);
}
INLINE scalar1 __device__ __HOST_V__ operator *(scalar1 a, int b)
{
    return make_scalar1(a.x * b);
}
INLINE scalar2 __device__ __HOST_V__ operator *(scalar2 a, int b)
{
    return make_scalar2(a.x * b, a.y * b);
}
INLINE scalar3 __device__ __HOST_V__ operator *(scalar3 a, int b)
{
    return make_scalar3(a.x * b, a.y * b, a.z * b);
}
INLINE scalar4 __device__ __HOST_V__ operator *(scalar4 a, int b)
{
    return make_scalar4(a.x * b, a.y * b, a.z * b, a.w * b);
}
template <int N> INLINE scalarN<N> __device__ __HOST_V__ operator *(scalarN<N> a, int b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] * b);
    return y;
}
template <int N> INLINE scalarN<N> __device__ __HOST_V__ operator *(int a, scalarN<N> b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a * b.el[i]);
    return y;
}
INLINE scalar1 __device__ __HOST_V__ operator *(int1 a, scalar b)
{
    return make_scalar1(a.x * b);
}
INLINE scalar2 __device__ __HOST_V__ operator *(int2 a, scalar b)
{
    return make_scalar2(a.x * b, a.y * b);
}
INLINE scalar3 __device__ __HOST_V__ operator *(int3 a, scalar b)
{
    return make_scalar3(a.x * b, a.y * b, a.z * b);
}
INLINE scalar4 __device__ __HOST_V__ operator *(int4 a, scalar b)
{
    return make_scalar4(a.x * b, a.y * b, a.z * b, a.w * b);
}
INLINE scalar1 __device__ __HOST_V__ operator *(scalar a, int1 b)
{
    return make_scalar1(a * b.x);
}
INLINE scalar2 __device__ __HOST_V__ operator *(scalar a, int2 b)
{
    return make_scalar2(a * b.x, a * b.y);
}
INLINE scalar3 __device__ __HOST_V__ operator *(scalar a, int3 b)
{
    return make_scalar3(a * b.x, a * b.y, a * b.z);
}
INLINE scalar4 __device__ __HOST_V__ operator *(scalar a, int4 b)
{
    return make_scalar4(a * b.x, a * b.y, a * b.z, a * b.w);
}
template <int N> INLINE scalarN<N> __device__ __HOST_V__ operator *(scalar a, intN<N> b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a * b.el[i]);
    return y;
}
template <int N> INLINE scalarN<N> __device__ __HOST_V__ operator *(intN<N> a, scalar b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] * b);
    return y;
}
INLINE cscalar1 __device__ __HOST_V__ operator *(scalar a, cscalar1 b)
{
    return make_cscalar1(a * b.x);
}
INLINE cscalar2 __device__ __HOST_V__ operator *(scalar a, cscalar2 b)
{
    return make_cscalar2(a * b.x, a * b.y);
}
INLINE cscalar3 __device__ __HOST_V__ operator *(scalar a, cscalar3 b)
{
    return make_cscalar3(a * b.x, a * b.y, a * b.z);
}
INLINE cscalar4 __device__ __HOST_V__ operator *(scalar a, cscalar4 b)
{
    return make_cscalar4(a * b.x, a * b.y, a * b.z, a * b.w);
}
INLINE cscalar1 __device__ __HOST_V__ operator *(cscalar1 a, scalar b)
{
    return make_cscalar1(a.x * b);
}
INLINE cscalar2 __device__ __HOST_V__ operator *(cscalar2 a, scalar b)
{
    return make_cscalar2(a.x * b, a.y * b);
}
INLINE cscalar3 __device__ __HOST_V__ operator *(cscalar3 a, scalar b)
{
    return make_cscalar3(a.x * b, a.y * b, a.z * b);
}
INLINE cscalar4 __device__ __HOST_V__ operator *(cscalar4 a, scalar b)
{
    return make_cscalar4(a.x * b, a.y * b, a.z * b, a.w * b);
}
template <int N> INLINE cscalarN<N> __device__ __HOST_V__ operator *(cscalarN<N> a, scalar b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] * b);
    return y;
}
template <int N> INLINE cscalarN<N> __device__ __HOST_V__ operator *(scalar a, cscalarN<N> b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a * b.el[i]);
    return y;
}
INLINE cscalar1 __device__ __HOST_V__ operator *(scalar1 a, cscalar b)
{
    return make_cscalar1(a.x * b);
}
INLINE cscalar2 __device__ __HOST_V__ operator *(scalar2 a, cscalar b)
{
    return make_cscalar2(a.x * b, a.y * b);
}
INLINE cscalar3 __device__ __HOST_V__ operator *(scalar3 a, cscalar b)
{
    return make_cscalar3(a.x * b, a.y * b, a.z * b);
}
INLINE cscalar4 __device__ __HOST_V__ operator *(scalar4 a, cscalar b)
{
    return make_cscalar4(a.x * b, a.y * b, a.z * b, a.w * b);
}
INLINE cscalar1 __device__ __HOST_V__ operator *(cscalar a, scalar1 b)
{
    return make_cscalar1(a * b.x);
}
INLINE cscalar2 __device__ __HOST_V__ operator *(cscalar a, scalar2 b)
{
    return make_cscalar2(a * b.x, a * b.y);
}
INLINE cscalar3 __device__ __HOST_V__ operator *(cscalar a, scalar3 b)
{
    return make_cscalar3(a * b.x, a * b.y, a * b.z);
}
INLINE cscalar4 __device__ __HOST_V__ operator *(cscalar a, scalar4 b)
{
    return make_cscalar4(a * b.x, a * b.y, a * b.z, a * b.w);
}
template <int N> INLINE cscalarN<N> __device__ __HOST_V__ operator *(cscalar a, scalarN<N> b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a * b.el[i]);
    return y;
}
template <int N> INLINE cscalarN<N> __device__ __HOST_V__ operator *(scalarN<N> a, cscalar b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] * b);
    return y;
}
INLINE scalar1 __device__ __HOST_V__ operator *(int1 a, scalar1 b)
{
    return make_scalar1(a.x * b.x);
}
INLINE scalar2 __device__ __HOST_V__ operator *(int2 a, scalar2 b)
{
    return make_scalar2(a.x * b.x, a.y * b.y);
}
INLINE scalar3 __device__ __HOST_V__ operator *(int3 a, scalar3 b)
{
    return make_scalar3(a.x * b.x, a.y * b.y, a.z * b.z);
}
INLINE scalar4 __device__ __HOST_V__ operator *(int4 a, scalar4 b)
{
    return make_scalar4(a.x * b.x, a.y * b.y, a.z * b.z, a.w * b.w);
}
INLINE scalar1 __device__ __HOST_V__ operator *(scalar1 a, int1 b)
{
    return make_scalar1(a.x * b.x);
}
INLINE scalar2 __device__ __HOST_V__ operator *(scalar2 a, int2 b)
{
    return make_scalar2(a.x * b.x, a.y * b.y);
}
INLINE scalar3 __device__ __HOST_V__ operator *(scalar3 a, int3 b)
{
    return make_scalar3(a.x * b.x, a.y * b.y, a.z * b.z);
}
INLINE scalar4 __device__ __HOST_V__ operator *(scalar4 a, int4 b)
{
    return make_scalar4(a.x * b.x, a.y * b.y, a.z * b.z, a.w * b.w);
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ operator *(scalarN<N> a, intN<N> b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] * b.el[i]);
    return y;
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ operator *(intN<N> a, scalarN<N> b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] * b.el[i]);
    return y;
}
INLINE cscalar1 __device__ __HOST_V__ operator *(scalar1 a, cscalar1 b)
{
    return make_cscalar1(a.x * b.x);
}
INLINE cscalar2 __device__ __HOST_V__ operator *(scalar2 a, cscalar2 b)
{
    return make_cscalar2(a.x * b.x, a.y * b.y);
}
INLINE cscalar3 __device__ __HOST_V__ operator *(scalar3 a, cscalar3 b)
{
    return make_cscalar3(a.x * b.x, a.y * b.y, a.z * b.z);
}
INLINE cscalar4 __device__ __HOST_V__ operator *(scalar4 a, cscalar4 b)
{
    return make_cscalar4(a.x * b.x, a.y * b.y, a.z * b.z, a.w * b.w);
}
INLINE cscalar1 __device__ __HOST_V__ operator *(cscalar1 a, scalar1 b)
{
    return make_cscalar1(a.x * b.x);
}
INLINE cscalar2 __device__ __HOST_V__ operator *(cscalar2 a, scalar2 b)
{
    return make_cscalar2(a.x * b.x, a.y * b.y);
}
INLINE cscalar3 __device__ __HOST_V__ operator *(cscalar3 a, scalar3 b)
{
    return make_cscalar3(a.x * b.x, a.y * b.y, a.z * b.z);
}
INLINE cscalar4 __device__ __HOST_V__ operator *(cscalar4 a, scalar4 b)
{
    return make_cscalar4(a.x * b.x, a.y * b.y, a.z * b.z, a.w * b.w);
}
template <int N>
INLINE cscalarN<N> __device__ __HOST_V__ operator *(cscalarN<N> a, scalarN<N> b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] * b.el[i]);
    return y;
}
template <int N>
INLINE cscalarN<N> __device__ __HOST_V__ operator *(scalarN<N> a, cscalarN<N> b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] * b.el[i]);
    return y;
}
INLINE scalar1 __device__ __HOST_V__ operator /(scalar1 a, scalar1 b)
{
    return make_scalar1(a.x / b.x);
}
INLINE scalar1 __device__ __HOST_V__ operator /(scalar a, scalar1 b)
{
    return make_scalar1(a / b.x);
}
INLINE scalar1 __device__ __HOST_V__ operator /(scalar1 a, scalar b)
{
    return make_scalar1(a.x / b);
}
INLINE scalar2 __device__ __HOST_V__ operator /(scalar2 a, scalar2 b)
{
    return make_scalar2(a.x / b.x, a.y / b.y);
}
INLINE scalar2 __device__ __HOST_V__ operator /(scalar a, scalar2 b)
{
    return make_scalar2(a / b.x, a / b.y);
}
INLINE scalar2 __device__ __HOST_V__ operator /(scalar2 a, scalar b)
{
    return make_scalar2(a.x / b, a.y / b);
}
INLINE scalar3 __device__ __HOST_V__ operator /(scalar3 a, scalar3 b)
{
    return make_scalar3(a.x / b.x, a.y / b.y, a.z / b.z);
}
INLINE scalar3 __device__ __HOST_V__ operator /(scalar a, scalar3 b)
{
    return make_scalar3(a / b.x, a / b.y, a / b.z);
}
INLINE scalar3 __device__ __HOST_V__ operator /(scalar3 a, scalar b)
{
    return make_scalar3(a.x / b, a.y / b, a.z / b);
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ operator /(scalarN<N> a, scalarN<N> b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] / b.el[i]);
    return y;
}
template <int N> INLINE scalarN<N> __device__ __HOST_V__ operator /(scalar a, scalarN<N> b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a / b.el[i]);
    return y;
}
template <int N> INLINE scalarN<N> __device__ __HOST_V__ operator /(scalarN<N> a, scalar b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] / b);
    return y;
}
INLINE int1 __device__ __HOST_V__ operator /(int1 a, int1 b)
{
    return make_int1(a.x / b.x);
}
INLINE int1 __device__ __HOST_V__ operator /(int a, int1 b)
{
    return make_int1(a / b.x);
}
INLINE int1 __device__ __HOST_V__ operator /(int1 a, int b)
{
    return make_int1(a.x / b);
}
INLINE int2 __device__ __HOST_V__ operator /(int2 a, int2 b)
{
    return make_int2(a.x / b.x, a.y / b.y);
}
INLINE int2 __device__ __HOST_V__ operator /(int a, int2 b)
{
    return make_int2(a / b.x, a / b.y);
}
INLINE int2 __device__ __HOST_V__ operator /(int2 a, int b)
{
    return make_int2(a.x / b, a.y / b);
}
INLINE int3 __device__ __HOST_V__ operator /(int3 a, int3 b)
{
    return make_int3(a.x / b.x, a.y / b.y, a.z / b.z);
}
INLINE int3 __device__ __HOST_V__ operator /(int a, int3 b)
{
    return make_int3(a / b.x, a / b.y, a / b.z);
}
INLINE int3 __device__ __HOST_V__ operator /(int3 a, int b)
{
    return make_int3(a.x / b, a.y / b, a.z / b);
}
INLINE int4 __device__ __HOST_V__ operator /(int4 a, int4 b)
{
    return make_int4(a.x / b.x, a.y / b.y, a.z / b.z, a.w / b.w);
}
INLINE int4 __device__ __HOST_V__ operator /(int a, int4 b)
{
    return make_int4(a / b.x, a / b.y, a / b.z, a / b.w);
}
INLINE int4 __device__ __HOST_V__ operator /(int4 a, int b)
{
    return make_int4(a.x / b, a.y / b, a.z / b, a.w / b);
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ operator /(intN<N> a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] / b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator /(int a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a / b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator /(intN<N> a, int b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] / b);
    return y;
}
INLINE cscalar1 __device__ __HOST_V__ operator /(cscalar1 a, cscalar1 b)
{
    return make_cscalar1(a.x / b.x);
}
INLINE cscalar1 __device__ __HOST_V__ operator /(cscalar a, cscalar1 b)
{
    return make_cscalar1(a / b.x);
}
INLINE cscalar1 __device__ __HOST_V__ operator /(cscalar1 a, cscalar b)
{
    return make_cscalar1(a.x / b);
}
INLINE cscalar2 __device__ __HOST_V__ operator /(cscalar2 a, cscalar2 b)
{
    return make_cscalar2(a.x / b.x, a.y / b.y);
}
INLINE cscalar2 __device__ __HOST_V__ operator /(cscalar a, cscalar2 b)
{
    return make_cscalar2(a / b.x, a / b.y);
}
INLINE cscalar2 __device__ __HOST_V__ operator /(cscalar2 a, cscalar b)
{
    return make_cscalar2(a.x / b, a.y / b);
}
INLINE cscalar3 __device__ __HOST_V__ operator /(cscalar3 a, cscalar3 b)
{
    return make_cscalar3(a.x / b.x, a.y / b.y, a.z / b.z);
}
INLINE cscalar3 __device__ __HOST_V__ operator /(cscalar a, cscalar3 b)
{
    return make_cscalar3(a / b.x, a / b.y, a / b.z);
}
INLINE cscalar3 __device__ __HOST_V__ operator /(cscalar3 a, cscalar b)
{
    return make_cscalar3(a.x / b, a.y / b, a.z / b);
}
INLINE cscalar4 __device__ __HOST_V__ operator /(cscalar4 a, cscalar4 b)
{
    return make_cscalar4(a.x / b.x, a.y / b.y, a.z / b.z, a.w / b.w);
}
INLINE cscalar4 __device__ __HOST_V__ operator /(cscalar a, cscalar4 b)
{
    return make_cscalar4(a / b.x, a / b.y, a / b.z, a / b.w);
}
INLINE cscalar4 __device__ __HOST_V__ operator /(cscalar4 a, cscalar b)
{
    return make_cscalar4(a.x / b, a.y / b, a.z / b, a.w / b);
}
template <int N>
INLINE cscalarN<N> __device__ __HOST_V__ operator /(cscalarN<N> a, cscalarN<N> b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] / b.el[i]);
    return y;
}
template <int N> INLINE cscalarN<N> __device__ __HOST_V__ operator /(cscalar a, cscalarN<N> b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a / b.el[i]);
    return y;
}
template <int N> INLINE cscalarN<N> __device__ __HOST_V__ operator /(cscalarN<N> a, cscalar b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] / b);
    return y;
}
INLINE scalar1 __device__ __HOST_V__ operator /(int a, scalar1 b)
{
    return make_scalar1(a / b.x);
}
INLINE scalar2 __device__ __HOST_V__ operator /(int a, scalar2 b)
{
    return make_scalar2(a / b.x, a / b.y);
}
INLINE scalar3 __device__ __HOST_V__ operator /(int a, scalar3 b)
{
    return make_scalar3(a / b.x, a / b.y, a / b.z);
}
INLINE scalar4 __device__ __HOST_V__ operator /(int a, scalar4 b)
{
    return make_scalar4(a / b.x, a / b.y, a / b.z, a / b.w);
}
INLINE scalar1 __device__ __HOST_V__ operator /(scalar1 a, int b)
{
    return make_scalar1(a.x / b);
}
INLINE scalar2 __device__ __HOST_V__ operator /(scalar2 a, int b)
{
    return make_scalar2(a.x / b, a.y / b);
}
INLINE scalar3 __device__ __HOST_V__ operator /(scalar3 a, int b)
{
    return make_scalar3(a.x / b, a.y / b, a.z / b);
}
INLINE scalar4 __device__ __HOST_V__ operator /(scalar4 a, int b)
{
    return make_scalar4(a.x / b, a.y / b, a.z / b, a.w / b);
}
template <int N> INLINE scalarN<N> __device__ __HOST_V__ operator /(scalarN<N> a, int b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] / b);
    return y;
}
template <int N> INLINE scalarN<N> __device__ __HOST_V__ operator /(int a, scalarN<N> b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a / b.el[i]);
    return y;
}
INLINE scalar1 __device__ __HOST_V__ operator /(int1 a, scalar b)
{
    return make_scalar1(a.x / b);
}
INLINE scalar2 __device__ __HOST_V__ operator /(int2 a, scalar b)
{
    return make_scalar2(a.x / b, a.y / b);
}
INLINE scalar3 __device__ __HOST_V__ operator /(int3 a, scalar b)
{
    return make_scalar3(a.x / b, a.y / b, a.z / b);
}
INLINE scalar4 __device__ __HOST_V__ operator /(int4 a, scalar b)
{
    return make_scalar4(a.x / b, a.y / b, a.z / b, a.w / b);
}
INLINE scalar1 __device__ __HOST_V__ operator /(scalar a, int1 b)
{
    return make_scalar1(a / b.x);
}
INLINE scalar2 __device__ __HOST_V__ operator /(scalar a, int2 b)
{
    return make_scalar2(a / b.x, a / b.y);
}
INLINE scalar3 __device__ __HOST_V__ operator /(scalar a, int3 b)
{
    return make_scalar3(a / b.x, a / b.y, a / b.z);
}
INLINE scalar4 __device__ __HOST_V__ operator /(scalar a, int4 b)
{
    return make_scalar4(a / b.x, a / b.y, a / b.z, a / b.w);
}
template <int N> INLINE scalarN<N> __device__ __HOST_V__ operator /(scalar a, intN<N> b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a / b.el[i]);
    return y;
}
template <int N> INLINE scalarN<N> __device__ __HOST_V__ operator /(intN<N> a, scalar b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] / b);
    return y;
}
INLINE cscalar1 __device__ __HOST_V__ operator /(scalar a, cscalar1 b)
{
    return make_cscalar1(a / b.x);
}
INLINE cscalar2 __device__ __HOST_V__ operator /(scalar a, cscalar2 b)
{
    return make_cscalar2(a / b.x, a / b.y);
}
INLINE cscalar3 __device__ __HOST_V__ operator /(scalar a, cscalar3 b)
{
    return make_cscalar3(a / b.x, a / b.y, a / b.z);
}
INLINE cscalar4 __device__ __HOST_V__ operator /(scalar a, cscalar4 b)
{
    return make_cscalar4(a / b.x, a / b.y, a / b.z, a / b.w);
}
INLINE cscalar1 __device__ __HOST_V__ operator /(cscalar1 a, scalar b)
{
    return make_cscalar1(a.x / b);
}
INLINE cscalar2 __device__ __HOST_V__ operator /(cscalar2 a, scalar b)
{
    return make_cscalar2(a.x / b, a.y / b);
}
INLINE cscalar3 __device__ __HOST_V__ operator /(cscalar3 a, scalar b)
{
    return make_cscalar3(a.x / b, a.y / b, a.z / b);
}
INLINE cscalar4 __device__ __HOST_V__ operator /(cscalar4 a, scalar b)
{
    return make_cscalar4(a.x / b, a.y / b, a.z / b, a.w / b);
}
template <int N> INLINE cscalarN<N> __device__ __HOST_V__ operator /(cscalarN<N> a, scalar b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] / b);
    return y;
}
template <int N> INLINE cscalarN<N> __device__ __HOST_V__ operator /(scalar a, cscalarN<N> b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a / b.el[i]);
    return y;
}
INLINE cscalar1 __device__ __HOST_V__ operator /(scalar1 a, cscalar b)
{
    return make_cscalar1(a.x / b);
}
INLINE cscalar2 __device__ __HOST_V__ operator /(scalar2 a, cscalar b)
{
    return make_cscalar2(a.x / b, a.y / b);
}
INLINE cscalar3 __device__ __HOST_V__ operator /(scalar3 a, cscalar b)
{
    return make_cscalar3(a.x / b, a.y / b, a.z / b);
}
INLINE cscalar4 __device__ __HOST_V__ operator /(scalar4 a, cscalar b)
{
    return make_cscalar4(a.x / b, a.y / b, a.z / b, a.w / b);
}
INLINE cscalar1 __device__ __HOST_V__ operator /(cscalar a, scalar1 b)
{
    return make_cscalar1(a / b.x);
}
INLINE cscalar2 __device__ __HOST_V__ operator /(cscalar a, scalar2 b)
{
    return make_cscalar2(a / b.x, a / b.y);
}
INLINE cscalar3 __device__ __HOST_V__ operator /(cscalar a, scalar3 b)
{
    return make_cscalar3(a / b.x, a / b.y, a / b.z);
}
INLINE cscalar4 __device__ __HOST_V__ operator /(cscalar a, scalar4 b)
{
    return make_cscalar4(a / b.x, a / b.y, a / b.z, a / b.w);
}
template <int N> INLINE cscalarN<N> __device__ __HOST_V__ operator /(cscalar a, scalarN<N> b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a / b.el[i]);
    return y;
}
template <int N> INLINE cscalarN<N> __device__ __HOST_V__ operator /(scalarN<N> a, cscalar b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] / b);
    return y;
}
INLINE scalar1 __device__ __HOST_V__ operator /(int1 a, scalar1 b)
{
    return make_scalar1(a.x / b.x);
}
INLINE scalar2 __device__ __HOST_V__ operator /(int2 a, scalar2 b)
{
    return make_scalar2(a.x / b.x, a.y / b.y);
}
INLINE scalar3 __device__ __HOST_V__ operator /(int3 a, scalar3 b)
{
    return make_scalar3(a.x / b.x, a.y / b.y, a.z / b.z);
}
INLINE scalar4 __device__ __HOST_V__ operator /(int4 a, scalar4 b)
{
    return make_scalar4(a.x / b.x, a.y / b.y, a.z / b.z, a.w / b.w);
}
INLINE scalar1 __device__ __HOST_V__ operator /(scalar1 a, int1 b)
{
    return make_scalar1(a.x / b.x);
}
INLINE scalar2 __device__ __HOST_V__ operator /(scalar2 a, int2 b)
{
    return make_scalar2(a.x / b.x, a.y / b.y);
}
INLINE scalar3 __device__ __HOST_V__ operator /(scalar3 a, int3 b)
{
    return make_scalar3(a.x / b.x, a.y / b.y, a.z / b.z);
}
INLINE scalar4 __device__ __HOST_V__ operator /(scalar4 a, int4 b)
{
    return make_scalar4(a.x / b.x, a.y / b.y, a.z / b.z, a.w / b.w);
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ operator /(scalarN<N> a, intN<N> b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] / b.el[i]);
    return y;
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ operator /(intN<N> a, scalarN<N> b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] / b.el[i]);
    return y;
}
INLINE cscalar1 __device__ __HOST_V__ operator /(scalar1 a, cscalar1 b)
{
    return make_cscalar1(a.x / b.x);
}
INLINE cscalar2 __device__ __HOST_V__ operator /(scalar2 a, cscalar2 b)
{
    return make_cscalar2(a.x / b.x, a.y / b.y);
}
INLINE cscalar3 __device__ __HOST_V__ operator /(scalar3 a, cscalar3 b)
{
    return make_cscalar3(a.x / b.x, a.y / b.y, a.z / b.z);
}
INLINE cscalar4 __device__ __HOST_V__ operator /(scalar4 a, cscalar4 b)
{
    return make_cscalar4(a.x / b.x, a.y / b.y, a.z / b.z, a.w / b.w);
}
INLINE cscalar1 __device__ __HOST_V__ operator /(cscalar1 a, scalar1 b)
{
    return make_cscalar1(a.x / b.x);
}
INLINE cscalar2 __device__ __HOST_V__ operator /(cscalar2 a, scalar2 b)
{
    return make_cscalar2(a.x / b.x, a.y / b.y);
}
INLINE cscalar3 __device__ __HOST_V__ operator /(cscalar3 a, scalar3 b)
{
    return make_cscalar3(a.x / b.x, a.y / b.y, a.z / b.z);
}
INLINE cscalar4 __device__ __HOST_V__ operator /(cscalar4 a, scalar4 b)
{
    return make_cscalar4(a.x / b.x, a.y / b.y, a.z / b.z, a.w / b.w);
}
template <int N>
INLINE cscalarN<N> __device__ __HOST_V__ operator /(cscalarN<N> a, scalarN<N> b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] / b.el[i]);
    return y;
}
template <int N>
INLINE cscalarN<N> __device__ __HOST_V__ operator /(scalarN<N> a, cscalarN<N> b)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = a.el[i] / b.el[i]);
    return y;
}
INLINE int1 __device__ __HOST_V__ operator >(scalar1 a, scalar1 b)
{
    return make_int1(a.x > b.x);
}
INLINE int1 __device__ __HOST_V__ operator >(scalar a, scalar1 b)
{
    return make_int1(a > b.x);
}
INLINE int1 __device__ __HOST_V__ operator >(scalar1 a, scalar b)
{
    return make_int1(a.x > b);
}
INLINE int2 __device__ __HOST_V__ operator >(scalar2 a, scalar2 b)
{
    return make_int2(a.x > b.x, a.y > b.y);
}
INLINE int2 __device__ __HOST_V__ operator >(scalar a, scalar2 b)
{
    return make_int2(a > b.x, a > b.y);
}
INLINE int2 __device__ __HOST_V__ operator >(scalar2 a, scalar b)
{
    return make_int2(a.x > b, a.y > b);
}
INLINE int3 __device__ __HOST_V__ operator >(scalar3 a, scalar3 b)
{
    return make_int3(a.x > b.x, a.y > b.y, a.z > b.z);
}
INLINE int3 __device__ __HOST_V__ operator >(scalar a, scalar3 b)
{
    return make_int3(a > b.x, a > b.y, a > b.z);
}
INLINE int3 __device__ __HOST_V__ operator >(scalar3 a, scalar b)
{
    return make_int3(a.x > b, a.y > b, a.z > b);
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ operator >(scalarN<N> a, scalarN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] > b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator >(scalar a, scalarN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a > b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator >(scalarN<N> a, scalar b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] > b);
    return y;
}
INLINE int1 __device__ __HOST_V__ operator >(int1 a, int1 b)
{
    return make_int1(a.x > b.x);
}
INLINE int1 __device__ __HOST_V__ operator >(int a, int1 b)
{
    return make_int1(a > b.x);
}
INLINE int1 __device__ __HOST_V__ operator >(int1 a, int b)
{
    return make_int1(a.x > b);
}
INLINE int2 __device__ __HOST_V__ operator >(int2 a, int2 b)
{
    return make_int2(a.x > b.x, a.y > b.y);
}
INLINE int2 __device__ __HOST_V__ operator >(int a, int2 b)
{
    return make_int2(a > b.x, a > b.y);
}
INLINE int2 __device__ __HOST_V__ operator >(int2 a, int b)
{
    return make_int2(a.x > b, a.y > b);
}
INLINE int3 __device__ __HOST_V__ operator >(int3 a, int3 b)
{
    return make_int3(a.x > b.x, a.y > b.y, a.z > b.z);
}
INLINE int3 __device__ __HOST_V__ operator >(int a, int3 b)
{
    return make_int3(a > b.x, a > b.y, a > b.z);
}
INLINE int3 __device__ __HOST_V__ operator >(int3 a, int b)
{
    return make_int3(a.x > b, a.y > b, a.z > b);
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ operator >(intN<N> a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] > b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator >(int a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a > b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator >(intN<N> a, int b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] > b);
    return y;
}
INLINE int1 __device__ __HOST_V__ operator >(int a, scalar1 b)
{
    return make_int1(a > b.x);
}
INLINE int2 __device__ __HOST_V__ operator >(int a, scalar2 b)
{
    return make_int2(a > b.x, a > b.y);
}
INLINE int3 __device__ __HOST_V__ operator >(int a, scalar3 b)
{
    return make_int3(a > b.x, a > b.y, a > b.z);
}
INLINE int4 __device__ __HOST_V__ operator >(int a, scalar4 b)
{
    return make_int4(a > b.x, a > b.y, a > b.z, a > b.w);
}
INLINE int1 __device__ __HOST_V__ operator >(scalar1 a, int b)
{
    return make_int1(a.x > b);
}
INLINE int2 __device__ __HOST_V__ operator >(scalar2 a, int b)
{
    return make_int2(a.x > b, a.y > b);
}
INLINE int3 __device__ __HOST_V__ operator >(scalar3 a, int b)
{
    return make_int3(a.x > b, a.y > b, a.z > b);
}
INLINE int4 __device__ __HOST_V__ operator >(scalar4 a, int b)
{
    return make_int4(a.x > b, a.y > b, a.z > b, a.w > b);
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator >(scalarN<N> a, int b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] > b);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator >(int a, scalarN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a > b.el[i]);
    return y;
}
INLINE int1 __device__ __HOST_V__ operator >(int1 a, scalar b)
{
    return make_int1(a.x > b);
}
INLINE int2 __device__ __HOST_V__ operator >(int2 a, scalar b)
{
    return make_int2(a.x > b, a.y > b);
}
INLINE int3 __device__ __HOST_V__ operator >(int3 a, scalar b)
{
    return make_int3(a.x > b, a.y > b, a.z > b);
}
INLINE int4 __device__ __HOST_V__ operator >(int4 a, scalar b)
{
    return make_int4(a.x > b, a.y > b, a.z > b, a.w > b);
}
INLINE int1 __device__ __HOST_V__ operator >(scalar a, int1 b)
{
    return make_int1(a > b.x);
}
INLINE int2 __device__ __HOST_V__ operator >(scalar a, int2 b)
{
    return make_int2(a > b.x, a > b.y);
}
INLINE int3 __device__ __HOST_V__ operator >(scalar a, int3 b)
{
    return make_int3(a > b.x, a > b.y, a > b.z);
}
INLINE int4 __device__ __HOST_V__ operator >(scalar a, int4 b)
{
    return make_int4(a > b.x, a > b.y, a > b.z, a > b.w);
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator >(scalar a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a > b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator >(intN<N> a, scalar b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] > b);
    return y;
}
INLINE int1 __device__ __HOST_V__ operator <(scalar1 a, scalar1 b)
{
    return make_int1(a.x < b.x);
}
INLINE int1 __device__ __HOST_V__ operator <(scalar a, scalar1 b)
{
    return make_int1(a < b.x);
}
INLINE int1 __device__ __HOST_V__ operator <(scalar1 a, scalar b)
{
    return make_int1(a.x < b);
}
INLINE int2 __device__ __HOST_V__ operator <(scalar2 a, scalar2 b)
{
    return make_int2(a.x < b.x, a.y < b.y);
}
INLINE int2 __device__ __HOST_V__ operator <(scalar a, scalar2 b)
{
    return make_int2(a < b.x, a < b.y);
}
INLINE int2 __device__ __HOST_V__ operator <(scalar2 a, scalar b)
{
    return make_int2(a.x < b, a.y < b);
}
INLINE int3 __device__ __HOST_V__ operator <(scalar3 a, scalar3 b)
{
    return make_int3(a.x < b.x, a.y < b.y, a.z < b.z);
}
INLINE int3 __device__ __HOST_V__ operator <(scalar a, scalar3 b)
{
    return make_int3(a < b.x, a < b.y, a < b.z);
}
INLINE int3 __device__ __HOST_V__ operator <(scalar3 a, scalar b)
{
    return make_int3(a.x < b, a.y < b, a.z < b);
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ operator <(scalarN<N> a, scalarN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] < b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator <(scalar a, scalarN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a < b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator <(scalarN<N> a, scalar b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] < b);
    return y;
}
INLINE int1 __device__ __HOST_V__ operator <(int1 a, int1 b)
{
    return make_int1(a.x < b.x);
}
INLINE int1 __device__ __HOST_V__ operator <(int a, int1 b)
{
    return make_int1(a < b.x);
}
INLINE int1 __device__ __HOST_V__ operator <(int1 a, int b)
{
    return make_int1(a.x < b);
}
INLINE int2 __device__ __HOST_V__ operator <(int2 a, int2 b)
{
    return make_int2(a.x < b.x, a.y < b.y);
}
INLINE int2 __device__ __HOST_V__ operator <(int a, int2 b)
{
    return make_int2(a < b.x, a < b.y);
}
INLINE int2 __device__ __HOST_V__ operator <(int2 a, int b)
{
    return make_int2(a.x < b, a.y < b);
}
INLINE int3 __device__ __HOST_V__ operator <(int3 a, int3 b)
{
    return make_int3(a.x < b.x, a.y < b.y, a.z < b.z);
}
INLINE int3 __device__ __HOST_V__ operator <(int a, int3 b)
{
    return make_int3(a < b.x, a < b.y, a < b.z);
}
INLINE int3 __device__ __HOST_V__ operator <(int3 a, int b)
{
    return make_int3(a.x < b, a.y < b, a.z < b);
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ operator <(intN<N> a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] < b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator <(int a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a < b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator <(intN<N> a, int b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] < b);
    return y;
}
INLINE int1 __device__ __HOST_V__ operator <(int a, scalar1 b)
{
    return make_int1(a < b.x);
}
INLINE int2 __device__ __HOST_V__ operator <(int a, scalar2 b)
{
    return make_int2(a < b.x, a < b.y);
}
INLINE int3 __device__ __HOST_V__ operator <(int a, scalar3 b)
{
    return make_int3(a < b.x, a < b.y, a < b.z);
}
INLINE int4 __device__ __HOST_V__ operator <(int a, scalar4 b)
{
    return make_int4(a < b.x, a < b.y, a < b.z, a < b.w);
}
INLINE int1 __device__ __HOST_V__ operator <(scalar1 a, int b)
{
    return make_int1(a.x < b);
}
INLINE int2 __device__ __HOST_V__ operator <(scalar2 a, int b)
{
    return make_int2(a.x < b, a.y < b);
}
INLINE int3 __device__ __HOST_V__ operator <(scalar3 a, int b)
{
    return make_int3(a.x < b, a.y < b, a.z < b);
}
INLINE int4 __device__ __HOST_V__ operator <(scalar4 a, int b)
{
    return make_int4(a.x < b, a.y < b, a.z < b, a.w < b);
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator <(scalarN<N> a, int b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] < b);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator <(int a, scalarN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a < b.el[i]);
    return y;
}
INLINE int1 __device__ __HOST_V__ operator <(int1 a, scalar b)
{
    return make_int1(a.x < b);
}
INLINE int2 __device__ __HOST_V__ operator <(int2 a, scalar b)
{
    return make_int2(a.x < b, a.y < b);
}
INLINE int3 __device__ __HOST_V__ operator <(int3 a, scalar b)
{
    return make_int3(a.x < b, a.y < b, a.z < b);
}
INLINE int4 __device__ __HOST_V__ operator <(int4 a, scalar b)
{
    return make_int4(a.x < b, a.y < b, a.z < b, a.w < b);
}
INLINE int1 __device__ __HOST_V__ operator <(scalar a, int1 b)
{
    return make_int1(a < b.x);
}
INLINE int2 __device__ __HOST_V__ operator <(scalar a, int2 b)
{
    return make_int2(a < b.x, a < b.y);
}
INLINE int3 __device__ __HOST_V__ operator <(scalar a, int3 b)
{
    return make_int3(a < b.x, a < b.y, a < b.z);
}
INLINE int4 __device__ __HOST_V__ operator <(scalar a, int4 b)
{
    return make_int4(a < b.x, a < b.y, a < b.z, a < b.w);
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator <(scalar a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a < b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator <(intN<N> a, scalar b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] < b);
    return y;
}
INLINE int1 __device__ __HOST_V__ operator >=(scalar1 a, scalar1 b)
{
    return make_int1(a.x >= b.x);
}
INLINE int1 __device__ __HOST_V__ operator >=(scalar a, scalar1 b)
{
    return make_int1(a >= b.x);
}
INLINE int1 __device__ __HOST_V__ operator >=(scalar1 a, scalar b)
{
    return make_int1(a.x >= b);
}
INLINE int2 __device__ __HOST_V__ operator >=(scalar2 a, scalar2 b)
{
    return make_int2(a.x >= b.x, a.y >= b.y);
}
INLINE int2 __device__ __HOST_V__ operator >=(scalar a, scalar2 b)
{
    return make_int2(a >= b.x, a >= b.y);
}
INLINE int2 __device__ __HOST_V__ operator >=(scalar2 a, scalar b)
{
    return make_int2(a.x >= b, a.y >= b);
}
INLINE int3 __device__ __HOST_V__ operator >=(scalar3 a, scalar3 b)
{
    return make_int3(a.x >= b.x, a.y >= b.y, a.z >= b.z);
}
INLINE int3 __device__ __HOST_V__ operator >=(scalar a, scalar3 b)
{
    return make_int3(a >= b.x, a >= b.y, a >= b.z);
}
INLINE int3 __device__ __HOST_V__ operator >=(scalar3 a, scalar b)
{
    return make_int3(a.x >= b, a.y >= b, a.z >= b);
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ operator >=(scalarN<N> a, scalarN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] >= b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator >=(scalar a, scalarN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a >= b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator >=(scalarN<N> a, scalar b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] >= b);
    return y;
}
INLINE int1 __device__ __HOST_V__ operator >=(int1 a, int1 b)
{
    return make_int1(a.x >= b.x);
}
INLINE int1 __device__ __HOST_V__ operator >=(int a, int1 b)
{
    return make_int1(a >= b.x);
}
INLINE int1 __device__ __HOST_V__ operator >=(int1 a, int b)
{
    return make_int1(a.x >= b);
}
INLINE int2 __device__ __HOST_V__ operator >=(int2 a, int2 b)
{
    return make_int2(a.x >= b.x, a.y >= b.y);
}
INLINE int2 __device__ __HOST_V__ operator >=(int a, int2 b)
{
    return make_int2(a >= b.x, a >= b.y);
}
INLINE int2 __device__ __HOST_V__ operator >=(int2 a, int b)
{
    return make_int2(a.x >= b, a.y >= b);
}
INLINE int3 __device__ __HOST_V__ operator >=(int3 a, int3 b)
{
    return make_int3(a.x >= b.x, a.y >= b.y, a.z >= b.z);
}
INLINE int3 __device__ __HOST_V__ operator >=(int a, int3 b)
{
    return make_int3(a >= b.x, a >= b.y, a >= b.z);
}
INLINE int3 __device__ __HOST_V__ operator >=(int3 a, int b)
{
    return make_int3(a.x >= b, a.y >= b, a.z >= b);
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ operator >=(intN<N> a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] >= b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator >=(int a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a >= b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator >=(intN<N> a, int b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] >= b);
    return y;
}
INLINE int1 __device__ __HOST_V__ operator >=(int a, scalar1 b)
{
    return make_int1(a >= b.x);
}
INLINE int2 __device__ __HOST_V__ operator >=(int a, scalar2 b)
{
    return make_int2(a >= b.x, a >= b.y);
}
INLINE int3 __device__ __HOST_V__ operator >=(int a, scalar3 b)
{
    return make_int3(a >= b.x, a >= b.y, a >= b.z);
}
INLINE int4 __device__ __HOST_V__ operator >=(int a, scalar4 b)
{
    return make_int4(a >= b.x, a >= b.y, a >= b.z, a >= b.w);
}
INLINE int1 __device__ __HOST_V__ operator >=(scalar1 a, int b)
{
    return make_int1(a.x >= b);
}
INLINE int2 __device__ __HOST_V__ operator >=(scalar2 a, int b)
{
    return make_int2(a.x >= b, a.y >= b);
}
INLINE int3 __device__ __HOST_V__ operator >=(scalar3 a, int b)
{
    return make_int3(a.x >= b, a.y >= b, a.z >= b);
}
INLINE int4 __device__ __HOST_V__ operator >=(scalar4 a, int b)
{
    return make_int4(a.x >= b, a.y >= b, a.z >= b, a.w >= b);
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator >=(scalarN<N> a, int b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] >= b);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator >=(int a, scalarN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a >= b.el[i]);
    return y;
}
INLINE int1 __device__ __HOST_V__ operator >=(int1 a, scalar b)
{
    return make_int1(a.x >= b);
}
INLINE int2 __device__ __HOST_V__ operator >=(int2 a, scalar b)
{
    return make_int2(a.x >= b, a.y >= b);
}
INLINE int3 __device__ __HOST_V__ operator >=(int3 a, scalar b)
{
    return make_int3(a.x >= b, a.y >= b, a.z >= b);
}
INLINE int4 __device__ __HOST_V__ operator >=(int4 a, scalar b)
{
    return make_int4(a.x >= b, a.y >= b, a.z >= b, a.w >= b);
}
INLINE int1 __device__ __HOST_V__ operator >=(scalar a, int1 b)
{
    return make_int1(a >= b.x);
}
INLINE int2 __device__ __HOST_V__ operator >=(scalar a, int2 b)
{
    return make_int2(a >= b.x, a >= b.y);
}
INLINE int3 __device__ __HOST_V__ operator >=(scalar a, int3 b)
{
    return make_int3(a >= b.x, a >= b.y, a >= b.z);
}
INLINE int4 __device__ __HOST_V__ operator >=(scalar a, int4 b)
{
    return make_int4(a >= b.x, a >= b.y, a >= b.z, a >= b.w);
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator >=(scalar a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a >= b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator >=(intN<N> a, scalar b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] >= b);
    return y;
}
INLINE int1 __device__ __HOST_V__ operator <=(scalar1 a, scalar1 b)
{
    return make_int1(a.x <= b.x);
}
INLINE int1 __device__ __HOST_V__ operator <=(scalar a, scalar1 b)
{
    return make_int1(a <= b.x);
}
INLINE int1 __device__ __HOST_V__ operator <=(scalar1 a, scalar b)
{
    return make_int1(a.x <= b);
}
INLINE int2 __device__ __HOST_V__ operator <=(scalar2 a, scalar2 b)
{
    return make_int2(a.x <= b.x, a.y <= b.y);
}
INLINE int2 __device__ __HOST_V__ operator <=(scalar a, scalar2 b)
{
    return make_int2(a <= b.x, a <= b.y);
}
INLINE int2 __device__ __HOST_V__ operator <=(scalar2 a, scalar b)
{
    return make_int2(a.x <= b, a.y <= b);
}
INLINE int3 __device__ __HOST_V__ operator <=(scalar3 a, scalar3 b)
{
    return make_int3(a.x <= b.x, a.y <= b.y, a.z <= b.z);
}
INLINE int3 __device__ __HOST_V__ operator <=(scalar a, scalar3 b)
{
    return make_int3(a <= b.x, a <= b.y, a <= b.z);
}
INLINE int3 __device__ __HOST_V__ operator <=(scalar3 a, scalar b)
{
    return make_int3(a.x <= b, a.y <= b, a.z <= b);
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ operator <=(scalarN<N> a, scalarN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] <= b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator <=(scalar a, scalarN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a <= b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator <=(scalarN<N> a, scalar b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] <= b);
    return y;
}
INLINE int1 __device__ __HOST_V__ operator <=(int1 a, int1 b)
{
    return make_int1(a.x <= b.x);
}
INLINE int1 __device__ __HOST_V__ operator <=(int a, int1 b)
{
    return make_int1(a <= b.x);
}
INLINE int1 __device__ __HOST_V__ operator <=(int1 a, int b)
{
    return make_int1(a.x <= b);
}
INLINE int2 __device__ __HOST_V__ operator <=(int2 a, int2 b)
{
    return make_int2(a.x <= b.x, a.y <= b.y);
}
INLINE int2 __device__ __HOST_V__ operator <=(int a, int2 b)
{
    return make_int2(a <= b.x, a <= b.y);
}
INLINE int2 __device__ __HOST_V__ operator <=(int2 a, int b)
{
    return make_int2(a.x <= b, a.y <= b);
}
INLINE int3 __device__ __HOST_V__ operator <=(int3 a, int3 b)
{
    return make_int3(a.x <= b.x, a.y <= b.y, a.z <= b.z);
}
INLINE int3 __device__ __HOST_V__ operator <=(int a, int3 b)
{
    return make_int3(a <= b.x, a <= b.y, a <= b.z);
}
INLINE int3 __device__ __HOST_V__ operator <=(int3 a, int b)
{
    return make_int3(a.x <= b, a.y <= b, a.z <= b);
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ operator <=(intN<N> a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] <= b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator <=(int a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a <= b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator <=(intN<N> a, int b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] <= b);
    return y;
}
INLINE int1 __device__ __HOST_V__ operator <=(int a, scalar1 b)
{
    return make_int1(a <= b.x);
}
INLINE int2 __device__ __HOST_V__ operator <=(int a, scalar2 b)
{
    return make_int2(a <= b.x, a <= b.y);
}
INLINE int3 __device__ __HOST_V__ operator <=(int a, scalar3 b)
{
    return make_int3(a <= b.x, a <= b.y, a <= b.z);
}
INLINE int4 __device__ __HOST_V__ operator <=(int a, scalar4 b)
{
    return make_int4(a <= b.x, a <= b.y, a <= b.z, a <= b.w);
}
INLINE int1 __device__ __HOST_V__ operator <=(scalar1 a, int b)
{
    return make_int1(a.x <= b);
}
INLINE int2 __device__ __HOST_V__ operator <=(scalar2 a, int b)
{
    return make_int2(a.x <= b, a.y <= b);
}
INLINE int3 __device__ __HOST_V__ operator <=(scalar3 a, int b)
{
    return make_int3(a.x <= b, a.y <= b, a.z <= b);
}
INLINE int4 __device__ __HOST_V__ operator <=(scalar4 a, int b)
{
    return make_int4(a.x <= b, a.y <= b, a.z <= b, a.w <= b);
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator <=(scalarN<N> a, int b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] <= b);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator <=(int a, scalarN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a <= b.el[i]);
    return y;
}
INLINE int1 __device__ __HOST_V__ operator <=(int1 a, scalar b)
{
    return make_int1(a.x <= b);
}
INLINE int2 __device__ __HOST_V__ operator <=(int2 a, scalar b)
{
    return make_int2(a.x <= b, a.y <= b);
}
INLINE int3 __device__ __HOST_V__ operator <=(int3 a, scalar b)
{
    return make_int3(a.x <= b, a.y <= b, a.z <= b);
}
INLINE int4 __device__ __HOST_V__ operator <=(int4 a, scalar b)
{
    return make_int4(a.x <= b, a.y <= b, a.z <= b, a.w <= b);
}
INLINE int1 __device__ __HOST_V__ operator <=(scalar a, int1 b)
{
    return make_int1(a <= b.x);
}
INLINE int2 __device__ __HOST_V__ operator <=(scalar a, int2 b)
{
    return make_int2(a <= b.x, a <= b.y);
}
INLINE int3 __device__ __HOST_V__ operator <=(scalar a, int3 b)
{
    return make_int3(a <= b.x, a <= b.y, a <= b.z);
}
INLINE int4 __device__ __HOST_V__ operator <=(scalar a, int4 b)
{
    return make_int4(a <= b.x, a <= b.y, a <= b.z, a <= b.w);
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator <=(scalar a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a <= b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator <=(intN<N> a, scalar b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] <= b);
    return y;
}
INLINE int1 __device__ __HOST_V__ operator ==(scalar1 a, scalar1 b)
{
    return make_int1(a.x == b.x);
}
INLINE int1 __device__ __HOST_V__ operator ==(scalar a, scalar1 b)
{
    return make_int1(a == b.x);
}
INLINE int1 __device__ __HOST_V__ operator ==(scalar1 a, scalar b)
{
    return make_int1(a.x == b);
}
INLINE int2 __device__ __HOST_V__ operator ==(scalar2 a, scalar2 b)
{
    return make_int2(a.x == b.x, a.y == b.y);
}
INLINE int2 __device__ __HOST_V__ operator ==(scalar a, scalar2 b)
{
    return make_int2(a == b.x, a == b.y);
}
INLINE int2 __device__ __HOST_V__ operator ==(scalar2 a, scalar b)
{
    return make_int2(a.x == b, a.y == b);
}
INLINE int3 __device__ __HOST_V__ operator ==(scalar3 a, scalar3 b)
{
    return make_int3(a.x == b.x, a.y == b.y, a.z == b.z);
}
INLINE int3 __device__ __HOST_V__ operator ==(scalar a, scalar3 b)
{
    return make_int3(a == b.x, a == b.y, a == b.z);
}
INLINE int3 __device__ __HOST_V__ operator ==(scalar3 a, scalar b)
{
    return make_int3(a.x == b, a.y == b, a.z == b);
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ operator ==(scalarN<N> a, scalarN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] == b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator ==(scalar a, scalarN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a == b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator ==(scalarN<N> a, scalar b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] == b);
    return y;
}
INLINE int1 __device__ __HOST_V__ operator ==(int1 a, int1 b)
{
    return make_int1(a.x == b.x);
}
INLINE int1 __device__ __HOST_V__ operator ==(int a, int1 b)
{
    return make_int1(a == b.x);
}
INLINE int1 __device__ __HOST_V__ operator ==(int1 a, int b)
{
    return make_int1(a.x == b);
}
INLINE int2 __device__ __HOST_V__ operator ==(int2 a, int2 b)
{
    return make_int2(a.x == b.x, a.y == b.y);
}
INLINE int2 __device__ __HOST_V__ operator ==(int a, int2 b)
{
    return make_int2(a == b.x, a == b.y);
}
INLINE int2 __device__ __HOST_V__ operator ==(int2 a, int b)
{
    return make_int2(a.x == b, a.y == b);
}
INLINE int3 __device__ __HOST_V__ operator ==(int3 a, int3 b)
{
    return make_int3(a.x == b.x, a.y == b.y, a.z == b.z);
}
INLINE int3 __device__ __HOST_V__ operator ==(int a, int3 b)
{
    return make_int3(a == b.x, a == b.y, a == b.z);
}
INLINE int3 __device__ __HOST_V__ operator ==(int3 a, int b)
{
    return make_int3(a.x == b, a.y == b, a.z == b);
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ operator ==(intN<N> a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] == b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator ==(int a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a == b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator ==(intN<N> a, int b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] == b);
    return y;
}
INLINE int1 __device__ __HOST_V__ operator ==(int a, scalar1 b)
{
    return make_int1(a == b.x);
}
INLINE int2 __device__ __HOST_V__ operator ==(int a, scalar2 b)
{
    return make_int2(a == b.x, a == b.y);
}
INLINE int3 __device__ __HOST_V__ operator ==(int a, scalar3 b)
{
    return make_int3(a == b.x, a == b.y, a == b.z);
}
INLINE int4 __device__ __HOST_V__ operator ==(int a, scalar4 b)
{
    return make_int4(a == b.x, a == b.y, a == b.z, a == b.w);
}
INLINE int1 __device__ __HOST_V__ operator ==(scalar1 a, int b)
{
    return make_int1(a.x == b);
}
INLINE int2 __device__ __HOST_V__ operator ==(scalar2 a, int b)
{
    return make_int2(a.x == b, a.y == b);
}
INLINE int3 __device__ __HOST_V__ operator ==(scalar3 a, int b)
{
    return make_int3(a.x == b, a.y == b, a.z == b);
}
INLINE int4 __device__ __HOST_V__ operator ==(scalar4 a, int b)
{
    return make_int4(a.x == b, a.y == b, a.z == b, a.w == b);
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator ==(scalarN<N> a, int b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] == b);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator ==(int a, scalarN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a == b.el[i]);
    return y;
}
INLINE int1 __device__ __HOST_V__ operator ==(int1 a, scalar b)
{
    return make_int1(a.x == b);
}
INLINE int2 __device__ __HOST_V__ operator ==(int2 a, scalar b)
{
    return make_int2(a.x == b, a.y == b);
}
INLINE int3 __device__ __HOST_V__ operator ==(int3 a, scalar b)
{
    return make_int3(a.x == b, a.y == b, a.z == b);
}
INLINE int4 __device__ __HOST_V__ operator ==(int4 a, scalar b)
{
    return make_int4(a.x == b, a.y == b, a.z == b, a.w == b);
}
INLINE int1 __device__ __HOST_V__ operator ==(scalar a, int1 b)
{
    return make_int1(a == b.x);
}
INLINE int2 __device__ __HOST_V__ operator ==(scalar a, int2 b)
{
    return make_int2(a == b.x, a == b.y);
}
INLINE int3 __device__ __HOST_V__ operator ==(scalar a, int3 b)
{
    return make_int3(a == b.x, a == b.y, a == b.z);
}
INLINE int4 __device__ __HOST_V__ operator ==(scalar a, int4 b)
{
    return make_int4(a == b.x, a == b.y, a == b.z, a == b.w);
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator ==(scalar a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a == b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator ==(intN<N> a, scalar b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] == b);
    return y;
}
INLINE int1 __device__ __HOST_V__ operator !=(scalar1 a, scalar1 b)
{
    return make_int1(a.x != b.x);
}
INLINE int1 __device__ __HOST_V__ operator !=(scalar a, scalar1 b)
{
    return make_int1(a != b.x);
}
INLINE int1 __device__ __HOST_V__ operator !=(scalar1 a, scalar b)
{
    return make_int1(a.x != b);
}
INLINE int2 __device__ __HOST_V__ operator !=(scalar2 a, scalar2 b)
{
    return make_int2(a.x != b.x, a.y != b.y);
}
INLINE int2 __device__ __HOST_V__ operator !=(scalar a, scalar2 b)
{
    return make_int2(a != b.x, a != b.y);
}
INLINE int2 __device__ __HOST_V__ operator !=(scalar2 a, scalar b)
{
    return make_int2(a.x != b, a.y != b);
}
INLINE int3 __device__ __HOST_V__ operator !=(scalar3 a, scalar3 b)
{
    return make_int3(a.x != b.x, a.y != b.y, a.z != b.z);
}
INLINE int3 __device__ __HOST_V__ operator !=(scalar a, scalar3 b)
{
    return make_int3(a != b.x, a != b.y, a != b.z);
}
INLINE int3 __device__ __HOST_V__ operator !=(scalar3 a, scalar b)
{
    return make_int3(a.x != b, a.y != b, a.z != b);
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ operator !=(scalarN<N> a, scalarN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] != b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator !=(scalar a, scalarN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a != b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator !=(scalarN<N> a, scalar b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] != b);
    return y;
}
INLINE int1 __device__ __HOST_V__ operator !=(int1 a, int1 b)
{
    return make_int1(a.x != b.x);
}
INLINE int1 __device__ __HOST_V__ operator !=(int a, int1 b)
{
    return make_int1(a != b.x);
}
INLINE int1 __device__ __HOST_V__ operator !=(int1 a, int b)
{
    return make_int1(a.x != b);
}
INLINE int2 __device__ __HOST_V__ operator !=(int2 a, int2 b)
{
    return make_int2(a.x != b.x, a.y != b.y);
}
INLINE int2 __device__ __HOST_V__ operator !=(int a, int2 b)
{
    return make_int2(a != b.x, a != b.y);
}
INLINE int2 __device__ __HOST_V__ operator !=(int2 a, int b)
{
    return make_int2(a.x != b, a.y != b);
}
INLINE int3 __device__ __HOST_V__ operator !=(int3 a, int3 b)
{
    return make_int3(a.x != b.x, a.y != b.y, a.z != b.z);
}
INLINE int3 __device__ __HOST_V__ operator !=(int a, int3 b)
{
    return make_int3(a != b.x, a != b.y, a != b.z);
}
INLINE int3 __device__ __HOST_V__ operator !=(int3 a, int b)
{
    return make_int3(a.x != b, a.y != b, a.z != b);
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ operator !=(intN<N> a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] != b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator !=(int a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a != b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator !=(intN<N> a, int b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] != b);
    return y;
}
INLINE int1 __device__ __HOST_V__ operator !=(int a, scalar1 b)
{
    return make_int1(a != b.x);
}
INLINE int2 __device__ __HOST_V__ operator !=(int a, scalar2 b)
{
    return make_int2(a != b.x, a != b.y);
}
INLINE int3 __device__ __HOST_V__ operator !=(int a, scalar3 b)
{
    return make_int3(a != b.x, a != b.y, a != b.z);
}
INLINE int4 __device__ __HOST_V__ operator !=(int a, scalar4 b)
{
    return make_int4(a != b.x, a != b.y, a != b.z, a != b.w);
}
INLINE int1 __device__ __HOST_V__ operator !=(scalar1 a, int b)
{
    return make_int1(a.x != b);
}
INLINE int2 __device__ __HOST_V__ operator !=(scalar2 a, int b)
{
    return make_int2(a.x != b, a.y != b);
}
INLINE int3 __device__ __HOST_V__ operator !=(scalar3 a, int b)
{
    return make_int3(a.x != b, a.y != b, a.z != b);
}
INLINE int4 __device__ __HOST_V__ operator !=(scalar4 a, int b)
{
    return make_int4(a.x != b, a.y != b, a.z != b, a.w != b);
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator !=(scalarN<N> a, int b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] != b);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator !=(int a, scalarN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a != b.el[i]);
    return y;
}
INLINE int1 __device__ __HOST_V__ operator !=(int1 a, scalar b)
{
    return make_int1(a.x != b);
}
INLINE int2 __device__ __HOST_V__ operator !=(int2 a, scalar b)
{
    return make_int2(a.x != b, a.y != b);
}
INLINE int3 __device__ __HOST_V__ operator !=(int3 a, scalar b)
{
    return make_int3(a.x != b, a.y != b, a.z != b);
}
INLINE int4 __device__ __HOST_V__ operator !=(int4 a, scalar b)
{
    return make_int4(a.x != b, a.y != b, a.z != b, a.w != b);
}
INLINE int1 __device__ __HOST_V__ operator !=(scalar a, int1 b)
{
    return make_int1(a != b.x);
}
INLINE int2 __device__ __HOST_V__ operator !=(scalar a, int2 b)
{
    return make_int2(a != b.x, a != b.y);
}
INLINE int3 __device__ __HOST_V__ operator !=(scalar a, int3 b)
{
    return make_int3(a != b.x, a != b.y, a != b.z);
}
INLINE int4 __device__ __HOST_V__ operator !=(scalar a, int4 b)
{
    return make_int4(a != b.x, a != b.y, a != b.z, a != b.w);
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator !=(scalar a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a != b.el[i]);
    return y;
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator !=(intN<N> a, scalar b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = a.el[i] != b);
    return y;
}
namespace quasar {
INLINE scalar1 __device__ __HOST_V__ autocast (int1 a, ReturnType<scalar1>)
{
    return make_scalar1(autocast(a.x, ReturnType<scalar>()));
}
INLINE scalar2 __device__ __HOST_V__ autocast(int2 a, ReturnType<scalar2>)
{
    return make_scalar2(autocast(a.x, ReturnType<scalar>()), autocast(a.y, ReturnType<scalar>()));
}
INLINE scalar3 __device__ __HOST_V__ autocast(int3 a, ReturnType<scalar3>)
{
    return make_scalar3(autocast(a.x, ReturnType<scalar>()), autocast(a.y, ReturnType<scalar>()), autocast(a.z, ReturnType<scalar>()));
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ autocast(intN<N> a, ReturnType<scalarN<N> >)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<scalar>()));
    return y;
}
INLINE int1 __device__ __HOST_V__ autocast (scalar1 a, ReturnType<int1>)
{
    return make_int1(autocast(a.x, ReturnType<int>()));
}
INLINE int2 __device__ __HOST_V__ autocast(scalar2 a, ReturnType<int2>)
{
    return make_int2(autocast(a.x, ReturnType<int>()), autocast(a.y, ReturnType<int>()));
}
INLINE int3 __device__ __HOST_V__ autocast(scalar3 a, ReturnType<int3>)
{
    return make_int3(autocast(a.x, ReturnType<int>()), autocast(a.y, ReturnType<int>()), autocast(a.z, ReturnType<int>()));
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ autocast(scalarN<N> a, ReturnType<intN<N> >)
{
    intN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<int>()));
    return y;
}
INLINE cscalar1 __device__ __HOST_V__ autocast (scalar1 a, ReturnType<cscalar1>)
{
    return make_cscalar1(autocast(a.x, ReturnType<cscalar>()));
}
INLINE cscalar2 __device__ __HOST_V__ autocast(scalar2 a, ReturnType<cscalar2>)
{
    return make_cscalar2(autocast(a.x, ReturnType<cscalar>()), autocast(a.y, ReturnType<cscalar>()));
}
INLINE cscalar3 __device__ __HOST_V__ autocast(scalar3 a, ReturnType<cscalar3>)
{
    return make_cscalar3(autocast(a.x, ReturnType<cscalar>()), autocast(a.y, ReturnType<cscalar>()), autocast(a.z, ReturnType<cscalar>()));
}
INLINE cscalar4 __device__ __HOST_V__ autocast(scalar4 a, ReturnType<cscalar4>)
{
    return make_cscalar4(autocast(a.x, ReturnType<cscalar>()), autocast(a.y, ReturnType<cscalar>()), autocast(a.z, ReturnType<cscalar>()), autocast(a.w, ReturnType<cscalar>()));
}
template <int N>
INLINE cscalarN<N> __device__ __HOST_V__ autocast(scalarN<N> a, ReturnType<cscalarN<N> >)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<cscalar>()));
    return y;
}
INLINE cscalar1 __device__ __HOST_V__ autocast (int1 a, ReturnType<cscalar1>)
{
    return make_cscalar1(autocast(a.x, ReturnType<cscalar>()));
}
INLINE cscalar2 __device__ __HOST_V__ autocast(int2 a, ReturnType<cscalar2>)
{
    return make_cscalar2(autocast(a.x, ReturnType<cscalar>()), autocast(a.y, ReturnType<cscalar>()));
}
INLINE cscalar3 __device__ __HOST_V__ autocast(int3 a, ReturnType<cscalar3>)
{
    return make_cscalar3(autocast(a.x, ReturnType<cscalar>()), autocast(a.y, ReturnType<cscalar>()), autocast(a.z, ReturnType<cscalar>()));
}
INLINE cscalar4 __device__ __HOST_V__ autocast(int4 a, ReturnType<cscalar4>)
{
    return make_cscalar4(autocast(a.x, ReturnType<cscalar>()), autocast(a.y, ReturnType<cscalar>()), autocast(a.z, ReturnType<cscalar>()), autocast(a.w, ReturnType<cscalar>()));
}
template <int N>
INLINE cscalarN<N> __device__ __HOST_V__ autocast(intN<N> a, ReturnType<cscalarN<N> >)
{
    cscalarN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<cscalar>()));
    return y;
}
#ifndef DBL_SCALAR
INLINE double1 __device__ __HOST_V__ autocast (scalar1 a, ReturnType<double1>)
{
    return make_double1(autocast(a.x, ReturnType<double>()));
}
INLINE double2 __device__ __HOST_V__ autocast(scalar2 a, ReturnType<double2>)
{
    return make_double2(autocast(a.x, ReturnType<double>()), autocast(a.y, ReturnType<double>()));
}
INLINE double3 __device__ __HOST_V__ autocast(scalar3 a, ReturnType<double3>)
{
    return make_double3(autocast(a.x, ReturnType<double>()), autocast(a.y, ReturnType<double>()), autocast(a.z, ReturnType<double>()));
}
INLINE double4 __device__ __HOST_V__ autocast(scalar4 a, ReturnType<double4>)
{
    return make_double4(autocast(a.x, ReturnType<double>()), autocast(a.y, ReturnType<double>()), autocast(a.z, ReturnType<double>()), autocast(a.w, ReturnType<double>()));
}
template <int N>
INLINE doubleN<N> __device__ __HOST_V__ autocast(scalarN<N> a, ReturnType<doubleN<N> >)
{
    doubleN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<double>()));
    return y;
}
INLINE scalar1 __device__ __HOST_V__ autocast (double1 a, ReturnType<scalar1>)
{
    return make_scalar1(autocast(a.x, ReturnType<scalar>()));
}
INLINE scalar2 __device__ __HOST_V__ autocast(double2 a, ReturnType<scalar2>)
{
    return make_scalar2(autocast(a.x, ReturnType<scalar>()), autocast(a.y, ReturnType<scalar>()));
}
INLINE scalar3 __device__ __HOST_V__ autocast(double3 a, ReturnType<scalar3>)
{
    return make_scalar3(autocast(a.x, ReturnType<scalar>()), autocast(a.y, ReturnType<scalar>()), autocast(a.z, ReturnType<scalar>()));
}
INLINE scalar4 __device__ __HOST_V__ autocast(double4 a, ReturnType<scalar4>)
{
    return make_scalar4(autocast(a.x, ReturnType<scalar>()), autocast(a.y, ReturnType<scalar>()), autocast(a.z, ReturnType<scalar>()), autocast(a.w, ReturnType<scalar>()));
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ autocast(doubleN<N> a, ReturnType<scalarN<N> >)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<scalar>()));
    return y;
}
#else
INLINE float1 __device__ __HOST_V__ autocast (scalar1 a, ReturnType<float1>)
{
    return make_float1(autocast(a.x, ReturnType<float>()));
}
INLINE float2 __device__ __HOST_V__ autocast(scalar2 a, ReturnType<float2>)
{
    return make_float2(autocast(a.x, ReturnType<float>()), autocast(a.y, ReturnType<float>()));
}
INLINE float3 __device__ __HOST_V__ autocast(scalar3 a, ReturnType<float3>)
{
    return make_float3(autocast(a.x, ReturnType<float>()), autocast(a.y, ReturnType<float>()), autocast(a.z, ReturnType<float>()));
}
INLINE float4 __device__ __HOST_V__ autocast(scalar4 a, ReturnType<float4>)
{
    return make_float4(autocast(a.x, ReturnType<float>()), autocast(a.y, ReturnType<float>()), autocast(a.z, ReturnType<float>()), autocast(a.w, ReturnType<float>()));
}
template <int N>
INLINE floatN<N> __device__ __HOST_V__ autocast(scalarN<N> a, ReturnType<floatN<N> >)
{
    floatN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<float>()));
    return y;
}
INLINE scalar1 __device__ __HOST_V__ autocast (float1 a, ReturnType<scalar1>)
{
    return make_scalar1(autocast(a.x, ReturnType<scalar>()));
}
INLINE scalar2 __device__ __HOST_V__ autocast(float2 a, ReturnType<scalar2>)
{
    return make_scalar2(autocast(a.x, ReturnType<scalar>()), autocast(a.y, ReturnType<scalar>()));
}
INLINE scalar3 __device__ __HOST_V__ autocast(float3 a, ReturnType<scalar3>)
{
    return make_scalar3(autocast(a.x, ReturnType<scalar>()), autocast(a.y, ReturnType<scalar>()), autocast(a.z, ReturnType<scalar>()));
}
INLINE scalar4 __device__ __HOST_V__ autocast(float4 a, ReturnType<scalar4>)
{
    return make_scalar4(autocast(a.x, ReturnType<scalar>()), autocast(a.y, ReturnType<scalar>()), autocast(a.z, ReturnType<scalar>()), autocast(a.w, ReturnType<scalar>()));
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ autocast(floatN<N> a, ReturnType<scalarN<N> >)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<scalar>()));
    return y;
}
#endif
INLINE char __device__  __HOST_V__ autocast_int(scalar val, IntegerConversionModes mode, ReturnType<char>)
{
    return autocast_int<scalar, char>(val, MINVAL_char, MAXVAL_char, mode);
}
INLINE char1 __device__  __HOST_V__ autocast_int(scalar1 val, IntegerConversionModes mode, ReturnType<char1>)
{
    return make_char1(autocast_int<scalar, char>(val.x, MINVAL_char, MAXVAL_char, mode));
}
INLINE char2 __device__  __HOST_V__ autocast_int(scalar2 val, IntegerConversionModes mode, ReturnType<char2>)
{
    return make_char2(autocast_int<scalar, char>(val.x, MINVAL_char, MAXVAL_char, mode),
						autocast_int<scalar, char>(val.y, MINVAL_char, MAXVAL_char, mode));
}
INLINE char3 __device__  __HOST_V__ autocast_int(scalar3 val, IntegerConversionModes mode, ReturnType<char3>)
{
    return make_char3(autocast_int<scalar, char>(val.x, MINVAL_char, MAXVAL_char, mode),
						autocast_int<scalar, char>(val.y, MINVAL_char, MAXVAL_char, mode),
						autocast_int<scalar, char>(val.z, MINVAL_char, MAXVAL_char, mode));
}
template <int N>
INLINE charN<N> __device__  __HOST_V__ autocast_int(scalarN<N> val, IntegerConversionModes mode, ReturnType<charN<N> >)
{
	charN<N> y;
	FOR_ALL((y.el[i] = autocast_int<scalar, char>(val.el[i], MINVAL_char, MAXVAL_char, mode)));
	return y;
}
INLINE short __device__  __HOST_V__ autocast_int(scalar val, IntegerConversionModes mode, ReturnType<short>)
{
    return autocast_int<scalar, short>(val, MINVAL_int16, MAXVAL_int16, mode);
}
INLINE short1 __device__  __HOST_V__ autocast_int(scalar1 val, IntegerConversionModes mode, ReturnType<short1>)
{
    return make_short1(autocast_int<scalar, short>(val.x, MINVAL_int16, MAXVAL_int16, mode));
}
INLINE short2 __device__  __HOST_V__ autocast_int(scalar2 val, IntegerConversionModes mode, ReturnType<short2>)
{
    return make_short2(autocast_int<scalar, short>(val.x, MINVAL_int16, MAXVAL_int16, mode),
						autocast_int<scalar, short>(val.y, MINVAL_int16, MAXVAL_int16, mode));
}
INLINE short3 __device__  __HOST_V__ autocast_int(scalar3 val, IntegerConversionModes mode, ReturnType<short3>)
{
    return make_short3(autocast_int<scalar, short>(val.x, MINVAL_int16, MAXVAL_int16, mode),
						autocast_int<scalar, short>(val.y, MINVAL_int16, MAXVAL_int16, mode),
						autocast_int<scalar, short>(val.z, MINVAL_int16, MAXVAL_int16, mode));
}
template <int N>
INLINE shortN<N> __device__  __HOST_V__ autocast_int(scalarN<N> val, IntegerConversionModes mode, ReturnType<shortN<N> >)
{
	shortN<N> y;
	FOR_ALL((y.el[i] = autocast_int<scalar, short>(val.el[i], MINVAL_int16, MAXVAL_int16, mode)));
	return y;
}
INLINE int __device__  __HOST_V__ autocast_int(scalar val, IntegerConversionModes mode, ReturnType<int>)
{
    return autocast_int<scalar, int>(val, MINVAL_int32, MAXVAL_int32, mode);
}
INLINE int1 __device__  __HOST_V__ autocast_int(scalar1 val, IntegerConversionModes mode, ReturnType<int1>)
{
    return make_int1(autocast_int<scalar, int>(val.x, MINVAL_int32, MAXVAL_int32, mode));
}
INLINE int2 __device__  __HOST_V__ autocast_int(scalar2 val, IntegerConversionModes mode, ReturnType<int2>)
{
    return make_int2(autocast_int<scalar, int>(val.x, MINVAL_int32, MAXVAL_int32, mode),
						autocast_int<scalar, int>(val.y, MINVAL_int32, MAXVAL_int32, mode));
}
INLINE int3 __device__  __HOST_V__ autocast_int(scalar3 val, IntegerConversionModes mode, ReturnType<int3>)
{
    return make_int3(autocast_int<scalar, int>(val.x, MINVAL_int32, MAXVAL_int32, mode),
						autocast_int<scalar, int>(val.y, MINVAL_int32, MAXVAL_int32, mode),
						autocast_int<scalar, int>(val.z, MINVAL_int32, MAXVAL_int32, mode));
}
INLINE int4 __device__  __HOST_V__ autocast_int(scalar4 val, IntegerConversionModes mode, ReturnType<int4>)
{
    return make_int4(autocast_int<scalar, int>(val.x, MINVAL_int32, MAXVAL_int32, mode),
						autocast_int<scalar, int>(val.y, MINVAL_int32, MAXVAL_int32, mode),
						autocast_int<scalar, int>(val.z, MINVAL_int32, MAXVAL_int32, mode),
						autocast_int<scalar, int>(val.w, MINVAL_int32, MAXVAL_int32, mode));
}
template <int N>
INLINE intN<N> __device__  __HOST_V__ autocast_int(scalarN<N> val, IntegerConversionModes mode, ReturnType<intN<N> >)
{
	intN<N> y;
	FOR_ALL((y.el[i] = autocast_int<scalar, int>(val.el[i], MINVAL_int32, MAXVAL_int32, mode)));
	return y;
}
INLINE longlong __device__  __HOST_V__ autocast_int(scalar val, IntegerConversionModes mode, ReturnType<longlong>)
{
    return autocast_int<scalar, longlong>(val, MINVAL_int64, MAXVAL_int64, mode);
}
INLINE longlong1 __device__  __HOST_V__ autocast_int(scalar1 val, IntegerConversionModes mode, ReturnType<longlong1>)
{
    return make_longlong1(autocast_int<scalar, longlong>(val.x, MINVAL_int64, MAXVAL_int64, mode));
}
INLINE longlong2 __device__  __HOST_V__ autocast_int(scalar2 val, IntegerConversionModes mode, ReturnType<longlong2>)
{
    return make_longlong2(autocast_int<scalar, longlong>(val.x, MINVAL_int64, MAXVAL_int64, mode),
						autocast_int<scalar, longlong>(val.y, MINVAL_int64, MAXVAL_int64, mode));
}
INLINE longlong3 __device__  __HOST_V__ autocast_int(scalar3 val, IntegerConversionModes mode, ReturnType<longlong3>)
{
    return make_longlong3(autocast_int<scalar, longlong>(val.x, MINVAL_int64, MAXVAL_int64, mode),
						autocast_int<scalar, longlong>(val.y, MINVAL_int64, MAXVAL_int64, mode),
						autocast_int<scalar, longlong>(val.z, MINVAL_int64, MAXVAL_int64, mode));
}
INLINE longlong4 __device__  __HOST_V__ autocast_int(scalar4 val, IntegerConversionModes mode, ReturnType<longlong4>)
{
    return make_longlong4(autocast_int<scalar, longlong>(val.x, MINVAL_int64, MAXVAL_int64, mode),
						autocast_int<scalar, longlong>(val.y, MINVAL_int64, MAXVAL_int64, mode),
						autocast_int<scalar, longlong>(val.z, MINVAL_int64, MAXVAL_int64, mode),
						autocast_int<scalar, longlong>(val.w, MINVAL_int64, MAXVAL_int64, mode));
}
template <int N>
INLINE longlongN<N> __device__  __HOST_V__ autocast_int(scalarN<N> val, IntegerConversionModes mode, ReturnType<longlongN<N> >)
{
	longlongN<N> y;
	FOR_ALL((y.el[i] = autocast_int<scalar, longlong>(val.el[i], MINVAL_int64, MAXVAL_int64, mode)));
	return y;
}
INLINE uchar __device__  __HOST_V__ autocast_int(scalar val, IntegerConversionModes mode, ReturnType<uchar>)
{
    return autocast_int<scalar, uchar>(val, MINVAL_uchar, MAXVAL_uchar, mode);
}
INLINE uchar1 __device__  __HOST_V__ autocast_int(scalar1 val, IntegerConversionModes mode, ReturnType<uchar1>)
{
    return make_uchar1(autocast_int<scalar, uchar>(val.x, MINVAL_uchar, MAXVAL_uchar, mode));
}
INLINE uchar2 __device__  __HOST_V__ autocast_int(scalar2 val, IntegerConversionModes mode, ReturnType<uchar2>)
{
    return make_uchar2(autocast_int<scalar, uchar>(val.x, MINVAL_uchar, MAXVAL_uchar, mode),
						autocast_int<scalar, uchar>(val.y, MINVAL_uchar, MAXVAL_uchar, mode));
}
INLINE uchar3 __device__  __HOST_V__ autocast_int(scalar3 val, IntegerConversionModes mode, ReturnType<uchar3>)
{
    return make_uchar3(autocast_int<scalar, uchar>(val.x, MINVAL_uchar, MAXVAL_uchar, mode),
						autocast_int<scalar, uchar>(val.y, MINVAL_uchar, MAXVAL_uchar, mode),
						autocast_int<scalar, uchar>(val.z, MINVAL_uchar, MAXVAL_uchar, mode));
}
template <int N>
INLINE ucharN<N> __device__  __HOST_V__ autocast_int(scalarN<N> val, IntegerConversionModes mode, ReturnType<ucharN<N> >)
{
	ucharN<N> y;
	FOR_ALL((y.el[i] = autocast_int<scalar, uchar>(val.el[i], MINVAL_uchar, MAXVAL_uchar, mode)));
	return y;
}
INLINE ushort __device__  __HOST_V__ autocast_int(scalar val, IntegerConversionModes mode, ReturnType<ushort>)
{
    return autocast_int<scalar, ushort>(val, MINVAL_uint16, MAXVAL_uint16, mode);
}
INLINE ushort1 __device__  __HOST_V__ autocast_int(scalar1 val, IntegerConversionModes mode, ReturnType<ushort1>)
{
    return make_ushort1(autocast_int<scalar, ushort>(val.x, MINVAL_uint16, MAXVAL_uint16, mode));
}
INLINE ushort2 __device__  __HOST_V__ autocast_int(scalar2 val, IntegerConversionModes mode, ReturnType<ushort2>)
{
    return make_ushort2(autocast_int<scalar, ushort>(val.x, MINVAL_uint16, MAXVAL_uint16, mode),
						autocast_int<scalar, ushort>(val.y, MINVAL_uint16, MAXVAL_uint16, mode));
}
INLINE ushort3 __device__  __HOST_V__ autocast_int(scalar3 val, IntegerConversionModes mode, ReturnType<ushort3>)
{
    return make_ushort3(autocast_int<scalar, ushort>(val.x, MINVAL_uint16, MAXVAL_uint16, mode),
						autocast_int<scalar, ushort>(val.y, MINVAL_uint16, MAXVAL_uint16, mode),
						autocast_int<scalar, ushort>(val.z, MINVAL_uint16, MAXVAL_uint16, mode));
}
template <int N>
INLINE ushortN<N> __device__  __HOST_V__ autocast_int(scalarN<N> val, IntegerConversionModes mode, ReturnType<ushortN<N> >)
{
	ushortN<N> y;
	FOR_ALL((y.el[i] = autocast_int<scalar, ushort>(val.el[i], MINVAL_uint16, MAXVAL_uint16, mode)));
	return y;
}
INLINE uint __device__  __HOST_V__ autocast_int(scalar val, IntegerConversionModes mode, ReturnType<uint>)
{
    return autocast_int<scalar, uint>(val, MINVAL_uint32, MAXVAL_uint32, mode);
}
INLINE uint1 __device__  __HOST_V__ autocast_int(scalar1 val, IntegerConversionModes mode, ReturnType<uint1>)
{
    return make_uint1(autocast_int<scalar, uint>(val.x, MINVAL_uint32, MAXVAL_uint32, mode));
}
INLINE uint2 __device__  __HOST_V__ autocast_int(scalar2 val, IntegerConversionModes mode, ReturnType<uint2>)
{
    return make_uint2(autocast_int<scalar, uint>(val.x, MINVAL_uint32, MAXVAL_uint32, mode),
						autocast_int<scalar, uint>(val.y, MINVAL_uint32, MAXVAL_uint32, mode));
}
INLINE uint3 __device__  __HOST_V__ autocast_int(scalar3 val, IntegerConversionModes mode, ReturnType<uint3>)
{
    return make_uint3(autocast_int<scalar, uint>(val.x, MINVAL_uint32, MAXVAL_uint32, mode),
						autocast_int<scalar, uint>(val.y, MINVAL_uint32, MAXVAL_uint32, mode),
						autocast_int<scalar, uint>(val.z, MINVAL_uint32, MAXVAL_uint32, mode));
}
INLINE uint4 __device__  __HOST_V__ autocast_int(scalar4 val, IntegerConversionModes mode, ReturnType<uint4>)
{
    return make_uint4(autocast_int<scalar, uint>(val.x, MINVAL_uint32, MAXVAL_uint32, mode),
						autocast_int<scalar, uint>(val.y, MINVAL_uint32, MAXVAL_uint32, mode),
						autocast_int<scalar, uint>(val.z, MINVAL_uint32, MAXVAL_uint32, mode),
						autocast_int<scalar, uint>(val.w, MINVAL_uint32, MAXVAL_uint32, mode));
}
template <int N>
INLINE uintN<N> __device__  __HOST_V__ autocast_int(scalarN<N> val, IntegerConversionModes mode, ReturnType<uintN<N> >)
{
	uintN<N> y;
	FOR_ALL((y.el[i] = autocast_int<scalar, uint>(val.el[i], MINVAL_uint32, MAXVAL_uint32, mode)));
	return y;
}
INLINE ulonglong __device__  __HOST_V__ autocast_int(scalar val, IntegerConversionModes mode, ReturnType<ulonglong>)
{
    return autocast_int<scalar, ulonglong>(val, MINVAL_uint64, MAXVAL_uint64, mode);
}
INLINE ulonglong1 __device__  __HOST_V__ autocast_int(scalar1 val, IntegerConversionModes mode, ReturnType<ulonglong1>)
{
    return make_ulonglong1(autocast_int<scalar, ulonglong>(val.x, MINVAL_uint64, MAXVAL_uint64, mode));
}
INLINE ulonglong2 __device__  __HOST_V__ autocast_int(scalar2 val, IntegerConversionModes mode, ReturnType<ulonglong2>)
{
    return make_ulonglong2(autocast_int<scalar, ulonglong>(val.x, MINVAL_uint64, MAXVAL_uint64, mode),
						autocast_int<scalar, ulonglong>(val.y, MINVAL_uint64, MAXVAL_uint64, mode));
}
INLINE ulonglong3 __device__  __HOST_V__ autocast_int(scalar3 val, IntegerConversionModes mode, ReturnType<ulonglong3>)
{
    return make_ulonglong3(autocast_int<scalar, ulonglong>(val.x, MINVAL_uint64, MAXVAL_uint64, mode),
						autocast_int<scalar, ulonglong>(val.y, MINVAL_uint64, MAXVAL_uint64, mode),
						autocast_int<scalar, ulonglong>(val.z, MINVAL_uint64, MAXVAL_uint64, mode));
}
INLINE ulonglong4 __device__  __HOST_V__ autocast_int(scalar4 val, IntegerConversionModes mode, ReturnType<ulonglong4>)
{
    return make_ulonglong4(autocast_int<scalar, ulonglong>(val.x, MINVAL_uint64, MAXVAL_uint64, mode),
						autocast_int<scalar, ulonglong>(val.y, MINVAL_uint64, MAXVAL_uint64, mode),
						autocast_int<scalar, ulonglong>(val.z, MINVAL_uint64, MAXVAL_uint64, mode),
						autocast_int<scalar, ulonglong>(val.w, MINVAL_uint64, MAXVAL_uint64, mode));
}
template <int N>
INLINE ulonglongN<N> __device__  __HOST_V__ autocast_int(scalarN<N> val, IntegerConversionModes mode, ReturnType<ulonglongN<N> >)
{
	ulonglongN<N> y;
	FOR_ALL((y.el[i] = autocast_int<scalar, ulonglong>(val.el[i], MINVAL_uint64, MAXVAL_uint64, mode)));
	return y;
}
INLINE char __device__  __HOST_V__ autocast_int(int val, IntegerConversionModes mode, ReturnType<char>)
{
    return autocast_int<int, char>(val, MINVAL_char, MAXVAL_char, mode);
}
INLINE char1 __device__  __HOST_V__ autocast_int(int1 val, IntegerConversionModes mode, ReturnType<char1>)
{
    return make_char1(autocast_int<int, char>(val.x, MINVAL_char, MAXVAL_char, mode));
}
INLINE char2 __device__  __HOST_V__ autocast_int(int2 val, IntegerConversionModes mode, ReturnType<char2>)
{
    return make_char2(autocast_int<int, char>(val.x, MINVAL_char, MAXVAL_char, mode),
						autocast_int<int, char>(val.y, MINVAL_char, MAXVAL_char, mode));
}
INLINE char3 __device__  __HOST_V__ autocast_int(int3 val, IntegerConversionModes mode, ReturnType<char3>)
{
    return make_char3(autocast_int<int, char>(val.x, MINVAL_char, MAXVAL_char, mode),
						autocast_int<int, char>(val.y, MINVAL_char, MAXVAL_char, mode),
						autocast_int<int, char>(val.z, MINVAL_char, MAXVAL_char, mode));
}
template <int N>
INLINE charN<N> __device__  __HOST_V__ autocast_int(intN<N> val, IntegerConversionModes mode, ReturnType<charN<N> >)
{
	charN<N> y;
	FOR_ALL((y.el[i] = autocast_int<int, char>(val.el[i], MINVAL_char, MAXVAL_char, mode)));
	return y;
}
INLINE short __device__  __HOST_V__ autocast_int(int val, IntegerConversionModes mode, ReturnType<short>)
{
    return autocast_int<int, short>(val, MINVAL_int16, MAXVAL_int16, mode);
}
INLINE short1 __device__  __HOST_V__ autocast_int(int1 val, IntegerConversionModes mode, ReturnType<short1>)
{
    return make_short1(autocast_int<int, short>(val.x, MINVAL_int16, MAXVAL_int16, mode));
}
INLINE short2 __device__  __HOST_V__ autocast_int(int2 val, IntegerConversionModes mode, ReturnType<short2>)
{
    return make_short2(autocast_int<int, short>(val.x, MINVAL_int16, MAXVAL_int16, mode),
						autocast_int<int, short>(val.y, MINVAL_int16, MAXVAL_int16, mode));
}
INLINE short3 __device__  __HOST_V__ autocast_int(int3 val, IntegerConversionModes mode, ReturnType<short3>)
{
    return make_short3(autocast_int<int, short>(val.x, MINVAL_int16, MAXVAL_int16, mode),
						autocast_int<int, short>(val.y, MINVAL_int16, MAXVAL_int16, mode),
						autocast_int<int, short>(val.z, MINVAL_int16, MAXVAL_int16, mode));
}
template <int N>
INLINE shortN<N> __device__  __HOST_V__ autocast_int(intN<N> val, IntegerConversionModes mode, ReturnType<shortN<N> >)
{
	shortN<N> y;
	FOR_ALL((y.el[i] = autocast_int<int, short>(val.el[i], MINVAL_int16, MAXVAL_int16, mode)));
	return y;
}
INLINE int __device__  __HOST_V__ autocast_int(int val, IntegerConversionModes mode, ReturnType<int>)
{
    return autocast_int<int, int>(val, MINVAL_int32, MAXVAL_int32, mode);
}
INLINE int1 __device__  __HOST_V__ autocast_int(int1 val, IntegerConversionModes mode, ReturnType<int1>)
{
    return make_int1(autocast_int<int, int>(val.x, MINVAL_int32, MAXVAL_int32, mode));
}
INLINE int2 __device__  __HOST_V__ autocast_int(int2 val, IntegerConversionModes mode, ReturnType<int2>)
{
    return make_int2(autocast_int<int, int>(val.x, MINVAL_int32, MAXVAL_int32, mode),
						autocast_int<int, int>(val.y, MINVAL_int32, MAXVAL_int32, mode));
}
INLINE int3 __device__  __HOST_V__ autocast_int(int3 val, IntegerConversionModes mode, ReturnType<int3>)
{
    return make_int3(autocast_int<int, int>(val.x, MINVAL_int32, MAXVAL_int32, mode),
						autocast_int<int, int>(val.y, MINVAL_int32, MAXVAL_int32, mode),
						autocast_int<int, int>(val.z, MINVAL_int32, MAXVAL_int32, mode));
}
INLINE int4 __device__  __HOST_V__ autocast_int(int4 val, IntegerConversionModes mode, ReturnType<int4>)
{
    return make_int4(autocast_int<int, int>(val.x, MINVAL_int32, MAXVAL_int32, mode),
						autocast_int<int, int>(val.y, MINVAL_int32, MAXVAL_int32, mode),
						autocast_int<int, int>(val.z, MINVAL_int32, MAXVAL_int32, mode),
						autocast_int<int, int>(val.w, MINVAL_int32, MAXVAL_int32, mode));
}
template <int N>
INLINE intN<N> __device__  __HOST_V__ autocast_int(intN<N> val, IntegerConversionModes mode, ReturnType<intN<N> >)
{
	intN<N> y;
	FOR_ALL((y.el[i] = autocast_int<int, int>(val.el[i], MINVAL_int32, MAXVAL_int32, mode)));
	return y;
}
INLINE longlong __device__  __HOST_V__ autocast_int(int val, IntegerConversionModes mode, ReturnType<longlong>)
{
    return autocast_int<int, longlong>(val, MINVAL_int64, MAXVAL_int64, mode);
}
INLINE longlong1 __device__  __HOST_V__ autocast_int(int1 val, IntegerConversionModes mode, ReturnType<longlong1>)
{
    return make_longlong1(autocast_int<int, longlong>(val.x, MINVAL_int64, MAXVAL_int64, mode));
}
INLINE longlong2 __device__  __HOST_V__ autocast_int(int2 val, IntegerConversionModes mode, ReturnType<longlong2>)
{
    return make_longlong2(autocast_int<int, longlong>(val.x, MINVAL_int64, MAXVAL_int64, mode),
						autocast_int<int, longlong>(val.y, MINVAL_int64, MAXVAL_int64, mode));
}
INLINE longlong3 __device__  __HOST_V__ autocast_int(int3 val, IntegerConversionModes mode, ReturnType<longlong3>)
{
    return make_longlong3(autocast_int<int, longlong>(val.x, MINVAL_int64, MAXVAL_int64, mode),
						autocast_int<int, longlong>(val.y, MINVAL_int64, MAXVAL_int64, mode),
						autocast_int<int, longlong>(val.z, MINVAL_int64, MAXVAL_int64, mode));
}
INLINE longlong4 __device__  __HOST_V__ autocast_int(int4 val, IntegerConversionModes mode, ReturnType<longlong4>)
{
    return make_longlong4(autocast_int<int, longlong>(val.x, MINVAL_int64, MAXVAL_int64, mode),
						autocast_int<int, longlong>(val.y, MINVAL_int64, MAXVAL_int64, mode),
						autocast_int<int, longlong>(val.z, MINVAL_int64, MAXVAL_int64, mode),
						autocast_int<int, longlong>(val.w, MINVAL_int64, MAXVAL_int64, mode));
}
template <int N>
INLINE longlongN<N> __device__  __HOST_V__ autocast_int(intN<N> val, IntegerConversionModes mode, ReturnType<longlongN<N> >)
{
	longlongN<N> y;
	FOR_ALL((y.el[i] = autocast_int<int, longlong>(val.el[i], MINVAL_int64, MAXVAL_int64, mode)));
	return y;
}
INLINE uchar __device__  __HOST_V__ autocast_int(int val, IntegerConversionModes mode, ReturnType<uchar>)
{
    return autocast_int<int, uchar>(val, MINVAL_uchar, MAXVAL_uchar, mode);
}
INLINE uchar1 __device__  __HOST_V__ autocast_int(int1 val, IntegerConversionModes mode, ReturnType<uchar1>)
{
    return make_uchar1(autocast_int<int, uchar>(val.x, MINVAL_uchar, MAXVAL_uchar, mode));
}
INLINE uchar2 __device__  __HOST_V__ autocast_int(int2 val, IntegerConversionModes mode, ReturnType<uchar2>)
{
    return make_uchar2(autocast_int<int, uchar>(val.x, MINVAL_uchar, MAXVAL_uchar, mode),
						autocast_int<int, uchar>(val.y, MINVAL_uchar, MAXVAL_uchar, mode));
}
INLINE uchar3 __device__  __HOST_V__ autocast_int(int3 val, IntegerConversionModes mode, ReturnType<uchar3>)
{
    return make_uchar3(autocast_int<int, uchar>(val.x, MINVAL_uchar, MAXVAL_uchar, mode),
						autocast_int<int, uchar>(val.y, MINVAL_uchar, MAXVAL_uchar, mode),
						autocast_int<int, uchar>(val.z, MINVAL_uchar, MAXVAL_uchar, mode));
}
template <int N>
INLINE ucharN<N> __device__  __HOST_V__ autocast_int(intN<N> val, IntegerConversionModes mode, ReturnType<ucharN<N> >)
{
	ucharN<N> y;
	FOR_ALL((y.el[i] = autocast_int<int, uchar>(val.el[i], MINVAL_uchar, MAXVAL_uchar, mode)));
	return y;
}
INLINE ushort __device__  __HOST_V__ autocast_int(int val, IntegerConversionModes mode, ReturnType<ushort>)
{
    return autocast_int<int, ushort>(val, MINVAL_uint16, MAXVAL_uint16, mode);
}
INLINE ushort1 __device__  __HOST_V__ autocast_int(int1 val, IntegerConversionModes mode, ReturnType<ushort1>)
{
    return make_ushort1(autocast_int<int, ushort>(val.x, MINVAL_uint16, MAXVAL_uint16, mode));
}
INLINE ushort2 __device__  __HOST_V__ autocast_int(int2 val, IntegerConversionModes mode, ReturnType<ushort2>)
{
    return make_ushort2(autocast_int<int, ushort>(val.x, MINVAL_uint16, MAXVAL_uint16, mode),
						autocast_int<int, ushort>(val.y, MINVAL_uint16, MAXVAL_uint16, mode));
}
INLINE ushort3 __device__  __HOST_V__ autocast_int(int3 val, IntegerConversionModes mode, ReturnType<ushort3>)
{
    return make_ushort3(autocast_int<int, ushort>(val.x, MINVAL_uint16, MAXVAL_uint16, mode),
						autocast_int<int, ushort>(val.y, MINVAL_uint16, MAXVAL_uint16, mode),
						autocast_int<int, ushort>(val.z, MINVAL_uint16, MAXVAL_uint16, mode));
}
template <int N>
INLINE ushortN<N> __device__  __HOST_V__ autocast_int(intN<N> val, IntegerConversionModes mode, ReturnType<ushortN<N> >)
{
	ushortN<N> y;
	FOR_ALL((y.el[i] = autocast_int<int, ushort>(val.el[i], MINVAL_uint16, MAXVAL_uint16, mode)));
	return y;
}
INLINE uint __device__  __HOST_V__ autocast_int(int val, IntegerConversionModes mode, ReturnType<uint>)
{
    return autocast_int<int, uint>(val, MINVAL_uint32, MAXVAL_uint32, mode);
}
INLINE uint1 __device__  __HOST_V__ autocast_int(int1 val, IntegerConversionModes mode, ReturnType<uint1>)
{
    return make_uint1(autocast_int<int, uint>(val.x, MINVAL_uint32, MAXVAL_uint32, mode));
}
INLINE uint2 __device__  __HOST_V__ autocast_int(int2 val, IntegerConversionModes mode, ReturnType<uint2>)
{
    return make_uint2(autocast_int<int, uint>(val.x, MINVAL_uint32, MAXVAL_uint32, mode),
						autocast_int<int, uint>(val.y, MINVAL_uint32, MAXVAL_uint32, mode));
}
INLINE uint3 __device__  __HOST_V__ autocast_int(int3 val, IntegerConversionModes mode, ReturnType<uint3>)
{
    return make_uint3(autocast_int<int, uint>(val.x, MINVAL_uint32, MAXVAL_uint32, mode),
						autocast_int<int, uint>(val.y, MINVAL_uint32, MAXVAL_uint32, mode),
						autocast_int<int, uint>(val.z, MINVAL_uint32, MAXVAL_uint32, mode));
}
INLINE uint4 __device__  __HOST_V__ autocast_int(int4 val, IntegerConversionModes mode, ReturnType<uint4>)
{
    return make_uint4(autocast_int<int, uint>(val.x, MINVAL_uint32, MAXVAL_uint32, mode),
						autocast_int<int, uint>(val.y, MINVAL_uint32, MAXVAL_uint32, mode),
						autocast_int<int, uint>(val.z, MINVAL_uint32, MAXVAL_uint32, mode),
						autocast_int<int, uint>(val.w, MINVAL_uint32, MAXVAL_uint32, mode));
}
template <int N>
INLINE uintN<N> __device__  __HOST_V__ autocast_int(intN<N> val, IntegerConversionModes mode, ReturnType<uintN<N> >)
{
	uintN<N> y;
	FOR_ALL((y.el[i] = autocast_int<int, uint>(val.el[i], MINVAL_uint32, MAXVAL_uint32, mode)));
	return y;
}
INLINE ulonglong __device__  __HOST_V__ autocast_int(int val, IntegerConversionModes mode, ReturnType<ulonglong>)
{
    return autocast_int<int, ulonglong>(val, MINVAL_uint64, MAXVAL_uint64, mode);
}
INLINE ulonglong1 __device__  __HOST_V__ autocast_int(int1 val, IntegerConversionModes mode, ReturnType<ulonglong1>)
{
    return make_ulonglong1(autocast_int<int, ulonglong>(val.x, MINVAL_uint64, MAXVAL_uint64, mode));
}
INLINE ulonglong2 __device__  __HOST_V__ autocast_int(int2 val, IntegerConversionModes mode, ReturnType<ulonglong2>)
{
    return make_ulonglong2(autocast_int<int, ulonglong>(val.x, MINVAL_uint64, MAXVAL_uint64, mode),
						autocast_int<int, ulonglong>(val.y, MINVAL_uint64, MAXVAL_uint64, mode));
}
INLINE ulonglong3 __device__  __HOST_V__ autocast_int(int3 val, IntegerConversionModes mode, ReturnType<ulonglong3>)
{
    return make_ulonglong3(autocast_int<int, ulonglong>(val.x, MINVAL_uint64, MAXVAL_uint64, mode),
						autocast_int<int, ulonglong>(val.y, MINVAL_uint64, MAXVAL_uint64, mode),
						autocast_int<int, ulonglong>(val.z, MINVAL_uint64, MAXVAL_uint64, mode));
}
INLINE ulonglong4 __device__  __HOST_V__ autocast_int(int4 val, IntegerConversionModes mode, ReturnType<ulonglong4>)
{
    return make_ulonglong4(autocast_int<int, ulonglong>(val.x, MINVAL_uint64, MAXVAL_uint64, mode),
						autocast_int<int, ulonglong>(val.y, MINVAL_uint64, MAXVAL_uint64, mode),
						autocast_int<int, ulonglong>(val.z, MINVAL_uint64, MAXVAL_uint64, mode),
						autocast_int<int, ulonglong>(val.w, MINVAL_uint64, MAXVAL_uint64, mode));
}
template <int N>
INLINE ulonglongN<N> __device__  __HOST_V__ autocast_int(intN<N> val, IntegerConversionModes mode, ReturnType<ulonglongN<N> >)
{
	ulonglongN<N> y;
	FOR_ALL((y.el[i] = autocast_int<int, ulonglong>(val.el[i], MINVAL_uint64, MAXVAL_uint64, mode)));
	return y;
}
INLINE uint __device__  __HOST_V__ autocast_int(unsigned int val, IntegerConversionModes mode, ReturnType<uint>)
{
    return autocast_int<unsigned int, uint>(val, MINVAL_uint32, MAXVAL_uint32, mode);
}
INLINE ulonglong __device__  __HOST_V__ autocast_int(unsigned int val, IntegerConversionModes mode, ReturnType<ulonglong>)
{
    return autocast_int<unsigned int, ulonglong>(val, MINVAL_uint64, MAXVAL_uint64, mode);
}
INLINE ulonglong __device__  __HOST_V__ autocast_int(unsigned long val, IntegerConversionModes mode, ReturnType<ulonglong>)
{
    return autocast_int<unsigned long, ulonglong>(val, MINVAL_uint64, MAXVAL_uint64, mode);
}
INLINE char __device__ __HOST_V__ autocast (int a, ReturnType<char>)
{
    return (char) a;
}
INLINE char1 __device__ __HOST_V__ autocast (int1 a, ReturnType<char1>)
{
    return make_char1(autocast(a.x, ReturnType<char>()));
}
INLINE char2 __device__ __HOST_V__ autocast(int2 a, ReturnType<char2>)
{
    return make_char2(autocast(a.x, ReturnType<char>()), autocast(a.y, ReturnType<char>()));
}
INLINE char3 __device__ __HOST_V__ autocast(int3 a, ReturnType<char3>)
{
    return make_char3(autocast(a.x, ReturnType<char>()), autocast(a.y, ReturnType<char>()), autocast(a.z, ReturnType<char>()));
}
INLINE char4 __device__ __HOST_V__ autocast(int4 a, ReturnType<char4>)
{
    return make_char4(autocast(a.x, ReturnType<char>()), autocast(a.y, ReturnType<char>()), autocast(a.z, ReturnType<char>()), autocast(a.w, ReturnType<char>()));
}
template <int N>
INLINE charN<N> __device__ __HOST_V__ autocast(intN<N> a, ReturnType<charN<N> >)
{
    charN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<char>()));
    return y;
}
INLINE short __device__ __HOST_V__ autocast (int a, ReturnType<short>)
{
    return (short) a;
}
INLINE short1 __device__ __HOST_V__ autocast (int1 a, ReturnType<short1>)
{
    return make_short1(autocast(a.x, ReturnType<short>()));
}
INLINE short2 __device__ __HOST_V__ autocast(int2 a, ReturnType<short2>)
{
    return make_short2(autocast(a.x, ReturnType<short>()), autocast(a.y, ReturnType<short>()));
}
INLINE short3 __device__ __HOST_V__ autocast(int3 a, ReturnType<short3>)
{
    return make_short3(autocast(a.x, ReturnType<short>()), autocast(a.y, ReturnType<short>()), autocast(a.z, ReturnType<short>()));
}
INLINE short4 __device__ __HOST_V__ autocast(int4 a, ReturnType<short4>)
{
    return make_short4(autocast(a.x, ReturnType<short>()), autocast(a.y, ReturnType<short>()), autocast(a.z, ReturnType<short>()), autocast(a.w, ReturnType<short>()));
}
template <int N>
INLINE shortN<N> __device__ __HOST_V__ autocast(intN<N> a, ReturnType<shortN<N> >)
{
    shortN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<short>()));
    return y;
}
INLINE longlong __device__ __HOST_V__ autocast (int a, ReturnType<longlong>)
{
    return (longlong) a;
}
INLINE longlong1 __device__ __HOST_V__ autocast (int1 a, ReturnType<longlong1>)
{
    return make_longlong1(autocast(a.x, ReturnType<longlong>()));
}
INLINE longlong2 __device__ __HOST_V__ autocast(int2 a, ReturnType<longlong2>)
{
    return make_longlong2(autocast(a.x, ReturnType<longlong>()), autocast(a.y, ReturnType<longlong>()));
}
INLINE longlong3 __device__ __HOST_V__ autocast(int3 a, ReturnType<longlong3>)
{
    return make_longlong3(autocast(a.x, ReturnType<longlong>()), autocast(a.y, ReturnType<longlong>()), autocast(a.z, ReturnType<longlong>()));
}
INLINE longlong4 __device__ __HOST_V__ autocast(int4 a, ReturnType<longlong4>)
{
    return make_longlong4(autocast(a.x, ReturnType<longlong>()), autocast(a.y, ReturnType<longlong>()), autocast(a.z, ReturnType<longlong>()), autocast(a.w, ReturnType<longlong>()));
}
template <int N>
INLINE longlongN<N> __device__ __HOST_V__ autocast(intN<N> a, ReturnType<longlongN<N> >)
{
    longlongN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<longlong>()));
    return y;
}
INLINE uchar __device__ __HOST_V__ autocast (int a, ReturnType<uchar>)
{
    return (uchar) a;
}
INLINE uchar1 __device__ __HOST_V__ autocast (int1 a, ReturnType<uchar1>)
{
    return make_uchar1(autocast(a.x, ReturnType<uchar>()));
}
INLINE uchar2 __device__ __HOST_V__ autocast(int2 a, ReturnType<uchar2>)
{
    return make_uchar2(autocast(a.x, ReturnType<uchar>()), autocast(a.y, ReturnType<uchar>()));
}
INLINE uchar3 __device__ __HOST_V__ autocast(int3 a, ReturnType<uchar3>)
{
    return make_uchar3(autocast(a.x, ReturnType<uchar>()), autocast(a.y, ReturnType<uchar>()), autocast(a.z, ReturnType<uchar>()));
}
INLINE uchar4 __device__ __HOST_V__ autocast(int4 a, ReturnType<uchar4>)
{
    return make_uchar4(autocast(a.x, ReturnType<uchar>()), autocast(a.y, ReturnType<uchar>()), autocast(a.z, ReturnType<uchar>()), autocast(a.w, ReturnType<uchar>()));
}
template <int N>
INLINE ucharN<N> __device__ __HOST_V__ autocast(intN<N> a, ReturnType<ucharN<N> >)
{
    ucharN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<uchar>()));
    return y;
}
INLINE ushort __device__ __HOST_V__ autocast (int a, ReturnType<ushort>)
{
    return (ushort) a;
}
INLINE ushort1 __device__ __HOST_V__ autocast (int1 a, ReturnType<ushort1>)
{
    return make_ushort1(autocast(a.x, ReturnType<ushort>()));
}
INLINE ushort2 __device__ __HOST_V__ autocast(int2 a, ReturnType<ushort2>)
{
    return make_ushort2(autocast(a.x, ReturnType<ushort>()), autocast(a.y, ReturnType<ushort>()));
}
INLINE ushort3 __device__ __HOST_V__ autocast(int3 a, ReturnType<ushort3>)
{
    return make_ushort3(autocast(a.x, ReturnType<ushort>()), autocast(a.y, ReturnType<ushort>()), autocast(a.z, ReturnType<ushort>()));
}
INLINE ushort4 __device__ __HOST_V__ autocast(int4 a, ReturnType<ushort4>)
{
    return make_ushort4(autocast(a.x, ReturnType<ushort>()), autocast(a.y, ReturnType<ushort>()), autocast(a.z, ReturnType<ushort>()), autocast(a.w, ReturnType<ushort>()));
}
template <int N>
INLINE ushortN<N> __device__ __HOST_V__ autocast(intN<N> a, ReturnType<ushortN<N> >)
{
    ushortN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<ushort>()));
    return y;
}
INLINE uint __device__ __HOST_V__ autocast (int a, ReturnType<uint>)
{
    return (uint) a;
}
INLINE uint1 __device__ __HOST_V__ autocast (int1 a, ReturnType<uint1>)
{
    return make_uint1(autocast(a.x, ReturnType<uint>()));
}
INLINE uint2 __device__ __HOST_V__ autocast(int2 a, ReturnType<uint2>)
{
    return make_uint2(autocast(a.x, ReturnType<uint>()), autocast(a.y, ReturnType<uint>()));
}
INLINE uint3 __device__ __HOST_V__ autocast(int3 a, ReturnType<uint3>)
{
    return make_uint3(autocast(a.x, ReturnType<uint>()), autocast(a.y, ReturnType<uint>()), autocast(a.z, ReturnType<uint>()));
}
INLINE uint4 __device__ __HOST_V__ autocast(int4 a, ReturnType<uint4>)
{
    return make_uint4(autocast(a.x, ReturnType<uint>()), autocast(a.y, ReturnType<uint>()), autocast(a.z, ReturnType<uint>()), autocast(a.w, ReturnType<uint>()));
}
template <int N>
INLINE uintN<N> __device__ __HOST_V__ autocast(intN<N> a, ReturnType<uintN<N> >)
{
    uintN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<uint>()));
    return y;
}
INLINE ulonglong __device__ __HOST_V__ autocast (int a, ReturnType<ulonglong>)
{
    return (ulonglong) a;
}
INLINE ulonglong1 __device__ __HOST_V__ autocast (int1 a, ReturnType<ulonglong1>)
{
    return make_ulonglong1(autocast(a.x, ReturnType<ulonglong>()));
}
INLINE ulonglong2 __device__ __HOST_V__ autocast(int2 a, ReturnType<ulonglong2>)
{
    return make_ulonglong2(autocast(a.x, ReturnType<ulonglong>()), autocast(a.y, ReturnType<ulonglong>()));
}
INLINE ulonglong3 __device__ __HOST_V__ autocast(int3 a, ReturnType<ulonglong3>)
{
    return make_ulonglong3(autocast(a.x, ReturnType<ulonglong>()), autocast(a.y, ReturnType<ulonglong>()), autocast(a.z, ReturnType<ulonglong>()));
}
INLINE ulonglong4 __device__ __HOST_V__ autocast(int4 a, ReturnType<ulonglong4>)
{
    return make_ulonglong4(autocast(a.x, ReturnType<ulonglong>()), autocast(a.y, ReturnType<ulonglong>()), autocast(a.z, ReturnType<ulonglong>()), autocast(a.w, ReturnType<ulonglong>()));
}
template <int N>
INLINE ulonglongN<N> __device__ __HOST_V__ autocast(intN<N> a, ReturnType<ulonglongN<N> >)
{
    ulonglongN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<ulonglong>()));
    return y;
}
INLINE int __device__ __HOST_V__ autocast (char a, ReturnType<int>)
{
    return (int) a;
}
INLINE int1 __device__ __HOST_V__ autocast (char1 a, ReturnType<int1>)
{
    return make_int1(autocast(a.x, ReturnType<int>()));
}
INLINE int2 __device__ __HOST_V__ autocast(char2 a, ReturnType<int2>)
{
    return make_int2(autocast(a.x, ReturnType<int>()), autocast(a.y, ReturnType<int>()));
}
INLINE int3 __device__ __HOST_V__ autocast(char3 a, ReturnType<int3>)
{
    return make_int3(autocast(a.x, ReturnType<int>()), autocast(a.y, ReturnType<int>()), autocast(a.z, ReturnType<int>()));
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ autocast(charN<N> a, ReturnType<intN<N> >)
{
    intN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<int>()));
    return y;
}
INLINE int __device__ __HOST_V__ autocast (short a, ReturnType<int>)
{
    return (int) a;
}
INLINE int1 __device__ __HOST_V__ autocast (short1 a, ReturnType<int1>)
{
    return make_int1(autocast(a.x, ReturnType<int>()));
}
INLINE int2 __device__ __HOST_V__ autocast(short2 a, ReturnType<int2>)
{
    return make_int2(autocast(a.x, ReturnType<int>()), autocast(a.y, ReturnType<int>()));
}
INLINE int3 __device__ __HOST_V__ autocast(short3 a, ReturnType<int3>)
{
    return make_int3(autocast(a.x, ReturnType<int>()), autocast(a.y, ReturnType<int>()), autocast(a.z, ReturnType<int>()));
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ autocast(shortN<N> a, ReturnType<intN<N> >)
{
    intN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<int>()));
    return y;
}
INLINE int __device__ __HOST_V__ autocast (longlong a, ReturnType<int>)
{
    return (int) a;
}
INLINE int1 __device__ __HOST_V__ autocast (longlong1 a, ReturnType<int1>)
{
    return make_int1(autocast(a.x, ReturnType<int>()));
}
INLINE int2 __device__ __HOST_V__ autocast(longlong2 a, ReturnType<int2>)
{
    return make_int2(autocast(a.x, ReturnType<int>()), autocast(a.y, ReturnType<int>()));
}
INLINE int3 __device__ __HOST_V__ autocast(longlong3 a, ReturnType<int3>)
{
    return make_int3(autocast(a.x, ReturnType<int>()), autocast(a.y, ReturnType<int>()), autocast(a.z, ReturnType<int>()));
}
INLINE int4 __device__ __HOST_V__ autocast(longlong4 a, ReturnType<int4>)
{
    return make_int4(autocast(a.x, ReturnType<int>()), autocast(a.y, ReturnType<int>()), autocast(a.z, ReturnType<int>()), autocast(a.w, ReturnType<int>()));
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ autocast(longlongN<N> a, ReturnType<intN<N> >)
{
    intN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<int>()));
    return y;
}
INLINE int __device__ __HOST_V__ autocast (uchar a, ReturnType<int>)
{
    return (int) a;
}
INLINE int1 __device__ __HOST_V__ autocast (uchar1 a, ReturnType<int1>)
{
    return make_int1(autocast(a.x, ReturnType<int>()));
}
INLINE int2 __device__ __HOST_V__ autocast(uchar2 a, ReturnType<int2>)
{
    return make_int2(autocast(a.x, ReturnType<int>()), autocast(a.y, ReturnType<int>()));
}
INLINE int3 __device__ __HOST_V__ autocast(uchar3 a, ReturnType<int3>)
{
    return make_int3(autocast(a.x, ReturnType<int>()), autocast(a.y, ReturnType<int>()), autocast(a.z, ReturnType<int>()));
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ autocast(ucharN<N> a, ReturnType<intN<N> >)
{
    intN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<int>()));
    return y;
}
INLINE int __device__ __HOST_V__ autocast (ushort a, ReturnType<int>)
{
    return (int) a;
}
INLINE int1 __device__ __HOST_V__ autocast (ushort1 a, ReturnType<int1>)
{
    return make_int1(autocast(a.x, ReturnType<int>()));
}
INLINE int2 __device__ __HOST_V__ autocast(ushort2 a, ReturnType<int2>)
{
    return make_int2(autocast(a.x, ReturnType<int>()), autocast(a.y, ReturnType<int>()));
}
INLINE int3 __device__ __HOST_V__ autocast(ushort3 a, ReturnType<int3>)
{
    return make_int3(autocast(a.x, ReturnType<int>()), autocast(a.y, ReturnType<int>()), autocast(a.z, ReturnType<int>()));
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ autocast(ushortN<N> a, ReturnType<intN<N> >)
{
    intN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<int>()));
    return y;
}
INLINE int __device__ __HOST_V__ autocast (uint a, ReturnType<int>)
{
    return (int) a;
}
INLINE int1 __device__ __HOST_V__ autocast (uint1 a, ReturnType<int1>)
{
    return make_int1(autocast(a.x, ReturnType<int>()));
}
INLINE int2 __device__ __HOST_V__ autocast(uint2 a, ReturnType<int2>)
{
    return make_int2(autocast(a.x, ReturnType<int>()), autocast(a.y, ReturnType<int>()));
}
INLINE int3 __device__ __HOST_V__ autocast(uint3 a, ReturnType<int3>)
{
    return make_int3(autocast(a.x, ReturnType<int>()), autocast(a.y, ReturnType<int>()), autocast(a.z, ReturnType<int>()));
}
INLINE int4 __device__ __HOST_V__ autocast(uint4 a, ReturnType<int4>)
{
    return make_int4(autocast(a.x, ReturnType<int>()), autocast(a.y, ReturnType<int>()), autocast(a.z, ReturnType<int>()), autocast(a.w, ReturnType<int>()));
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ autocast(uintN<N> a, ReturnType<intN<N> >)
{
    intN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<int>()));
    return y;
}
INLINE int __device__ __HOST_V__ autocast (ulonglong a, ReturnType<int>)
{
    return (int) a;
}
INLINE int1 __device__ __HOST_V__ autocast (ulonglong1 a, ReturnType<int1>)
{
    return make_int1(autocast(a.x, ReturnType<int>()));
}
INLINE int2 __device__ __HOST_V__ autocast(ulonglong2 a, ReturnType<int2>)
{
    return make_int2(autocast(a.x, ReturnType<int>()), autocast(a.y, ReturnType<int>()));
}
INLINE int3 __device__ __HOST_V__ autocast(ulonglong3 a, ReturnType<int3>)
{
    return make_int3(autocast(a.x, ReturnType<int>()), autocast(a.y, ReturnType<int>()), autocast(a.z, ReturnType<int>()));
}
INLINE int4 __device__ __HOST_V__ autocast(ulonglong4 a, ReturnType<int4>)
{
    return make_int4(autocast(a.x, ReturnType<int>()), autocast(a.y, ReturnType<int>()), autocast(a.z, ReturnType<int>()), autocast(a.w, ReturnType<int>()));
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ autocast(ulonglongN<N> a, ReturnType<intN<N> >)
{
    intN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<int>()));
    return y;
}
INLINE ulonglong __device__  __HOST_V__ autocast_int(unsigned long long val, IntegerConversionModes mode, ReturnType<ulonglong>)
{
    return autocast_int<unsigned long long, ulonglong>(val, MINVAL_uint64, MAXVAL_uint64, mode);
}
INLINE longlong __device__  __HOST_V__ autocast_int(unsigned long long val, IntegerConversionModes mode, ReturnType<longlong>)
{
    return autocast_int<unsigned long long, longlong>(val, MINVAL_int64, MAXVAL_int64, mode);
}
INLINE char __device__ __HOST_V__ autocast (scalar a, ReturnType<char>)
{
    return (char) a;
}
INLINE char1 __device__ __HOST_V__ autocast (scalar1 a, ReturnType<char1>)
{
    return make_char1(autocast(a.x, ReturnType<char>()));
}
INLINE char2 __device__ __HOST_V__ autocast(scalar2 a, ReturnType<char2>)
{
    return make_char2(autocast(a.x, ReturnType<char>()), autocast(a.y, ReturnType<char>()));
}
INLINE char3 __device__ __HOST_V__ autocast(scalar3 a, ReturnType<char3>)
{
    return make_char3(autocast(a.x, ReturnType<char>()), autocast(a.y, ReturnType<char>()), autocast(a.z, ReturnType<char>()));
}
INLINE char4 __device__ __HOST_V__ autocast(scalar4 a, ReturnType<char4>)
{
    return make_char4(autocast(a.x, ReturnType<char>()), autocast(a.y, ReturnType<char>()), autocast(a.z, ReturnType<char>()), autocast(a.w, ReturnType<char>()));
}
template <int N>
INLINE charN<N> __device__ __HOST_V__ autocast(scalarN<N> a, ReturnType<charN<N> >)
{
    charN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<char>()));
    return y;
}
INLINE short __device__ __HOST_V__ autocast (scalar a, ReturnType<short>)
{
    return (short) a;
}
INLINE short1 __device__ __HOST_V__ autocast (scalar1 a, ReturnType<short1>)
{
    return make_short1(autocast(a.x, ReturnType<short>()));
}
INLINE short2 __device__ __HOST_V__ autocast(scalar2 a, ReturnType<short2>)
{
    return make_short2(autocast(a.x, ReturnType<short>()), autocast(a.y, ReturnType<short>()));
}
INLINE short3 __device__ __HOST_V__ autocast(scalar3 a, ReturnType<short3>)
{
    return make_short3(autocast(a.x, ReturnType<short>()), autocast(a.y, ReturnType<short>()), autocast(a.z, ReturnType<short>()));
}
INLINE short4 __device__ __HOST_V__ autocast(scalar4 a, ReturnType<short4>)
{
    return make_short4(autocast(a.x, ReturnType<short>()), autocast(a.y, ReturnType<short>()), autocast(a.z, ReturnType<short>()), autocast(a.w, ReturnType<short>()));
}
template <int N>
INLINE shortN<N> __device__ __HOST_V__ autocast(scalarN<N> a, ReturnType<shortN<N> >)
{
    shortN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<short>()));
    return y;
}
INLINE longlong __device__ __HOST_V__ autocast (scalar a, ReturnType<longlong>)
{
    return (longlong) a;
}
INLINE longlong1 __device__ __HOST_V__ autocast (scalar1 a, ReturnType<longlong1>)
{
    return make_longlong1(autocast(a.x, ReturnType<longlong>()));
}
INLINE longlong2 __device__ __HOST_V__ autocast(scalar2 a, ReturnType<longlong2>)
{
    return make_longlong2(autocast(a.x, ReturnType<longlong>()), autocast(a.y, ReturnType<longlong>()));
}
INLINE longlong3 __device__ __HOST_V__ autocast(scalar3 a, ReturnType<longlong3>)
{
    return make_longlong3(autocast(a.x, ReturnType<longlong>()), autocast(a.y, ReturnType<longlong>()), autocast(a.z, ReturnType<longlong>()));
}
INLINE longlong4 __device__ __HOST_V__ autocast(scalar4 a, ReturnType<longlong4>)
{
    return make_longlong4(autocast(a.x, ReturnType<longlong>()), autocast(a.y, ReturnType<longlong>()), autocast(a.z, ReturnType<longlong>()), autocast(a.w, ReturnType<longlong>()));
}
template <int N>
INLINE longlongN<N> __device__ __HOST_V__ autocast(scalarN<N> a, ReturnType<longlongN<N> >)
{
    longlongN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<longlong>()));
    return y;
}
INLINE uchar __device__ __HOST_V__ autocast (scalar a, ReturnType<uchar>)
{
    return (uchar) a;
}
INLINE uchar1 __device__ __HOST_V__ autocast (scalar1 a, ReturnType<uchar1>)
{
    return make_uchar1(autocast(a.x, ReturnType<uchar>()));
}
INLINE uchar2 __device__ __HOST_V__ autocast(scalar2 a, ReturnType<uchar2>)
{
    return make_uchar2(autocast(a.x, ReturnType<uchar>()), autocast(a.y, ReturnType<uchar>()));
}
INLINE uchar3 __device__ __HOST_V__ autocast(scalar3 a, ReturnType<uchar3>)
{
    return make_uchar3(autocast(a.x, ReturnType<uchar>()), autocast(a.y, ReturnType<uchar>()), autocast(a.z, ReturnType<uchar>()));
}
INLINE uchar4 __device__ __HOST_V__ autocast(scalar4 a, ReturnType<uchar4>)
{
    return make_uchar4(autocast(a.x, ReturnType<uchar>()), autocast(a.y, ReturnType<uchar>()), autocast(a.z, ReturnType<uchar>()), autocast(a.w, ReturnType<uchar>()));
}
template <int N>
INLINE ucharN<N> __device__ __HOST_V__ autocast(scalarN<N> a, ReturnType<ucharN<N> >)
{
    ucharN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<uchar>()));
    return y;
}
INLINE ushort __device__ __HOST_V__ autocast (scalar a, ReturnType<ushort>)
{
    return (ushort) a;
}
INLINE ushort1 __device__ __HOST_V__ autocast (scalar1 a, ReturnType<ushort1>)
{
    return make_ushort1(autocast(a.x, ReturnType<ushort>()));
}
INLINE ushort2 __device__ __HOST_V__ autocast(scalar2 a, ReturnType<ushort2>)
{
    return make_ushort2(autocast(a.x, ReturnType<ushort>()), autocast(a.y, ReturnType<ushort>()));
}
INLINE ushort3 __device__ __HOST_V__ autocast(scalar3 a, ReturnType<ushort3>)
{
    return make_ushort3(autocast(a.x, ReturnType<ushort>()), autocast(a.y, ReturnType<ushort>()), autocast(a.z, ReturnType<ushort>()));
}
INLINE ushort4 __device__ __HOST_V__ autocast(scalar4 a, ReturnType<ushort4>)
{
    return make_ushort4(autocast(a.x, ReturnType<ushort>()), autocast(a.y, ReturnType<ushort>()), autocast(a.z, ReturnType<ushort>()), autocast(a.w, ReturnType<ushort>()));
}
template <int N>
INLINE ushortN<N> __device__ __HOST_V__ autocast(scalarN<N> a, ReturnType<ushortN<N> >)
{
    ushortN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<ushort>()));
    return y;
}
INLINE uint __device__ __HOST_V__ autocast (scalar a, ReturnType<uint>)
{
    return (uint) a;
}
INLINE uint1 __device__ __HOST_V__ autocast (scalar1 a, ReturnType<uint1>)
{
    return make_uint1(autocast(a.x, ReturnType<uint>()));
}
INLINE uint2 __device__ __HOST_V__ autocast(scalar2 a, ReturnType<uint2>)
{
    return make_uint2(autocast(a.x, ReturnType<uint>()), autocast(a.y, ReturnType<uint>()));
}
INLINE uint3 __device__ __HOST_V__ autocast(scalar3 a, ReturnType<uint3>)
{
    return make_uint3(autocast(a.x, ReturnType<uint>()), autocast(a.y, ReturnType<uint>()), autocast(a.z, ReturnType<uint>()));
}
INLINE uint4 __device__ __HOST_V__ autocast(scalar4 a, ReturnType<uint4>)
{
    return make_uint4(autocast(a.x, ReturnType<uint>()), autocast(a.y, ReturnType<uint>()), autocast(a.z, ReturnType<uint>()), autocast(a.w, ReturnType<uint>()));
}
template <int N>
INLINE uintN<N> __device__ __HOST_V__ autocast(scalarN<N> a, ReturnType<uintN<N> >)
{
    uintN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<uint>()));
    return y;
}
INLINE ulonglong __device__ __HOST_V__ autocast (scalar a, ReturnType<ulonglong>)
{
    return (ulonglong) a;
}
INLINE ulonglong1 __device__ __HOST_V__ autocast (scalar1 a, ReturnType<ulonglong1>)
{
    return make_ulonglong1(autocast(a.x, ReturnType<ulonglong>()));
}
INLINE ulonglong2 __device__ __HOST_V__ autocast(scalar2 a, ReturnType<ulonglong2>)
{
    return make_ulonglong2(autocast(a.x, ReturnType<ulonglong>()), autocast(a.y, ReturnType<ulonglong>()));
}
INLINE ulonglong3 __device__ __HOST_V__ autocast(scalar3 a, ReturnType<ulonglong3>)
{
    return make_ulonglong3(autocast(a.x, ReturnType<ulonglong>()), autocast(a.y, ReturnType<ulonglong>()), autocast(a.z, ReturnType<ulonglong>()));
}
INLINE ulonglong4 __device__ __HOST_V__ autocast(scalar4 a, ReturnType<ulonglong4>)
{
    return make_ulonglong4(autocast(a.x, ReturnType<ulonglong>()), autocast(a.y, ReturnType<ulonglong>()), autocast(a.z, ReturnType<ulonglong>()), autocast(a.w, ReturnType<ulonglong>()));
}
template <int N>
INLINE ulonglongN<N> __device__ __HOST_V__ autocast(scalarN<N> a, ReturnType<ulonglongN<N> >)
{
    ulonglongN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<ulonglong>()));
    return y;
}
INLINE scalar __device__ __HOST_V__ autocast (char a, ReturnType<scalar>)
{
    return (scalar) a;
}
INLINE scalar1 __device__ __HOST_V__ autocast (char1 a, ReturnType<scalar1>)
{
    return make_scalar1(autocast(a.x, ReturnType<scalar>()));
}
INLINE scalar2 __device__ __HOST_V__ autocast(char2 a, ReturnType<scalar2>)
{
    return make_scalar2(autocast(a.x, ReturnType<scalar>()), autocast(a.y, ReturnType<scalar>()));
}
INLINE scalar3 __device__ __HOST_V__ autocast(char3 a, ReturnType<scalar3>)
{
    return make_scalar3(autocast(a.x, ReturnType<scalar>()), autocast(a.y, ReturnType<scalar>()), autocast(a.z, ReturnType<scalar>()));
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ autocast(charN<N> a, ReturnType<scalarN<N> >)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<scalar>()));
    return y;
}
INLINE scalar __device__ __HOST_V__ autocast (short a, ReturnType<scalar>)
{
    return (scalar) a;
}
INLINE scalar1 __device__ __HOST_V__ autocast (short1 a, ReturnType<scalar1>)
{
    return make_scalar1(autocast(a.x, ReturnType<scalar>()));
}
INLINE scalar2 __device__ __HOST_V__ autocast(short2 a, ReturnType<scalar2>)
{
    return make_scalar2(autocast(a.x, ReturnType<scalar>()), autocast(a.y, ReturnType<scalar>()));
}
INLINE scalar3 __device__ __HOST_V__ autocast(short3 a, ReturnType<scalar3>)
{
    return make_scalar3(autocast(a.x, ReturnType<scalar>()), autocast(a.y, ReturnType<scalar>()), autocast(a.z, ReturnType<scalar>()));
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ autocast(shortN<N> a, ReturnType<scalarN<N> >)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<scalar>()));
    return y;
}
INLINE scalar __device__ __HOST_V__ autocast (longlong a, ReturnType<scalar>)
{
    return (scalar) a;
}
INLINE scalar1 __device__ __HOST_V__ autocast (longlong1 a, ReturnType<scalar1>)
{
    return make_scalar1(autocast(a.x, ReturnType<scalar>()));
}
INLINE scalar2 __device__ __HOST_V__ autocast(longlong2 a, ReturnType<scalar2>)
{
    return make_scalar2(autocast(a.x, ReturnType<scalar>()), autocast(a.y, ReturnType<scalar>()));
}
INLINE scalar3 __device__ __HOST_V__ autocast(longlong3 a, ReturnType<scalar3>)
{
    return make_scalar3(autocast(a.x, ReturnType<scalar>()), autocast(a.y, ReturnType<scalar>()), autocast(a.z, ReturnType<scalar>()));
}
INLINE scalar4 __device__ __HOST_V__ autocast(longlong4 a, ReturnType<scalar4>)
{
    return make_scalar4(autocast(a.x, ReturnType<scalar>()), autocast(a.y, ReturnType<scalar>()), autocast(a.z, ReturnType<scalar>()), autocast(a.w, ReturnType<scalar>()));
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ autocast(longlongN<N> a, ReturnType<scalarN<N> >)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<scalar>()));
    return y;
}
INLINE scalar __device__ __HOST_V__ autocast (uchar a, ReturnType<scalar>)
{
    return (scalar) a;
}
INLINE scalar1 __device__ __HOST_V__ autocast (uchar1 a, ReturnType<scalar1>)
{
    return make_scalar1(autocast(a.x, ReturnType<scalar>()));
}
INLINE scalar2 __device__ __HOST_V__ autocast(uchar2 a, ReturnType<scalar2>)
{
    return make_scalar2(autocast(a.x, ReturnType<scalar>()), autocast(a.y, ReturnType<scalar>()));
}
INLINE scalar3 __device__ __HOST_V__ autocast(uchar3 a, ReturnType<scalar3>)
{
    return make_scalar3(autocast(a.x, ReturnType<scalar>()), autocast(a.y, ReturnType<scalar>()), autocast(a.z, ReturnType<scalar>()));
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ autocast(ucharN<N> a, ReturnType<scalarN<N> >)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<scalar>()));
    return y;
}
INLINE scalar __device__ __HOST_V__ autocast (ushort a, ReturnType<scalar>)
{
    return (scalar) a;
}
INLINE scalar1 __device__ __HOST_V__ autocast (ushort1 a, ReturnType<scalar1>)
{
    return make_scalar1(autocast(a.x, ReturnType<scalar>()));
}
INLINE scalar2 __device__ __HOST_V__ autocast(ushort2 a, ReturnType<scalar2>)
{
    return make_scalar2(autocast(a.x, ReturnType<scalar>()), autocast(a.y, ReturnType<scalar>()));
}
INLINE scalar3 __device__ __HOST_V__ autocast(ushort3 a, ReturnType<scalar3>)
{
    return make_scalar3(autocast(a.x, ReturnType<scalar>()), autocast(a.y, ReturnType<scalar>()), autocast(a.z, ReturnType<scalar>()));
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ autocast(ushortN<N> a, ReturnType<scalarN<N> >)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<scalar>()));
    return y;
}
INLINE scalar __device__ __HOST_V__ autocast (uint a, ReturnType<scalar>)
{
    return (scalar) a;
}
INLINE scalar1 __device__ __HOST_V__ autocast (uint1 a, ReturnType<scalar1>)
{
    return make_scalar1(autocast(a.x, ReturnType<scalar>()));
}
INLINE scalar2 __device__ __HOST_V__ autocast(uint2 a, ReturnType<scalar2>)
{
    return make_scalar2(autocast(a.x, ReturnType<scalar>()), autocast(a.y, ReturnType<scalar>()));
}
INLINE scalar3 __device__ __HOST_V__ autocast(uint3 a, ReturnType<scalar3>)
{
    return make_scalar3(autocast(a.x, ReturnType<scalar>()), autocast(a.y, ReturnType<scalar>()), autocast(a.z, ReturnType<scalar>()));
}
INLINE scalar4 __device__ __HOST_V__ autocast(uint4 a, ReturnType<scalar4>)
{
    return make_scalar4(autocast(a.x, ReturnType<scalar>()), autocast(a.y, ReturnType<scalar>()), autocast(a.z, ReturnType<scalar>()), autocast(a.w, ReturnType<scalar>()));
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ autocast(uintN<N> a, ReturnType<scalarN<N> >)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<scalar>()));
    return y;
}
INLINE scalar __device__ __HOST_V__ autocast (ulonglong a, ReturnType<scalar>)
{
    return (scalar) a;
}
INLINE scalar1 __device__ __HOST_V__ autocast (ulonglong1 a, ReturnType<scalar1>)
{
    return make_scalar1(autocast(a.x, ReturnType<scalar>()));
}
INLINE scalar2 __device__ __HOST_V__ autocast(ulonglong2 a, ReturnType<scalar2>)
{
    return make_scalar2(autocast(a.x, ReturnType<scalar>()), autocast(a.y, ReturnType<scalar>()));
}
INLINE scalar3 __device__ __HOST_V__ autocast(ulonglong3 a, ReturnType<scalar3>)
{
    return make_scalar3(autocast(a.x, ReturnType<scalar>()), autocast(a.y, ReturnType<scalar>()), autocast(a.z, ReturnType<scalar>()));
}
INLINE scalar4 __device__ __HOST_V__ autocast(ulonglong4 a, ReturnType<scalar4>)
{
    return make_scalar4(autocast(a.x, ReturnType<scalar>()), autocast(a.y, ReturnType<scalar>()), autocast(a.z, ReturnType<scalar>()), autocast(a.w, ReturnType<scalar>()));
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ autocast(ulonglongN<N> a, ReturnType<scalarN<N> >)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = autocast(a.el[i], ReturnType<scalar>()));
    return y;
}
}	// namespace quasar
INLINE int1 __device__ __HOST_V__ _not_(int1 a)
{
	return make_int1(_not_(a.x));
}
INLINE int2 __device__ __HOST_V__ _not_(int2 a)
{
	return make_int2(_not_(a.x), _not_(a.y));
}
INLINE int3 __device__ __HOST_V__ _not_(int3 a)
{
    return make_int3(_not_(a.x), _not_(a.y), _not_(a.z));
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ _not_(intN<N> a)
{
    intN<N> y;
    FOR_ALL(y.el[i] = _not_(a.el[i]));
    return y;
}
INLINE int1 __device__ __HOST_V__ lognot(int1 a)
{
	return make_int1(lognot(a.x));
}
INLINE int2 __device__ __HOST_V__ lognot(int2 a)
{
	return make_int2(lognot(a.x), lognot(a.y));
}
INLINE int3 __device__ __HOST_V__ lognot(int3 a)
{
    return make_int3(lognot(a.x), lognot(a.y), lognot(a.z));
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ lognot(intN<N> a)
{
    intN<N> y;
    FOR_ALL(y.el[i] = lognot(a.el[i]));
    return y;
}
INLINE int1 __device__ __HOST_V__ abs(int1 a)
{
	return make_int1(abs(a.x));
}
INLINE int2 __device__ __HOST_V__ abs(int2 a)
{
	return make_int2(abs(a.x), abs(a.y));
}
INLINE int3 __device__ __HOST_V__ abs(int3 a)
{
    return make_int3(abs(a.x), abs(a.y), abs(a.z));
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ abs(intN<N> a)
{
    intN<N> y;
    FOR_ALL(y.el[i] = abs(a.el[i]));
    return y;
}
INLINE int1 __device__ __HOST_V__ sign(int1 a)
{
	return make_int1(sign(a.x));
}
INLINE int2 __device__ __HOST_V__ sign(int2 a)
{
	return make_int2(sign(a.x), sign(a.y));
}
INLINE int3 __device__ __HOST_V__ sign(int3 a)
{
    return make_int3(sign(a.x), sign(a.y), sign(a.z));
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ sign(intN<N> a)
{
    intN<N> y;
    FOR_ALL(y.el[i] = sign(a.el[i]));
    return y;
}
INLINE int1 __device__ __HOST_V__ _and_(int1 a, int1 b)
{
    return make_int1(_and_(a.x, b.x));
}
INLINE int2 __device__ __HOST_V__ _and_(int2 a, int2 b)
{
    return make_int2(_and_(a.x, b.x), _and_(a.y, b.y));
}
INLINE int3 __device__ __HOST_V__ _and_(int3 a, int3 b)
{
    return make_int3(_and_(a.x, b.x), _and_(a.y, b.y), _and_(a.z, b.z));
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ _and_(intN<N> a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = _and_(a.el[i], b.el[i]));
    return y;
}
INLINE int1 __device__ __HOST_V__ _or_(int1 a, int1 b)
{
    return make_int1(_or_(a.x, b.x));
}
INLINE int2 __device__ __HOST_V__ _or_(int2 a, int2 b)
{
    return make_int2(_or_(a.x, b.x), _or_(a.y, b.y));
}
INLINE int3 __device__ __HOST_V__ _or_(int3 a, int3 b)
{
    return make_int3(_or_(a.x, b.x), _or_(a.y, b.y), _or_(a.z, b.z));
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ _or_(intN<N> a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = _or_(a.el[i], b.el[i]));
    return y;
}
INLINE int1 __device__ __HOST_V__ _xor_(int1 a, int1 b)
{
    return make_int1(_xor_(a.x, b.x));
}
INLINE int2 __device__ __HOST_V__ _xor_(int2 a, int2 b)
{
    return make_int2(_xor_(a.x, b.x), _xor_(a.y, b.y));
}
INLINE int3 __device__ __HOST_V__ _xor_(int3 a, int3 b)
{
    return make_int3(_xor_(a.x, b.x), _xor_(a.y, b.y), _xor_(a.z, b.z));
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ _xor_(intN<N> a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = _xor_(a.el[i], b.el[i]));
    return y;
}
INLINE int1 __device__ __HOST_V__ min(int1 a, int1 b)
{
    return make_int1(min(a.x, b.x));
}
INLINE int2 __device__ __HOST_V__ min(int2 a, int2 b)
{
    return make_int2(min(a.x, b.x), min(a.y, b.y));
}
INLINE int3 __device__ __HOST_V__ min(int3 a, int3 b)
{
    return make_int3(min(a.x, b.x), min(a.y, b.y), min(a.z, b.z));
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ min(intN<N> a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = min(a.el[i], b.el[i]));
    return y;
}
INLINE int1 __device__ __HOST_V__ max(int1 a, int1 b)
{
    return make_int1(max(a.x, b.x));
}
INLINE int2 __device__ __HOST_V__ max(int2 a, int2 b)
{
    return make_int2(max(a.x, b.x), max(a.y, b.y));
}
INLINE int3 __device__ __HOST_V__ max(int3 a, int3 b)
{
    return make_int3(max(a.x, b.x), max(a.y, b.y), max(a.z, b.z));
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ max(intN<N> a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = max(a.el[i], b.el[i]));
    return y;
}
INLINE int1 __device__ __HOST_V__ logor(int1 a, int1 b)
{
    return make_int1(logor(a.x, b.x));
}
INLINE int2 __device__ __HOST_V__ logor(int2 a, int2 b)
{
    return make_int2(logor(a.x, b.x), logor(a.y, b.y));
}
INLINE int3 __device__ __HOST_V__ logor(int3 a, int3 b)
{
    return make_int3(logor(a.x, b.x), logor(a.y, b.y), logor(a.z, b.z));
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ logor(intN<N> a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = logor(a.el[i], b.el[i]));
    return y;
}
INLINE int1 __device__ __HOST_V__ logand(int1 a, int1 b)
{
    return make_int1(logand(a.x, b.x));
}
INLINE int2 __device__ __HOST_V__ logand(int2 a, int2 b)
{
    return make_int2(logand(a.x, b.x), logand(a.y, b.y));
}
INLINE int3 __device__ __HOST_V__ logand(int3 a, int3 b)
{
    return make_int3(logand(a.x, b.x), logand(a.y, b.y), logand(a.z, b.z));
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ logand(intN<N> a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = logand(a.el[i], b.el[i]));
    return y;
}
INLINE int1 __device__ __HOST_V__ logxor(int1 a, int1 b)
{
    return make_int1(logxor(a.x, b.x));
}
INLINE int2 __device__ __HOST_V__ logxor(int2 a, int2 b)
{
    return make_int2(logxor(a.x, b.x), logxor(a.y, b.y));
}
INLINE int3 __device__ __HOST_V__ logxor(int3 a, int3 b)
{
    return make_int3(logxor(a.x, b.x), logxor(a.y, b.y), logxor(a.z, b.z));
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ logxor(intN<N> a, intN<N> b)
{
    intN<N> y;
    FOR_ALL(y.el[i] = logxor(a.el[i], b.el[i]));
    return y;
}
INLINE scalar1 __device__ __HOST_V__ abs(scalar1 a)
{
	return make_scalar1(abs(a.x));
}
INLINE scalar2 __device__ __HOST_V__ abs(scalar2 a)
{
	return make_scalar2(abs(a.x), abs(a.y));
}
INLINE scalar3 __device__ __HOST_V__ abs(scalar3 a)
{
    return make_scalar3(abs(a.x), abs(a.y), abs(a.z));
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ abs(scalarN<N> a)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = abs(a.el[i]));
    return y;
}
INLINE scalar1 __device__ __HOST_V__ round(scalar1 a)
{
	return make_scalar1(round(a.x));
}
INLINE scalar2 __device__ __HOST_V__ round(scalar2 a)
{
	return make_scalar2(round(a.x), round(a.y));
}
INLINE scalar3 __device__ __HOST_V__ round(scalar3 a)
{
    return make_scalar3(round(a.x), round(a.y), round(a.z));
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ round(scalarN<N> a)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = round(a.el[i]));
    return y;
}
INLINE scalar1 __device__ __HOST_V__ floor(scalar1 a)
{
	return make_scalar1(floor(a.x));
}
INLINE scalar2 __device__ __HOST_V__ floor(scalar2 a)
{
	return make_scalar2(floor(a.x), floor(a.y));
}
INLINE scalar3 __device__ __HOST_V__ floor(scalar3 a)
{
    return make_scalar3(floor(a.x), floor(a.y), floor(a.z));
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ floor(scalarN<N> a)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = floor(a.el[i]));
    return y;
}
INLINE scalar1 __device__ __HOST_V__ ceil(scalar1 a)
{
	return make_scalar1(ceil(a.x));
}
INLINE scalar2 __device__ __HOST_V__ ceil(scalar2 a)
{
	return make_scalar2(ceil(a.x), ceil(a.y));
}
INLINE scalar3 __device__ __HOST_V__ ceil(scalar3 a)
{
    return make_scalar3(ceil(a.x), ceil(a.y), ceil(a.z));
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ ceil(scalarN<N> a)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = ceil(a.el[i]));
    return y;
}
INLINE scalar1 __device__ __HOST_V__ sqrt(scalar1 a)
{
	return make_scalar1(sqrt(a.x));
}
INLINE scalar2 __device__ __HOST_V__ sqrt(scalar2 a)
{
	return make_scalar2(sqrt(a.x), sqrt(a.y));
}
INLINE scalar3 __device__ __HOST_V__ sqrt(scalar3 a)
{
    return make_scalar3(sqrt(a.x), sqrt(a.y), sqrt(a.z));
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ sqrt(scalarN<N> a)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = sqrt(a.el[i]));
    return y;
}
INLINE scalar1 __device__ __HOST_V__ rsqrt(scalar1 a)
{
	return make_scalar1(rsqrt(a.x));
}
INLINE scalar2 __device__ __HOST_V__ rsqrt(scalar2 a)
{
	return make_scalar2(rsqrt(a.x), rsqrt(a.y));
}
INLINE scalar3 __device__ __HOST_V__ rsqrt(scalar3 a)
{
    return make_scalar3(rsqrt(a.x), rsqrt(a.y), rsqrt(a.z));
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ rsqrt(scalarN<N> a)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = rsqrt(a.el[i]));
    return y;
}
INLINE scalar1 __device__ __HOST_V__ frac(scalar1 a)
{
	return make_scalar1(frac(a.x));
}
INLINE scalar2 __device__ __HOST_V__ frac(scalar2 a)
{
	return make_scalar2(frac(a.x), frac(a.y));
}
INLINE scalar3 __device__ __HOST_V__ frac(scalar3 a)
{
    return make_scalar3(frac(a.x), frac(a.y), frac(a.z));
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ frac(scalarN<N> a)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = frac(a.el[i]));
    return y;
}
INLINE scalar1 __device__ __HOST_V__ sign(scalar1 a)
{
	return make_scalar1(sign(a.x));
}
INLINE scalar2 __device__ __HOST_V__ sign(scalar2 a)
{
	return make_scalar2(sign(a.x), sign(a.y));
}
INLINE scalar3 __device__ __HOST_V__ sign(scalar3 a)
{
    return make_scalar3(sign(a.x), sign(a.y), sign(a.z));
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ sign(scalarN<N> a)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = sign(a.el[i]));
    return y;
}
INLINE scalar1 __device__ __HOST_V__ rcp_rn(scalar1 a)
{
	return make_scalar1(rcp_rn(a.x));
}
INLINE scalar2 __device__ __HOST_V__ rcp_rn(scalar2 a)
{
	return make_scalar2(rcp_rn(a.x), rcp_rn(a.y));
}
INLINE scalar3 __device__ __HOST_V__ rcp_rn(scalar3 a)
{
    return make_scalar3(rcp_rn(a.x), rcp_rn(a.y), rcp_rn(a.z));
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ rcp_rn(scalarN<N> a)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = rcp_rn(a.el[i]));
    return y;
}
INLINE scalar1 __device__ __HOST_V__ rcp_rz(scalar1 a)
{
	return make_scalar1(rcp_rz(a.x));
}
INLINE scalar2 __device__ __HOST_V__ rcp_rz(scalar2 a)
{
	return make_scalar2(rcp_rz(a.x), rcp_rz(a.y));
}
INLINE scalar3 __device__ __HOST_V__ rcp_rz(scalar3 a)
{
    return make_scalar3(rcp_rz(a.x), rcp_rz(a.y), rcp_rz(a.z));
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ rcp_rz(scalarN<N> a)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = rcp_rz(a.el[i]));
    return y;
}
INLINE scalar1 __device__ __HOST_V__ rcp_ru(scalar1 a)
{
	return make_scalar1(rcp_ru(a.x));
}
INLINE scalar2 __device__ __HOST_V__ rcp_ru(scalar2 a)
{
	return make_scalar2(rcp_ru(a.x), rcp_ru(a.y));
}
INLINE scalar3 __device__ __HOST_V__ rcp_ru(scalar3 a)
{
    return make_scalar3(rcp_ru(a.x), rcp_ru(a.y), rcp_ru(a.z));
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ rcp_ru(scalarN<N> a)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = rcp_ru(a.el[i]));
    return y;
}
INLINE scalar1 __device__ __HOST_V__ _or_(scalar1 a, scalar1 b)
{
    return make_scalar1(_or_(a.x, b.x));
}
INLINE scalar2 __device__ __HOST_V__ _or_(scalar2 a, scalar2 b)
{
    return make_scalar2(_or_(a.x, b.x), _or_(a.y, b.y));
}
INLINE scalar3 __device__ __HOST_V__ _or_(scalar3 a, scalar3 b)
{
    return make_scalar3(_or_(a.x, b.x), _or_(a.y, b.y), _or_(a.z, b.z));
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ _or_(scalarN<N> a, scalarN<N> b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = _or_(a.el[i], b.el[i]));
    return y;
}
INLINE scalar1 __device__ __HOST_V__ _xor_(scalar1 a, scalar1 b)
{
    return make_scalar1(_xor_(a.x, b.x));
}
INLINE scalar2 __device__ __HOST_V__ _xor_(scalar2 a, scalar2 b)
{
    return make_scalar2(_xor_(a.x, b.x), _xor_(a.y, b.y));
}
INLINE scalar3 __device__ __HOST_V__ _xor_(scalar3 a, scalar3 b)
{
    return make_scalar3(_xor_(a.x, b.x), _xor_(a.y, b.y), _xor_(a.z, b.z));
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ _xor_(scalarN<N> a, scalarN<N> b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = _xor_(a.el[i], b.el[i]));
    return y;
}
INLINE scalar1 __device__ __HOST_V__ min(scalar1 a, scalar1 b)
{
    return make_scalar1(min(a.x, b.x));
}
INLINE scalar2 __device__ __HOST_V__ min(scalar2 a, scalar2 b)
{
    return make_scalar2(min(a.x, b.x), min(a.y, b.y));
}
INLINE scalar3 __device__ __HOST_V__ min(scalar3 a, scalar3 b)
{
    return make_scalar3(min(a.x, b.x), min(a.y, b.y), min(a.z, b.z));
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ min(scalarN<N> a, scalarN<N> b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = min(a.el[i], b.el[i]));
    return y;
}
INLINE scalar1 __device__ __HOST_V__ max(scalar1 a, scalar1 b)
{
    return make_scalar1(max(a.x, b.x));
}
INLINE scalar2 __device__ __HOST_V__ max(scalar2 a, scalar2 b)
{
    return make_scalar2(max(a.x, b.x), max(a.y, b.y));
}
INLINE scalar3 __device__ __HOST_V__ max(scalar3 a, scalar3 b)
{
    return make_scalar3(max(a.x, b.x), max(a.y, b.y), max(a.z, b.z));
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ max(scalarN<N> a, scalarN<N> b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = max(a.el[i], b.el[i]));
    return y;
}
INLINE scalar1 __device__ __HOST_V__ div_approx(scalar1 a, scalar1 b)
{
    return make_scalar1(div_approx(a.x, b.x));
}
INLINE scalar2 __device__ __HOST_V__ div_approx(scalar2 a, scalar2 b)
{
    return make_scalar2(div_approx(a.x, b.x), div_approx(a.y, b.y));
}
INLINE scalar3 __device__ __HOST_V__ div_approx(scalar3 a, scalar3 b)
{
    return make_scalar3(div_approx(a.x, b.x), div_approx(a.y, b.y), div_approx(a.z, b.z));
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ div_approx(scalarN<N> a, scalarN<N> b)
{
    scalarN<N> y;
    FOR_ALL(y.el[i] = div_approx(a.el[i], b.el[i]));
    return y;
}
INLINE scalar __device__ __HOST_V__ dotprod(scalar a, scalar b) { return a * b; }
INLINE scalar __device__ __HOST_V__ dotprod(scalar1 a, scalar1 b) { return a.x * b.x; }
INLINE scalar __device__ __HOST_V__ dotprod(scalar2 a, scalar2 b)
{
    return a.x * b.x + a.y * b.y;
}
INLINE scalar __device__  __HOST_V__ dotprod(scalar3 a, scalar3 b)
{
    return a.x * b.x + a.y * b.y + a.z * b.z;
}
template <int N> scalar __device__  __HOST_V__ dotprod(scalarN<N> a, scalarN<N> b)
{
    scalar t = scalar(0);
    FOR_ALL(t += a.el[i] * b.el[i]);
    return t;
}
INLINE int __device__ __HOST_V__ dotprod(int a, int b) { return a * b; }
INLINE int __device__ __HOST_V__ dotprod(int1 a, int1 b) { return a.x * b.x; }
INLINE int __device__ __HOST_V__ dotprod(int2 a, int2 b)
{
    return a.x * b.x + a.y * b.y;
}
INLINE int __device__  __HOST_V__ dotprod(int3 a, int3 b)
{
    return a.x * b.x + a.y * b.y + a.z * b.z;
}
template <int N> int __device__  __HOST_V__ dotprod(intN<N> a, intN<N> b)
{
    int t = int(0);
    FOR_ALL(t += a.el[i] * b.el[i]);
    return t;
}
INLINE cscalar __device__ __HOST_V__ dotprod(cscalar a, cscalar b) { return a * b; }
INLINE cscalar __device__ __HOST_V__ dotprod(cscalar1 a, cscalar1 b) { return a.x * b.x; }
INLINE cscalar __device__ __HOST_V__ dotprod(cscalar2 a, cscalar2 b)
{
    return a.x * b.x + a.y * b.y;
}
INLINE cscalar __device__  __HOST_V__ dotprod(cscalar3 a, cscalar3 b)
{
    return a.x * b.x + a.y * b.y + a.z * b.z;
}
INLINE cscalar __device__  __HOST_V__ dotprod(cscalar4 a, cscalar4 b)
{
    return a.x * b.x + a.y * b.y + a.z * b.z + a.w * b.w;
}
template <int N> cscalar __device__  __HOST_V__ dotprod(cscalarN<N> a, cscalarN<N> b)
{
    cscalar t = cscalar(0);
    FOR_ALL(t += a.el[i] * b.el[i]);
    return t;
}
INLINE scalar __device__ __HOST_V__ sum(scalar x) { return x; }
INLINE scalar __device__ __HOST_V__ sum(scalar1 a) { return a.x; }
INLINE scalar __device__ __HOST_V__ sum(scalar2 a) { return a.x + a.y; }
INLINE scalar __device__ __HOST_V__ sum(scalar3 a) { return a.x + a.y + a.z; }
INLINE scalar __device__ __HOST_V__ sum(scalar4 a);
INLINE int __device__ __HOST_V__ sum(int x) { return x; }
INLINE int __device__ __HOST_V__ sum(int1 a) { return a.x; }
INLINE int __device__ __HOST_V__ sum(int2 a) { return a.x + a.y; }
INLINE int __device__ __HOST_V__ sum(int3 a) { return a.x + a.y + a.z; }
INLINE int __device__ __HOST_V__ sum(int4 a);
INLINE scalar __device__ __HOST_V__ prod(scalar x) { return x; }
INLINE scalar __device__ __HOST_V__ prod(scalar1 a) { return a.x; }
INLINE scalar __device__ __HOST_V__ prod(scalar2 a) { return a.x * a.y; }
INLINE scalar __device__ __HOST_V__ prod(scalar3 a) { return a.x * a.y * a.z; }
INLINE int __device__ __HOST_V__ prod(int x) { return x; }
INLINE int __device__ __HOST_V__ prod(int1 a) { return a.x; }
INLINE int __device__ __HOST_V__ prod(int2 a) { return a.x * a.y; }
INLINE int __device__ __HOST_V__ prod(int3 a) { return a.x * a.y * a.z; }
INLINE scalar __device__ __HOST_V__ mean(scalar x) { return x; }
INLINE scalar __device__ __HOST_V__ mean(scalar1 a) { return a.x; }
INLINE scalar __device__ __HOST_V__ mean(scalar2 a) { return scalar(0.5) * (a.x + a.y); }
INLINE scalar __device__ __HOST_V__ mean(scalar3 a) { return scalar(1.0 / 3) * sum(a); }
INLINE scalar __device__ __HOST_V__ mean(scalar4 a) { return scalar(1.0 / 4) * sum(a); }
INLINE scalar __device__ __HOST_V__ mean(int x) { return x; }
INLINE scalar __device__ __HOST_V__ mean(int1 a) { return a.x; }
INLINE scalar __device__ __HOST_V__ mean(int2 a) { return scalar(0.5) * sum(a); }
INLINE scalar __device__ __HOST_V__ mean(int3 a) { return scalar(1.0 / 3) * sum(a); }
INLINE scalar __device__ __HOST_V__ mean(int4 a) { return scalar(1.0 / 4) * sum(a); }
INLINE int __device__ __HOST_V__ _any_(int a) { return a != 0; }
INLINE int __device__ __HOST_V__ _any_(int2 a) { return a.x || a.y; }
INLINE int __device__ __HOST_V__ _any_(int3 a) { return a.x || a.y || a.z; }
INLINE int __device__ __HOST_V__ _all_(int a) { return a != 0; }
INLINE int __device__ __HOST_V__ _all_(int2 a) { return a.x && a.y; }
INLINE int __device__ __HOST_V__ _all_(int3 a) { return a.x && a.y && a.z; }
INLINE scalar __device__  __HOST_V__ min(scalar2 a) { return min(a.x, a.y); }
INLINE scalar __device__  __HOST_V__ min(scalar3 a) { return min(a.x, min(a.y, a.z)); }
INLINE scalar __device__  __HOST_V__ max(scalar2 a) { return max(a.x, a.y); }
INLINE scalar __device__  __HOST_V__ max(scalar3 a) { return max(a.x, max(a.y, a.z)); }
template <int N>
scalar __device__  __HOST_V__ min(scalarN<N> a)
{
    scalar y = a.el[0];
    for (int i = 1; i < N; i++)
        y = min(y, a.el[i]);
    return y;
}
template <int N>
scalar __device__  __HOST_V__ max(scalarN<N> a)
{
    scalar y = a.el[0];
    for (int i = 1; i < N; i++)
        y = max(y, a.el[i]);
    return y;
}
INLINE int __device__  __HOST_V__ min(int2 a) { return min(a.x, a.y); }
INLINE int __device__  __HOST_V__ min(int3 a) { return min(a.x, min(a.y, a.z)); }
INLINE int __device__  __HOST_V__ max(int2 a) { return max(a.x, a.y); }
INLINE int __device__  __HOST_V__ max(int3 a) { return max(a.x, max(a.y, a.z)); }
template <int N> int __device__  __HOST_V__ min(intN<N> a)
{
    int y = a.el[0];
    for (int i = 1; i < N; i++)
        y = min(y, a.el[i]);
    return y;
}
template <int N> int __device__  __HOST_V__ max(intN<N> a)
{
    int y = a.el[0];
    for (int i = 1; i < N; i++)
        y = max(y, a.el[i]);
    return y;
}
INLINE scalar4 __device__ __HOST_V__ operator-(scalar4 a)
{
	scalar4 y;
#if defined(HAS_SSE) && !defined(DBL_SCALAR)
	__m128 A = _mm_set1_ps(0);
	__m128 B = _mm_load_ps((float *)&a);
	__m128 C = _mm_sub_ps(A, B);
	_mm_store_ps((float *)&y, C);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A = _mm256_set1_pd(0);
	__m256d B = _mm256_load_pd((double *)&a);
	__m256d C = _mm256_sub_pd(A, B);
	_mm256_store_pd((double *)&y, C);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128d A = _mm_set1_pd(0);
	__m128d B1 = _mm_load_pd((double *)&a.x);
	__m128d B2 = _mm_load_pd((double *)&a.z);
	__m128d C1 = _mm_sub_pd(A, B1);
	__m128d C2 = _mm_sub_pd(A, B2);
	_mm_store_pd((double*)&y.x, C1);
	_mm_store_pd((double*)&y.z, C2);
#else
	y = make_scalar4(-a.x, -a.y, -a.z, -a.w);
#endif
	return y;
}
INLINE scalar4 __device__ __HOST_V__ operator +(scalar4 a, scalar4 b)
{
	scalar4 y;
#if defined(HAS_SSE) && !defined(DBL_SCALAR)
	__m128 A = _mm_load_ps((float *)&a);
	__m128 B = _mm_load_ps((float *)&b);
	__m128 C = _mm_add_ps(A, B);
	_mm_store_ps((float *)&y, C);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A = _mm256_loadu_pd((double *)&a);
	__m256d B = _mm256_loadu_pd((double *)&b);
	__m256d C = _mm256_add_pd(A, B);
	_mm256_store_pd((double*)&y, C);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128d A1 = _mm_load_pd((double *)&a.x);
	__m128d A2 = _mm_load_pd((double *)&a.z);
	__m128d B1 = _mm_load_pd((double *)&b.x);
	__m128d B2 = _mm_load_pd((double *)&b.z);
	__m128d C1 = _mm_add_pd(A1,B1);
	__m128d C2 = _mm_add_pd(A2,B2);
	_mm_store_pd((double*)&y.x, C1);
	_mm_store_pd((double*)&y.z, C2);
#else
    y = make_scalar4(a.x + b.x, a.y + b.y, a.z + b.z, a.w + b.w);
#endif
	return y;
}
INLINE scalar4 __device__ __HOST_V__ operator +(scalar a, scalar4 b)
{
	scalar4 y;
#if defined(HAS_SSE) && !defined(DBL_SCALAR)
	__m128 A = _mm_set1_ps(a);
	__m128 B = _mm_load_ps((float *)&b);
	__m128 C = _mm_add_ps(A, B);
	_mm_store_ps((float *)&y, C);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A = _mm256_load1_pd(a);
	__m256d B = _mm256_loadu_pd((double *)&b);
	__m256d C = _mm256_add_pd(A, B);
	_mm256_store_pd((double*)&y, C);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128d A = _mm_load1_pd((double *)&a);
	__m128d B1 = _mm_load_pd((double *)&b.x);
	__m128d B2 = _mm_load_pd((double *)&b.z);
	__m128d C1 = _mm_add_pd(A,B1);
	__m128d C2 = _mm_add_pd(A,B2);
	_mm_store_pd((double*)&y.x, C1);
	_mm_store_pd((double*)&y.z, C2);
#else
    y = make_scalar4(a + b.x, a + b.y, a + b.z, a + b.w);
#endif
	return y;
}
INLINE scalar4 __device__ __HOST_V__ operator +(scalar4 a, scalar b)
{
	scalar4 y;
#if defined(HAS_SSE) && !defined(DBL_SCALAR)
	__m128 A = _mm_load_ps((float *)&a);
	__m128 B = _mm_set1_ps(b);
	__m128 C = _mm_add_ps(A, B);
	_mm_store_ps((float *)&y, C);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A = _mm256_loadu_pd((double *)&a);
	__m256d B = _mm256_load1_pd(b);
	__m256d C = _mm256_add_pd(A, B);
	_mm256_store_pd((double*)&y, C);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128d A1 = _mm_load_pd((double *)&a.x);
	__m128d A2 = _mm_load_pd((double *)&a.z);
	__m128d B = _mm_load1_pd((double *)&b);
	__m128d C1 = _mm_add_pd(A1,B);
	__m128d C2 = _mm_add_pd(A2,B);
	_mm_store_pd((double*)&y.x, C1);
	_mm_store_pd((double*)&y.z, C2);
#else
    y = make_scalar4(a.x + b, a.y + b, a.z + b, a.w + b);
#endif
	return y;
}
INLINE scalar4 __device__ __HOST_V__ operator -(scalar4 a, scalar4 b)
{
	scalar4 y;
#if defined(HAS_SSE) && !defined(DBL_SCALAR)
	__m128 A = _mm_load_ps((float *)&a);
	__m128 B = _mm_load_ps((float *)&b);
	__m128 C = _mm_sub_ps(A, B);
	_mm_store_ps((float *)&y, C);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A = _mm256_loadu_pd((double *)&a);
	__m256d B = _mm256_loadu_pd((double *)&b);
	__m256d C = _mm256_sub_pd(A, B);
	_mm256_store_pd((double*)&y, C);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128d A1 = _mm_load_pd((double *)&a.x);
	__m128d A2 = _mm_load_pd((double *)&a.z);
	__m128d B1 = _mm_load_pd((double *)&b.x);
	__m128d B2 = _mm_load_pd((double *)&b.z);
	__m128d C1 = _mm_sub_pd(A1,B1);
	__m128d C2 = _mm_sub_pd(A2,B2);
	_mm_store_pd((double*)&y.x, C1);
	_mm_store_pd((double*)&y.z, C2);
#else
    y = make_scalar4(a.x - b.x, a.y - b.y, a.z - b.z, a.w - b.w);
#endif
	return y;
}
INLINE scalar4 __device__ __HOST_V__ operator -(scalar a, scalar4 b)
{
	scalar4 y;
#if defined(HAS_SSE) && !defined(DBL_SCALAR)
	__m128 A = _mm_set1_ps(a);
	__m128 B = _mm_load_ps((float *)&b);
	__m128 C = _mm_sub_ps(A, B);
	_mm_store_ps((float *)&y, C);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A = _mm256_load1_pd(a);
	__m256d B = _mm256_loadu_pd((double *)&b);
	__m256d C = _mm256_sub_pd(A, B);
	_mm256_store_pd((double*)&y, C);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128d A = _mm_load1_pd((double *)&a);
	__m128d B1 = _mm_load_pd((double *)&b.x);
	__m128d B2 = _mm_load_pd((double *)&b.z);
	__m128d C1 = _mm_sub_pd(A,B1);
	__m128d C2 = _mm_sub_pd(A,B2);
	_mm_store_pd((double*)&y.x, C1);
	_mm_store_pd((double*)&y.z, C2);
#else
    y = make_scalar4(a - b.x, a - b.y, a - b.z, a - b.w);
#endif
	return y;
}
INLINE scalar4 __device__ __HOST_V__ operator -(scalar4 a, scalar b)
{
	scalar4 y;
#if defined(HAS_SSE) && !defined(DBL_SCALAR)
	__m128 A = _mm_load_ps((float *)&a);
	__m128 B = _mm_set1_ps(b);
	__m128 C = _mm_sub_ps(A, B);
	_mm_store_ps((float *)&y, C);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A = _mm256_loadu_pd((double *)&a);
	__m256d B = _mm256_load1_pd(b);
	__m256d C = _mm256_sub_pd(A, B);
	_mm256_store_pd((double*)&y, C);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128d A1 = _mm_load_pd((double *)&a.x);
	__m128d A2 = _mm_load_pd((double *)&a.z);
	__m128d B = _mm_load1_pd((double *)&b);
	__m128d C1 = _mm_sub_pd(A1,B);
	__m128d C2 = _mm_sub_pd(A2,B);
	_mm_store_pd((double*)&y.x, C1);
	_mm_store_pd((double*)&y.z, C2);
#else
    y = make_scalar4(a.x - b, a.y - b, a.z - b, a.w - b);
#endif
	return y;
}
INLINE scalar4 __device__ __HOST_V__ operator *(scalar4 a, scalar4 b)
{
	scalar4 y;
#if defined(HAS_SSE) && !defined(DBL_SCALAR)
	__m128 A = _mm_load_ps((float *)&a);
	__m128 B = _mm_load_ps((float *)&b);
	__m128 C = _mm_mul_ps(A, B);
	_mm_store_ps((float *)&y, C);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A = _mm256_loadu_pd((double *)&a);
	__m256d B = _mm256_loadu_pd((double *)&b);
	__m256d C = _mm256_mul_pd(A, B);
	_mm256_store_pd((double*)&y, C);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128d A1 = _mm_load_pd((double *)&a.x);
	__m128d A2 = _mm_load_pd((double *)&a.z);
	__m128d B1 = _mm_load_pd((double *)&b.x);
	__m128d B2 = _mm_load_pd((double *)&b.z);
	__m128d C1 = _mm_mul_pd(A1,B1);
	__m128d C2 = _mm_mul_pd(A2,B2);
	_mm_store_pd((double*)&y.x, C1);
	_mm_store_pd((double*)&y.z, C2);
#else
    y = make_scalar4(a.x * b.x, a.y * b.y, a.z * b.z, a.w * b.w);
#endif
	return y;
}
INLINE scalar4 __device__ __HOST_V__ operator *(scalar a, scalar4 b)
{
	scalar4 y;
#if defined(HAS_SSE) && !defined(DBL_SCALAR)
	__m128 A = _mm_set1_ps(a);
	__m128 B = _mm_load_ps((float *)&b);
	__m128 C = _mm_mul_ps(A, B);
	_mm_store_ps((float *)&y, C);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A = _mm256_load1_pd(a);
	__m256d B = _mm256_loadu_pd((double *)&b);
	__m256d C = _mm256_mul_pd(A, B);
	_mm256_store_pd((double*)&y, C);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128d A = _mm_load1_pd((double *)&a);
	__m128d B1 = _mm_load_pd((double *)&b.x);
	__m128d B2 = _mm_load_pd((double *)&b.z);
	__m128d C1 = _mm_mul_pd(A,B1);
	__m128d C2 = _mm_mul_pd(A,B2);
	_mm_store_pd((double*)&y.x, C1);
	_mm_store_pd((double*)&y.z, C2);
#else
    y = make_scalar4(a * b.x, a * b.y, a * b.z, a * b.w);
#endif
	return y;
}
INLINE scalar4 __device__ __HOST_V__ operator *(scalar4 a, scalar b)
{
	scalar4 y;
#if defined(HAS_SSE) && !defined(DBL_SCALAR)
	__m128 A = _mm_load_ps((float *)&a);
	__m128 B = _mm_set1_ps(b);
	__m128 C = _mm_mul_ps(A, B);
	_mm_store_ps((float *)&y, C);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A = _mm256_loadu_pd((double *)&a);
	__m256d B = _mm256_load1_pd(b);
	__m256d C = _mm256_mul_pd(A, B);
	_mm256_store_pd((double*)&y, C);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128d A1 = _mm_load_pd((double *)&a.x);
	__m128d A2 = _mm_load_pd((double *)&a.z);
	__m128d B = _mm_load1_pd((double *)&b);
	__m128d C1 = _mm_mul_pd(A1,B);
	__m128d C2 = _mm_mul_pd(A2,B);
	_mm_store_pd((double*)&y.x, C1);
	_mm_store_pd((double*)&y.z, C2);
#else
    y = make_scalar4(a.x * b, a.y * b, a.z * b, a.w * b);
#endif
	return y;
}
INLINE scalar4 __device__ __HOST_V__ operator /(scalar4 a, scalar4 b)
{
	scalar4 y;
#if defined(HAS_SSE) && !defined(DBL_SCALAR)
	__m128 A = _mm_load_ps((float *)&a);
	__m128 B = _mm_load_ps((float *)&b);
	__m128 C = _mm_div_ps(A, B);
	_mm_store_ps((float *)&y, C);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A = _mm256_loadu_pd((double *)&a);
	__m256d B = _mm256_loadu_pd((double *)&b);
	__m256d C = _mm256_div_pd(A, B);
	_mm256_store_pd((double*)&y, C);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128d A1 = _mm_load_pd((double *)&a.x);
	__m128d A2 = _mm_load_pd((double *)&a.z);
	__m128d B1 = _mm_load_pd((double *)&b.x);
	__m128d B2 = _mm_load_pd((double *)&b.z);
	__m128d C1 = _mm_div_pd(A1,B1);
	__m128d C2 = _mm_div_pd(A2,B2);
	_mm_store_pd((double*)&y.x, C1);
	_mm_store_pd((double*)&y.z, C2);
#else
    y = make_scalar4(a.x / b.x, a.y / b.y, a.z / b.z, a.w / b.w);
#endif
	return y;
}
INLINE scalar4 __device__ __HOST_V__ operator /(scalar a, scalar4 b)
{
	scalar4 y;
#if defined(HAS_SSE) && !defined(DBL_SCALAR)
	__m128 A = _mm_set1_ps(a);
	__m128 B = _mm_load_ps((float *)&b);
	__m128 C = _mm_div_ps(A, B);
	_mm_store_ps((float *)&y, C);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A = _mm256_load1_pd(a);
	__m256d B = _mm256_loadu_pd((double *)&b);
	__m256d C = _mm256_div_pd(A, B);
	_mm256_store_pd((double*)&y, C);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128d A = _mm_load1_pd((double *)&a);
	__m128d B1 = _mm_load_pd((double *)&b.x);
	__m128d B2 = _mm_load_pd((double *)&b.z);
	__m128d C1 = _mm_div_pd(A,B1);
	__m128d C2 = _mm_div_pd(A,B2);
	_mm_store_pd((double*)&y.x, C1);
	_mm_store_pd((double*)&y.z, C2);
#else
    y = make_scalar4(a / b.x, a / b.y, a / b.z, a / b.w);
#endif
	return y;
}
INLINE scalar4 __device__ __HOST_V__ operator /(scalar4 a, scalar b)
{
	scalar4 y;
#if defined(HAS_SSE) && !defined(DBL_SCALAR)
	__m128 A = _mm_load_ps((float *)&a);
	__m128 B = _mm_set1_ps(b);
	__m128 C = _mm_div_ps(A, B);
	_mm_store_ps((float *)&y, C);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A = _mm256_loadu_pd((double *)&a);
	__m256d B = _mm256_load1_pd(b);
	__m256d C = _mm256_div_pd(A, B);
	_mm256_store_pd((double*)&y, C);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128d A1 = _mm_load_pd((double *)&a.x);
	__m128d A2 = _mm_load_pd((double *)&a.z);
	__m128d B = _mm_load1_pd((double *)&b);
	__m128d C1 = _mm_div_pd(A1,B);
	__m128d C2 = _mm_div_pd(A2,B);
	_mm_store_pd((double*)&y.x, C1);
	_mm_store_pd((double*)&y.z, C2);
#else
    y = make_scalar4(a.x / b, a.y / b, a.z / b, a.w / b);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator >(scalar4 a, scalar4 b)
{
	int4 y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128 A = _mm_load_ps((float *)&a);
	__m128 B = _mm_load_ps((float *)&b);
	__m128i C = _mm_castps_si128(_mm_cmpgt_ps(A, B)); // Result is 0 or -1
	__m128i D = _mm_and_si128(C, mask); // AND to get 0 or 1
	_mm_store_si128((__m128i*)&y, D);
#elif defined(HAS_AVX2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m256d A = _mm256_loadu_pd((double *)&a);
	__m256d B = _mm256_loadu_pd((double *)&b);
	__m256i C = _mm256_castpd_si256(_mm256_cmp_pd(A, B, _CMP_GT_OQ));
	__m128i D1 = _mm256_extracti128_si256(C, 0);
	__m128i D2 = _mm256_extracti128_si256(C, 1);
	__m128i D3 = _mm_shuffle_epi32(D1, 0x4); // 1000b
	__m128i D4 = _mm_shuffle_epi32(D2, 0x4); // 1000b
	__m128i D5 = _mm_unpacklo_epi32(D3, D4); // D1[0] D2[0] D1[1] D2[1]
	__m128i D6 = _mm_shuffle_epi32(D5, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D6, mask);
	_mm_store_si128((__m128i*)&y, E);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128d A1 = _mm_load_pd((double *)&a.x);
	__m128d A2 = _mm_load_pd((double *)&a.z);
	__m128d B1 = _mm_load_pd((double *)&b.x);
	__m128d B2 = _mm_load_pd((double *)&b.z);
	__m128i C1 = _mm_castpd_si128(_mm_cmpgt_pd(A1,B1));
	__m128i C2 = _mm_castpd_si128(_mm_cmpgt_pd(A2,B2));
	__m128i D1 = _mm_shuffle_epi32(C1, 0x8); // 1000b
	__m128i D2 = _mm_shuffle_epi32(C2, 0x8); // 1000b
	__m128i D3 = _mm_unpacklo_epi32(D1, D2); // D1[0] D2[0] D1[1] D2[1]
	__m128i D4 = _mm_shuffle_epi32(D3, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D4, mask);
	_mm_store_si128((__m128i*)&y, E);
#else
    y = make_int4(a.x > b.x, a.y > b.y, a.z > b.z, a.w > b.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator >(scalar a, scalar4 b)
{
	int4 y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128 A = _mm_load1_ps((float *)&a);
	__m128 B = _mm_load_ps((float *)&b);
	__m128i C = _mm_castps_si128(_mm_cmpgt_ps(A, B)); // Result is 0 or -1
	__m128i D = _mm_and_si128(C, mask); // AND to get 0 or 1
	_mm_store_si128((__m128i*)&y, D);
#elif defined(HAS_AVX2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m256d A = _mm256_load1_pd(a);
	__m256d B = _mm256_loadu_pd((double *)&b);
	__m256i C = _mm256_castpd_si256(_mm256_cmp_pd(A, B, _CMP_GT_OQ));
	__m128i D1 = _mm256_extracti128_si256(C, 0);
	__m128i D2 = _mm256_extracti128_si256(C, 1);
	__m128i D3 = _mm_shuffle_epi32(D1, 0x4); // 1000b
	__m128i D4 = _mm_shuffle_epi32(D2, 0x4); // 1000b
	__m128i D5 = _mm_unpacklo_epi32(D3, D4); // D1[0] D2[0] D1[1] D2[1]
	__m128i D6 = _mm_shuffle_epi32(D5, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D6, mask);
	_mm_store_si128((__m128i*)&y, E);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128d A = _mm_load1_pd((double *)&a);
	__m128d B1 = _mm_load_pd((double *)&b.x);
	__m128d B2 = _mm_load_pd((double *)&b.z);
	__m128i C1 = _mm_castpd_si128(_mm_cmpgt_pd(A,B1));
	__m128i C2 = _mm_castpd_si128(_mm_cmpgt_pd(A,B2));
	__m128i D1 = _mm_shuffle_epi32(C1, 0x8); // 1000b
	__m128i D2 = _mm_shuffle_epi32(C2, 0x8); // 1000b
	__m128i D3 = _mm_unpacklo_epi32(D1, D2); // D1[0] D2[0] D1[1] D2[1]
	__m128i D4 = _mm_shuffle_epi32(D3, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D4, mask);
	_mm_store_si128((__m128i*)&y, E);
#else
    y = make_int4(a > b.x, a > b.y, a > b.z, a > b.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator >(scalar4 a, scalar b)
{
	int4 y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128 A = _mm_load_ps((float *)&a);
	__m128 B = _mm_load1_ps((float *)&b);
	__m128i C = _mm_castps_si128(_mm_cmpgt_ps(A, B)); // Result is 0 or -1
	__m128i D = _mm_and_si128(C, mask); // AND to get 0 or 1
	_mm_store_si128((__m128i*)&y, D);
#elif defined(HAS_AVX2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m256d A = _mm256_loadu_pd((double *)&a);
	__m256d B = _mm256_load1_pd(b);
	__m256i C = _mm256_castpd_si256(_mm256_cmp_pd(A, B, _CMP_GT_OQ));
	__m128i D1 = _mm256_extracti128_si256(C, 0);
	__m128i D2 = _mm256_extracti128_si256(C, 1);
	__m128i D3 = _mm_shuffle_epi32(D1, 0x4); // 1000b
	__m128i D4 = _mm_shuffle_epi32(D2, 0x4); // 1000b
	__m128i D5 = _mm_unpacklo_epi32(D3, D4); // D1[0] D2[0] D1[1] D2[1]
	__m128i D6 = _mm_shuffle_epi32(D5, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D6, mask);
	_mm_store_si128((__m128i*)&y, E);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128d A1 = _mm_load_pd((double *)&a.x);
	__m128d A2 = _mm_load_pd((double *)&a.z);
	__m128d B = _mm_load1_pd((double *)&b);
	__m128i C1 = _mm_castpd_si128(_mm_cmpgt_pd(A1,B));
	__m128i C2 = _mm_castpd_si128(_mm_cmpgt_pd(A2,B));
	__m128i D1 = _mm_shuffle_epi32(C1, 0x8); // 1000b
	__m128i D2 = _mm_shuffle_epi32(C2, 0x8); // 1000b
	__m128i D3 = _mm_unpacklo_epi32(D1, D2); // D1[0] D2[0] D1[1] D2[1]
	__m128i D4 = _mm_shuffle_epi32(D3, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D4, mask);
	_mm_store_si128((__m128i*)&y, E);
#else
    y = make_int4(a.x > b, a.y > b, a.z > b, a.w > b);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator <(scalar4 a, scalar4 b)
{
	int4 y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128 A = _mm_load_ps((float *)&a);
	__m128 B = _mm_load_ps((float *)&b);
	__m128i C = _mm_castps_si128(_mm_cmplt_ps(A, B)); // Result is 0 or -1
	__m128i D = _mm_and_si128(C, mask); // AND to get 0 or 1
	_mm_store_si128((__m128i*)&y, D);
#elif defined(HAS_AVX2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m256d A = _mm256_loadu_pd((double *)&a);
	__m256d B = _mm256_loadu_pd((double *)&b);
	__m256i C = _mm256_castpd_si256(_mm256_cmp_pd(A, B, _CMP_LT_OQ));
	__m128i D1 = _mm256_extracti128_si256(C, 0);
	__m128i D2 = _mm256_extracti128_si256(C, 1);
	__m128i D3 = _mm_shuffle_epi32(D1, 0x4); // 1000b
	__m128i D4 = _mm_shuffle_epi32(D2, 0x4); // 1000b
	__m128i D5 = _mm_unpacklo_epi32(D3, D4); // D1[0] D2[0] D1[1] D2[1]
	__m128i D6 = _mm_shuffle_epi32(D5, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D6, mask);
	_mm_store_si128((__m128i*)&y, E);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128d A1 = _mm_load_pd((double *)&a.x);
	__m128d A2 = _mm_load_pd((double *)&a.z);
	__m128d B1 = _mm_load_pd((double *)&b.x);
	__m128d B2 = _mm_load_pd((double *)&b.z);
	__m128i C1 = _mm_castpd_si128(_mm_cmplt_pd(A1,B1));
	__m128i C2 = _mm_castpd_si128(_mm_cmplt_pd(A2,B2));
	__m128i D1 = _mm_shuffle_epi32(C1, 0x8); // 1000b
	__m128i D2 = _mm_shuffle_epi32(C2, 0x8); // 1000b
	__m128i D3 = _mm_unpacklo_epi32(D1, D2); // D1[0] D2[0] D1[1] D2[1]
	__m128i D4 = _mm_shuffle_epi32(D3, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D4, mask);
	_mm_store_si128((__m128i*)&y, E);
#else
    y = make_int4(a.x < b.x, a.y < b.y, a.z < b.z, a.w < b.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator <(scalar a, scalar4 b)
{
	int4 y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128 A = _mm_load1_ps((float *)&a);
	__m128 B = _mm_load_ps((float *)&b);
	__m128i C = _mm_castps_si128(_mm_cmplt_ps(A, B)); // Result is 0 or -1
	__m128i D = _mm_and_si128(C, mask); // AND to get 0 or 1
	_mm_store_si128((__m128i*)&y, D);
#elif defined(HAS_AVX2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m256d A = _mm256_load1_pd(a);
	__m256d B = _mm256_loadu_pd((double *)&b);
	__m256i C = _mm256_castpd_si256(_mm256_cmp_pd(A, B, _CMP_LT_OQ));
	__m128i D1 = _mm256_extracti128_si256(C, 0);
	__m128i D2 = _mm256_extracti128_si256(C, 1);
	__m128i D3 = _mm_shuffle_epi32(D1, 0x4); // 1000b
	__m128i D4 = _mm_shuffle_epi32(D2, 0x4); // 1000b
	__m128i D5 = _mm_unpacklo_epi32(D3, D4); // D1[0] D2[0] D1[1] D2[1]
	__m128i D6 = _mm_shuffle_epi32(D5, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D6, mask);
	_mm_store_si128((__m128i*)&y, E);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128d A = _mm_load1_pd((double *)&a);
	__m128d B1 = _mm_load_pd((double *)&b.x);
	__m128d B2 = _mm_load_pd((double *)&b.z);
	__m128i C1 = _mm_castpd_si128(_mm_cmplt_pd(A,B1));
	__m128i C2 = _mm_castpd_si128(_mm_cmplt_pd(A,B2));
	__m128i D1 = _mm_shuffle_epi32(C1, 0x8); // 1000b
	__m128i D2 = _mm_shuffle_epi32(C2, 0x8); // 1000b
	__m128i D3 = _mm_unpacklo_epi32(D1, D2); // D1[0] D2[0] D1[1] D2[1]
	__m128i D4 = _mm_shuffle_epi32(D3, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D4, mask);
	_mm_store_si128((__m128i*)&y, E);
#else
    y = make_int4(a < b.x, a < b.y, a < b.z, a < b.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator <(scalar4 a, scalar b)
{
	int4 y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128 A = _mm_load_ps((float *)&a);
	__m128 B = _mm_load1_ps((float *)&b);
	__m128i C = _mm_castps_si128(_mm_cmplt_ps(A, B)); // Result is 0 or -1
	__m128i D = _mm_and_si128(C, mask); // AND to get 0 or 1
	_mm_store_si128((__m128i*)&y, D);
#elif defined(HAS_AVX2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m256d A = _mm256_loadu_pd((double *)&a);
	__m256d B = _mm256_load1_pd(b);
	__m256i C = _mm256_castpd_si256(_mm256_cmp_pd(A, B, _CMP_LT_OQ));
	__m128i D1 = _mm256_extracti128_si256(C, 0);
	__m128i D2 = _mm256_extracti128_si256(C, 1);
	__m128i D3 = _mm_shuffle_epi32(D1, 0x4); // 1000b
	__m128i D4 = _mm_shuffle_epi32(D2, 0x4); // 1000b
	__m128i D5 = _mm_unpacklo_epi32(D3, D4); // D1[0] D2[0] D1[1] D2[1]
	__m128i D6 = _mm_shuffle_epi32(D5, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D6, mask);
	_mm_store_si128((__m128i*)&y, E);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128d A1 = _mm_load_pd((double *)&a.x);
	__m128d A2 = _mm_load_pd((double *)&a.z);
	__m128d B = _mm_load1_pd((double *)&b);
	__m128i C1 = _mm_castpd_si128(_mm_cmplt_pd(A1,B));
	__m128i C2 = _mm_castpd_si128(_mm_cmplt_pd(A2,B));
	__m128i D1 = _mm_shuffle_epi32(C1, 0x8); // 1000b
	__m128i D2 = _mm_shuffle_epi32(C2, 0x8); // 1000b
	__m128i D3 = _mm_unpacklo_epi32(D1, D2); // D1[0] D2[0] D1[1] D2[1]
	__m128i D4 = _mm_shuffle_epi32(D3, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D4, mask);
	_mm_store_si128((__m128i*)&y, E);
#else
    y = make_int4(a.x < b, a.y < b, a.z < b, a.w < b);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator >=(scalar4 a, scalar4 b)
{
	int4 y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128 A = _mm_load_ps((float *)&a);
	__m128 B = _mm_load_ps((float *)&b);
	__m128i C = _mm_castps_si128(_mm_cmpge_ps(A, B)); // Result is 0 or -1
	__m128i D = _mm_and_si128(C, mask); // AND to get 0 or 1
	_mm_store_si128((__m128i*)&y, D);
#elif defined(HAS_AVX2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m256d A = _mm256_loadu_pd((double *)&a);
	__m256d B = _mm256_loadu_pd((double *)&b);
	__m256i C = _mm256_castpd_si256(_mm256_cmp_pd(A, B, _CMP_GE_OQ));
	__m128i D1 = _mm256_extracti128_si256(C, 0);
	__m128i D2 = _mm256_extracti128_si256(C, 1);
	__m128i D3 = _mm_shuffle_epi32(D1, 0x4); // 1000b
	__m128i D4 = _mm_shuffle_epi32(D2, 0x4); // 1000b
	__m128i D5 = _mm_unpacklo_epi32(D3, D4); // D1[0] D2[0] D1[1] D2[1]
	__m128i D6 = _mm_shuffle_epi32(D5, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D6, mask);
	_mm_store_si128((__m128i*)&y, E);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128d A1 = _mm_load_pd((double *)&a.x);
	__m128d A2 = _mm_load_pd((double *)&a.z);
	__m128d B1 = _mm_load_pd((double *)&b.x);
	__m128d B2 = _mm_load_pd((double *)&b.z);
	__m128i C1 = _mm_castpd_si128(_mm_cmpge_pd(A1,B1));
	__m128i C2 = _mm_castpd_si128(_mm_cmpge_pd(A2,B2));
	__m128i D1 = _mm_shuffle_epi32(C1, 0x8); // 1000b
	__m128i D2 = _mm_shuffle_epi32(C2, 0x8); // 1000b
	__m128i D3 = _mm_unpacklo_epi32(D1, D2); // D1[0] D2[0] D1[1] D2[1]
	__m128i D4 = _mm_shuffle_epi32(D3, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D4, mask);
	_mm_store_si128((__m128i*)&y, E);
#else
    y = make_int4(a.x >= b.x, a.y >= b.y, a.z >= b.z, a.w >= b.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator >=(scalar a, scalar4 b)
{
	int4 y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128 A = _mm_load1_ps((float *)&a);
	__m128 B = _mm_load_ps((float *)&b);
	__m128i C = _mm_castps_si128(_mm_cmpge_ps(A, B)); // Result is 0 or -1
	__m128i D = _mm_and_si128(C, mask); // AND to get 0 or 1
	_mm_store_si128((__m128i*)&y, D);
#elif defined(HAS_AVX2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m256d A = _mm256_load1_pd(a);
	__m256d B = _mm256_loadu_pd((double *)&b);
	__m256i C = _mm256_castpd_si256(_mm256_cmp_pd(A, B, _CMP_GE_OQ));
	__m128i D1 = _mm256_extracti128_si256(C, 0);
	__m128i D2 = _mm256_extracti128_si256(C, 1);
	__m128i D3 = _mm_shuffle_epi32(D1, 0x4); // 1000b
	__m128i D4 = _mm_shuffle_epi32(D2, 0x4); // 1000b
	__m128i D5 = _mm_unpacklo_epi32(D3, D4); // D1[0] D2[0] D1[1] D2[1]
	__m128i D6 = _mm_shuffle_epi32(D5, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D6, mask);
	_mm_store_si128((__m128i*)&y, E);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128d A = _mm_load1_pd((double *)&a);
	__m128d B1 = _mm_load_pd((double *)&b.x);
	__m128d B2 = _mm_load_pd((double *)&b.z);
	__m128i C1 = _mm_castpd_si128(_mm_cmpge_pd(A,B1));
	__m128i C2 = _mm_castpd_si128(_mm_cmpge_pd(A,B2));
	__m128i D1 = _mm_shuffle_epi32(C1, 0x8); // 1000b
	__m128i D2 = _mm_shuffle_epi32(C2, 0x8); // 1000b
	__m128i D3 = _mm_unpacklo_epi32(D1, D2); // D1[0] D2[0] D1[1] D2[1]
	__m128i D4 = _mm_shuffle_epi32(D3, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D4, mask);
	_mm_store_si128((__m128i*)&y, E);
#else
    y = make_int4(a >= b.x, a >= b.y, a >= b.z, a >= b.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator >=(scalar4 a, scalar b)
{
	int4 y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128 A = _mm_load_ps((float *)&a);
	__m128 B = _mm_load1_ps((float *)&b);
	__m128i C = _mm_castps_si128(_mm_cmpge_ps(A, B)); // Result is 0 or -1
	__m128i D = _mm_and_si128(C, mask); // AND to get 0 or 1
	_mm_store_si128((__m128i*)&y, D);
#elif defined(HAS_AVX2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m256d A = _mm256_loadu_pd((double *)&a);
	__m256d B = _mm256_load1_pd(b);
	__m256i C = _mm256_castpd_si256(_mm256_cmp_pd(A, B, _CMP_GE_OQ));
	__m128i D1 = _mm256_extracti128_si256(C, 0);
	__m128i D2 = _mm256_extracti128_si256(C, 1);
	__m128i D3 = _mm_shuffle_epi32(D1, 0x4); // 1000b
	__m128i D4 = _mm_shuffle_epi32(D2, 0x4); // 1000b
	__m128i D5 = _mm_unpacklo_epi32(D3, D4); // D1[0] D2[0] D1[1] D2[1]
	__m128i D6 = _mm_shuffle_epi32(D5, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D6, mask);
	_mm_store_si128((__m128i*)&y, E);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128d A1 = _mm_load_pd((double *)&a.x);
	__m128d A2 = _mm_load_pd((double *)&a.z);
	__m128d B = _mm_load1_pd((double *)&b);
	__m128i C1 = _mm_castpd_si128(_mm_cmpge_pd(A1,B));
	__m128i C2 = _mm_castpd_si128(_mm_cmpge_pd(A2,B));
	__m128i D1 = _mm_shuffle_epi32(C1, 0x8); // 1000b
	__m128i D2 = _mm_shuffle_epi32(C2, 0x8); // 1000b
	__m128i D3 = _mm_unpacklo_epi32(D1, D2); // D1[0] D2[0] D1[1] D2[1]
	__m128i D4 = _mm_shuffle_epi32(D3, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D4, mask);
	_mm_store_si128((__m128i*)&y, E);
#else
    y = make_int4(a.x >= b, a.y >= b, a.z >= b, a.w >= b);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator <=(scalar4 a, scalar4 b)
{
	int4 y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128 A = _mm_load_ps((float *)&a);
	__m128 B = _mm_load_ps((float *)&b);
	__m128i C = _mm_castps_si128(_mm_cmple_ps(A, B)); // Result is 0 or -1
	__m128i D = _mm_and_si128(C, mask); // AND to get 0 or 1
	_mm_store_si128((__m128i*)&y, D);
#elif defined(HAS_AVX2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m256d A = _mm256_loadu_pd((double *)&a);
	__m256d B = _mm256_loadu_pd((double *)&b);
	__m256i C = _mm256_castpd_si256(_mm256_cmp_pd(A, B, _CMP_LE_OQ));
	__m128i D1 = _mm256_extracti128_si256(C, 0);
	__m128i D2 = _mm256_extracti128_si256(C, 1);
	__m128i D3 = _mm_shuffle_epi32(D1, 0x4); // 1000b
	__m128i D4 = _mm_shuffle_epi32(D2, 0x4); // 1000b
	__m128i D5 = _mm_unpacklo_epi32(D3, D4); // D1[0] D2[0] D1[1] D2[1]
	__m128i D6 = _mm_shuffle_epi32(D5, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D6, mask);
	_mm_store_si128((__m128i*)&y, E);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128d A1 = _mm_load_pd((double *)&a.x);
	__m128d A2 = _mm_load_pd((double *)&a.z);
	__m128d B1 = _mm_load_pd((double *)&b.x);
	__m128d B2 = _mm_load_pd((double *)&b.z);
	__m128i C1 = _mm_castpd_si128(_mm_cmple_pd(A1,B1));
	__m128i C2 = _mm_castpd_si128(_mm_cmple_pd(A2,B2));
	__m128i D1 = _mm_shuffle_epi32(C1, 0x8); // 1000b
	__m128i D2 = _mm_shuffle_epi32(C2, 0x8); // 1000b
	__m128i D3 = _mm_unpacklo_epi32(D1, D2); // D1[0] D2[0] D1[1] D2[1]
	__m128i D4 = _mm_shuffle_epi32(D3, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D4, mask);
	_mm_store_si128((__m128i*)&y, E);
#else
    y = make_int4(a.x <= b.x, a.y <= b.y, a.z <= b.z, a.w <= b.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator <=(scalar a, scalar4 b)
{
	int4 y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128 A = _mm_load1_ps((float *)&a);
	__m128 B = _mm_load_ps((float *)&b);
	__m128i C = _mm_castps_si128(_mm_cmple_ps(A, B)); // Result is 0 or -1
	__m128i D = _mm_and_si128(C, mask); // AND to get 0 or 1
	_mm_store_si128((__m128i*)&y, D);
#elif defined(HAS_AVX2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m256d A = _mm256_load1_pd(a);
	__m256d B = _mm256_loadu_pd((double *)&b);
	__m256i C = _mm256_castpd_si256(_mm256_cmp_pd(A, B, _CMP_LE_OQ));
	__m128i D1 = _mm256_extracti128_si256(C, 0);
	__m128i D2 = _mm256_extracti128_si256(C, 1);
	__m128i D3 = _mm_shuffle_epi32(D1, 0x4); // 1000b
	__m128i D4 = _mm_shuffle_epi32(D2, 0x4); // 1000b
	__m128i D5 = _mm_unpacklo_epi32(D3, D4); // D1[0] D2[0] D1[1] D2[1]
	__m128i D6 = _mm_shuffle_epi32(D5, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D6, mask);
	_mm_store_si128((__m128i*)&y, E);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128d A = _mm_load1_pd((double *)&a);
	__m128d B1 = _mm_load_pd((double *)&b.x);
	__m128d B2 = _mm_load_pd((double *)&b.z);
	__m128i C1 = _mm_castpd_si128(_mm_cmple_pd(A,B1));
	__m128i C2 = _mm_castpd_si128(_mm_cmple_pd(A,B2));
	__m128i D1 = _mm_shuffle_epi32(C1, 0x8); // 1000b
	__m128i D2 = _mm_shuffle_epi32(C2, 0x8); // 1000b
	__m128i D3 = _mm_unpacklo_epi32(D1, D2); // D1[0] D2[0] D1[1] D2[1]
	__m128i D4 = _mm_shuffle_epi32(D3, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D4, mask);
	_mm_store_si128((__m128i*)&y, E);
#else
    y = make_int4(a <= b.x, a <= b.y, a <= b.z, a <= b.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator <=(scalar4 a, scalar b)
{
	int4 y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128 A = _mm_load_ps((float *)&a);
	__m128 B = _mm_load1_ps((float *)&b);
	__m128i C = _mm_castps_si128(_mm_cmple_ps(A, B)); // Result is 0 or -1
	__m128i D = _mm_and_si128(C, mask); // AND to get 0 or 1
	_mm_store_si128((__m128i*)&y, D);
#elif defined(HAS_AVX2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m256d A = _mm256_loadu_pd((double *)&a);
	__m256d B = _mm256_load1_pd(b);
	__m256i C = _mm256_castpd_si256(_mm256_cmp_pd(A, B, _CMP_LE_OQ));
	__m128i D1 = _mm256_extracti128_si256(C, 0);
	__m128i D2 = _mm256_extracti128_si256(C, 1);
	__m128i D3 = _mm_shuffle_epi32(D1, 0x4); // 1000b
	__m128i D4 = _mm_shuffle_epi32(D2, 0x4); // 1000b
	__m128i D5 = _mm_unpacklo_epi32(D3, D4); // D1[0] D2[0] D1[1] D2[1]
	__m128i D6 = _mm_shuffle_epi32(D5, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D6, mask);
	_mm_store_si128((__m128i*)&y, E);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128d A1 = _mm_load_pd((double *)&a.x);
	__m128d A2 = _mm_load_pd((double *)&a.z);
	__m128d B = _mm_load1_pd((double *)&b);
	__m128i C1 = _mm_castpd_si128(_mm_cmple_pd(A1,B));
	__m128i C2 = _mm_castpd_si128(_mm_cmple_pd(A2,B));
	__m128i D1 = _mm_shuffle_epi32(C1, 0x8); // 1000b
	__m128i D2 = _mm_shuffle_epi32(C2, 0x8); // 1000b
	__m128i D3 = _mm_unpacklo_epi32(D1, D2); // D1[0] D2[0] D1[1] D2[1]
	__m128i D4 = _mm_shuffle_epi32(D3, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D4, mask);
	_mm_store_si128((__m128i*)&y, E);
#else
    y = make_int4(a.x <= b, a.y <= b, a.z <= b, a.w <= b);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator ==(scalar4 a, scalar4 b)
{
	int4 y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128 A = _mm_load_ps((float *)&a);
	__m128 B = _mm_load_ps((float *)&b);
	__m128i C = _mm_castps_si128(_mm_cmpeq_ps(A, B)); // Result is 0 or -1
	__m128i D = _mm_and_si128(C, mask); // AND to get 0 or 1
	_mm_store_si128((__m128i*)&y, D);
#elif defined(HAS_AVX2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m256d A = _mm256_loadu_pd((double *)&a);
	__m256d B = _mm256_loadu_pd((double *)&b);
	__m256i C = _mm256_castpd_si256(_mm256_cmp_pd(A, B, _CMP_EQ_OQ));
	__m128i D1 = _mm256_extracti128_si256(C, 0);
	__m128i D2 = _mm256_extracti128_si256(C, 1);
	__m128i D3 = _mm_shuffle_epi32(D1, 0x4); // 1000b
	__m128i D4 = _mm_shuffle_epi32(D2, 0x4); // 1000b
	__m128i D5 = _mm_unpacklo_epi32(D3, D4); // D1[0] D2[0] D1[1] D2[1]
	__m128i D6 = _mm_shuffle_epi32(D5, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D6, mask);
	_mm_store_si128((__m128i*)&y, E);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128d A1 = _mm_load_pd((double *)&a.x);
	__m128d A2 = _mm_load_pd((double *)&a.z);
	__m128d B1 = _mm_load_pd((double *)&b.x);
	__m128d B2 = _mm_load_pd((double *)&b.z);
	__m128i C1 = _mm_castpd_si128(_mm_cmpeq_pd(A1,B1));
	__m128i C2 = _mm_castpd_si128(_mm_cmpeq_pd(A2,B2));
	__m128i D1 = _mm_shuffle_epi32(C1, 0x8); // 1000b
	__m128i D2 = _mm_shuffle_epi32(C2, 0x8); // 1000b
	__m128i D3 = _mm_unpacklo_epi32(D1, D2); // D1[0] D2[0] D1[1] D2[1]
	__m128i D4 = _mm_shuffle_epi32(D3, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D4, mask);
	_mm_store_si128((__m128i*)&y, E);
#else
    y = make_int4(a.x == b.x, a.y == b.y, a.z == b.z, a.w == b.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator ==(scalar a, scalar4 b)
{
	int4 y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128 A = _mm_load1_ps((float *)&a);
	__m128 B = _mm_load_ps((float *)&b);
	__m128i C = _mm_castps_si128(_mm_cmpeq_ps(A, B)); // Result is 0 or -1
	__m128i D = _mm_and_si128(C, mask); // AND to get 0 or 1
	_mm_store_si128((__m128i*)&y, D);
#elif defined(HAS_AVX2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m256d A = _mm256_load1_pd(a);
	__m256d B = _mm256_loadu_pd((double *)&b);
	__m256i C = _mm256_castpd_si256(_mm256_cmp_pd(A, B, _CMP_EQ_OQ));
	__m128i D1 = _mm256_extracti128_si256(C, 0);
	__m128i D2 = _mm256_extracti128_si256(C, 1);
	__m128i D3 = _mm_shuffle_epi32(D1, 0x4); // 1000b
	__m128i D4 = _mm_shuffle_epi32(D2, 0x4); // 1000b
	__m128i D5 = _mm_unpacklo_epi32(D3, D4); // D1[0] D2[0] D1[1] D2[1]
	__m128i D6 = _mm_shuffle_epi32(D5, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D6, mask);
	_mm_store_si128((__m128i*)&y, E);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128d A = _mm_load1_pd((double *)&a);
	__m128d B1 = _mm_load_pd((double *)&b.x);
	__m128d B2 = _mm_load_pd((double *)&b.z);
	__m128i C1 = _mm_castpd_si128(_mm_cmpeq_pd(A,B1));
	__m128i C2 = _mm_castpd_si128(_mm_cmpeq_pd(A,B2));
	__m128i D1 = _mm_shuffle_epi32(C1, 0x8); // 1000b
	__m128i D2 = _mm_shuffle_epi32(C2, 0x8); // 1000b
	__m128i D3 = _mm_unpacklo_epi32(D1, D2); // D1[0] D2[0] D1[1] D2[1]
	__m128i D4 = _mm_shuffle_epi32(D3, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D4, mask);
	_mm_store_si128((__m128i*)&y, E);
#else
    y = make_int4(a == b.x, a == b.y, a == b.z, a == b.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator ==(scalar4 a, scalar b)
{
	int4 y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128 A = _mm_load_ps((float *)&a);
	__m128 B = _mm_load1_ps((float *)&b);
	__m128i C = _mm_castps_si128(_mm_cmpeq_ps(A, B)); // Result is 0 or -1
	__m128i D = _mm_and_si128(C, mask); // AND to get 0 or 1
	_mm_store_si128((__m128i*)&y, D);
#elif defined(HAS_AVX2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m256d A = _mm256_loadu_pd((double *)&a);
	__m256d B = _mm256_load1_pd(b);
	__m256i C = _mm256_castpd_si256(_mm256_cmp_pd(A, B, _CMP_EQ_OQ));
	__m128i D1 = _mm256_extracti128_si256(C, 0);
	__m128i D2 = _mm256_extracti128_si256(C, 1);
	__m128i D3 = _mm_shuffle_epi32(D1, 0x4); // 1000b
	__m128i D4 = _mm_shuffle_epi32(D2, 0x4); // 1000b
	__m128i D5 = _mm_unpacklo_epi32(D3, D4); // D1[0] D2[0] D1[1] D2[1]
	__m128i D6 = _mm_shuffle_epi32(D5, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D6, mask);
	_mm_store_si128((__m128i*)&y, E);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128d A1 = _mm_load_pd((double *)&a.x);
	__m128d A2 = _mm_load_pd((double *)&a.z);
	__m128d B = _mm_load1_pd((double *)&b);
	__m128i C1 = _mm_castpd_si128(_mm_cmpeq_pd(A1,B));
	__m128i C2 = _mm_castpd_si128(_mm_cmpeq_pd(A2,B));
	__m128i D1 = _mm_shuffle_epi32(C1, 0x8); // 1000b
	__m128i D2 = _mm_shuffle_epi32(C2, 0x8); // 1000b
	__m128i D3 = _mm_unpacklo_epi32(D1, D2); // D1[0] D2[0] D1[1] D2[1]
	__m128i D4 = _mm_shuffle_epi32(D3, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D4, mask);
	_mm_store_si128((__m128i*)&y, E);
#else
    y = make_int4(a.x == b, a.y == b, a.z == b, a.w == b);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator !=(scalar4 a, scalar4 b)
{
	int4 y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128 A = _mm_load_ps((float *)&a);
	__m128 B = _mm_load_ps((float *)&b);
	__m128i C = _mm_castps_si128(_mm_cmpneq_ps(A, B)); // Result is 0 or -1
	__m128i D = _mm_and_si128(C, mask); // AND to get 0 or 1
	_mm_store_si128((__m128i*)&y, D);
#elif defined(HAS_AVX2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m256d A = _mm256_loadu_pd((double *)&a);
	__m256d B = _mm256_loadu_pd((double *)&b);
	__m256i C = _mm256_castpd_si256(_mm256_cmp_pd(A, B, _CMP_NEQ_OQ));
	__m128i D1 = _mm256_extracti128_si256(C, 0);
	__m128i D2 = _mm256_extracti128_si256(C, 1);
	__m128i D3 = _mm_shuffle_epi32(D1, 0x4); // 1000b
	__m128i D4 = _mm_shuffle_epi32(D2, 0x4); // 1000b
	__m128i D5 = _mm_unpacklo_epi32(D3, D4); // D1[0] D2[0] D1[1] D2[1]
	__m128i D6 = _mm_shuffle_epi32(D5, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D6, mask);
	_mm_store_si128((__m128i*)&y, E);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128d A1 = _mm_load_pd((double *)&a.x);
	__m128d A2 = _mm_load_pd((double *)&a.z);
	__m128d B1 = _mm_load_pd((double *)&b.x);
	__m128d B2 = _mm_load_pd((double *)&b.z);
	__m128i C1 = _mm_castpd_si128(_mm_cmpneq_pd(A1,B1));
	__m128i C2 = _mm_castpd_si128(_mm_cmpneq_pd(A2,B2));
	__m128i D1 = _mm_shuffle_epi32(C1, 0x8); // 1000b
	__m128i D2 = _mm_shuffle_epi32(C2, 0x8); // 1000b
	__m128i D3 = _mm_unpacklo_epi32(D1, D2); // D1[0] D2[0] D1[1] D2[1]
	__m128i D4 = _mm_shuffle_epi32(D3, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D4, mask);
	_mm_store_si128((__m128i*)&y, E);
#else
    y = make_int4(a.x != b.x, a.y != b.y, a.z != b.z, a.w != b.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator !=(scalar a, scalar4 b)
{
	int4 y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128 A = _mm_load1_ps((float *)&a);
	__m128 B = _mm_load_ps((float *)&b);
	__m128i C = _mm_castps_si128(_mm_cmpneq_ps(A, B)); // Result is 0 or -1
	__m128i D = _mm_and_si128(C, mask); // AND to get 0 or 1
	_mm_store_si128((__m128i*)&y, D);
#elif defined(HAS_AVX2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m256d A = _mm256_load1_pd(a);
	__m256d B = _mm256_loadu_pd((double *)&b);
	__m256i C = _mm256_castpd_si256(_mm256_cmp_pd(A, B, _CMP_NEQ_OQ));
	__m128i D1 = _mm256_extracti128_si256(C, 0);
	__m128i D2 = _mm256_extracti128_si256(C, 1);
	__m128i D3 = _mm_shuffle_epi32(D1, 0x4); // 1000b
	__m128i D4 = _mm_shuffle_epi32(D2, 0x4); // 1000b
	__m128i D5 = _mm_unpacklo_epi32(D3, D4); // D1[0] D2[0] D1[1] D2[1]
	__m128i D6 = _mm_shuffle_epi32(D5, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D6, mask);
	_mm_store_si128((__m128i*)&y, E);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128d A = _mm_load1_pd((double *)&a);
	__m128d B1 = _mm_load_pd((double *)&b.x);
	__m128d B2 = _mm_load_pd((double *)&b.z);
	__m128i C1 = _mm_castpd_si128(_mm_cmpneq_pd(A,B1));
	__m128i C2 = _mm_castpd_si128(_mm_cmpneq_pd(A,B2));
	__m128i D1 = _mm_shuffle_epi32(C1, 0x8); // 1000b
	__m128i D2 = _mm_shuffle_epi32(C2, 0x8); // 1000b
	__m128i D3 = _mm_unpacklo_epi32(D1, D2); // D1[0] D2[0] D1[1] D2[1]
	__m128i D4 = _mm_shuffle_epi32(D3, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D4, mask);
	_mm_store_si128((__m128i*)&y, E);
#else
    y = make_int4(a != b.x, a != b.y, a != b.z, a != b.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator !=(scalar4 a, scalar b)
{
	int4 y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128 A = _mm_load_ps((float *)&a);
	__m128 B = _mm_load1_ps((float *)&b);
	__m128i C = _mm_castps_si128(_mm_cmpneq_ps(A, B)); // Result is 0 or -1
	__m128i D = _mm_and_si128(C, mask); // AND to get 0 or 1
	_mm_store_si128((__m128i*)&y, D);
#elif defined(HAS_AVX2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m256d A = _mm256_loadu_pd((double *)&a);
	__m256d B = _mm256_load1_pd(b);
	__m256i C = _mm256_castpd_si256(_mm256_cmp_pd(A, B, _CMP_NEQ_OQ));
	__m128i D1 = _mm256_extracti128_si256(C, 0);
	__m128i D2 = _mm256_extracti128_si256(C, 1);
	__m128i D3 = _mm_shuffle_epi32(D1, 0x4); // 1000b
	__m128i D4 = _mm_shuffle_epi32(D2, 0x4); // 1000b
	__m128i D5 = _mm_unpacklo_epi32(D3, D4); // D1[0] D2[0] D1[1] D2[1]
	__m128i D6 = _mm_shuffle_epi32(D5, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D6, mask);
	_mm_store_si128((__m128i*)&y, E);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128i mask = _mm_set1_epi32(1);
	__m128d A1 = _mm_load_pd((double *)&a.x);
	__m128d A2 = _mm_load_pd((double *)&a.z);
	__m128d B = _mm_load1_pd((double *)&b);
	__m128i C1 = _mm_castpd_si128(_mm_cmpneq_pd(A1,B));
	__m128i C2 = _mm_castpd_si128(_mm_cmpneq_pd(A2,B));
	__m128i D1 = _mm_shuffle_epi32(C1, 0x8); // 1000b
	__m128i D2 = _mm_shuffle_epi32(C2, 0x8); // 1000b
	__m128i D3 = _mm_unpacklo_epi32(D1, D2); // D1[0] D2[0] D1[1] D2[1]
	__m128i D4 = _mm_shuffle_epi32(D3, 0xd8); // 11011000b
	__m128i E = _mm_and_si128(D4, mask);
	_mm_store_si128((__m128i*)&y, E);
#else
    y = make_int4(a.x != b, a.y != b, a.z != b, a.w != b);
#endif
	return y;
}
namespace quasar {
INLINE scalar4 __device__ __HOST_V__ autocast(int4 a, ReturnType<scalar4>)
{
	scalar4 y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128i A = _mm_load_si128((__m128i*)&a);
	__m128 B = _mm_cvtepi32_ps(A);
	_mm_store_ps((float *)&y, B);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m128i A = _mm_load_si128((__m128i*)&a);
	__m256d B = _mm256_cvtepi32_pd(A);
	_mm256_store_pd((double*)&y, B);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128i A1 = _mm_load_si128((__m128i*)&a);
	__m128i A2 = _mm_shuffle_epi32(A1, 0xE); // 1110b
	__m128d B1 = _mm_cvtepi32_pd(A1);
	__m128d B2 = _mm_cvtepi32_pd(A2);
	_mm_store_pd((double *)&y.x, B1);
	_mm_store_pd((double *)&y.z, B2);
#else
	y = make_scalar4(scalar(a.x), scalar(a.y), scalar(a.z), scalar(a.z));
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ autocast(scalar4 a, ReturnType<int4>)
{
	int4 y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128 A = _mm_load_ps((float *)&a.x);
	__m128i B = _mm_cvtps_epi32(A);
	_mm_store_si128((__m128i*)&y, B);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A = _mm256_load_pd((double *)&a.x);
	__m128i B = _mm256_cvtpd_epi32(A);
	_mm_store_si128((__m128i*)&y, B);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128d A1 = _mm_load_pd((double *)&a.x);
	__m128d A2 = _mm_load_pd((double *)&a.z);
	__m128i B1 = _mm_cvtpd_epi32(A1);
	__m128i B2 = _mm_cvtpd_epi32(A2);
	__m128i C = _mm_unpacklo_epi32(B1,B2);
	__m128i D = _mm_shuffle_epi32(C, 0xd8); // 11011000b
	_mm_store_si128((__m128i*)&y, D);
#else
	y = make_int4(int(a.x), int(a.y), int(a.z), int(a.w));
#endif
    return y;
}

} // namespace quasar
INLINE scalar4 __device__ __HOST_V__ abs(scalar4 a)
{
	scalar4 y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128 A = _mm_load_ps((float *)&a);
	__m128 mask = _mm_castsi128_ps(_mm_set1_epi32(0x7FFFFFFF));
	__m128 B = _mm_and_ps(A, mask);
	_mm_store_ps((float*)&y, B);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A = _mm256_load_pd((double *)&a);
	__m256d mask = _mm256_castsi256_pd(_mm256_set1_epi64x(0x7FFFFFFFFFFFFFFFLL));
	__m256d B = _mm256_and_pd(A, mask);
	_mm256_store_pd((double*)&y, B);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR) && defined(HAS_mm_set1_epi64x)
	__m128d A1 = _mm_load_pd((double *)&a.x);
	__m128d A2 = _mm_load_pd((double *)&a.z);
	__m128d mask = _mm_castsi128_pd(_mm_set1_epi64x(0x7FFFFFFFFFFFFFFFLL));
	__m128d B1 = _mm_and_pd(A1, mask);
	__m128d B2 = _mm_and_pd(A2, mask);
	_mm_store_pd((double*)&y.x, B1);
	_mm_store_pd((double*)&y.z, B2);
#else
	y = make_scalar4(fabs(a.x), fabs(a.y), fabs(a.z), fabs(a.w));
#endif
	return y;
}
INLINE scalar4 __device__ __HOST_V__ min(scalar4 a, scalar4 b)
{
	scalar4 y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128 A = _mm_load_ps((float *)&a);
	__m128 B = _mm_load_ps((float *)&b);
	__m128 C = _mm_min_ps(A, B);
	_mm_store_ps((float*)&y, C);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A = _mm256_load_pd((double *)&a);
	__m256d B = _mm256_load_pd((double *)&b);
	__m256d C = _mm256_min_pd(A, B);
	_mm256_store_pd((double*)&y, C);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128d A1 = _mm_load_pd((double *)&a.x);
	__m128d A2 = _mm_load_pd((double *)&a.z);
	__m128d B1 = _mm_load_pd((double *)&b.x);
	__m128d B2 = _mm_load_pd((double *)&b.z);
	_mm_store_pd((double*)&y.x, _mm_min_pd(A1, B1));
	_mm_store_pd((double*)&y.z, _mm_min_pd(A2, B2));
#else
	y = make_scalar4(min(a.x,b.x), min(a.y,b.y), min(a.z,b.z), min(a.w,b.w));
#endif
	return y;
}
INLINE scalar4 __device__ __HOST_V__ max(scalar4 a, scalar4 b)
{
	scalar4 y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128 A = _mm_load_ps((float *)&a);
	__m128 B = _mm_load_ps((float *)&b);
	__m128 C = _mm_max_ps(A, B);
	_mm_store_ps((float*)&y, C);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A = _mm256_load_pd((double *)&a);
	__m256d B = _mm256_load_pd((double *)&b);
	__m256d C = _mm256_max_pd(A, B);
	_mm256_store_pd((double*)&y, C);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128d A1 = _mm_load_pd((double *)&a.x);
	__m128d A2 = _mm_load_pd((double *)&a.z);
	__m128d B1 = _mm_load_pd((double *)&b.x);
	__m128d B2 = _mm_load_pd((double *)&b.z);
	_mm_store_pd((double*)&y.x, _mm_max_pd(A1, B1));
	_mm_store_pd((double*)&y.z, _mm_max_pd(A2, B2));
#else
	y = make_scalar4(max(a.x,b.x), max(a.y,b.y), max(a.z,b.z), max(a.w,b.w));
#endif
	return y;
}
INLINE scalar4 __device__ __HOST_V__ floor(scalar4 a)
{
	scalar4 y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128 A = _mm_load_ps((float *)&a);
	_mm_store_ps((float*)&y, _mm_floor_ps(A));
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A = _mm256_load_pd((double *)&a);
	_mm256_store_pd((double*)&y, _mm256_floor_pd(A));
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128d A1 = _mm_load_pd((double *)&a.x);
	__m128d A2 = _mm_load_pd((double *)&a.z);
	_mm_store_pd((double*)&y.x, _mm_floor_pd(A1));
	_mm_store_pd((double*)&y.z, _mm_floor_pd(A2));
#else
	y = make_scalar4(floor(a.x), floor(a.y), floor(a.z), floor(a.w));
#endif
	return y;
}
INLINE scalar4 __device__ __HOST_V__ ceil(scalar4 a)
{
	scalar4 y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128 A = _mm_load_ps((float *)&a);
	_mm_store_ps((float*)&y, _mm_ceil_ps(A));
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A = _mm256_load_pd((double *)&a);
	_mm256_store_pd((double*)&y, _mm256_ceil_pd(A));
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128d A1 = _mm_load_pd((double *)&a.x);
	__m128d A2 = _mm_load_pd((double *)&a.z);
	_mm_store_pd((double*)&y.x, _mm_ceil_pd(A1));
	_mm_store_pd((double*)&y.z, _mm_ceil_pd(A2));
#else
	y = make_scalar4(ceil(a.x), ceil(a.y), ceil(a.z), ceil(a.w));
#endif
	return y;
}
INLINE scalar4 __device__ __HOST_V__ sqrt(scalar4 a)
{
	scalar4 y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128 A = _mm_load_ps((float *)&a);
	_mm_store_ps((float*)&y, _mm_sqrt_ps(A));
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A = _mm256_load_pd((double *)&a);
	_mm256_store_pd((double*)&y, _mm256_sqrt_pd(A));
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128d A1 = _mm_load_pd((double *)&a.x);
	__m128d A2 = _mm_load_pd((double *)&a.z);
	_mm_store_pd((double*)&y.x, _mm_sqrt_pd(A1));
	_mm_store_pd((double*)&y.z, _mm_sqrt_pd(A2));
#else
	y = make_scalar4(sqrt(a.x), sqrt(a.y), sqrt(a.z), sqrt(a.w));
#endif
	return y;
}
INLINE scalar4 __device__ __HOST_V__ rsqrt(scalar4 a)
{
	scalar4 y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128 A = _mm_load_ps((float *)&a);
	_mm_store_ps((float*)&y, _mm_rsqrt_ps(A));
#else
	y = make_scalar4(rsqrt(a.x), rsqrt(a.y), rsqrt(a.z), rsqrt(a.w));
#endif
	return y;
}

INLINE scalar4 __device__ __HOST_V__ round(scalar4 a)
{
	scalar4 y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	const int RM = _MM_FROUND_TO_NEAREST_INT | _MM_FROUND_NO_EXC;
	__m128 A = _mm_load_ps((float *)&a);
	_mm_store_ps((float*)&y, _mm_round_ps(A, RM));
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	const int RM = _MM_FROUND_TO_NEAREST_INT | _MM_FROUND_NO_EXC;
	__m256d A = _mm256_load_pd((double *)&a);
	_mm256_store_pd((double*)&y, _mm256_round_pd(A, RM));
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	const int RM = _MM_FROUND_TO_NEAREST_INT | _MM_FROUND_NO_EXC;
	__m128d A1 = _mm_load_pd((double *)&a.x);
	__m128d A2 = _mm_load_pd((double *)&a.z);
	_mm_store_pd((double*)&y.x, _mm_round_pd(A1, RM));
	_mm_store_pd((double*)&y.z, _mm_round_pd(A2, RM));
#else
	y = make_scalar4(round(a.x), round(a.y), round(a.z), round(a.w));
#endif
	return y;
}

INLINE scalar4 __device__ __HOST_V__ frac(scalar4 a)
{
	return a - floor(a);
}

INLINE scalar4 __device__ __HOST_V__ sign(scalar4 a)
{
	scalar4 y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128 A = _mm_load_ps((float *)&a);
	__m128 Z = _mm_cmpeq_ps(A, _mm_set1_ps(0));
	__m128 one = _mm_set1_ps(1);
	__m128 mask = _mm_castsi128_ps(_mm_set1_epi32(0x80000000U));
	__m128 B = _mm_or_ps(_mm_and_ps(A, mask), one); // -1 or 1
	__m128 E = _mm_andnot_ps(Z, B);
	_mm_store_ps((float*)&y, E);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A = _mm256_load_pd((double *)&a);
	__m256d Z = _mm256_cmp_pd(A, _mm256_set1_pd(0), _CMP_EQ_US);
	__m256d one = _mm256_set1_pd(1);
	__m256d mask = _mm256_castsi256_pd(_mm256_set1_epi64x(0x8000000000000000ULL));
	__m256d B = _mm256_or_pd(_mm256_and_pd(A, mask), one); // -1 or 1
	__m256d E = _mm256_andnot_pd(Z, B);
	_mm256_store_pd((double*)&y, E);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR) && defined(HAS_mm_set1_epi64x)
	__m128d A1 = _mm_load_pd((double *)&a.x);
	__m128d A2 = _mm_load_pd((double *)&a.z);
	__m128d zero = _mm_set1_pd(0);
	__m128d one = _mm_set1_pd(1);
	__m128d Z1 = _mm_cmpeq_pd(A1, zero);
	__m128d Z2 = _mm_cmpeq_pd(A2, zero);
	__m128d mask = _mm_castsi128_pd(_mm_set1_epi64x(0x8000000000000000ULL));
	__m128d B1 = _mm_or_pd(_mm_and_pd(A1, mask), one); // -1 or 1
	__m128d B2 = _mm_or_pd(_mm_and_pd(A2, mask), one); // -1 or 1
	__m128d E1 = _mm_andnot_pd(Z1, B1);
	__m128d E2 = _mm_andnot_pd(Z2, B2);
	_mm_store_pd((double*)&y.x, E1);
	_mm_store_pd((double*)&y.z, E2);
#else
	y = make_scalar4(sign(a.x), sign(a.y), sign(a.z), sign(a.w));
#endif
	return y;
}
INLINE scalar __device__ __HOST_V__ sum(scalar4 a)
{
	scalar y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128 A = _mm_load_ps((float *)&a);
	__m128 B = _mm_shuffle_ps(A, A, 0xE); // 1110b
	__m128 C = _mm_add_ps(A, B);
	__m128 D = _mm_add_ps(C, _mm_shuffle_ps(C, C, 1)); // 01b
	y = _mm_cvtss_f32(D);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128d A = _mm_load_pd((double *)&a.x);
	__m128d B = _mm_load_pd((double *)&a.z);
	__m128d C = _mm_add_pd(A, B);
	__m128d D = _mm_add_pd(C, _mm_shuffle_pd(C, C, 1)); // 01b
	y = _mm_cvtsd_f64(D);
#else
	y = a.x + a.y + a.z + a.w;
#endif
	return y;
}

INLINE scalar __device__ __HOST_V__ prod(scalar4 a)
{
	scalar y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128 A = _mm_load_ps((float *)&a);
	__m128 B = _mm_shuffle_ps(A, A, 0xE); // 1110b
	__m128 C = _mm_mul_ps(A, B);
	__m128 D = _mm_mul_ps(C, _mm_shuffle_ps(C, C, 1)); // 01b
	y = _mm_cvtss_f32(D);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128d A = _mm_load_pd((double *)&a.x);
	__m128d B = _mm_load_pd((double *)&a.z);
	__m128d C = _mm_mul_pd(A, B);
	__m128d D = _mm_mul_pd(C, _mm_shuffle_pd(C, C, 1)); // 01b
	y = _mm_cvtsd_f64(D);
#else
	y = a.x * a.y * a.z * a.w;
#endif
	return y;
}
INLINE scalar __device__ __HOST_V__ min(scalar4 a)
{
	scalar y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128 A = _mm_load_ps((float *)&a);
	__m128 B = _mm_shuffle_ps(A, A, 0xE); // 1110b
	__m128 C = _mm_min_ps(A, B);
	__m128 D = _mm_min_ps(C, _mm_shuffle_ps(C, C, 1)); // 01b
	y = _mm_cvtss_f32(D);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128d A = _mm_load_pd((double *)&a.x);
	__m128d B = _mm_load_pd((double *)&a.z);
	__m128d C = _mm_min_pd(A, B);
	__m128d D = _mm_min_pd(C, _mm_shuffle_pd(C, C, 1)); // 01b
	y = _mm_cvtsd_f64(D);
#else
	y = min(min(a.x, a.y), min(a.z, a.w));
#endif
	return y;
}
INLINE scalar __device__ __HOST_V__ max(scalar4 a)
{
	scalar y;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128 A = _mm_load_ps((float *)&a);
	__m128 B = _mm_shuffle_ps(A, A, 0xE); // 1110b
	__m128 C = _mm_max_ps(A, B);
	__m128 D = _mm_max_ps(C, _mm_shuffle_ps(C, C, 1)); // 01b
	y = _mm_cvtss_f32(D);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128d A = _mm_load_pd((double *)&a.x);
	__m128d B = _mm_load_pd((double *)&a.z);
	__m128d C = _mm_max_pd(A, B);
	__m128d D = _mm_max_pd(C, _mm_shuffle_pd(C, C, 1)); // 01b
	y = _mm_cvtsd_f64(D);
#else
	y = max(max(a.x, a.y), max(a.z, a.w));
#endif
	return y;
}

INLINE scalar __device__ __HOST_V__ dotprod(scalar4 a1, scalar4 a2)
{
	scalar y;
#if defined(HAS_SSE) && !defined(DBL_SCALAR)
	__m128 A1 = _mm_load_ps((float *)&a1);
	__m128 A2 = _mm_load_ps((float *)&a2);
	__m128 A = _mm_mul_ps(A1, A2);
	__m128 B = _mm_shuffle_ps(A, A, 0xE); // 1110b
	__m128 C = _mm_add_ps(A, B);
	__m128 D = _mm_add_ps(C, _mm_shuffle_ps(C, C, 1)); // 01b
	y = _mm_cvtss_f32(D);
#elif defined(HAS_SSE2) && defined(DBL_SCALAR)
	__m128d A1 = _mm_load_pd((double *)&a1.x);
	__m128d A2 = _mm_load_pd((double *)&a2.x);
	__m128d B1 = _mm_load_pd((double *)&a1.z);
	__m128d B2 = _mm_load_pd((double *)&a2.z);
	__m128d C = _mm_add_pd(_mm_mul_pd(A1,A2), _mm_mul_pd(B1,B2));
	__m128d D = _mm_add_pd(C, _mm_shuffle_pd(C, C, 1)); // 01b
	y = _mm_cvtsd_f64(D);
#else
	y = a1.x*a2.x + a1.y*a2.y + a1.z*a2.z + a1.w*a2.w;
#endif
	return y;
}

INLINE int4 __device__ __HOST_V__ operator-(int4 a)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_set1_epi32(0);
	__m128i B = _mm_load_si128((__m128i *)&a);
	__m128i C = _mm_sub_epi32(A, B);
	_mm_store_si128((__m128i *)&y, C);
#else
	y = make_int4(-a.x, -a.y, -a.z, -a.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator +(int4 a, int4 b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i *)&a);
	__m128i B = _mm_load_si128((__m128i *)&b);
	__m128i C = _mm_add_epi32(A, B);
	_mm_store_si128((__m128i *)&y, C);
#else
    y = make_int4(a.x + b.x, a.y + b.y, a.z + b.z, a.w + b.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator +(int a, int4 b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_set1_epi32(a);
	__m128i B = _mm_load_si128((__m128i *)&b);
	__m128i C = _mm_add_epi32(A, B);
	_mm_store_si128((__m128i *)&y, C);
#else
    y = make_int4(a + b.x, a + b.y, a + b.z, a + b.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator +(int4 a, int b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i *)&a);
	__m128i B = _mm_set1_epi32(b);
	__m128i C = _mm_add_epi32(A, B);
	_mm_store_si128((__m128i *)&y, C);
#else
    y = make_int4(a.x + b, a.y + b, a.z + b, a.w + b);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator -(int4 a, int4 b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i *)&a);
	__m128i B = _mm_load_si128((__m128i *)&b);
	__m128i C = _mm_sub_epi32(A, B);
	_mm_store_si128((__m128i *)&y, C);
#else
    y = make_int4(a.x - b.x, a.y - b.y, a.z - b.z, a.w - b.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator -(int a, int4 b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_set1_epi32(a);
	__m128i B = _mm_load_si128((__m128i *)&b);
	__m128i C = _mm_sub_epi32(A, B);
	_mm_store_si128((__m128i *)&y, C);
#else
    y = make_int4(a - b.x, a - b.y, a - b.z, a - b.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator -(int4 a, int b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i *)&a);
	__m128i B = _mm_set1_epi32(b);
	__m128i C = _mm_sub_epi32(A, B);
	_mm_store_si128((__m128i *)&y, C);
#else
    y = make_int4(a.x - b, a.y - b, a.z - b, a.w - b);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator *(int4 a, int4 b)
{
	int4 y;
#if defined(HAS_SSE41)
	__m128i A1 = _mm_load_si128((__m128i *)&a); // a[0] ... a[2] ...
	__m128i A2 = _mm_shuffle_epi32(A1, 0x31); // 110001b  a[1] ... a[3]
	__m128i B1 = _mm_load_si128((__m128i *)&b); // b[0] ... b[2] ...
	__m128i B2 = _mm_shuffle_epi32(B1, 0x31); // 110001b  b[1] ... b[3]
	__m128i C1 = _mm_mul_epi32(A1, B1);
	__m128i C2 = _mm_mul_epi32(A2, B2);
	__m128i D1 = _mm_shuffle_epi32(C1, 0x8); // 1000b
	__m128i D2 = _mm_shuffle_epi32(C2, 0x8); // 1000b
	__m128i D3 = _mm_unpacklo_epi32(D1, D2); // D1[0] D2[0] D1[1] D2[1]
	_mm_store_si128((__m128i *)&y, D3);
#else
    y = make_int4(a.x * b.x, a.y * b.y, a.z * b.z, a.w * b.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator *(int a, int4 b)
{
	int4 y;
#if defined(HAS_SSE41)
	__m128i A1 = _mm_set1_epi32(a);
	__m128i A2 = _mm_set1_epi32(a);
	__m128i B1 = _mm_load_si128((__m128i *)&b); // b[0] ... b[2] ...
	__m128i B2 = _mm_shuffle_epi32(B1, 0x31); // 110001b  b[1] ... b[3]
	__m128i C1 = _mm_mul_epi32(A1, B1);
	__m128i C2 = _mm_mul_epi32(A2, B2);
	__m128i D1 = _mm_shuffle_epi32(C1, 0x8); // 1000b
	__m128i D2 = _mm_shuffle_epi32(C2, 0x8); // 1000b
	__m128i D3 = _mm_unpacklo_epi32(D1, D2); // D1[0] D2[0] D1[1] D2[1]
	_mm_store_si128((__m128i *)&y, D3);
#else
    y = make_int4(a * b.x, a * b.y, a * b.z, a * b.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator *(int4 a, int b)
{
	int4 y;
#if defined(HAS_SSE41)
	__m128i A1 = _mm_load_si128((__m128i *)&a); // a[0] ... a[2] ...
	__m128i A2 = _mm_shuffle_epi32(A1, 0x31); // 110001b  a[1] ... a[3]
	__m128i B1 = _mm_set1_epi32(b);
	__m128i B2 = _mm_set1_epi32(b);
	__m128i C1 = _mm_mul_epi32(A1, B1);
	__m128i C2 = _mm_mul_epi32(A2, B2);
	__m128i D1 = _mm_shuffle_epi32(C1, 0x8); // 1000b
	__m128i D2 = _mm_shuffle_epi32(C2, 0x8); // 1000b
	__m128i D3 = _mm_unpacklo_epi32(D1, D2); // D1[0] D2[0] D1[1] D2[1]
	_mm_store_si128((__m128i *)&y, D3);
#else
    y = make_int4(a.x * b, a.y * b, a.z * b, a.w * b);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator >(int4 a, int4 b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i *)&a);
	__m128i B = _mm_load_si128((__m128i *)&b);
	__m128i C = _mm_cmpgt_epi32(A, B);
	__m128i D = _mm_and_si128(C, _mm_set1_epi32(1));
	_mm_store_si128((__m128i *)&y, D);
#else
    y = make_int4(a.x > b.x, a.y > b.y, a.z > b.z, a.w > b.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator >(int a, int4 b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_set1_epi32(a);
	__m128i B = _mm_load_si128((__m128i *)&b);
	__m128i C = _mm_cmpgt_epi32(A, B);
	__m128i D = _mm_and_si128(C, _mm_set1_epi32(1));
	_mm_store_si128((__m128i *)&y, D);
#else
    y = make_int4(a > b.x, a > b.y, a > b.z, a > b.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator >(int4 a, int b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i *)&a);
	__m128i B = _mm_set1_epi32(b);
	__m128i C = _mm_cmpgt_epi32(A, B);
	__m128i D = _mm_and_si128(C, _mm_set1_epi32(1));
	_mm_store_si128((__m128i *)&y, D);
#else
    y = make_int4(a.x > b, a.y > b, a.z > b, a.w > b);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator <(int4 a, int4 b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i *)&a);
	__m128i B = _mm_load_si128((__m128i *)&b);
	__m128i C = _mm_cmplt_epi32(A, B);
	__m128i D = _mm_and_si128(C, _mm_set1_epi32(1));
	_mm_store_si128((__m128i *)&y, D);
#else
    y = make_int4(a.x < b.x, a.y < b.y, a.z < b.z, a.w < b.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator <(int a, int4 b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_set1_epi32(a);
	__m128i B = _mm_load_si128((__m128i *)&b);
	__m128i C = _mm_cmplt_epi32(A, B);
	__m128i D = _mm_and_si128(C, _mm_set1_epi32(1));
	_mm_store_si128((__m128i *)&y, D);
#else
    y = make_int4(a < b.x, a < b.y, a < b.z, a < b.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator <(int4 a, int b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i *)&a);
	__m128i B = _mm_set1_epi32(b);
	__m128i C = _mm_cmplt_epi32(A, B);
	__m128i D = _mm_and_si128(C, _mm_set1_epi32(1));
	_mm_store_si128((__m128i *)&y, D);
#else
    y = make_int4(a.x < b, a.y < b, a.z < b, a.w < b);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator >=(int4 a, int4 b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i *)&a);
	__m128i B = _mm_load_si128((__m128i *)&b);
	__m128i C = _mm_cmpge_epi32(A, B);
	__m128i D = _mm_and_si128(C, _mm_set1_epi32(1));
	_mm_store_si128((__m128i *)&y, D);
#else
    y = make_int4(a.x >= b.x, a.y >= b.y, a.z >= b.z, a.w >= b.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator >=(int a, int4 b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_set1_epi32(a);
	__m128i B = _mm_load_si128((__m128i *)&b);
	__m128i C = _mm_cmpge_epi32(A, B);
	__m128i D = _mm_and_si128(C, _mm_set1_epi32(1));
	_mm_store_si128((__m128i *)&y, D);
#else
    y = make_int4(a >= b.x, a >= b.y, a >= b.z, a >= b.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator >=(int4 a, int b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i *)&a);
	__m128i B = _mm_set1_epi32(b);
	__m128i C = _mm_cmpge_epi32(A, B);
	__m128i D = _mm_and_si128(C, _mm_set1_epi32(1));
	_mm_store_si128((__m128i *)&y, D);
#else
    y = make_int4(a.x >= b, a.y >= b, a.z >= b, a.w >= b);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator <=(int4 a, int4 b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i *)&a);
	__m128i B = _mm_load_si128((__m128i *)&b);
	__m128i C = _mm_cmple_epi32(A, B);
	__m128i D = _mm_and_si128(C, _mm_set1_epi32(1));
	_mm_store_si128((__m128i *)&y, D);
#else
    y = make_int4(a.x <= b.x, a.y <= b.y, a.z <= b.z, a.w <= b.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator <=(int a, int4 b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_set1_epi32(a);
	__m128i B = _mm_load_si128((__m128i *)&b);
	__m128i C = _mm_cmple_epi32(A, B);
	__m128i D = _mm_and_si128(C, _mm_set1_epi32(1));
	_mm_store_si128((__m128i *)&y, D);
#else
    y = make_int4(a <= b.x, a <= b.y, a <= b.z, a <= b.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator <=(int4 a, int b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i *)&a);
	__m128i B = _mm_set1_epi32(b);
	__m128i C = _mm_cmple_epi32(A, B);
	__m128i D = _mm_and_si128(C, _mm_set1_epi32(1));
	_mm_store_si128((__m128i *)&y, D);
#else
    y = make_int4(a.x <= b, a.y <= b, a.z <= b, a.w <= b);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator ==(int4 a, int4 b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i *)&a);
	__m128i B = _mm_load_si128((__m128i *)&b);
	__m128i C = _mm_cmpeq_epi32(A, B);
	__m128i D = _mm_and_si128(C, _mm_set1_epi32(1));
	_mm_store_si128((__m128i *)&y, D);
#else
    y = make_int4(a.x == b.x, a.y == b.y, a.z == b.z, a.w == b.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator ==(int a, int4 b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_set1_epi32(a);
	__m128i B = _mm_load_si128((__m128i *)&b);
	__m128i C = _mm_cmpeq_epi32(A, B);
	__m128i D = _mm_and_si128(C, _mm_set1_epi32(1));
	_mm_store_si128((__m128i *)&y, D);
#else
    y = make_int4(a == b.x, a == b.y, a == b.z, a == b.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator ==(int4 a, int b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i *)&a);
	__m128i B = _mm_set1_epi32(b);
	__m128i C = _mm_cmpeq_epi32(A, B);
	__m128i D = _mm_and_si128(C, _mm_set1_epi32(1));
	_mm_store_si128((__m128i *)&y, D);
#else
    y = make_int4(a.x == b, a.y == b, a.z == b, a.w == b);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator !=(int4 a, int4 b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i *)&a);
	__m128i B = _mm_load_si128((__m128i *)&b);
	__m128i C = _mm_cmpneq_epi32(A, B);
	__m128i D = _mm_and_si128(C, _mm_set1_epi32(1));
	_mm_store_si128((__m128i *)&y, D);
#else
    y = make_int4(a.x != b.x, a.y != b.y, a.z != b.z, a.w != b.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator !=(int a, int4 b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_set1_epi32(a);
	__m128i B = _mm_load_si128((__m128i *)&b);
	__m128i C = _mm_cmpneq_epi32(A, B);
	__m128i D = _mm_and_si128(C, _mm_set1_epi32(1));
	_mm_store_si128((__m128i *)&y, D);
#else
    y = make_int4(a != b.x, a != b.y, a != b.z, a != b.w);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ operator !=(int4 a, int b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i *)&a);
	__m128i B = _mm_set1_epi32(b);
	__m128i C = _mm_cmpneq_epi32(A, B);
	__m128i D = _mm_and_si128(C, _mm_set1_epi32(1));
	_mm_store_si128((__m128i *)&y, D);
#else
    y = make_int4(a.x != b, a.y != b, a.z != b, a.w != b);
#endif
	return y;
}
INLINE int4 __device__ __HOST_V__ _not_(int4 a)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i*)&a);
	__m128i B = _mm_not_epi32(A);
	_mm_store_si128((__m128i*)&y, B);
#else
    y = make_int4(_not_(a.x), _not_(a.y), _not_(a.z), _not_(a.w));
#endif
	return y;
}

INLINE int4 __device__ __HOST_V__ _and_(int4 a, int4 b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i*)&a);
	__m128i B = _mm_load_si128((__m128i*)&b);
	__m128i C = _mm_and_si128(A, B);
	_mm_store_si128((__m128i*)&y, C);
#else
    y = make_int4(_and_(a.x,b.x), _and_(a.y,b.y), _and_(a.z,b.z), _and_(a.w,b.w));
#endif
	return y;
}

INLINE int4 __device__ __HOST_V__ _or_(int4 a, int4 b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i*)&a);
	__m128i B = _mm_load_si128((__m128i*)&b);
	__m128i C = _mm_or_si128(A, B);
	_mm_store_si128((__m128i*)&y, C);
#else
    y = make_int4(_or_(a.x,b.x), _or_(a.y,b.y), _or_(a.z,b.z), _or_(a.w,b.w));
#endif
	return y;
}

INLINE int4 __device__ __HOST_V__ _xor_(int4 a, int4 b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i*)&a);
	__m128i B = _mm_load_si128((__m128i*)&b);
	__m128i C = _mm_xor_si128(A, B);
	_mm_store_si128((__m128i*)&y, C);
#else
    y = make_int4(_xor_(a.x,b.x), _xor_(a.y,b.y), _xor_(a.z,b.z), _xor_(a.w,b.w));
#endif
	return y;
}

INLINE int4 __device__ __HOST_V__ lognot(int4 a)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i*)&a);
	__m128i B = _mm_cmpeq_epi32(A, _mm_set1_epi32(0));
	__m128i C = _mm_and_si128(B, _mm_set1_epi32(1));
	_mm_store_si128((__m128i*)&y, C);
#else
    y = make_int4(a.x == 0, a.y == 0, a.z == 0, a.w == 0);
#endif
	return y;
}

INLINE int4 __device__ __HOST_V__ logand(int4 a, int4 b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i P1 = _mm_set1_epi32(0);
	__m128i P2 = _mm_set1_epi32(1);
	__m128i A = _mm_add_epi32(_mm_cmpeq_epi32(_mm_load_si128((__m128i*)&a),P1),P2);
	__m128i B = _mm_add_epi32(_mm_cmpeq_epi32(_mm_load_si128((__m128i*)&b),P1),P2);
	__m128i C = _mm_and_si128(A, B);
	_mm_store_si128((__m128i*)&y, C);
#else
    y = make_int4(logand(a.x,b.x), logand(a.y,b.y), logand(a.z,b.z), logand(a.w,b.w));
#endif
	return y;
}

INLINE int4 __device__ __HOST_V__ logor(int4 a, int4 b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i P1 = _mm_set1_epi32(0);
	__m128i P2 = _mm_set1_epi32(1);
	__m128i A = _mm_add_epi32(_mm_cmpeq_epi32(_mm_load_si128((__m128i*)&a),P1),P2);
	__m128i B = _mm_add_epi32(_mm_cmpeq_epi32(_mm_load_si128((__m128i*)&b),P1),P2);
	__m128i C = _mm_or_si128(A, B);
	_mm_store_si128((__m128i*)&y, C);
#else
    y = make_int4(logor(a.x,b.x), logor(a.y,b.y), logor(a.z,b.z), logor(a.w,b.w));
#endif
	return y;
}

INLINE int4 __device__ __HOST_V__ logxor(int4 a, int4 b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i P1 = _mm_set1_epi32(0);
	__m128i P2 = _mm_set1_epi32(1);
	__m128i A = _mm_add_epi32(_mm_cmpeq_epi32(_mm_load_si128((__m128i*)&a),P1),P2);
	__m128i B = _mm_add_epi32(_mm_cmpeq_epi32(_mm_load_si128((__m128i*)&b),P1),P2);
	__m128i C = _mm_xor_si128(A, B);
	_mm_store_si128((__m128i*)&y, C);
#else
    y = make_int4(logxor(a.x,b.x), logxor(a.y,b.y), logxor(a.z,b.z), logxor(a.w,b.w));
#endif
	return y;
}

INLINE int4 __device__ __HOST_V__ abs(int4 a)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i *)&a);
	__m128i B = _mm_abs_epi32(A);
	_mm_store_si128((__m128i*)&y, B);
#else
	y = make_int4(abs(a.x), abs(a.y), abs(a.z), abs(a.w));
#endif
	return y;
}

INLINE int4 __device__ __HOST_V__ min(int4 a, int4 b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i *)&a);
	__m128i B = _mm_load_si128((__m128i *)&b);
	__m128i C = _mm_min_epi32(A, B);
	_mm_store_si128((__m128i*)&y, C);
#else
	y = make_int4(min(a.x,b.x), min(a.y,b.y), min(a.z,b.z), min(a.w,b.w));
#endif
	return y;
}

INLINE int4 __device__ __HOST_V__ max(int4 a, int4 b)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i *)&a);
	__m128i B = _mm_load_si128((__m128i *)&b);
	__m128i C = _mm_max_epi32(A, B);
	_mm_store_si128((__m128i*)&y, C);
#else
	y = make_int4(max(a.x, b.x), max(a.y, b.y), max(a.z, b.z), max(a.w, b.w));
#endif
	return y;
}

INLINE int4 __device__ __HOST_V__ sign(int4 a)
{
	int4 y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i *)&a);
	__m128i Z = _mm_cmpeq_epi32(A, _mm_set1_epi32(0));
	__m128i mask = _mm_set1_epi32(0x80000000);
	__m128i B = _mm_and_si128(A, mask);
	__m128i C = _mm_srli_epi32(B, 30);
	__m128i D = _mm_sub_epi32(_mm_set1_epi32(1), C);
	__m128i E = _mm_andnot_si128(Z, D); // Handling zero
	_mm_store_si128((__m128i*)&y, E);
#else
	y = make_int4(sign(a.x), sign(a.y), sign(a.z), sign(a.w));
#endif
	return y;
}
INLINE int __device__ __HOST_V__ sum(int4 a)
{
	int y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i *)&a);
	__m128i B = _mm_shuffle_epi32(A, 0xE); // 1110b
	__m128i C = _mm_add_epi32(A, B);
	__m128i D = _mm_add_epi32(C, _mm_shuffle_epi32(C, 1)); // 01b
	y = _mm_cvtsi128_si32(D);
#else
	y = a.x + a.y + a.z + a.w;
#endif
	return y;
}

INLINE int __device__ __HOST_V__ prod(int4 a)
{
	int y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i *)&a);
	__m128i B = _mm_shuffle_epi32(A, 0xB1); // 10110001b
	__m128i C = _mm_mul_epi32(A, B); // A[0]*A[1], 0, A[2]*A[3], 0
	__m128i D = _mm_mul_epi32(C, _mm_shuffle_epi32(C, 2)); // 10b
	y = _mm_cvtsi128_si32(D);
#else
	y = a.x * a.y * a.z * a.w;
#endif
	return y;
}

INLINE int __device__ __HOST_V__ min(int4 a)
{
	int y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i *)&a);
	__m128i B = _mm_shuffle_epi32(A, 0xE); // 1110b
	__m128i C = _mm_min_epi32(A, B);
	__m128i D = _mm_min_epi32(C, _mm_shuffle_epi32(C, 1)); // 01b
	y = _mm_cvtsi128_si32(D);
#else
	y = min(min(a.x, a.y), min(a.z, a.w));
#endif
	return y;
}
INLINE int __device__ __HOST_V__ max(int4 a)
{
	int y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i *)&a);
	__m128i B = _mm_shuffle_epi32(A, 0xE); // 1110b
	__m128i C = _mm_max_epi32(A, B);
	__m128i D = _mm_max_epi32(C, _mm_shuffle_epi32(C, 1)); // 01b
	y = _mm_cvtsi128_si32(D);
#else
	y = max(max(a.x, a.y), max(a.z, a.w));
#endif
	return y;
}

INLINE int __device__ __HOST_V__ _any_(int4 a)
{
	int y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i *)&a);
	__m128i B = _mm_cmpneq_epi32(A, _mm_set1_epi32(0));
	__m128i C = _mm_shuffle_epi32(B, 0xE); // 1110b
	__m128i D = _mm_or_si128(B, C);
	__m128i E = _mm_or_si128(D, _mm_shuffle_epi32(D, 1)); // 01b
	y = -_mm_cvtsi128_si32(E);
#else
	y = a.x || a.y || a.z || a.w;
#endif
	return y;
}

INLINE int __device__ __HOST_V__ _all_(int4 a)
{
	int y;
#if defined(HAS_SSE2)
	__m128i A = _mm_load_si128((__m128i *)&a);
	__m128i B = _mm_cmpneq_epi32(A, _mm_set1_epi32(0));
	__m128i C = _mm_shuffle_epi32(B, 0xE); // 1110b
	__m128i D = _mm_and_si128(B, C);
	__m128i E = _mm_and_si128(D, _mm_shuffle_epi32(D, 1)); // 01b
	y = -_mm_cvtsi128_si32(E);
#else
	y = a.x && a.y && a.z && a.w;
#endif
	return y;
}

INLINE int __device__ __HOST_V__ dotprod(int4 a1, int4 a2)
{
	int y;
#if defined(HAS_SSE2)
	__m128i A1 = _mm_load_si128((__m128i *)&a1);
	__m128i A2 = _mm_load_si128((__m128i *)&a2);
	__m128i A3 = _mm_shuffle_epi32(A1, 0xB1); // 10110001b
	__m128i A4 = _mm_shuffle_epi32(A2, 0xB1); // 10110001b
	__m128i B1 = _mm_mul_epi32(A1, A2); // A1[0]*A2[0] ... A1[2]*A2[2]
	__m128i B2 = _mm_mul_epi32(A3, A4); // A1[1]*A2[1] ... A1[3]*A2[3]
	__m128i C = _mm_add_epi32(B1, B2);
	__m128i D = _mm_add_epi32(C, _mm_shuffle_epi32(C, 2)); // 10b
	y = _mm_cvtsi128_si32(D);
#else
	y = a1.x*a2.x + a1.y*a2.y + a1.z*a2.z + a1.w*a2.w;
#endif
	return y;
}

namespace quasar {

INLINE ushort4 __device__  __HOST_V__ autocast_int(int4 val, IntegerConversionModes mode, ReturnType<ushort4>)
{
	ushort4 b;
	switch (mode)
	{
#if defined(HAS_SSE2) && defined(HAS_mm_set1_epi64x)
	case ConvSaturated:
	{
		__m128i A = _mm_load_si128((__m128i *)&val);
		__m128i B = _mm_max_epi32(_mm_min_epi32(A, _mm_set1_epi32(MAXVAL_uint16)), _mm_set1_epi32(MINVAL_uint16));
		_mm_storel_epi64((__m128i *)&b,  _mm_packus_epi32(B, _mm_set1_epi32(0)));
		break;
	}
	case ConvUnchecked:
	{
		__m128i A = _mm_load_si128((__m128i *)&val);
		__m128i B = _mm_shuffle_epi8(A, _mm_set1_epi64x(0x0d0c090805040100ull));
		_mm_storel_epi64((__m128i *)&b, B);
		break;
	}
#endif
	default:
		b = make_ushort4(autocast_int<int, ushort>(val.x, MINVAL_uint16, MAXVAL_uint16, mode),
						autocast_int<int, ushort>(val.y, MINVAL_uint16, MAXVAL_uint16, mode),
						autocast_int<int, ushort>(val.z, MINVAL_uint16, MAXVAL_uint16, mode),
						autocast_int<int, ushort>(val.w, MINVAL_uint16, MAXVAL_uint16, mode));
		break;
	}
	return b;
}

INLINE uchar4 __device__  __HOST_V__ autocast_int(int4 val, IntegerConversionModes mode, ReturnType<uchar4>)
{
	uchar4 b;
	switch (mode)
	{
#if defined(HAS_SSE2)
	case ConvSaturated:
	{
		__m128i A = _mm_load_si128((__m128i *)&val);
		__m128i B = B = _mm_max_epi32(_mm_min_epi32(A, _mm_set1_epi32(MAXVAL_uchar)), _mm_set1_epi32(MINVAL_uchar));
		__m128i C = _mm_shuffle_epi8(B, _mm_set1_epi32(0x0c080400));
		(int &)b = _mm_extract_epi32(C, 0);
		break;
	}
	case ConvUnchecked:
	{
		__m128i A = _mm_load_si128((__m128i *)&val);
		__m128i B = _mm_min_epu32(A, _mm_set1_epi32(255));
		__m128i C = _mm_shuffle_epi8(B, _mm_set1_epi32(0x0c080400));
		(int &)b = _mm_extract_epi32(C, 0);
		break;
	}
#endif
	default:
		b = make_uchar4(autocast_int<int, uchar>(val.x, MINVAL_uchar, MAXVAL_uchar, mode),
						autocast_int<int, uchar>(val.y, MINVAL_uchar, MAXVAL_uchar, mode),
						autocast_int<int, uchar>(val.z, MINVAL_uchar, MAXVAL_uchar, mode),
						autocast_int<int, uchar>(val.w, MINVAL_uchar, MAXVAL_uchar, mode));
		break;
	}
	return b;
}

INLINE short4 __device__  __HOST_V__ autocast_int(int4 val, IntegerConversionModes mode, ReturnType<short4>)
{
	short4 b;
	switch (mode)
	{
#if defined(HAS_SSE2) && defined(HAS_mm_set1_epi64x)
	{
	case ConvSaturated:
		__m128i A = _mm_load_si128((__m128i *)&val);
		__m128i B = _mm_packs_epi32(A, _mm_set1_epi32(0));
		_mm_storel_epi64((__m128i *)&b, B);
		break;
	}
	case ConvUnchecked:
	{
		__m128i A = _mm_load_si128((__m128i *)&val);
		__m128i B = _mm_shuffle_epi8(A, _mm_set1_epi64x(0x0d0c090805040100ull));
		_mm_storel_epi64((__m128i *)&b, B);
		break;
	}
#endif
	default:
		b = make_short4(autocast_int<int, short>(val.x, MINVAL_int16, MAXVAL_int16, mode),
						autocast_int<int, short>(val.y, MINVAL_int16, MAXVAL_int16, mode),
						autocast_int<int, short>(val.z, MINVAL_int16, MAXVAL_int16, mode),
						autocast_int<int, short>(val.w, MINVAL_int16, MAXVAL_int16, mode));
		break;
	}
	return b;
}

INLINE char4 __device__  __HOST_V__ autocast_int(int4 val, IntegerConversionModes mode, ReturnType<char4>)
{
	char4 b;
	switch (mode)
	{
#if defined(HAS_SSE2)
	case ConvSaturated:
	{
		__m128i A0 = _mm_load_si128((__m128i *)&val);
		__m128i A1 = _mm_max_epi32(A0, _mm_set1_epi32(-128));
		__m128i A2 = _mm_min_epi32(A1, _mm_set1_epi32(127));
		__m128i B = _mm_shuffle_epi8(A2, _mm_set1_epi32(0x0c080400));
		(int &)b = _mm_extract_epi32(B, 0);
		break;
	}
	case ConvUnchecked:
	{
		__m128i A = _mm_load_si128((__m128i *)&val);
		__m128i mask = _mm_set1_epi32(0x0c080400);
		__m128i B = _mm_shuffle_epi8(A, mask);
		(int &)b = _mm_extract_epi32(B, 0);
		break;
	}
#endif
	default:
		b =  make_char4(autocast_int<int, char>(val.x, MINVAL_char, MAXVAL_char, mode),
						autocast_int<int, char>(val.y, MINVAL_char, MAXVAL_char, mode),
						autocast_int<int, char>(val.z, MINVAL_char, MAXVAL_char, mode),
						autocast_int<int, char>(val.w, MINVAL_char, MAXVAL_char, mode));
		break;
	}
	return b;
}

INLINE char4 __device__  __HOST_V__ autocast_int(scalar4 val, IntegerConversionModes mode, ReturnType<char4>)
{
	char4 b;
	switch (mode)
	{
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	case ConvSaturated:
	{
		__m128i A0 = _mm_cvtps_epi32(_mm_load_ps((float *)&val));
		__m128i A1 = _mm_max_epi32(A0, _mm_set1_epi32(-128));
		__m128i A2 = _mm_min_epi32(A1, _mm_set1_epi32(127));
		__m128i B = _mm_shuffle_epi8(A2, _mm_set1_epi32(0x0c080400));
		(int &)b = _mm_extract_epi32(B, 0);
		break;
	}
	case ConvUnchecked:
	{
		__m128i A = _mm_cvtps_epi32(_mm_load_ps((float *)&val));
		__m128i B = _mm_shuffle_epi8(A, _mm_set1_epi32(0x0c080400));
		(int &)b = _mm_extract_epi32(B, 0);
		break;
	}
#elif defined(HAS_SSE4) && defined(DBL_SCALAR)
	case ConvSaturated:
	{
		__m128i A = _mm_cvtpd_epi32(_mm_load_pd((double *)&val));
		__m128i B = _mm_cvtpd_epi32(_mm_load_pd((double *)&val + 2));
		__m128i B2 = _mm_shuffle_epi32(B, 0x40); // 01000000b
		__m128i A0 = _mm_blendv_epi8(A, B2, _mm_set_epi64x(0xFFFFFFFFFFFFFFFFull, 0));
		__m128i A1 = _mm_max_epi32(A0, _mm_set1_epi32(-128));
		__m128i A2 = _mm_min_epi32(A1, _mm_set1_epi32(127));
		__m128i C = _mm_shuffle_epi8(A2, _mm_set1_epi32(0x0c080400));
		(int &)b = _mm_extract_epi32(C, 0);
		break;
	}
	case ConvUnchecked:
	{
		__m128i A = _mm_cvtpd_epi32(_mm_load_pd((double *)&val));
		__m128i B = _mm_cvtpd_epi32(_mm_load_pd((double *)&val + 2));
		__m128i B2 = _mm_shuffle_epi32(B, 0x40); // 01000000b
		__m128i A0 = _mm_blendv_epi8(A, B2, _mm_set_epi64x(0xFFFFFFFFFFFFFFFFull, 0));
		__m128i C = _mm_shuffle_epi8(A0, _mm_set1_epi32(0x0c080400));
		(int &)b = _mm_extract_epi32(C, 0);
		break;
	}
#endif
	default:
		b =  make_char4(autocast_int<scalar, char>(val.x, MINVAL_char, MAXVAL_char, mode),
						autocast_int<scalar, char>(val.y, MINVAL_char, MAXVAL_char, mode),
						autocast_int<scalar, char>(val.z, MINVAL_char, MAXVAL_char, mode),
						autocast_int<scalar, char>(val.w, MINVAL_char, MAXVAL_char, mode));
		break;
	}
	return b;
}

INLINE short4 __device__  __HOST_V__ autocast_int(scalar4 val, IntegerConversionModes mode, ReturnType<short4>)
{
	short4 b;
	switch (mode)
	{
#if defined(HAS_SSE2) && !defined(DBL_SCALAR) && defined(HAS_mm_set1_epi64x)
	case ConvSaturated:
	{
		__m128i A = _mm_cvtps_epi32(_mm_load_ps((float *)&val));
		__m128i B = _mm_packs_epi32(A, _mm_set1_epi32(0));
		_mm_storel_epi64((__m128i *)&b, B);
		break;
	}
	case ConvUnchecked:
	{
		__m128i A = _mm_cvtps_epi32(_mm_load_ps((float *)&val));
		__m128i B = _mm_shuffle_epi8(A, _mm_set1_epi64x(0x0d0c090805040100ull));
		_mm_storel_epi64((__m128i *)&b, B);
		break;
	}
#elif defined(HAS_SSE4) && defined(DBL_SCALAR) && defined(HAS_mm_set1_epi64x)
	case ConvSaturated:
	{
		__m128i A = _mm_cvtpd_epi32(_mm_load_pd((double *)&val));
		__m128i B = _mm_cvtpd_epi32(_mm_load_pd((double *)&val + 2));
		__m128i B2 = _mm_shuffle_epi32(B, 0x40); // 01000000b
		__m128i C = _mm_blendv_epi8(A, B2, _mm_set_epi64x(0xFFFFFFFFFFFFFFFFull, 0));
		__m128i D = _mm_packs_epi32(C, _mm_set1_epi32(0));
		_mm_storel_epi64((__m128i *)&b, D);
		break;
	}
	case ConvUnchecked:
	{
		__m128i A = _mm_cvtpd_epi32(_mm_load_pd((double *)&val));
		__m128i B = _mm_cvtpd_epi32(_mm_load_pd((double *)&val + 2));
		__m128i B2 = _mm_shuffle_epi32(B, 0x40); // 01000000b
		__m128i C = _mm_blendv_epi8(A, B2, _mm_set_epi64x(0xFFFFFFFFFFFFFFFFull, 0));
		__m128i D = _mm_shuffle_epi8(C, _mm_set1_epi64x(0x0d0c090805040100ull));
		_mm_storel_epi64((__m128i *)&b, D);
		break;
	}
#endif
	default:
		b = make_short4(autocast_int<scalar, short>(val.x, MINVAL_int16, MAXVAL_int16, mode),
						autocast_int<scalar, short>(val.y, MINVAL_int16, MAXVAL_int16, mode),
						autocast_int<scalar, short>(val.z, MINVAL_int16, MAXVAL_int16, mode),
						autocast_int<scalar, short>(val.w, MINVAL_int16, MAXVAL_int16, mode));
		break;
	}
	return b;
}

INLINE uchar4 __device__  __HOST_V__ autocast_int(scalar4 val, IntegerConversionModes mode, ReturnType<uchar4>)
{
	uchar4 b;
	switch (mode)
	{
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	case ConvSaturated:
	{
		__m128i A = _mm_cvtps_epi32(_mm_load_ps((float *)&val));
		__m128i B = _mm_max_epi32(_mm_min_epi32(A, _mm_set1_epi32(255)), _mm_set1_epi32(0));
		__m128i C = _mm_shuffle_epi8(B, _mm_set1_epi32(0x0c080400));
		(int &)b = _mm_extract_epi32(C, 0);
		break;
	}
	case ConvUnchecked:
	{
		__m128i A = _mm_cvtps_epi32(_mm_load_ps((float *)&val));
		__m128i B = _mm_shuffle_epi8(A, _mm_set1_epi32(0x0c080400));
		(int &)b = _mm_extract_epi32(B, 0);
		break;
	}
#elif defined(HAS_SSE4) && defined(DBL_SCALAR)
	case ConvSaturated:
	{
		__m128i A = _mm_cvtpd_epi32(_mm_load_pd((double *)&val));
		__m128i B = _mm_cvtpd_epi32(_mm_load_pd((double *)&val + 2));
		__m128i B2 = _mm_shuffle_epi32(B, 0x40); // 01000000b
		__m128i C = _mm_blendv_epi8(A, B2, _mm_set_epi64x(0xFFFFFFFFFFFFFFFFull, 0));
		__m128i D = _mm_max_epi32(_mm_min_epi32(C, _mm_set1_epi32(255)), _mm_set1_epi32(0));
		__m128i E = _mm_shuffle_epi8(D, _mm_set1_epi32(0x0c080400));
		(int &)b = _mm_extract_epi32(E, 0);
		break;
	}
	case ConvUnchecked:
	{
		__m128i A = _mm_cvtpd_epi32(_mm_load_pd((double *)&val));
		__m128i B = _mm_cvtpd_epi32(_mm_load_pd((double *)&val + 2));
		__m128i B2 = _mm_shuffle_epi32(B, 0x40); // 01000000b
		__m128i C = _mm_blendv_epi8(A, B2, _mm_set_epi64x(0xFFFFFFFFFFFFFFFFull, 0));
		__m128i D = _mm_shuffle_epi8(C, _mm_set1_epi32(0x0c080400));
		(int &)b = _mm_extract_epi32(D, 0);
		break;
	}
#endif
	default:
		b = make_uchar4(autocast_int<scalar, uchar>(val.x, MINVAL_uchar, MAXVAL_uchar, mode),
						autocast_int<scalar, uchar>(val.y, MINVAL_uchar, MAXVAL_uchar, mode),
						autocast_int<scalar, uchar>(val.z, MINVAL_uchar, MAXVAL_uchar, mode),
						autocast_int<scalar, uchar>(val.w, MINVAL_uchar, MAXVAL_uchar, mode));
		break;
	}
	return b;
}

INLINE ushort4 __device__  __HOST_V__ autocast_int(scalar4 val, IntegerConversionModes mode, ReturnType<ushort4>)
{
	ushort4 b;
	switch (mode)
	{
#if defined(HAS_SSE2) && !defined(DBL_SCALAR) && defined(HAS_mm_set1_epi64x)
	case ConvSaturated:
	{
		__m128i A = _mm_cvtps_epi32(_mm_load_ps((float *)&val));
		__m128i B = _mm_packus_epi32(A, _mm_set1_epi32(0));
		_mm_storel_epi64((__m128i *)&b, B);
		break;
	}
	case ConvUnchecked:
	{
		__m128i A = _mm_cvtps_epi32(_mm_load_ps((float *)&val));
		__m128i B = _mm_shuffle_epi8(A, _mm_set1_epi64x(0x0d0c090805040100ull));
		_mm_storel_epi64((__m128i *)&b, B);
		break;
	}
#elif defined(HAS_SSE4) && defined(DBL_SCALAR) && defined(HAS_mm_set1_epi64x)
	case ConvSaturated:
	{
		__m128i A = _mm_cvtpd_epi32(_mm_load_pd((double *)&val));
		__m128i B = _mm_cvtpd_epi32(_mm_load_pd((double *)&val + 2));
		__m128i B2 = _mm_shuffle_epi32(B, 0x40); // 01000000b
		__m128i C = _mm_blendv_epi8(A, B2, _mm_set_epi64x(0xFFFFFFFFFFFFFFFFull, 0));
		__m128i D = _mm_packus_epi32(C, _mm_set1_epi32(0));
		_mm_storel_epi64((__m128i *)&b, D);
		break;
	}
	case ConvUnchecked:
	{
		__m128i A = _mm_cvtpd_epi32(_mm_load_pd((double *)&val));
		__m128i B = _mm_cvtpd_epi32(_mm_load_pd((double *)&val + 2));
		__m128i B2 = _mm_shuffle_epi32(B, 0x40); // 01000000b
		__m128i C = _mm_blendv_epi8(A, B2, _mm_set_epi64x(0xFFFFFFFFFFFFFFFFull, 0));
		__m128i D = _mm_shuffle_epi8(C, _mm_set1_epi64x(0x0d0c090805040100ull));
		_mm_storel_epi64((__m128i *)&b, D);
		break;
	}
#endif
	default:
		b = make_ushort4(autocast_int<scalar, ushort>(val.x, MINVAL_uint16, MAXVAL_uint16, mode),
						autocast_int<scalar, ushort>(val.y, MINVAL_uint16, MAXVAL_uint16, mode),
						autocast_int<scalar, ushort>(val.z, MINVAL_uint16, MAXVAL_uint16, mode),
						autocast_int<scalar, ushort>(val.w, MINVAL_uint16, MAXVAL_uint16, mode));
		break;
	}
	return b;
}

INLINE scalar4 __device__ __HOST_V__ autocast(char4 a, ReturnType<scalar4>)
{
	scalar4 b;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128i A = _mm_set1_epi32((int&)a);
	__m128i B = _mm_cvtepi8_epi32(A);
	__m128 C = _mm_cvtepi32_ps(B);
	_mm_store_ps((float *)&b, C);
#elif defined(HAS_SSE4) && defined(DBL_SCALAR)
	__m128i A = _mm_set1_epi32((int&)a);
	__m128i B = _mm_cvtepi8_epi32(A);
	__m128d C = _mm_cvtepi32_pd(B);
	__m128d D = _mm_cvtepi32_pd(_mm_shuffle_epi32(B, 0xe)); // 1110b
	_mm_store_pd((double *)&b.x, C);
	_mm_store_pd((double *)&b.z, D);
#else
	b = make_scalar4(autocast(a.x, ReturnType<scalar>()), autocast(a.y, ReturnType<scalar>()), autocast(a.z, ReturnType<scalar>()), autocast(a.w, ReturnType<scalar>()));
#endif
    return b;
}
INLINE scalar4 __device__ __HOST_V__ autocast(short4 a, ReturnType<scalar4>)
{
	scalar4 b;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128i A = _mm_loadl_epi64((__m128i *)&a);
	__m128i B = _mm_cvtepi16_epi32(A);
	__m128 C = _mm_cvtepi32_ps(B);
	_mm_store_ps((float *)&b, C);
#elif defined(HAS_SSE4) && defined(DBL_SCALAR)
	__m128i A = _mm_loadl_epi64((__m128i *)&a);
	__m128i B = _mm_cvtepi16_epi32(A);
	__m128d C = _mm_cvtepi32_pd(B);
	__m128d D = _mm_cvtepi32_pd(_mm_shuffle_epi32(B, 0xe)); // 1110b
	_mm_store_pd((double *)&b.x, C);
	_mm_store_pd((double *)&b.z, D);
#else
    b = make_scalar4(autocast(a.x, ReturnType<scalar>()), autocast(a.y, ReturnType<scalar>()), autocast(a.z, ReturnType<scalar>()), autocast(a.w, ReturnType<scalar>()));
#endif
    return b;
}
INLINE scalar4 __device__ __HOST_V__ autocast(uchar4 a, ReturnType<scalar4>)
{
 	scalar4 b;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128i A = _mm_set1_epi32((int&)a);
	__m128i B = _mm_cvtepu8_epi32(A);
	__m128 C = _mm_cvtepi32_ps(B);
	_mm_store_ps((float *)&b, C);
#elif defined(HAS_SSE4) && defined(DBL_SCALAR)
	__m128i A = _mm_set1_epi32((int&)a);
	__m128i B = _mm_cvtepu8_epi32(A);
	__m128d C = _mm_cvtepi32_pd(B);
	__m128d D = _mm_cvtepi32_pd(_mm_shuffle_epi32(B, 0xe)); // 1110b
	_mm_store_pd((double *)&b.x, C);
	_mm_store_pd((double *)&b.z, D);
#else
    b = make_scalar4(autocast(a.x, ReturnType<scalar>()), autocast(a.y, ReturnType<scalar>()), autocast(a.z, ReturnType<scalar>()), autocast(a.w, ReturnType<scalar>()));
#endif
	return b;
}
INLINE scalar4 __device__ __HOST_V__ autocast(ushort4 a, ReturnType<scalar4>)
{
	scalar4 b;
#if defined(HAS_SSE2) && !defined(DBL_SCALAR)
	__m128i A = _mm_loadl_epi64((__m128i *)&a);
	__m128i B = _mm_cvtepu16_epi32(A);
	__m128 C = _mm_cvtepi32_ps(B);
	_mm_store_ps((float *)&b, C);
#elif defined(HAS_SSE4) && defined(DBL_SCALAR)
	__m128i A = _mm_loadl_epi64((__m128i *)&a);
	__m128i B = _mm_cvtepu16_epi32(A);
	__m128d C = _mm_cvtepi32_pd(B);
	__m128d D = _mm_cvtepi32_pd(_mm_shuffle_epi32(B, 0xe)); // 1110b
	_mm_store_pd((double *)&b.x, C);
	_mm_store_pd((double *)&b.z, D);
#else
    b = make_scalar4(autocast(a.x, ReturnType<scalar>()), autocast(a.y, ReturnType<scalar>()), autocast(a.z, ReturnType<scalar>()), autocast(a.w, ReturnType<scalar>()));
#endif
	return b;
}
INLINE int4 __device__ __HOST_V__ autocast(char4 a, ReturnType<int4>)
{
 	int4 b;
#if defined(HAS_SSE2)
	__m128i A = _mm_loadl_epi64((__m128i *)&a);
	__m128i B = _mm_cvtepi8_epi32(A);
	_mm_store_si128((__m128i *)&b, B);
#else
    b = make_int4(autocast(a.x, ReturnType<int>()), autocast(a.y, ReturnType<int>()), autocast(a.z, ReturnType<int>()), autocast(a.w, ReturnType<int>()));
#endif
	return b;
}
INLINE int4 __device__ __HOST_V__ autocast(short4 a, ReturnType<int4>)
{
 	int4 b;
#if defined(HAS_SSE2)
	__m128i A = _mm_loadl_epi64((__m128i *)&a);
	__m128i B = _mm_cvtepi16_epi32(A);
	_mm_store_si128((__m128i *)&b, B);
#else
    b = make_int4(autocast(a.x, ReturnType<int>()), autocast(a.y, ReturnType<int>()), autocast(a.z, ReturnType<int>()), autocast(a.w, ReturnType<int>()));
#endif
	return b;
}
INLINE int4 __device__ __HOST_V__ autocast(uchar4 a, ReturnType<int4>)
{
 	int4 b;
#if defined(HAS_SSE2)
	__m128i A = _mm_loadl_epi64((__m128i *)&a);
	__m128i B = _mm_cvtepu8_epi32(A);
	_mm_store_si128((__m128i *)&b, B);
#else
    b = make_int4(autocast(a.x, ReturnType<int>()), autocast(a.y, ReturnType<int>()), autocast(a.z, ReturnType<int>()), autocast(a.w, ReturnType<int>()));
#endif
	return b;
}
INLINE int4 __device__ __HOST_V__ autocast(ushort4 a, ReturnType<int4>)
{
 	int4 b;
#if defined(HAS_SSE2)
	__m128i A = _mm_loadl_epi64((__m128i *)&a);
	__m128i B = _mm_cvtepu16_epi32(A);
	_mm_store_si128((__m128i *)&b, B);
#else
    b = make_int4(autocast(a.x, ReturnType<int>()), autocast(a.y, ReturnType<int>()), autocast(a.z, ReturnType<int>()), autocast(a.w, ReturnType<int>()));
#endif
	return b;
}
}	// namespace quasar
template <>
INLINE scalarN<8> __device__ __HOST_V__ operator +(scalarN<8> a, scalarN<8> b)
{
	scalarN<8> y;
#if defined(HAS_AVX) && !defined(DBL_SCALAR)
	__m256 A = _mm256_load_ps((float *)&a);
	__m256 B = _mm256_load_ps((float *)&b);
	__m256 C = _mm256_add_ps(A, B);
	_mm256_store_ps((float *)&y, C);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A1 = _mm256_load_pd((double *)&a.el[0]);
	__m256d A2 = _mm256_load_pd((double *)&a.el[4]);
	__m256d B1 = _mm256_load_pd((double *)&b.el[0]);
	__m256d B2 = _mm256_load_pd((double *)&b.el[4]);
	__m256d C1 = _mm256_add_pd(A1,B1);
	__m256d C2 = _mm256_add_pd(A2,B1);
	_mm256_store_pd((double*)&y.el[0], C1);
	_mm256_store_pd((double*)&y.el[4], C2);
#else
    y = make_scalarN(a.el[0] + b.el[0], a.el[1] + b.el[1], a.el[2] + b.el[2], a.el[3] + b.el[3],
					 a.el[4] + b.el[4], a.el[5] + b.el[5], a.el[6] + b.el[6], a.el[7] + b.el[7]);
#endif
	return y;
}
template <>
INLINE scalarN<8> __device__ __HOST_V__ operator +(scalar a, scalarN<8> b)
{
	scalarN<8> y;
#if defined(HAS_AVX) && !defined(DBL_SCALAR)
	__m256 A = _mm256_set1_ps(a);
	__m256 B = _mm256_load_ps((float *)&b);
	__m256 C = _mm256_add_ps(A, B);
	_mm256_store_ps((float *)&y, C);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A = _mm256_set1_pd(a);
	__m256d B1 = _mm256_load_pd((double *)&b.el[0]);
	__m256d B2 = _mm256_load_pd((double *)&b.el[4]);
	__m256d C1 = _mm256_add_pd(A,B1);
	__m256d C2 = _mm256_add_pd(A,B2);
	_mm256_store_pd((double*)&y.el[0], C1);
	_mm256_store_pd((double*)&y.el[4], C2);
#else
    y = make_scalarN(a + b.el[0], a + b.el[1], a + b.el[2], a + b.el[3],
					 a + b.el[4], a + b.el[5], a + b.el[6], a + b.el[7]);
#endif
	return y;
}
template <>
INLINE scalarN<8> __device__ __HOST_V__ operator +(scalarN<8> a, scalar b)
{
	scalarN<8> y;
#if defined(HAS_AVX) && !defined(DBL_SCALAR)
	__m256 A = _mm256_load_ps((float *)&a);
	__m256 B = _mm256_set1_ps(b);
	__m256 C = _mm256_add_ps(A, B);
	_mm256_store_ps((float *)&y, C);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A1 = _mm256_load_pd((double *)&a.el[0]);
	__m256d A2 = _mm256_load_pd((double *)&a.el[4]);
	__m256d B = _mm256_set1_pd(b);
	__m256d C1 = _mm256_add_pd(A1,B);
	__m256d C2 = _mm256_add_pd(A2,B);
	_mm256_store_pd((double*)&y.el[0], C1);
	_mm256_store_pd((double*)&y.el[4], C2);
#else
    y = make_scalarN(a.el[0] + b, a.el[1] + b, a.el[2] + b, a.el[3] + b,
					 a.el[4] + b, a.el[5] + b, a.el[6] + b, a.el[7] + b);
#endif
	return y;
}
template <>
INLINE scalarN<8> __device__ __HOST_V__ operator -(scalarN<8> a, scalarN<8> b)
{
	scalarN<8> y;
#if defined(HAS_AVX) && !defined(DBL_SCALAR)
	__m256 A = _mm256_load_ps((float *)&a);
	__m256 B = _mm256_load_ps((float *)&b);
	__m256 C = _mm256_sub_ps(A, B);
	_mm256_store_ps((float *)&y, C);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A1 = _mm256_load_pd((double *)&a.el[0]);
	__m256d A2 = _mm256_load_pd((double *)&a.el[4]);
	__m256d B1 = _mm256_load_pd((double *)&b.el[0]);
	__m256d B2 = _mm256_load_pd((double *)&b.el[4]);
	__m256d C1 = _mm256_sub_pd(A1,B1);
	__m256d C2 = _mm256_sub_pd(A2,B1);
	_mm256_store_pd((double*)&y.el[0], C1);
	_mm256_store_pd((double*)&y.el[4], C2);
#else
    y = make_scalarN(a.el[0] - b.el[0], a.el[1] - b.el[1], a.el[2] - b.el[2], a.el[3] - b.el[3],
					 a.el[4] - b.el[4], a.el[5] - b.el[5], a.el[6] - b.el[6], a.el[7] - b.el[7]);
#endif
	return y;
}
template <>
INLINE scalarN<8> __device__ __HOST_V__ operator -(scalar a, scalarN<8> b)
{
	scalarN<8> y;
#if defined(HAS_AVX) && !defined(DBL_SCALAR)
	__m256 A = _mm256_set1_ps(a);
	__m256 B = _mm256_load_ps((float *)&b);
	__m256 C = _mm256_sub_ps(A, B);
	_mm256_store_ps((float *)&y, C);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A = _mm256_set1_pd(a);
	__m256d B1 = _mm256_load_pd((double *)&b.el[0]);
	__m256d B2 = _mm256_load_pd((double *)&b.el[4]);
	__m256d C1 = _mm256_sub_pd(A,B1);
	__m256d C2 = _mm256_sub_pd(A,B2);
	_mm256_store_pd((double*)&y.el[0], C1);
	_mm256_store_pd((double*)&y.el[4], C2);
#else
    y = make_scalarN(a - b.el[0], a - b.el[1], a - b.el[2], a - b.el[3],
					 a - b.el[4], a - b.el[5], a - b.el[6], a - b.el[7]);
#endif
	return y;
}
template <>
INLINE scalarN<8> __device__ __HOST_V__ operator -(scalarN<8> a, scalar b)
{
	scalarN<8> y;
#if defined(HAS_AVX) && !defined(DBL_SCALAR)
	__m256 A = _mm256_load_ps((float *)&a);
	__m256 B = _mm256_set1_ps(b);
	__m256 C = _mm256_sub_ps(A, B);
	_mm256_store_ps((float *)&y, C);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A1 = _mm256_load_pd((double *)&a.el[0]);
	__m256d A2 = _mm256_load_pd((double *)&a.el[4]);
	__m256d B = _mm256_set1_pd(b);
	__m256d C1 = _mm256_sub_pd(A1,B);
	__m256d C2 = _mm256_sub_pd(A2,B);
	_mm256_store_pd((double*)&y.el[0], C1);
	_mm256_store_pd((double*)&y.el[4], C2);
#else
    y = make_scalarN(a.el[0] - b, a.el[1] - b, a.el[2] - b, a.el[3] - b,
					 a.el[4] - b, a.el[5] - b, a.el[6] - b, a.el[7] - b);
#endif
	return y;
}
template <>
INLINE scalarN<8> __device__ __HOST_V__ operator *(scalarN<8> a, scalarN<8> b)
{
	scalarN<8> y;
#if defined(HAS_AVX) && !defined(DBL_SCALAR)
	__m256 A = _mm256_load_ps((float *)&a);
	__m256 B = _mm256_load_ps((float *)&b);
	__m256 C = _mm256_mul_ps(A, B);
	_mm256_store_ps((float *)&y, C);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A1 = _mm256_load_pd((double *)&a.el[0]);
	__m256d A2 = _mm256_load_pd((double *)&a.el[4]);
	__m256d B1 = _mm256_load_pd((double *)&b.el[0]);
	__m256d B2 = _mm256_load_pd((double *)&b.el[4]);
	__m256d C1 = _mm256_mul_pd(A1,B1);
	__m256d C2 = _mm256_mul_pd(A2,B1);
	_mm256_store_pd((double*)&y.el[0], C1);
	_mm256_store_pd((double*)&y.el[4], C2);
#else
    y = make_scalarN(a.el[0] * b.el[0], a.el[1] * b.el[1], a.el[2] * b.el[2], a.el[3] * b.el[3],
					 a.el[4] * b.el[4], a.el[5] * b.el[5], a.el[6] * b.el[6], a.el[7] * b.el[7]);
#endif
	return y;
}
template <>
INLINE scalarN<8> __device__ __HOST_V__ operator *(scalar a, scalarN<8> b)
{
	scalarN<8> y;
#if defined(HAS_AVX) && !defined(DBL_SCALAR)
	__m256 A = _mm256_set1_ps(a);
	__m256 B = _mm256_load_ps((float *)&b);
	__m256 C = _mm256_mul_ps(A, B);
	_mm256_store_ps((float *)&y, C);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A = _mm256_set1_pd(a);
	__m256d B1 = _mm256_load_pd((double *)&b.el[0]);
	__m256d B2 = _mm256_load_pd((double *)&b.el[4]);
	__m256d C1 = _mm256_mul_pd(A,B1);
	__m256d C2 = _mm256_mul_pd(A,B2);
	_mm256_store_pd((double*)&y.el[0], C1);
	_mm256_store_pd((double*)&y.el[4], C2);
#else
    y = make_scalarN(a * b.el[0], a * b.el[1], a * b.el[2], a * b.el[3],
					 a * b.el[4], a * b.el[5], a * b.el[6], a * b.el[7]);
#endif
	return y;
}
template <>
INLINE scalarN<8> __device__ __HOST_V__ operator *(scalarN<8> a, scalar b)
{
	scalarN<8> y;
#if defined(HAS_AVX) && !defined(DBL_SCALAR)
	__m256 A = _mm256_load_ps((float *)&a);
	__m256 B = _mm256_set1_ps(b);
	__m256 C = _mm256_mul_ps(A, B);
	_mm256_store_ps((float *)&y, C);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A1 = _mm256_load_pd((double *)&a.el[0]);
	__m256d A2 = _mm256_load_pd((double *)&a.el[4]);
	__m256d B = _mm256_set1_pd(b);
	__m256d C1 = _mm256_mul_pd(A1,B);
	__m256d C2 = _mm256_mul_pd(A2,B);
	_mm256_store_pd((double*)&y.el[0], C1);
	_mm256_store_pd((double*)&y.el[4], C2);
#else
    y = make_scalarN(a.el[0] * b, a.el[1] * b, a.el[2] * b, a.el[3] * b,
					 a.el[4] * b, a.el[5] * b, a.el[6] * b, a.el[7] * b);
#endif
	return y;
}
template <>
INLINE scalarN<8> __device__ __HOST_V__ operator /(scalarN<8> a, scalarN<8> b)
{
	scalarN<8> y;
#if defined(HAS_AVX) && !defined(DBL_SCALAR)
	__m256 A = _mm256_load_ps((float *)&a);
	__m256 B = _mm256_load_ps((float *)&b);
	__m256 C = _mm256_div_ps(A, B);
	_mm256_store_ps((float *)&y, C);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A1 = _mm256_load_pd((double *)&a.el[0]);
	__m256d A2 = _mm256_load_pd((double *)&a.el[4]);
	__m256d B1 = _mm256_load_pd((double *)&b.el[0]);
	__m256d B2 = _mm256_load_pd((double *)&b.el[4]);
	__m256d C1 = _mm256_div_pd(A1,B1);
	__m256d C2 = _mm256_div_pd(A2,B1);
	_mm256_store_pd((double*)&y.el[0], C1);
	_mm256_store_pd((double*)&y.el[4], C2);
#else
    y = make_scalarN(a.el[0] / b.el[0], a.el[1] / b.el[1], a.el[2] / b.el[2], a.el[3] / b.el[3],
					 a.el[4] / b.el[4], a.el[5] / b.el[5], a.el[6] / b.el[6], a.el[7] / b.el[7]);
#endif
	return y;
}
template <>
INLINE scalarN<8> __device__ __HOST_V__ operator /(scalar a, scalarN<8> b)
{
	scalarN<8> y;
#if defined(HAS_AVX) && !defined(DBL_SCALAR)
	__m256 A = _mm256_set1_ps(a);
	__m256 B = _mm256_load_ps((float *)&b);
	__m256 C = _mm256_div_ps(A, B);
	_mm256_store_ps((float *)&y, C);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A = _mm256_set1_pd(a);
	__m256d B1 = _mm256_load_pd((double *)&b.el[0]);
	__m256d B2 = _mm256_load_pd((double *)&b.el[4]);
	__m256d C1 = _mm256_div_pd(A,B1);
	__m256d C2 = _mm256_div_pd(A,B2);
	_mm256_store_pd((double*)&y.el[0], C1);
	_mm256_store_pd((double*)&y.el[4], C2);
#else
    y = make_scalarN(a / b.el[0], a / b.el[1], a / b.el[2], a / b.el[3],
					 a / b.el[4], a / b.el[5], a / b.el[6], a / b.el[7]);
#endif
	return y;
}
template <>
INLINE scalarN<8> __device__ __HOST_V__ operator /(scalarN<8> a, scalar b)
{
	scalarN<8> y;
#if defined(HAS_AVX) && !defined(DBL_SCALAR)
	__m256 A = _mm256_load_ps((float *)&a);
	__m256 B = _mm256_set1_ps(b);
	__m256 C = _mm256_div_ps(A, B);
	_mm256_store_ps((float *)&y, C);
#elif defined(HAS_AVX) && defined(DBL_SCALAR)
	__m256d A1 = _mm256_load_pd((double *)&a.el[0]);
	__m256d A2 = _mm256_load_pd((double *)&a.el[4]);
	__m256d B = _mm256_set1_pd(b);
	__m256d C1 = _mm256_div_pd(A1,B);
	__m256d C2 = _mm256_div_pd(A2,B);
	_mm256_store_pd((double*)&y.el[0], C1);
	_mm256_store_pd((double*)&y.el[4], C2);
#else
    y = make_scalarN(a.el[0] / b, a.el[1] / b, a.el[2] / b, a.el[3] / b,
					 a.el[4] / b, a.el[5] / b, a.el[6] / b, a.el[7] / b);
#endif
	return y;
}
INLINE scalar1 __device__ __HOST_V__ operator +=(scalar1& a, scalar1 b)
{
	return a = (a + b);
}
INLINE scalar1 __device__ __HOST_V__ operator +=(scalar1& a, scalar b)
{
	return a = (a + b);
}
INLINE scalar2 __device__ __HOST_V__ operator +=(scalar2& a, scalar2 b)
{
	return a = (a + b);
}
INLINE scalar2 __device__ __HOST_V__ operator +=(scalar2& a, scalar b)
{
	return a = (a + b);
}
INLINE scalar3 __device__ __HOST_V__ operator +=(scalar3& a, scalar3 b)
{
	return a = (a + b);
}
INLINE scalar3 __device__ __HOST_V__ operator +=(scalar3& a, scalar b)
{
	return a = (a + b);
}
INLINE scalar4 __device__ __HOST_V__ operator +=(scalar4& a, scalar4 b)
{
	return a = (a + b);
}
INLINE scalar4 __device__ __HOST_V__ operator +=(scalar4& a, scalar b)
{
	return a = (a + b);
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ operator +=(scalarN<N>& a, scalarN<N> b)
{
	return a = (a + b);
}
template <int N> INLINE scalarN<N> __device__ __HOST_V__ operator +=(scalarN<N>& a, scalar b)
{
	return a = (a + b);
}
INLINE int1 __device__ __HOST_V__ operator +=(int1& a, int1 b)
{
	return a = (a + b);
}
INLINE int1 __device__ __HOST_V__ operator +=(int1& a, int b)
{
	return a = (a + b);
}
INLINE int2 __device__ __HOST_V__ operator +=(int2& a, int2 b)
{
	return a = (a + b);
}
INLINE int2 __device__ __HOST_V__ operator +=(int2& a, int b)
{
	return a = (a + b);
}
INLINE int3 __device__ __HOST_V__ operator +=(int3& a, int3 b)
{
	return a = (a + b);
}
INLINE int3 __device__ __HOST_V__ operator +=(int3& a, int b)
{
	return a = (a + b);
}
INLINE int4 __device__ __HOST_V__ operator +=(int4& a, int4 b)
{
	return a = (a + b);
}
INLINE int4 __device__ __HOST_V__ operator +=(int4& a, int b)
{
	return a = (a + b);
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ operator +=(intN<N>& a, intN<N> b)
{
	return a = (a + b);
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator +=(intN<N>& a, int b)
{
	return a = (a + b);
}
INLINE cscalar1 __device__ __HOST_V__ operator +=(cscalar1& a, cscalar1 b)
{
	return a = (a + b);
}
INLINE cscalar1 __device__ __HOST_V__ operator +=(cscalar1& a, cscalar b)
{
	return a = (a + b);
}
INLINE cscalar2 __device__ __HOST_V__ operator +=(cscalar2& a, cscalar2 b)
{
	return a = (a + b);
}
INLINE cscalar2 __device__ __HOST_V__ operator +=(cscalar2& a, cscalar b)
{
	return a = (a + b);
}
INLINE cscalar3 __device__ __HOST_V__ operator +=(cscalar3& a, cscalar3 b)
{
	return a = (a + b);
}
INLINE cscalar3 __device__ __HOST_V__ operator +=(cscalar3& a, cscalar b)
{
	return a = (a + b);
}
INLINE cscalar4 __device__ __HOST_V__ operator +=(cscalar4& a, cscalar4 b)
{
	return a = (a + b);
}
INLINE cscalar4 __device__ __HOST_V__ operator +=(cscalar4& a, cscalar b)
{
	return a = (a + b);
}
template <int N>
INLINE cscalarN<N> __device__ __HOST_V__ operator +=(cscalarN<N>& a, cscalarN<N> b)
{
	return a = (a + b);
}
template <int N> INLINE cscalarN<N> __device__ __HOST_V__ operator +=(cscalarN<N>& a, cscalar b)
{
	return a = (a + b);
}
INLINE scalar1 __device__ __HOST_V__ operator -=(scalar1& a, scalar1 b)
{
	return a = (a - b);
}
INLINE scalar1 __device__ __HOST_V__ operator -=(scalar1& a, scalar b)
{
	return a = (a - b);
}
INLINE scalar2 __device__ __HOST_V__ operator -=(scalar2& a, scalar2 b)
{
	return a = (a - b);
}
INLINE scalar2 __device__ __HOST_V__ operator -=(scalar2& a, scalar b)
{
	return a = (a - b);
}
INLINE scalar3 __device__ __HOST_V__ operator -=(scalar3& a, scalar3 b)
{
	return a = (a - b);
}
INLINE scalar3 __device__ __HOST_V__ operator -=(scalar3& a, scalar b)
{
	return a = (a - b);
}
INLINE scalar4 __device__ __HOST_V__ operator -=(scalar4& a, scalar4 b)
{
	return a = (a - b);
}
INLINE scalar4 __device__ __HOST_V__ operator -=(scalar4& a, scalar b)
{
	return a = (a - b);
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ operator -=(scalarN<N>& a, scalarN<N> b)
{
	return a = (a - b);
}
template <int N> INLINE scalarN<N> __device__ __HOST_V__ operator -=(scalarN<N>& a, scalar b)
{
	return a = (a - b);
}
INLINE int1 __device__ __HOST_V__ operator -=(int1& a, int1 b)
{
	return a = (a - b);
}
INLINE int1 __device__ __HOST_V__ operator -=(int1& a, int b)
{
	return a = (a - b);
}
INLINE int2 __device__ __HOST_V__ operator -=(int2& a, int2 b)
{
	return a = (a - b);
}
INLINE int2 __device__ __HOST_V__ operator -=(int2& a, int b)
{
	return a = (a - b);
}
INLINE int3 __device__ __HOST_V__ operator -=(int3& a, int3 b)
{
	return a = (a - b);
}
INLINE int3 __device__ __HOST_V__ operator -=(int3& a, int b)
{
	return a = (a - b);
}
INLINE int4 __device__ __HOST_V__ operator -=(int4& a, int4 b)
{
	return a = (a - b);
}
INLINE int4 __device__ __HOST_V__ operator -=(int4& a, int b)
{
	return a = (a - b);
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ operator -=(intN<N>& a, intN<N> b)
{
	return a = (a - b);
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator -=(intN<N>& a, int b)
{
	return a = (a - b);
}
INLINE cscalar1 __device__ __HOST_V__ operator -=(cscalar1& a, cscalar1 b)
{
	return a = (a - b);
}
INLINE cscalar1 __device__ __HOST_V__ operator -=(cscalar1& a, cscalar b)
{
	return a = (a - b);
}
INLINE cscalar2 __device__ __HOST_V__ operator -=(cscalar2& a, cscalar2 b)
{
	return a = (a - b);
}
INLINE cscalar2 __device__ __HOST_V__ operator -=(cscalar2& a, cscalar b)
{
	return a = (a - b);
}
INLINE cscalar3 __device__ __HOST_V__ operator -=(cscalar3& a, cscalar3 b)
{
	return a = (a - b);
}
INLINE cscalar3 __device__ __HOST_V__ operator -=(cscalar3& a, cscalar b)
{
	return a = (a - b);
}
INLINE cscalar4 __device__ __HOST_V__ operator -=(cscalar4& a, cscalar4 b)
{
	return a = (a - b);
}
INLINE cscalar4 __device__ __HOST_V__ operator -=(cscalar4& a, cscalar b)
{
	return a = (a - b);
}
template <int N>
INLINE cscalarN<N> __device__ __HOST_V__ operator -=(cscalarN<N>& a, cscalarN<N> b)
{
	return a = (a - b);
}
template <int N> INLINE cscalarN<N> __device__ __HOST_V__ operator -=(cscalarN<N>& a, cscalar b)
{
	return a = (a - b);
}
INLINE scalar1 __device__ __HOST_V__ operator *=(scalar1& a, scalar1 b)
{
	return a = (a * b);
}
INLINE scalar1 __device__ __HOST_V__ operator *=(scalar1& a, scalar b)
{
	return a = (a * b);
}
INLINE scalar2 __device__ __HOST_V__ operator *=(scalar2& a, scalar2 b)
{
	return a = (a * b);
}
INLINE scalar2 __device__ __HOST_V__ operator *=(scalar2& a, scalar b)
{
	return a = (a * b);
}
INLINE scalar3 __device__ __HOST_V__ operator *=(scalar3& a, scalar3 b)
{
	return a = (a * b);
}
INLINE scalar3 __device__ __HOST_V__ operator *=(scalar3& a, scalar b)
{
	return a = (a * b);
}
INLINE scalar4 __device__ __HOST_V__ operator *=(scalar4& a, scalar4 b)
{
	return a = (a * b);
}
INLINE scalar4 __device__ __HOST_V__ operator *=(scalar4& a, scalar b)
{
	return a = (a * b);
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ operator *=(scalarN<N>& a, scalarN<N> b)
{
	return a = (a * b);
}
template <int N> INLINE scalarN<N> __device__ __HOST_V__ operator *=(scalarN<N>& a, scalar b)
{
	return a = (a * b);
}
INLINE int1 __device__ __HOST_V__ operator *=(int1& a, int1 b)
{
	return a = (a * b);
}
INLINE int1 __device__ __HOST_V__ operator *=(int1& a, int b)
{
	return a = (a * b);
}
INLINE int2 __device__ __HOST_V__ operator *=(int2& a, int2 b)
{
	return a = (a * b);
}
INLINE int2 __device__ __HOST_V__ operator *=(int2& a, int b)
{
	return a = (a * b);
}
INLINE int3 __device__ __HOST_V__ operator *=(int3& a, int3 b)
{
	return a = (a * b);
}
INLINE int3 __device__ __HOST_V__ operator *=(int3& a, int b)
{
	return a = (a * b);
}
INLINE int4 __device__ __HOST_V__ operator *=(int4& a, int4 b)
{
	return a = (a * b);
}
INLINE int4 __device__ __HOST_V__ operator *=(int4& a, int b)
{
	return a = (a * b);
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ operator *=(intN<N>& a, intN<N> b)
{
	return a = (a * b);
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator *=(intN<N>& a, int b)
{
	return a = (a * b);
}
INLINE cscalar1 __device__ __HOST_V__ operator *=(cscalar1& a, cscalar1 b)
{
	return a = (a * b);
}
INLINE cscalar1 __device__ __HOST_V__ operator *=(cscalar1& a, cscalar b)
{
	return a = (a * b);
}
INLINE cscalar2 __device__ __HOST_V__ operator *=(cscalar2& a, cscalar2 b)
{
	return a = (a * b);
}
INLINE cscalar2 __device__ __HOST_V__ operator *=(cscalar2& a, cscalar b)
{
	return a = (a * b);
}
INLINE cscalar3 __device__ __HOST_V__ operator *=(cscalar3& a, cscalar3 b)
{
	return a = (a * b);
}
INLINE cscalar3 __device__ __HOST_V__ operator *=(cscalar3& a, cscalar b)
{
	return a = (a * b);
}
INLINE cscalar4 __device__ __HOST_V__ operator *=(cscalar4& a, cscalar4 b)
{
	return a = (a * b);
}
INLINE cscalar4 __device__ __HOST_V__ operator *=(cscalar4& a, cscalar b)
{
	return a = (a * b);
}
template <int N>
INLINE cscalarN<N> __device__ __HOST_V__ operator *=(cscalarN<N>& a, cscalarN<N> b)
{
	return a = (a * b);
}
template <int N> INLINE cscalarN<N> __device__ __HOST_V__ operator *=(cscalarN<N>& a, cscalar b)
{
	return a = (a * b);
}
INLINE scalar1 __device__ __HOST_V__ operator /=(scalar1& a, scalar1 b)
{
	return a = (a / b);
}
INLINE scalar1 __device__ __HOST_V__ operator /=(scalar1& a, scalar b)
{
	return a = (a / b);
}
INLINE scalar2 __device__ __HOST_V__ operator /=(scalar2& a, scalar2 b)
{
	return a = (a / b);
}
INLINE scalar2 __device__ __HOST_V__ operator /=(scalar2& a, scalar b)
{
	return a = (a / b);
}
INLINE scalar3 __device__ __HOST_V__ operator /=(scalar3& a, scalar3 b)
{
	return a = (a / b);
}
INLINE scalar3 __device__ __HOST_V__ operator /=(scalar3& a, scalar b)
{
	return a = (a / b);
}
INLINE scalar4 __device__ __HOST_V__ operator /=(scalar4& a, scalar4 b)
{
	return a = (a / b);
}
INLINE scalar4 __device__ __HOST_V__ operator /=(scalar4& a, scalar b)
{
	return a = (a / b);
}
template <int N>
INLINE scalarN<N> __device__ __HOST_V__ operator /=(scalarN<N>& a, scalarN<N> b)
{
	return a = (a / b);
}
template <int N> INLINE scalarN<N> __device__ __HOST_V__ operator /=(scalarN<N>& a, scalar b)
{
	return a = (a / b);
}
INLINE int1 __device__ __HOST_V__ operator /=(int1& a, int1 b)
{
	return a = (a / b);
}
INLINE int1 __device__ __HOST_V__ operator /=(int1& a, int b)
{
	return a = (a / b);
}
INLINE int2 __device__ __HOST_V__ operator /=(int2& a, int2 b)
{
	return a = (a / b);
}
INLINE int2 __device__ __HOST_V__ operator /=(int2& a, int b)
{
	return a = (a / b);
}
INLINE int3 __device__ __HOST_V__ operator /=(int3& a, int3 b)
{
	return a = (a / b);
}
INLINE int3 __device__ __HOST_V__ operator /=(int3& a, int b)
{
	return a = (a / b);
}
INLINE int4 __device__ __HOST_V__ operator /=(int4& a, int4 b)
{
	return a = (a / b);
}
INLINE int4 __device__ __HOST_V__ operator /=(int4& a, int b)
{
	return a = (a / b);
}
template <int N>
INLINE intN<N> __device__ __HOST_V__ operator /=(intN<N>& a, intN<N> b)
{
	return a = (a / b);
}
template <int N> INLINE intN<N> __device__ __HOST_V__ operator /=(intN<N>& a, int b)
{
	return a = (a / b);
}
INLINE cscalar1 __device__ __HOST_V__ operator /=(cscalar1& a, cscalar1 b)
{
	return a = (a / b);
}
INLINE cscalar1 __device__ __HOST_V__ operator /=(cscalar1& a, cscalar b)
{
	return a = (a / b);
}
INLINE cscalar2 __device__ __HOST_V__ operator /=(cscalar2& a, cscalar2 b)
{
	return a = (a / b);
}
INLINE cscalar2 __device__ __HOST_V__ operator /=(cscalar2& a, cscalar b)
{
	return a = (a / b);
}
INLINE cscalar3 __device__ __HOST_V__ operator /=(cscalar3& a, cscalar3 b)
{
	return a = (a / b);
}
INLINE cscalar3 __device__ __HOST_V__ operator /=(cscalar3& a, cscalar b)
{
	return a = (a / b);
}
INLINE cscalar4 __device__ __HOST_V__ operator /=(cscalar4& a, cscalar4 b)
{
	return a = (a / b);
}
INLINE cscalar4 __device__ __HOST_V__ operator /=(cscalar4& a, cscalar b)
{
	return a = (a / b);
}
template <int N>
INLINE cscalarN<N> __device__ __HOST_V__ operator /=(cscalarN<N>& a, cscalarN<N> b)
{
	return a = (a / b);
}
template <int N> INLINE cscalarN<N> __device__ __HOST_V__ operator /=(cscalarN<N>& a, cscalar b)
{
	return a = (a / b);
}
//==================================================
// CUDA SIMD operations
//==================================================
#if defined(TARGET_CUDA) && __CUDA_ARCH__ >= 650

INLINE __device__ uint broadcast_u8(uchar a)
{
	return ((uint)(a) | ((uint)(a) << 8) | ((uint)(a) << 16) | ((uint)(a) << 24));
}
INLINE __device__ uint broadcast_u16(ushort a)
{
	return ((uint)(a) | ((uint)(a) << 16));
}

INLINE __HOST_V__ __device__ vecTN<uchar, 4> operator-(vecTN<uchar, 4> a)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vneg4((unsigned int &)a);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<uchar, 4> operator-(vecTN<uchar, 4> a, vecTN<uchar, 4> b)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vsub4((unsigned int &)a, (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<uchar, 4> operator-(int a, vecTN<uchar, 4> b)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vsub4(broadcast_u8(a), (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<uchar, 4> operator-(vecTN<uchar, 4> a, int b)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vsub4((unsigned int &)a, broadcast_u8(b));
	return c;
}
INLINE __HOST_V__ __device__ vecTN<uchar, 4> operator+(vecTN<uchar, 4> a, vecTN<uchar, 4> b)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vadd4((unsigned int &)a, (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<uchar, 4> operator+(int a, vecTN<uchar, 4> b)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vadd4(broadcast_u8(a), (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<uchar, 4> operator+(vecTN<uchar, 4> a, int b)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vadd4((unsigned int &)a, broadcast_u8(b));
	return c;
}
INLINE __HOST_V__ __device__ vecTN<uchar, 4> operator ==(vecTN<uchar, 4> a, vecTN<uchar, 4> b)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vcmpeq4((unsigned int &)a, (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<uchar, 4> operator ==(int a, vecTN<uchar, 4> b)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vcmpeq4(broadcast_u8(a), (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<uchar, 4> operator ==(vecTN<uchar, 4> a, int b)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vcmpeq4((unsigned int &)a, broadcast_u8(b)) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<uchar, 4> operator !=(vecTN<uchar, 4> a, vecTN<uchar, 4> b)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vcmpne4((unsigned int &)a, (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<uchar, 4> operator !=(int a, vecTN<uchar, 4> b)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vcmpne4(broadcast_u8(a), (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<uchar, 4> operator !=(vecTN<uchar, 4> a, int b)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vcmpne4((unsigned int &)a, broadcast_u8(b)) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<uchar, 4> operator <(vecTN<uchar, 4> a, vecTN<uchar, 4> b)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vcmpltu4((unsigned int &)a, (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<uchar, 4> operator <(int a, vecTN<uchar, 4> b)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vcmpltu4(broadcast_u8(a), (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<uchar, 4> operator <(vecTN<uchar, 4> a, int b)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vcmpltu4((unsigned int &)a, broadcast_u8(b)) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<uchar, 4> operator >(vecTN<uchar, 4> a, vecTN<uchar, 4> b)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vcmpgtu4((unsigned int &)a, (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<uchar, 4> operator >(int a, vecTN<uchar, 4> b)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vcmpgtu4(broadcast_u8(a), (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<uchar, 4> operator >(vecTN<uchar, 4> a, int b)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vcmpgtu4((unsigned int &)a, broadcast_u8(b)) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<uchar, 4> operator <=(vecTN<uchar, 4> a, vecTN<uchar, 4> b)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vcmpleu4((unsigned int &)a, (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<uchar, 4> operator <=(int a, vecTN<uchar, 4> b)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vcmpleu4(broadcast_u8(a), (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<uchar, 4> operator <=(vecTN<uchar, 4> a, int b)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vcmpleu4((unsigned int &)a, broadcast_u8(b)) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<uchar, 4> operator >=(vecTN<uchar, 4> a, vecTN<uchar, 4> b)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vcmpgeu4((unsigned int &)a, (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<uchar, 4> operator >=(int a, vecTN<uchar, 4> b)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vcmpgeu4(broadcast_u8(a), (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<uchar, 4> operator >=(vecTN<uchar, 4> a, int b)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vcmpgeu4((unsigned int &)a, broadcast_u8(b)) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<uchar, 4> min(vecTN<uchar, 4> a, vecTN<uchar, 4> b)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vminu4((unsigned int &)a, (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<uchar, 4> min(int a, vecTN<uchar, 4> b)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vminu4(broadcast_u8(a), (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<uchar, 4> min(vecTN<uchar, 4> a, int b)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vminu4((unsigned int &)a, broadcast_u8(b));
	return c;
}
INLINE __HOST_V__ __device__ vecTN<uchar, 4> max(vecTN<uchar, 4> a, vecTN<uchar, 4> b)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vmaxu4((unsigned int &)a, (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<uchar, 4> max(int a, vecTN<uchar, 4> b)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vmaxu4(broadcast_u8(a), (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<uchar, 4> max(vecTN<uchar, 4> a, int b)
{
	vecTN<uchar, 4> c;
	(unsigned int &)c = __vmaxu4((unsigned int &)a, broadcast_u8(b));
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> operator-(vecTN<char, 4> a)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vneg4((unsigned int &)a);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> operator-(vecTN<char, 4> a, vecTN<char, 4> b)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vsub4((unsigned int &)a, (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> operator-(int a, vecTN<char, 4> b)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vsub4(broadcast_u8(a), (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> operator-(vecTN<char, 4> a, int b)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vsub4((unsigned int &)a, broadcast_u8(b));
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> operator+(vecTN<char, 4> a, vecTN<char, 4> b)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vadd4((unsigned int &)a, (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> operator+(int a, vecTN<char, 4> b)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vadd4(broadcast_u8(a), (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> operator+(vecTN<char, 4> a, int b)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vadd4((unsigned int &)a, broadcast_u8(b));
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> operator ==(vecTN<char, 4> a, vecTN<char, 4> b)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vcmpeq4((unsigned int &)a, (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> operator ==(int a, vecTN<char, 4> b)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vcmpeq4(broadcast_u8(a), (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> operator ==(vecTN<char, 4> a, int b)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vcmpeq4((unsigned int &)a, broadcast_u8(b)) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> operator !=(vecTN<char, 4> a, vecTN<char, 4> b)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vcmpne4((unsigned int &)a, (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> operator !=(int a, vecTN<char, 4> b)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vcmpne4(broadcast_u8(a), (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> operator !=(vecTN<char, 4> a, int b)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vcmpne4((unsigned int &)a, broadcast_u8(b)) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> operator <(vecTN<char, 4> a, vecTN<char, 4> b)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vcmplts4((unsigned int &)a, (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> operator <(int a, vecTN<char, 4> b)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vcmplts4(broadcast_u8(a), (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> operator <(vecTN<char, 4> a, int b)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vcmplts4((unsigned int &)a, broadcast_u8(b)) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> operator >(vecTN<char, 4> a, vecTN<char, 4> b)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vcmpgts4((unsigned int &)a, (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> operator >(int a, vecTN<char, 4> b)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vcmpgts4(broadcast_u8(a), (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> operator >(vecTN<char, 4> a, int b)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vcmpgts4((unsigned int &)a, broadcast_u8(b)) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> operator <=(vecTN<char, 4> a, vecTN<char, 4> b)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vcmples4((unsigned int &)a, (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> operator <=(int a, vecTN<char, 4> b)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vcmples4(broadcast_u8(a), (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> operator <=(vecTN<char, 4> a, int b)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vcmples4((unsigned int &)a, broadcast_u8(b)) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> operator >=(vecTN<char, 4> a, vecTN<char, 4> b)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vcmpges4((unsigned int &)a, (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> operator >=(int a, vecTN<char, 4> b)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vcmpges4(broadcast_u8(a), (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> operator >=(vecTN<char, 4> a, int b)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vcmpges4((unsigned int &)a, broadcast_u8(b)) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> min(vecTN<char, 4> a, vecTN<char, 4> b)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vmins4((unsigned int &)a, (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> min(int a, vecTN<char, 4> b)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vmins4(broadcast_u8(a), (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> min(vecTN<char, 4> a, int b)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vmins4((unsigned int &)a, broadcast_u8(b));
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> max(vecTN<char, 4> a, vecTN<char, 4> b)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vmaxs4((unsigned int &)a, (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> max(int a, vecTN<char, 4> b)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vmaxs4(broadcast_u8(a), (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> max(vecTN<char, 4> a, int b)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vmaxs4((unsigned int &)a, broadcast_u8(b));
	return c;
}
INLINE __HOST_V__ __device__ vecTN<char, 4> abs(vecTN<char, 4> a)
{
	vecTN<char, 4> c;
	(unsigned int &)c = __vabs4((unsigned int &)a);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> operator-(vecTN<ushort, 2> a)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vneg2((unsigned int &)a);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> operator-(vecTN<ushort, 2> a, vecTN<ushort, 2> b)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vsub2((unsigned int &)a, (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> operator-(int a, vecTN<ushort, 2> b)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vsub2(broadcast_u8(a), (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> operator-(vecTN<ushort, 2> a, int b)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vsub2((unsigned int &)a, broadcast_u8(b));
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> operator+(vecTN<ushort, 2> a, vecTN<ushort, 2> b)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vadd2((unsigned int &)a, (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> operator+(int a, vecTN<ushort, 2> b)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vadd2(broadcast_u8(a), (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> operator+(vecTN<ushort, 2> a, int b)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vadd2((unsigned int &)a, broadcast_u8(b));
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> operator ==(vecTN<ushort, 2> a, vecTN<ushort, 2> b)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vcmpeq2((unsigned int &)a, (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> operator ==(int a, vecTN<ushort, 2> b)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vcmpeq2(broadcast_u8(a), (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> operator ==(vecTN<ushort, 2> a, int b)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vcmpeq2((unsigned int &)a, broadcast_u8(b)) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> operator !=(vecTN<ushort, 2> a, vecTN<ushort, 2> b)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vcmpne2((unsigned int &)a, (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> operator !=(int a, vecTN<ushort, 2> b)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vcmpne2(broadcast_u8(a), (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> operator !=(vecTN<ushort, 2> a, int b)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vcmpne2((unsigned int &)a, broadcast_u8(b)) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> operator <(vecTN<ushort, 2> a, vecTN<ushort, 2> b)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vcmpltu2((unsigned int &)a, (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> operator <(int a, vecTN<ushort, 2> b)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vcmpltu2(broadcast_u8(a), (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> operator <(vecTN<ushort, 2> a, int b)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vcmpltu2((unsigned int &)a, broadcast_u8(b)) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> operator >(vecTN<ushort, 2> a, vecTN<ushort, 2> b)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vcmpgtu2((unsigned int &)a, (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> operator >(int a, vecTN<ushort, 2> b)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vcmpgtu2(broadcast_u8(a), (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> operator >(vecTN<ushort, 2> a, int b)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vcmpgtu2((unsigned int &)a, broadcast_u8(b)) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> operator <=(vecTN<ushort, 2> a, vecTN<ushort, 2> b)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vcmpleu2((unsigned int &)a, (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> operator <=(int a, vecTN<ushort, 2> b)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vcmpleu2(broadcast_u8(a), (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> operator <=(vecTN<ushort, 2> a, int b)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vcmpleu2((unsigned int &)a, broadcast_u8(b)) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> operator >=(vecTN<ushort, 2> a, vecTN<ushort, 2> b)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vcmpgeu2((unsigned int &)a, (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> operator >=(int a, vecTN<ushort, 2> b)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vcmpgeu2(broadcast_u8(a), (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> operator >=(vecTN<ushort, 2> a, int b)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vcmpgeu2((unsigned int &)a, broadcast_u8(b)) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> min(vecTN<ushort, 2> a, vecTN<ushort, 2> b)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vminu2((unsigned int &)a, (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> min(int a, vecTN<ushort, 2> b)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vminu2(broadcast_u8(a), (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> min(vecTN<ushort, 2> a, int b)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vminu2((unsigned int &)a, broadcast_u8(b));
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> max(vecTN<ushort, 2> a, vecTN<ushort, 2> b)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vmaxu2((unsigned int &)a, (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> max(int a, vecTN<ushort, 2> b)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vmaxu2(broadcast_u8(a), (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<ushort, 2> max(vecTN<ushort, 2> a, int b)
{
	vecTN<ushort, 2> c;
	(unsigned int &)c = __vmaxu2((unsigned int &)a, broadcast_u8(b));
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> operator-(vecTN<short, 2> a)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vneg2((unsigned int &)a);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> operator-(vecTN<short, 2> a, vecTN<short, 2> b)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vsub2((unsigned int &)a, (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> operator-(int a, vecTN<short, 2> b)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vsub2(broadcast_u8(a), (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> operator-(vecTN<short, 2> a, int b)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vsub2((unsigned int &)a, broadcast_u8(b));
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> operator+(vecTN<short, 2> a, vecTN<short, 2> b)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vadd2((unsigned int &)a, (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> operator+(int a, vecTN<short, 2> b)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vadd2(broadcast_u8(a), (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> operator+(vecTN<short, 2> a, int b)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vadd2((unsigned int &)a, broadcast_u8(b));
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> operator ==(vecTN<short, 2> a, vecTN<short, 2> b)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vcmpeq2((unsigned int &)a, (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> operator ==(int a, vecTN<short, 2> b)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vcmpeq2(broadcast_u8(a), (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> operator ==(vecTN<short, 2> a, int b)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vcmpeq2((unsigned int &)a, broadcast_u8(b)) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> operator !=(vecTN<short, 2> a, vecTN<short, 2> b)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vcmpne2((unsigned int &)a, (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> operator !=(int a, vecTN<short, 2> b)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vcmpne2(broadcast_u8(a), (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> operator !=(vecTN<short, 2> a, int b)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vcmpne2((unsigned int &)a, broadcast_u8(b)) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> operator <(vecTN<short, 2> a, vecTN<short, 2> b)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vcmplts2((unsigned int &)a, (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> operator <(int a, vecTN<short, 2> b)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vcmplts2(broadcast_u8(a), (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> operator <(vecTN<short, 2> a, int b)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vcmplts2((unsigned int &)a, broadcast_u8(b)) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> operator >(vecTN<short, 2> a, vecTN<short, 2> b)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vcmpgts2((unsigned int &)a, (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> operator >(int a, vecTN<short, 2> b)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vcmpgts2(broadcast_u8(a), (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> operator >(vecTN<short, 2> a, int b)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vcmpgts2((unsigned int &)a, broadcast_u8(b)) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> operator <=(vecTN<short, 2> a, vecTN<short, 2> b)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vcmples2((unsigned int &)a, (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> operator <=(int a, vecTN<short, 2> b)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vcmples2(broadcast_u8(a), (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> operator <=(vecTN<short, 2> a, int b)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vcmples2((unsigned int &)a, broadcast_u8(b)) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> operator >=(vecTN<short, 2> a, vecTN<short, 2> b)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vcmpges2((unsigned int &)a, (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> operator >=(int a, vecTN<short, 2> b)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vcmpges2(broadcast_u8(a), (unsigned int &)b) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> operator >=(vecTN<short, 2> a, int b)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vcmpges2((unsigned int &)a, broadcast_u8(b)) & 0x01010101;
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> min(vecTN<short, 2> a, vecTN<short, 2> b)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vmins2((unsigned int &)a, (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> min(int a, vecTN<short, 2> b)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vmins2(broadcast_u8(a), (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> min(vecTN<short, 2> a, int b)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vmins2((unsigned int &)a, broadcast_u8(b));
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> max(vecTN<short, 2> a, vecTN<short, 2> b)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vmaxs2((unsigned int &)a, (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> max(int a, vecTN<short, 2> b)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vmaxs2(broadcast_u8(a), (unsigned int &)b);
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> max(vecTN<short, 2> a, int b)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vmaxs2((unsigned int &)a, broadcast_u8(b));
	return c;
}
INLINE __HOST_V__ __device__ vecTN<short, 2> abs(vecTN<short, 2> a)
{
	vecTN<short, 2> c;
	(unsigned int &)c = __vabs2((unsigned int &)a);
	return c;
}
#endif	// TARGET_CUDA
#ifdef __CUDA_NO_HALF_OPERATORS__
INLINE __device__ __half operator -(__half a)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __hneg(a);
#else
	return float2half_rn(- (float)a);
#endif
}
#endif
#ifdef __CUDA_NO_HALF_OPERATORS__
INLINE __device__ __half operator -(__half a, __half b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __hsub(a, b);
#else
	return float2half_rn((float)a - (float)b);
#endif
}
#endif
#ifdef __CUDA_NO_HALF_OPERATORS__
INLINE __device__ __half operator +(__half a, __half b)
{
#ifdef HAS_CUDA_HALF_SUPPORT
	return __hadd(a, b);
#else
	return float2half_rn((float)a + (float)b);
#endif
}
#endif
#ifdef __CUDA_NO_HALF_OPERATORS__
INLINE __device__ __half operator *(__half a, __half b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __hmul(a, b);
#else
	return float2half_rn((float)a * (float)b);
#endif
}
#endif
#ifdef __CUDA_NO_HALF_OPERATORS__
INLINE __device__ __half operator /(__half a, __half b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __hdiv(a, b);
#else
	return float2half_rn((float)a / (float)b);
#endif
}
#endif
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator-(vecTN<__half, 2> a)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __hneg2((__half2)a);
#else
	return floats2half2_rn(-half2float(a[0]), -half2float(a[1]));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator-(vecTN<__half, 2> a, vecTN<__half, 2> b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __hsub2((__half2)a, (__half2)b);
#else
	return floats2half2_rn(half2float(a[0]) - half2float(b[0]), half2float(a[1]) - half2float(b[1]));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator-(scalar a, vecTN<__half, 2> b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __hsub2(floats2half2_rn(a, a), (__half2)b);
#else
	return floats2half2_rn(a - half2float(b[0]), a - half2float(b[1]));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator-(vecTN<__half, 2> a, scalar b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __hsub2((__half2)a, floats2half2_rn(b, b));
#else
	return floats2half2_rn(half2float(a[0]) - b, half2float(a[1]) - b);
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator+(vecTN<__half, 2> a, vecTN<__half, 2> b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __hadd2((__half2)a, (__half2)b);
#else
	return floats2half2_rn(half2float(a[0]) + half2float(b[0]), half2float(a[1]) + half2float(b[1]));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator+(scalar a, vecTN<__half, 2> b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __hadd2(floats2half2_rn(a, a), (__half2)b);
#else
	return floats2half2_rn(a + half2float(b[0]), a + half2float(b[1]));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator+(vecTN<__half, 2> a, scalar b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __hadd2((__half2)a, floats2half2_rn(b, b));
#else
	return floats2half2_rn(half2float(a[0]) + b, half2float(a[1]) + b);
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator*(vecTN<__half, 2> a, vecTN<__half, 2> b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __hmul2((__half2)a, (__half2)b);
#else
	return floats2half2_rn(half2float(a[0]) * half2float(b[0]), half2float(a[1]) * half2float(b[1]));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator*(scalar a, vecTN<__half, 2> b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __hmul2(floats2half2_rn(a, a), (__half2)b);
#else
	return floats2half2_rn(a * half2float(b[0]), a * half2float(b[1]));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator*(vecTN<__half, 2> a, scalar b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __hmul2((__half2)a, floats2half2_rn(b, b));
#else
	return floats2half2_rn(half2float(a[0]) * b, half2float(a[1]) * b);
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator/(vecTN<__half, 2> a, vecTN<__half, 2> b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __h2div((__half2)a, (__half2)b);
#else
	return floats2half2_rn(half2float(a[0]) / half2float(b[0]), half2float(a[1]) / half2float(b[1]));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator/(scalar a, vecTN<__half, 2> b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __h2div(floats2half2_rn(a, a), (__half2)b);
#else
	return floats2half2_rn(a / half2float(b[0]), a / half2float(b[1]));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator/(vecTN<__half, 2> a, scalar b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __h2div((__half2)a, floats2half2_rn(b, b));
#else
	return floats2half2_rn(half2float(a[0]) / b, half2float(a[1]) / b);
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator==(vecTN<__half, 2> a, vecTN<__half, 2> b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __heq2((__half2)a, (__half2)b);
#else
	return floats2half2_rn(half2float(a[0]) == half2float(b[0]), half2float(a[1]) == half2float(b[1]));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator==(scalar a, vecTN<__half, 2> b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __heq2(floats2half2_rn(a, a), (__half2)b);
#else
	return floats2half2_rn(a == half2float(b[0]), a == half2float(b[1]));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator==(vecTN<__half, 2> a, scalar b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __heq2((__half2)a, floats2half2_rn(b, b));
#else
	return floats2half2_rn(half2float(a[0]) == b, half2float(a[1]) == b);
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator!=(vecTN<__half, 2> a, vecTN<__half, 2> b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __hne2((__half2)a, (__half2)b);
#else
	return floats2half2_rn(half2float(a[0]) != half2float(b[0]), half2float(a[1]) != half2float(b[1]));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator!=(scalar a, vecTN<__half, 2> b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __hne2(floats2half2_rn(a, a), (__half2)b);
#else
	return floats2half2_rn(a != half2float(b[0]), a != half2float(b[1]));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator!=(vecTN<__half, 2> a, scalar b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __hne2((__half2)a, floats2half2_rn(b, b));
#else
	return floats2half2_rn(half2float(a[0]) != b, half2float(a[1]) != b);
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator<(vecTN<__half, 2> a, vecTN<__half, 2> b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __hlt2((__half2)a, (__half2)b);
#else
	return floats2half2_rn(half2float(a[0]) < half2float(b[0]), half2float(a[1]) < half2float(b[1]));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator<(scalar a, vecTN<__half, 2> b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __hlt2(floats2half2_rn(a, a), (__half2)b);
#else
	return floats2half2_rn(a < half2float(b[0]), a < half2float(b[1]));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator<(vecTN<__half, 2> a, scalar b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __hlt2((__half2)a, floats2half2_rn(b, b));
#else
	return floats2half2_rn(half2float(a[0]) < b, half2float(a[1]) < b);
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator>(vecTN<__half, 2> a, vecTN<__half, 2> b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __hgt2((__half2)a, (__half2)b);
#else
	return floats2half2_rn(half2float(a[0]) > half2float(b[0]), half2float(a[1]) > half2float(b[1]));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator>(scalar a, vecTN<__half, 2> b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __hgt2(floats2half2_rn(a, a), (__half2)b);
#else
	return floats2half2_rn(a > half2float(b[0]), a > half2float(b[1]));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator>(vecTN<__half, 2> a, scalar b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __hgt2((__half2)a, floats2half2_rn(b, b));
#else
	return floats2half2_rn(half2float(a[0]) > b, half2float(a[1]) > b);
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator<=(vecTN<__half, 2> a, vecTN<__half, 2> b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __hle2((__half2)a, (__half2)b);
#else
	return floats2half2_rn(half2float(a[0]) <= half2float(b[0]), half2float(a[1]) <= half2float(b[1]));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator<=(scalar a, vecTN<__half, 2> b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __hle2(floats2half2_rn(a, a), (__half2)b);
#else
	return floats2half2_rn(a <= half2float(b[0]), a <= half2float(b[1]));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator<=(vecTN<__half, 2> a, scalar b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __hle2((__half2)a, floats2half2_rn(b, b));
#else
	return floats2half2_rn(half2float(a[0]) <= b, half2float(a[1]) <= b);
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator>=(vecTN<__half, 2> a, vecTN<__half, 2> b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __hge2((__half2)a, (__half2)b);
#else
	return floats2half2_rn(half2float(a[0]) >= half2float(b[0]), half2float(a[1]) >= half2float(b[1]));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator>=(scalar a, vecTN<__half, 2> b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __hge2(floats2half2_rn(a, a), (__half2)b);
#else
	return floats2half2_rn(a >= half2float(b[0]), a >= half2float(b[1]));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> operator>=(vecTN<__half, 2> a, scalar b)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return __hge2((__half2)a, floats2half2_rn(b, b));
#else
	return floats2half2_rn(half2float(a[0]) >= b, half2float(a[1]) >= b);
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> sin(vecTN<__half, 2> a)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return h2sin((__half2)a);
#else
	return floats2half2_rn(sin(half2float(a[0])), sin(half2float(a[1])));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> cos(vecTN<__half, 2> a)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return h2cos((__half2)a);
#else
	return floats2half2_rn(cos(half2float(a[0])), cos(half2float(a[1])));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> exp(vecTN<__half, 2> a)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return h2exp((__half2)a);
#else
	return floats2half2_rn(exp(half2float(a[0])), exp(half2float(a[1])));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> exp2(vecTN<__half, 2> a)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return h2exp2((__half2)a);
#else
	return floats2half2_rn(exp2(half2float(a[0])), exp2(half2float(a[1])));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> floor(vecTN<__half, 2> a)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return h2floor((__half2)a);
#else
	return floats2half2_rn(floor(half2float(a[0])), floor(half2float(a[1])));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> log(vecTN<__half, 2> a)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return h2log((__half2)a);
#else
	return floats2half2_rn(log(half2float(a[0])), log(half2float(a[1])));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> log10(vecTN<__half, 2> a)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return h2log10((__half2)a);
#else
	return floats2half2_rn(log10(half2float(a[0])), log10(half2float(a[1])));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> log2(vecTN<__half, 2> a)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return h2log2((__half2)a);
#else
	return floats2half2_rn(log2(half2float(a[0])), log2(half2float(a[1])));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> sqrt(vecTN<__half, 2> a)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return h2sqrt((__half2)a);
#else
	return floats2half2_rn(sqrt(half2float(a[0])), sqrt(half2float(a[1])));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> rsqrt(vecTN<__half, 2> a)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return h2rsqrt((__half2)a);
#else
	return floats2half2_rn(rsqrt(half2float(a[0])), rsqrt(half2float(a[1])));
#endif
}
INLINE __HOST_V__ __device__ vecTN<__half, 2> round(vecTN<__half, 2> a)
{
#if defined(HAS_CUDA_HALF_SUPPORT) && __CUDA_ARCH__ >= 530
	return h2rint((__half2)a);
#else
	return floats2half2_rn(round(half2float(a[0])), round(half2float(a[1])));
#endif
}
#endif
